/************************************************************************************/
/*																				  	*/
/*	NAME:		boot_mean_validation.mac.sas										*/
/*  TITLE: 		Bootstrap- Validierung (Bootstrap- Variante 'Mittleres Modell')		*/
/*  KEYS: 		bootstrap, validation, logistic regression, roc analysis 			*/
/*	VERSION:	SAS 8.2																*/
/* 	PRODUCT:	BASE, STAT															*/
/*  PROCS:		LOGISTIC, FREQ, MEANS												*/
/*																					*/
/*																					*/
/*	Verzeichnis/Dateiname:			F:\Prognose\boot_mean_validation.mac.sas		*/
/*  Autoren:						R. Muche, C. Ring, I. Assfalg, C. Ziegler		*/
/*	Programmierer:					Ina Assfalg/ Christoph Ziegler					*/
/*	Datum der Ersterstellung:		9/2002											*/
/*  Aktuelle Makro- Version:		12/2003											*/
/*																					*/
/*	Kurzbeschreibung:				Allgemeines Makro zur internen Validierung von 	*/
/*							    	logistischen Regressionsmodellen mittels 		*/
/*									Bootstrap (Bootstrap- Variante   				*/
/*									'Mittleres Modell')								*/
/*																					*/
/*	Hilfs-/Untermakro von:			PM_BOOTSTRAP_VALIDATION.MAC.SAS					*/
/*																					*/
/*  Genauere Beschreibung siehe:	PM_BOOTSTRAP_VALIDATION.MAC.SAS					*/
/*																					*/
/*  Verwendete externe Untermakros:	numberobs.mac.sas								*/
/*									pm_logreg.mac.sas								*/
/*									pm_roc.mac.sas									*/
/*																					*/
/*	Bentigte Dateien:				Datensatz (Parameter 'data=')					*/
/*																					*/
/*																					*/
/*  Erzeugte Dateien:				-----											*/
/*																					*/
/*  Programmnderungen 				Sandra Mller, 04.05.2016, Makrovariable		*/
/*  und -ergnzungen 				binwidth hinzugefgt							*/
/*  (Name, Datum, Beschreibung):     												*/
/*   																   			    */
/*																					*/
/************************************************************************************/

%MACRO BOOT_MEAN_VALIDATION(	data			= ,
								resp_var		= , 
								event			= 1,
								xvar			= ,
								cvar			= ,
								ref				= ,
								miss 			= 0,
								random			= 0,
								anzahl_samples	= 200,
								mean_flag		= 1,
								macro_path		= ,
								binwidth		= 0
			    			  );

/*(1) Externe Makros (Untermakros) einbinden (%include)	*/

FILENAME y &macro_path;

%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('pm_logreg.mac.sas');
%INCLUDE y ('pm_roc.mac.sas');

/* Wird auch im Untermakro %numberobs bentigt*/

%GLOBAL nnnn1;
%LET datensatz_ori=&data;


ODS SELECT NONE;

/*(2)	Erzeugen der Bootstrap-Samples								    */
/*(2.1) Makro, welches die Anzahl der Beobachtungen ausgibt (%numberobs)*/
/*      (Anzahl der Beobachtungen im bergebenen Datensatz ausgeben)    */

%NUMBEROBS (datensatz=&data);

/*(2.2) Aufruf der PROC LOGISTIC  (%logreg) fr den Originaldatenstz (OD)*/
/*	    -> Kenngren bestimmen (fr Ausgangsmodell)					 */

%PM_LOGREG ( 	data			= &data,
				resp_var		= &resp_var,
				event			= &event,
				xvar			= &xvar,
				cvar			= &cvar,
				ref				= &ref,
				fxvar			= ,
				fcvar			= ,
				miss			= &miss,
				sel				= NONE,
				sel_details		= 1,
				slstay			= 0.05,
				slentry			= 0.05,
				clodds			= 1,
				alpha			= 0.05,
				rsquare			= 1,
				ctable			= 1,
				ctable_pprob	= ,
				out_pred		= work.gof_ori,
				pred			= 1,
				predprobs_art	= INDIVIDUAL,
				roc				= 1,
				out_roc			= work.roc_ori,
				est				= 0,
				out_est			= ,
				inest_flag		= 0,
				inest_data		= ,
				boot_data		= ,
				c_wert			= 1,
				c_data			= work.association_ori,
				test			=,
				mi				= 0,
				out_mi			= work.out_mi_log,
				macro_path		= &macro_path,
				binwidth		= &binwidth
		);

/*Event-Rate der Ausgangsdatei bestimmen*/

DATA eventOri (KEEP=&resp_var);
	SET &data;
RUN; 

/*Hufigkeitsauszhlung von Event -> in Extra Datei*/

ODS LISTING CLOSE;
ODS OUTPUT OneWayFreqs=freqsOri;
	PROC FREQ DATA=eventOri;
	RUN;
ODS OUTPUT CLOSE;
ODS LISTING;

/*ROC-Analyse (OD) mit Makro %ROC*/

%PM_ROC			(
				 data				=&data,
				 resp_var			=&resp_var,
				 event				=&event,
				 data_roc			=work.roc_ori,
				 data_pred			=work.gof_ori,
				 data_asso			=work.association_ori,
				 validation			=BOOTSTRAP,
				 praevalenz			= ,
				 all_cp				=0,
			     cp					=,
				 cibinom			=0,
				 rocdat				=0,
				 auroc				=0,
				 printit			=0,
				 alpha_cl			=0.95,
				 ext				=tiffp,
				 winkel				=1,
				 point				=1,
				 path				='c:\temp',
				 mi					=0,
				 out_mi				=work.out_mi_roc,
				 macro_path			=&macro_path,
				 binwidth			=&binwidth
			    );

DM "clear log;";

/*ROC- / Sonstige Kenngren des Originaldatensatzes zusammenfassen*/

DATA result_roc_ori1;
	SET result;
	RENAME maxyouden 	= maxyouden_ori;
	RENAME _PROB_ 		= prob_ori;
	RENAME _SENSIT_		= sens_ori;
	RENAME spez 		= spez_ori;
	RENAME ppv 			= ppv_ori;
	RENAME npv 			= npv_ori;
	RENAME psep 		= psep_ori;
	dummy				= 1;
RUN;

DATA result_roc_ori2;
	SET index_ori;
	dummy				= 1;
	emax_ori			= 0;
	intercept_ori		= 0;
	slope_ori			= 1;
RUN;

/*Cutpoint des Ausgangsdatensatzes zur spteren Verwendung in einer Makrovariable */
/*zwischenspeichern*/

DATA result_roc_ori1;
	SET result_roc_ori1;
	call symput('prob_original',prob_ori);
RUN;

/*Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE result roc_ori gof_ori association_ori index_ori eventori result2;
RUN;
QUIT;

DATA boot;
	SET &data;
RUN;

/*Nullinitialisierung der Dateien die innerhalb der Schleife angelegt werden*/

DATA freqsBootGesamt;  	SET _NULL_;
DATA boot_est;  		SET _NULL_;

/*(2.3) Schleife zur Erzeugung der Bootstrap-Samples	*/

/*Schleife*/

%NUMBEROBS (datensatz=&data);

%LET samples_nnnn1=&nnnn1;
%LET boot=boot;

/* Gesamten Bootstrap-Datensatz erzeugen */
PROC SURVEYSELECT DATA=&boot OUT=boot_all
	seed=&random
	method=urs 
	samprate=1 
	outhits 
	rep=&anzahl_samples
	NOPRINT; 
RUN;

%DO km=1 %TO &anzahl_samples;
	DATA boot&km (DROP=Replicate);
	SET boot_all (WHERE=(Replicate=&km));
	RUN;

/*(2.4) Aufruf der PROC LOGISTIC (%logreg) fr jeden Bootstrap-Sample        */
/*      --> Bootstrap-Modelle (BM): insgesamt soviel BM wie Bootstrap-Samples*/

%PM_LOGREG ( 	data			= boot&km,
				resp_var		= &resp_var,
				event			= &event,
				xvar			= &xvar,
				cvar			= &cvar,
				ref				= &ref,
				fxvar			= ,
				fcvar			= ,
				miss			= &miss,
				sel				= NONE,
				sel_details		= 1,
				slstay			= 0.05,
				slentry			= 0.05,
				clodds			= 1,
				alpha			= 0.05,
				rsquare			= 1,
				ctable			= 1,
				ctable_pprob	= ,
				out_pred		= ,
				pred			= 0,
				predprobs_art	=INDIVIDUAL,
				roc				= 0,
				out_roc			= ,
				est				= 1,
				out_est			= work.boot_est&km,
				inest_flag		= 0,
				inest_data		= ,
				boot_data		= ,
				c_wert			= 0,
				c_data			= ,
				test			= ,
				mi				= 0,
				out_mi			= work.out_mi_log,
				macro_path		= &macro_path,
				binwidth		= &binwidth
		);

/*Event-Rate pro Sample bestimmen*/

DATA eventBoot&km (KEEP=&resp_var);
	SET boot&km;
RUN; 
		
ODS LISTING CLOSE;
ODS OUTPUT OneWayFreqs=freqsBoot&km;
	PROC FREQ DATA=eventBoot&km;
	RUN;
ODS OUTPUT CLOSE;
ODS LISTING;

/*Event-Raten aller Samples in einer Datei zusammengefasst*/

DATA freqsBootGesamt;
	SET freqsBootGesamt freqsBoot&km;
RUN;

DM "clear log;";

/*Die s der einzelnen Bootstrap-Samples in eine Datei zusammenfhren (Wird bei jedem*/
/*Schleifen-Durchlauf unten angehngt)*/

DATA boot_est;
	SET boot_est boot_est&km;
RUN;

PROC DATASETS NOLIST; 
		DELETE eventBoot&km freqsBoot&km boot_est&km ;

		%IF &mean_flag=1 %THEN %DO;
	
		DELETE boot&km;

		%END;

RUN; 
QUIT;

/*Ende der Bootstrap- Schleife*/

%END;

/*-----------------------------------------------------------------------------------*/

/*(3) Mittleres Modell:*/

/*(3.1) Konvergenz sicherstellen: Nur die Estimates ohne Warnung (-> 0 Converged) */
/*      werden ausgewertet -> Ergebnisdatei "&boot_est"*/

/*DATA boot_est;*/
/*	SET boot_est;*/
/*	IF _status_='0 Converged';*/
/*RUN;*/

/*Test ob es berhaupt Estimates ohne Warnung gibt:*/

/*%NUMBEROBS (datensatz=boot_est);*/
/**/
/*%let boot_est_count=&nnnn1;*/
/**/
/*%IF &boot_est_count=0 %THEN %DO;*/
/**/
/*DATA warning;*/
/*	warning='Schtzung der Modellparameter nicht gesichert. Konvergenz nicht erfllt!';*/
/*RUN;*/
/**/
/*PROC PRINT DATA=warning;*/
/*	VAR warning;*/
/*RUN;*/
/**/
/*%GOTO ende;*/
/**/
/*%END;*/

/*wenn nicht, Programm abbrechen und Meldung ausgeben (siehe Ende)*/

/*(3.2) Mittleres Modell (mit PROC MEANS) bestimmen*/

PROC MEANS NOPRINT DATA=boot_est;
	OUTPUT OUT=meansEstimates;
	TITLE1 "Datei: boot_est (Bootstrap-Zusammenfassung)";
	TITLE2 'Gemittelte Parameterschtzungen aller BM';
RUN;

/*Da nur mean interessant, alle anderen Statistiken lschen*/

DATA meansEstimates1;
	SET meansEstimates;
	IF _STAT_='N' THEN DELETE;
	IF _STAT_='MIN' THEN DELETE;
	IF _STAT_='MAX' THEN DELETE;
	IF _STAT_='STD' THEN DELETE;
RUN;

/*Datei "out_bootx" wird an das Makro %LOGREG bergeben (INEST-Datei), Dateiname in der*/
/*Form erforderlich, da im Makro %LOGREG so definiert*/

DATA out_bootx (DROP=_TYPE_ _FREQ_ _STAT_);
	SET meansEstimates1;
RUN;

/*(4) Mittleres Modell angewandt auf den Originaldatensatz (Parameter &mean_flag=1)*/
/*   (Option inest=work.out_bootx)												   */

%IF &mean_flag = 1 %THEN %DO;

/*(4.1) Aufruf der PROC LOGISTIC (%logreg) fr den Ausgangsdatensatz			*/
/*	    --> Kenngren bestimmen												*/

%PM_LOGREG ( 	data			= &data,
				resp_var		= &resp_var,
				event			= &event,
				xvar			= &xvar,
				cvar			= &cvar,
				ref				= &ref,
				fxvar			= ,
				fcvar			= ,
				miss			= &miss,
				sel				= NONE,
				sel_details		= 1,
				slstay			= 0.05,
				slentry			= 0.05,
				clodds			= 1,
				alpha			= 0.05,
				rsquare			= 1,
				ctable			= 1,
				ctable_pprob	= ,
				out_pred		= work.gof_boot,
				pred			= 1,
				predprobs_art	= INDIVIDUAL,
				roc				= 1,
				out_roc			= work.roc_boot,
				est				= 1,
				out_est			= work.estMeanOD,
				inest_flag		= 2,
				inest_data		= work.out_bootx,
				boot_data		= ,
				c_wert			= 1,
				c_data			= association_boot,
				test			= ,
				mi				= 0,
				out_mi			= work.out_mi_log,
				macro_path		= &macro_path,
				binwidth		= &binwidth
		);

/*Berechnung der Kenngren bei angewandtem Mittleren Modell auf OD. */

%PM_ROC			(
				 data				=&data,
				 resp_var			=&resp_var,
				 event				=&event,
				 data_roc			=work.roc_boot,
				 data_pred			=work.gof_boot,
				 data_asso			=work.association_boot,
				 validation			=BOOTSTRAP,
				 praevalenz			= ,
				 all_cp				=0,
			     cp					=&prob_original,
				 cibinom			=0,
				 rocdat				=0,
				 auroc				=0,
				 printit			=0,
				 alpha_cl			=0.95,
				 ext				=tiffp,
				 winkel				=1,
				 point				=1,
				 path				='c:\temp',
				 mi					=0,
				 out_mi				=work.out_mi_roc,
				 macro_path			=&macro_path,
			 	 binwidth			=&binwidth
			    );

DM "clear log;";

DATA result_roc_boot2;
	SET index_ori;
	RENAME Max_resca_ori	= max_resca_boot;
	RENAME R_Square_ori		= r_square_boot;
	RENAME brier_ori		= brier_boot;
	RENAME emax_ori			= emax_boot;
	RENAME som_ori			= som_boot;
	RENAME auc_ori			= auc_boot;
	RENAME slope_ori		= slope_boot;
	RENAME intercept_ori	= intercept_boot;
	dummy					= 1;
RUN;

/*(4.2) Optimismus der Kenngren nach Harrell (AUC, SomersD, Brier-Score, Emax) bestimmen:*/
/*	  Differenzen Ausgangsmodell - "Mittleres Modell angewandt auf OD" */

/*Mit der Ergebnisdatei des Ausgangsdatensatzes zusammenmergen;*/

DATA optimism_index;
	MERGE result_roc_ori2 result_roc_boot2;
	BY dummy;
RUN;

/*Differenzen berechnen*/

DATA optimism_index1 (DROP=dummy);
	SET optimism_index;
	variante='Mittleres Modell';
	op_auc=auc_ori-auc_boot;
	op_som=som_ori-som_boot;
	op_brier=brier_ori-brier_boot;
	op_slope=slope_ori-slope_boot;
	op_intercept=intercept_ori-intercept_boot;
	op_emax=emax_ori-emax_boot;
	op_max_resca=max_resca_ori-max_resca_boot;
	op_r_square=r_square_ori-r_square_boot;
RUN;

/*(4.3) Optimismus zustzlich fr ROC-Kenngren (Sensitivitt, Spezifitt, Youden-Index)*/
/*	    zum Cutpoint des Ausgangsmodells (Ergebnisdatei: "optimism_croc")				 */
/*	    zum jeweiligen neuen, maximalen Youden-Index (Ergebnisdatei: "optimism_yroc")    */

/*Variablen in Ergebnisdatei zusammenfassen und umbenennen*/
/*Fr angegebenen cp (hier: cp des Ausgangsmodells) result2*/

DATA result_roc_cboot;
	SET result2;
	RENAME next_cutpoint 	= prob_boot;
	RENAME _SENSIT_ 		= sens_boot;
	RENAME spez 			= spez_boot;
	RENAME youden			= youden_boot;
	RENAME ppv 				= ppv_boot;
	RENAME npv 				= npv_boot;
	RENAME psep 			= psep_boot;
	dummy					=1;
RUN;

/*Variablen in Ergebnisdatei zusammenfassen und umbenennen*/
/*Fr max. Youden des Mittleren Modells angewandt auf OD: result*/

DATA result_roc_yboot;
	SET result;
	RENAME maxyouden 		= maxyouden_boot;
	RENAME _PROB_ 			= prob_boot;
	RENAME _SENSIT_ 		= sens_boot;
	RENAME spez 			= spez_boot;
	RENAME ppv 				= ppv_boot;
	RENAME npv 				= npv_boot;
	RENAME psep				= psep_boot;
	dummy					= 1;
RUN;

/*Zum jeweiligen neuen, maximalen Youden-Index (Ergebnisdatei: "optimism_yroc")*/
/*Mit Ergebnisdatei des OD zusammenmergen	*/

DATA optimism_yroc;
	MERGE result_roc_ori1 result_roc_yboot;
	BY dummy;
RUN;

/*Differenzen berechnen*/

DATA optimism_yroc;
	SET optimism_yroc;
	op_sens			=sens_ori - sens_boot;
	op_spez			=spez_ori - spez_boot;
	op_youden		=maxyouden_ori - maxyouden_boot;
	op_ppv			=ppv_ori - ppv_boot;
	op_npv			=npv_ori - npv_boot;
	op_psep			=psep_ori - psep_boot;
RUN;

/*zum Cutpoint des Ausgangsmodells (Ergebnisdatei: "optimism_croc")*/
/*Mit Ergebnisdatei des OD zusammenmergen*/

DATA optimism_croc;
	MERGE result_roc_ori1 result_roc_cboot;
	BY dummy;
RUN;

/*Differenzen bestimmen*/

DATA optimism_croc;
	SET optimism_croc;
	op_sens			=sens_ori - sens_boot;
	op_spez			=spez_ori - spez_boot;
	op_youden		=maxyouden_ori - youden_boot;
	op_ppv			=ppv_ori - ppv_boot;
	op_npv			=npv_ori - npv_boot;
	op_psep			=psep_ori - psep_boot;

RUN;

/*Ergebnisdatei optimism_index1 fr schneren Output aufsplitten*/

/*AUC*/

DATA auc (KEEP=auc_ori auc_boot op_auc auc_percent);
	SET optimism_index1;
	auc_percent=(auc_ori-auc_boot)/auc_ori;
RUN;

/*SomersD*/

DATA som (KEEP=som_ori som_boot op_som som_percent);
	SET optimism_index1;
	som_percent=(som_ori-som_boot)/som_ori;
RUN;

/*Brier Score*/

DATA brier (KEEP=brier_ori brier_boot op_brier brier_percent);
	SET optimism_index1;
	brier_percent=(brier_ori-brier_boot)/brier_ori;
RUN;

/*Intercept*/

DATA intercept (KEEP=intercept_ori intercept_boot op_intercept intercept_percent);
	SET optimism_index1;
	intercept_percent=(intercept_ori-intercept_boot)/intercept_ori;
RUN;

/*Slope*/

DATA slope (KEEP=slope_ori slope_boot op_slope slope_percent);
	SET optimism_index1;
	slope_percent=(slope_ori-slope_boot)/slope_ori;
RUN;

/*EMax*/

DATA emax (KEEP=emax_ori emax_boot op_emax emax_percent);
	SET optimism_index1;
	emax_percent=(emax_ori-emax_boot)/emax_ori;
RUN;

/*R-Square*/

DATA rsquare (KEEP=r_square_ori r_square_boot op_r_square r_square_percent);
	SET optimism_index1;
	r_square_percent=(r_square_ori-r_square_boot)/r_square_ori;
RUN;

/*MaxRescaled R-Square*/

DATA maxresca (KEEP=max_resca_ori max_resca_boot op_max_resca max_resca_percent);
	SET optimism_index1;
	max_resca_percent=(max_resca_ori-max_resca_boot)/max_resca_ori;
RUN;

ODS SELECT ALL;

/*++++++++++++++++++++*/

/*Standardoutput ausgeben*/

/*Fallzahl bestimmen:*/

%IF &miss=1 %THEN %DO;

%NUMBEROBS(datensatz=&datensatz);

%LET fallzahl=&nnnn1;

%END;

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/
%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls miss=0 angegeben wurde*/


		DATA work.fallzahl;
		SET  &data;
		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

		RUN;

%LET fallzahldata=fallzahl;

%NUMBEROBS(datensatz=&fallzahldata);

%LET fallzahl=&nnnn1;

%END;

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatz_ori   ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

PROC DATASETS NOLIST;
	DELETE standardoutput fallzahl;
RUN;
QUIT;

/*++++++++++++++++++++*/

/*Ergebnisse ausgeben*/

PROC PRINT DATA= auc NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR auc_ori auc_boot op_auc auc_percent; 

	TITLE1 "Bootstrap-Zusammenfassung Variante 'Mittleres Modell'";
	TITLE2 "(Angewendet auf den Ausgangsdatensatz)    Anzahl Samples: &anzahl_samples";
	TITLE3 'O=Original / B=Bootstrap / OP=Optimismus';
	TITLE4 'Kenngren nach Harrell';
	TITLE5 '(Cutpoint- unabhngige Kenngren / Prognosegtemaen)';
	LABEL 	auc_ori			='AUC O' 
			auc_boot 		='AUC B'
			op_auc 			='AUC OP'
			auc_percent		='%';
RUN;

PROC PRINT DATA= som NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR som_ori som_boot op_som som_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL 	som_ori 		='SomersD O'
			som_boot 		='SomersD B'
			op_som 			='SomersD OP'
			som_percent		='%';
RUN;

PROC PRINT DATA= brier NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  brier_ori brier_boot op_brier brier_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	brier_ori 		='Brier Score O'
			brier_boot 		='Brier Score B'
			op_brier		='Brier Score OP'
			brier_percent	='%';
RUN;

PROC PRINT DATA= intercept NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  intercept_ori intercept_boot op_intercept intercept_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	intercept_ori 		='Intercept O'
			intercept_boot 		='Intercept B'
			op_intercept		='Intercept OP'
			intercept_percent	='%';
RUN;

PROC PRINT DATA= slope NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  slope_ori slope_boot op_slope slope_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	slope_ori 		='Slope O'
			slope_boot 		='Slope B'
			op_slope		='Slope OP'
			slope_percent	='%';
RUN;

PROC PRINT DATA= emax NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR emax_ori emax_boot op_emax emax_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	emax_ori 		='Emax O'
			emax_boot 		='Emax B'
			op_emax 		='Emax OP'
			emax_percent	='%';
RUN;

PROC PRINT DATA= rsquare NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR r_square_ori r_square_boot op_r_square r_square_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	r_square_ori 	='R-Square O'
			r_square_boot 	='R-Square B'
			op_r_square		='R-Square OP'
			r_square_percent='%';
RUN;

PROC PRINT DATA= maxresca NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR max_resca_ori max_resca_boot op_max_resca max_resca_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL 	max_resca_ori 		='Max-Rescaled R-Square O'
			max_resca_boot 		='Max-Rescaled R-Square B'
			op_max_resca		='Max-Rescaled R-Square OP'
			max_resca_percent	='%';
RUN;

PROC DATASETS NOLIST; 
	DELETE auc som brier emax slope intercept maxresca rsquare optimism_index1
		   estmeanod gof_boot roc_boot result_roc_yboot
		   result_roc_cboot
		   result_roc_boot2 result result2
		   optimism_index index_ori association_boot;
RUN; 
QUIT;

/*Ergebnisdatei optimism_yroc fr schneren Output aufsplitten*/

/*Sensitivitt*/

DATA sens (KEEP=op_sens sens_ori sens_boot sens_percent);
	SET optimism_yroc;
	sens_percent=(sens_ori-sens_boot)/sens_ori;
RUN;

/*Spezifitt*/

DATA spez (KEEP=op_spez spez_ori spez_boot spez_percent);
	SET optimism_yroc;
	spez_percent=(spez_ori-spez_boot)/spez_ori;
RUN;

/*YoudenIndex*/

DATA youden (KEEP=op_youden maxyouden_ori maxyouden_boot maxyouden_percent);
	SET optimism_yroc;
	maxyouden_percent=(maxyouden_ori-maxyouden_boot)/maxyouden_ori;
RUN;

/*PPV*/

DATA ppv (KEEP=op_ppv ppv_ori ppv_boot ppv_percent);
	SET optimism_yroc;
	ppv_percent=(ppv_ori-ppv_boot)/ppv_ori;
RUN;

/*NPV*/

DATA npv (KEEP=op_npv npv_ori npv_boot npv_percent);
	SET optimism_yroc;
	npv_percent=(npv_ori-npv_boot)/npv_ori;
RUN;

/*PSEP*/

DATA psep (KEEP=op_psep psep_ori psep_boot psep_percent);
	SET optimism_yroc;
	psep_percent=(psep_ori-psep_boot)/psep_ori;
RUN;

/*Ergebnisse ausgeben*/

PROC PRINT DATA= sens NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR sens_ori sens_boot op_sens sens_percent; 
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5 "ROC-Kenngren berechnet zum maximalen Youden-Index";
	LABEL 	sens_ori		='Sensitivitt O' 
			sens_boot 		='Sensitivitt B'
			op_sens 		='Sensitivitt OP'
			sens_percent	='%';
RUN;

PROC PRINT DATA= spez NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR spez_ori spez_boot op_spez spez_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL 	spez_ori 		='Spezifitt O'
			spez_boot 		='Spezifitt B'
			op_spez			='Spezifitt OP'
			spez_percent	='%';
RUN;

PROC PRINT DATA= youden NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR maxyouden_ori maxyouden_boot op_youden maxyouden_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	maxyouden_ori 		='Max. YoudenIndex O'
			maxyouden_boot		='YoudenIndex B'
			op_youden			='YoudenIndex OP'
			maxyouden_percent	='%';
RUN;

PROC PRINT DATA= ppv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR ppv_ori ppv_boot op_ppv ppv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	ppv_ori			='Positive Korrektheit O'
			ppv_boot 		='Positive Korrektheit B'
			op_ppv 			='Positive Korrektheit OP'
			ppv_percent		='%';
RUN;

PROC PRINT DATA= npv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  npv_ori npv_boot op_npv npv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	npv_ori 		='Negative Korrektheit O'
			npv_boot 		='Negative Korrektheit B'
			op_npv			='Negative Korrektheit OP'
			npv_percent		='%';
RUN;

PROC PRINT DATA= psep NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR psep_ori psep_boot op_psep psep_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL 	psep_ori		='Separation Index PSEP O'
			psep_boot 		='Separation Index PSEP B'
			op_psep			='Separation Index PSEP OP'
			psep_percent	='%';
RUN;

PROC DATASETS NOLIST; 
	DELETE sens spez youden ppv npv psep optimism_yroc;
RUN; 
QUIT;

/*Ergebnisdatei optimism_croc fr schneren Output aufsplitten*/

/*Sensitivitt*/

DATA sens (KEEP=op_sens sens_ori sens_boot sens_percent);
	SET optimism_croc;
	sens_percent=(sens_ori-sens_boot)/sens_ori;
RUN;

/*Spezifitt*/

DATA spez (KEEP=op_spez spez_ori spez_boot spez_percent);
	SET optimism_croc;
	spez_percent=(spez_ori-spez_boot)/spez_ori;
RUN;

/*YoudenIndex*/

DATA youden (KEEP=op_youden maxyouden_ori youden_boot youden_percent);
	SET optimism_croc;
	youden_percent=(maxyouden_ori-youden_boot)/maxyouden_ori;
RUN;

/*PPV*/

DATA ppv (KEEP=op_ppv ppv_ori ppv_boot ppv_percent);
	SET optimism_croc;
	ppv_percent=(ppv_ori-ppv_boot)/ppv_ori;
RUN;

/*NPV*/

DATA npv (KEEP=op_npv npv_ori npv_boot npv_percent);
	SET optimism_croc;
	npv_percent=(npv_ori-npv_boot)/npv_ori;
RUN;

/*PSEP*/

DATA psep (KEEP=op_psep psep_ori psep_boot psep_percent);
	SET optimism_croc;
	psep_percent=(psep_ori-psep_boot)/psep_ori;
RUN;

/*Ergebnisse ausgeben*/

PROC PRINT DATA= sens NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR sens_ori sens_boot op_sens sens_percent; 
	TITLE1 ;
	TITLE2 ;
	TITLE3 ;
	TITLE4 ;
	TITLE5 "ROC-Kenngren berechnet zum Cutpoint des Ausgangsmodell (&prob_original)";
	LABEL 	sens_ori		='Sensitivitt O' 
			sens_boot 		='Sensitivitt B'
			op_sens 		='Sensitivitt OP'
			sens_percent	='%';
RUN;

PROC PRINT DATA= spez NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR spez_ori spez_boot op_spez spez_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL 	spez_ori 		='Spezifitt O'
			spez_boot 		='Spezifitt B'
			op_spez			='Spezifitt OP'
			spez_percent	='%';
RUN;

PROC PRINT DATA= youden NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR maxyouden_ori youden_boot op_youden youden_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	maxyouden_ori 	='Max. YoudenIndex O'
			youden_boot		='YoudenIndex B'
			op_youden		='YoudenIndex OP'
			youden_percent	='%';
RUN;

PROC PRINT DATA= ppv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR ppv_ori ppv_boot op_ppv ppv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	ppv_ori			='Positive Korrektheit O'
			ppv_boot 		='Positive Korrektheit B'
			op_ppv 			='Positive Korrektheit OP'
			ppv_percent		='%';
RUN;

PROC PRINT DATA= npv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  npv_ori npv_boot op_npv npv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	npv_ori 		='Negative Korrektheit O'
			npv_boot 		='Negative Korrektheit B'
			op_npv			='Negative Korrektheit OP'
			npv_percent		='%';
RUN;

PROC PRINT DATA= psep NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR psep_ori psep_boot op_psep psep_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL 	psep_ori		='Separation Index PSEP O'
			psep_boot 		='Separation Index PSEP B'
			op_psep			='Separation Index PSEP OP'
			psep_percent	='%';
RUN;

PROC DATASETS NOLIST; 
	DELETE sens spez youden ppv npv psep optimism_croc;
RUN; 
QUIT;


/*Ende der if-Anweisung*/
%END;

/*(5) Mittleres Modell angewandt auf jeden Bootstrap-Sample (Option inest=work.out_bootx)*/
/*    Mittleres Modell angewandt auf jeden BS  											 */
/*    (Parameter &mean_flag=2)															 */

%IF &mean_flag = 2 %THEN %DO;

/*Dateien, die innerhalb der Schleife angewandt werden mit 0 initialisieren*/

DATA result_roc_cmean;	SET _NULL_;
DATA result_roc_ymean;	SET _NULL_;
DATA index_mean;		SET _NULL_;

/*Schleife*/
%DO zm=1 %TO &km-1; 

/*(5.1) Aufruf der PROC LOGISTIC (%logreg) fr jeden Bootstrap-Sample */
/*      --> Kenngren bestimmen*/

%PM_LOGREG ( 	data			= boot&zm,
				resp_var		= &resp_var,
				event			= &event,
				xvar			= &xvar,
				cvar			= &cvar,
				ref				= &ref,
				fxvar			= ,
				fcvar			= ,
				miss			= &miss,
				sel				= NONE,
				sel_details		= 1,
				slstay			= 0.05,
				slentry			= 0.05,
				clodds			= 1,
				alpha			= 0.05,
				rsquare			= 1,
				ctable			= 1,
				ctable_pprob	= ,
				out_pred		= work.gof_mean&zm,
				pred			= 1,
				predprobs_art	= INDIVIDUAL,
				roc				= 1,
				out_roc			= work.roc_mean&zm,
				est				= 1,
				out_est			= work.estMean2&zm,
				inest_flag		= 2,
				inest_data		= work.out_bootx,
				boot_data		= ,
				c_wert			= 1,
				c_data			= association_mean&zm,
				test			= ,
				mi				= 0,
				out_mi			= work.out_mi_log,
				macro_path		= &macro_path,
				binwidth		= &binwidth
		);

/*Kenngren bei angewandtem Mittleren Modell auf jeden BS*/
/*ROC-Analyse fr "Mittleres Modell" angewandt auf jeden BS mit Makro (%ROC)*/

%PM_ROC			(
				 data				=boot&zm,
				 resp_var			=&resp_var,
				 event				=&event,
				 data_roc			=work.roc_mean&zm,
				 data_pred			=work.gof_mean&zm,
				 data_asso			=work.association_mean&zm,
				 validation			=BOOTSTRAP,
				 praevalenz			= ,
				 all_cp				=0,
			     cp					=&prob_original,
				 cibinom			=0,
				 rocdat				=0,
				 auroc				=0,
				 printit			=0,
				 alpha_cl			=0.95,
				 ext				=tiffp,
				 winkel				=1,
				 point				=1,
				 path				='c:\temp',
				 mi					=0,
				 out_mi				=work.out_mi_roc,
				 macro_path			=&macro_path,
				 binwidth			=&binwidth
			    );

DM "clear log;";


/*Variablen in Ergebnisdatei zusammenfassen und umbenennen*/
/*Fr angegebenen cp (hier: cp des Ausgangsmodells) result2*/
/*Wird nach jedem Durchlauf unten angehngt*/

DATA result_roc_cmean;
	SET result_roc_cmean result2;
RUN;

/*Fr max. Youden des Mittleren Modells angewandt auf jeden BS: result*/

DATA result_roc_ymean;
	SET result_roc_ymean result;
RUN;

/*Berechnung der Kenngren bei angewandtem Mittleren Modell auf jeden BS */

DATA index_mean;
	SET index_mean index_ori;
RUN;

PROC DATASETS NOLIST; 
		DELETE boot&zm roc_mean&zm result result2 index_ori gof_mean&zm estmean2&zm
		       association_mean&zm
; 
RUN;
QUIT;

%END;

/*(5.2) Optimismus der Kenngren nach Harrell (AUC, SomersD, Brier-Score, Emax) bestimmen:*/
/*	    Differenzen Ausgangsmodell - "Mittleres Modell angewandt auf OD" */

/*Umbenennung von Variablen*/

DATA index_mean;
	SET index_mean;
	RENAME brier_ori		= brier_boot;
	RENAME emax_ori			= emax_boot;
	RENAME slope_ori		= slope_boot;
	RENAME intercept_ori	= intercept_boot;
	RENAME som_ori			= som_boot;
	RENAME auc_ori			= auc_boot;
	RENAME Max_resca_ori	= max_resca_boot;
	RENAME R_Square_ori		= r_square_boot;
RUN;

/*Zunchst Mittelwerte bestimmen;*/

PROC MEANS NOPRINT DATA=index_mean FW=8 MAXDEC=2; 
	OUTPUT OUT=result_roc_others_mean;
RUN;

/*Da nur mean interessiert, restliche Statistiken lschen;*/

DATA result_roc_others_mean1;
	SET result_roc_others_mean;
	IF _STAT_='N' 	THEN DELETE;
	IF _STAT_='MIN' THEN DELETE;
	IF _STAT_='MAX' THEN DELETE;
	IF _STAT_='STD' THEN DELETE;
	dummy=1;
RUN;

/*Mit Ergebnisdatei des Ausgangsmodells zusammenmergen*/

DATA optimism_index;
	MERGE result_roc_ori2 result_roc_others_mean1;
	BY dummy;
RUN;

/*Differenzen berechnen*/

DATA optimism_index (DROP=dummy _Type_ _Freq_ _Stat_);
	SET optimism_index;
	op_auc=auc_ori-auc_boot;
	op_som=som_ori-som_boot;
	op_brier=brier_ori-brier_boot;
	op_slope=slope_ori-slope_boot;
	op_intercept=intercept_ori-intercept_boot;
	op_emax=emax_ori-emax_boot;
	op_max_resca=max_resca_ori-max_resca_boot;
	op_r_square=r_square_ori-r_square_boot;
RUN;

/*Ergebnisdatei optimism_index fr schneren Output aufsplitten*/

/*AUC*/

DATA auc (KEEP=auc_ori auc_boot op_auc auc_percent);
	SET optimism_index;
	auc_percent=(auc_ori-auc_boot)/auc_ori;
RUN;

/*SomersD*/

DATA som (KEEP=som_ori som_boot op_som som_percent);
	SET optimism_index;
	som_percent=(som_ori-som_boot)/som_ori;
RUN;

/*Brier Score*/

DATA brier (KEEP=brier_ori brier_boot op_brier brier_percent);
	SET optimism_index;
	brier_percent=(brier_ori-brier_boot)/brier_ori;
RUN;

/*Intercept*/

DATA intercept (KEEP=intercept_ori intercept_boot op_intercept intercept_percent);
	SET optimism_index;
	intercept_percent=(intercept_ori-intercept_boot)/intercept_ori;
RUN;

/*Slope*/

DATA slope (KEEP=slope_ori slope_boot op_slope slope_percent);
	SET optimism_index;
	slope_percent=(slope_ori-slope_boot)/slope_ori;
RUN;

/*EMax*/

DATA emax (KEEP=emax_ori emax_boot op_emax emax_percent);
	SET optimism_index;
	emax_percent=(emax_ori-emax_boot)/emax_ori;
RUN;

/*R-Square*/

DATA rsquare (KEEP=r_square_ori r_square_boot op_r_square r_square_percent);
	SET optimism_index;
	r_square_percent=(r_square_ori-r_square_boot)/r_square_ori;
RUN;

/*MaxRescaled R-Square*/

DATA maxresca (KEEP=max_resca_ori max_resca_boot op_max_resca max_resca_percent);
	SET optimism_index;
	max_resca_percent=(max_resca_ori-max_resca_boot)/max_resca_ori;
RUN;

ODS SELECT ALL;

/*++++++++++++++++++++*/

/*Standardoutput ausgeben*/

/*Fallzahl bestimmen:*/

%IF &miss=1 %THEN %DO;

%NUMBEROBS(datensatz=&datensatz_ori);

%LET fallzahl=&nnnn1;

%END;

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/
%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls miss=0 angegeben wurde*/


		DATA work.fallzahl;
		SET  &data;
		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

		RUN;

%LET fallzahldata=fallzahl;

%NUMBEROBS(datensatz=&fallzahldata);

%LET fallzahl=&nnnn1;

%END;

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatz_ori   ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

PROC DATASETS NOLIST;
	DELETE standardoutput fallzahl;
RUN;
QUIT;

/*++++++++++++++++++++*/

/*Ergebnisse ausgeben*/

PROC PRINT DATA= auc NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR auc_ori auc_boot op_auc auc_percent; 
    TITLE1 "Bootstrap-Zusammenfassung Variante 'Mittleres Modell'";
	TITLE2 "(Angewandt auf jeden Bootstrap-Sample)   Anzahl Samples: &anzahl_samples";
	TITLE3 'O=Original / B=Bootstrap / OP=Optimismus';
	TITLE4 'Cutpoint- unabhngige Kenngren / Prognosegtemaen (Kenngren nach Harrell)';
	LABEL 	auc_ori			='AUC O' 
			auc_boot 		='AUC B'
			op_auc 			='AUC OP'
			auc_percent		='%';
RUN;

PROC PRINT DATA= som NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR som_ori som_boot op_som som_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	som_ori 		='SomersD O'
			som_boot 		='SomersD B'
			op_som 			='SomersD OP'
			som_percent		='%';
RUN;

PROC PRINT DATA= brier NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  brier_ori brier_boot op_brier brier_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	brier_ori 		='Brier Score O'
			brier_boot 		='Brier Score B'
			op_brier		='Brier Score OP'
			brier_percent	='%';
RUN;

PROC PRINT DATA= intercept NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  intercept_ori intercept_boot op_intercept intercept_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	intercept_ori 		='Intercept O'
			intercept_boot 		='Intercept B'
			op_intercept		='Intercept OP'
			intercept_percent	='%';
RUN;

PROC PRINT DATA= slope NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  slope_ori slope_boot op_slope slope_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	slope_ori 		='Slope O'
			slope_boot 		='Slope B'
			op_slope		='Slope OP'
			slope_percent	='%';
RUN;

PROC PRINT DATA= emax NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR emax_ori emax_boot op_emax emax_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	emax_ori 		='Emax O'
			emax_boot 		='Emax B'
			op_emax 		='Emax OP'
			emax_percent	='%';
RUN;

PROC PRINT DATA= rsquare NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR r_square_ori r_square_boot op_r_square r_square_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	r_square_ori 		='R-Square O'
			r_square_boot 		='R-Square B'
			op_r_square			='R-Square OP'
			r_square_percent	='%';
RUN;

PROC PRINT DATA= maxresca NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR max_resca_ori max_resca_boot op_max_resca max_resca_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	max_resca_ori 		='Max-Rescaled R-Square O'
			max_resca_boot 		='Max-Rescaled R-Square B'
			op_max_resca		='Max-Rescaled R-Square OP'
			max_resca_percent	='%';
RUN;

/*Zwischendatenstze lschen*/

PROC DATASETS NOLIST; 
	DELETE result_roc_others_mean1 auc som brier emax slope intercept rsquare maxresca optimism_index
		   result_roc_others_mean index_mean;
RUN; 
QUIT;

/*(5.3) Optimismus zustzlich fr ROC-Kenngren (Sensitivitt, Spezifitt, Youden-Index)*/
/*	    zum Cutpoint des Ausgangsmodells (Ergebnisdatei: "work.optimism_croc")*/
/*	    zum jeweiligen neuen, maximalen Youden-Index (Ergebnisdatei: "work.optimism_yroc")*/

/*Variablen umbenennen*/

DATA result_roc_cmean (DROP=_Pos_ _Neg_ _Falpos_ _Falneg_);
	SET result_roc_cmean;
	RENAME next_cutpoint 	= prob_boot;
	RENAME _SENSIT_ 		= sens_boot;
	RENAME spez 			= spez_boot;
	RENAME youden			= youden_boot;
	RENAME ppv 				= ppv_boot;
	RENAME npv 				= npv_boot;
	RENAME psep 			= psep_boot;
	dummy					= 1;
RUN;

/*Zunchst Mittelwerte bestimmen;*/

PROC MEANS NOPRINT DATA=result_roc_cmean FW=8 MAXDEC=2; 
	OUTPUT OUT=result_roc_cboot_mean;
RUN;

/*Da nur mean interessiert, restliche Statistiken lschen;*/

DATA result_roc_cboot_mean1;
	SET result_roc_cboot_mean;
	IF _STAT_='N' 	THEN DELETE;
	IF _STAT_='MIN' THEN DELETE;
	IF _STAT_='MAX' THEN DELETE;
	IF _STAT_='STD' THEN DELETE;
	dummy=1;
RUN;

/*Mit Ergebnisdatei des Ausgangsmodells zusammenmergen*/

DATA optimism_croc;
	MERGE result_roc_cboot_mean1 result_roc_ori1;
	BY dummy;
RUN;

/*Differenzen berechnen*/

DATA optimism_croc (DROP=dummy _Type_ _Freq_ _Stat_  prob_ori);
	SET optimism_croc;
	op_sens=sens_ori- sens_boot;
	op_spez=spez_ori-spez_boot;
	op_youden=maxyouden_ori-youden_boot;
	op_ppv=ppv_ori-ppv_boot;
	op_npv=npv_ori-npv_boot;
	op_psep=psep_ori-psep_boot;
RUN;

/*Ergebnisdatei optimism_croc fr schneren Output aufsplitten*/

/*Sensitivitt*/

DATA sens (KEEP=op_sens sens_ori sens_boot sens_percent);
	SET optimism_croc;
	sens_percent=(sens_ori-sens_boot)/sens_ori;
RUN;

/*Spezifitt*/

DATA spez (KEEP=op_spez spez_ori spez_boot spez_percent);
	SET optimism_croc;
	spez_percent=(spez_ori-spez_boot)/spez_ori;
RUN;

/*YoudenIndex*/

DATA youden (KEEP=op_youden maxyouden_ori youden_boot youden_percent);
	SET optimism_croc;
	youden_percent=(maxyouden_ori-youden_boot)/maxyouden_ori;
RUN;

/*PPV*/

DATA ppv (KEEP=op_ppv ppv_ori ppv_boot ppv_percent);
	SET optimism_croc;
	ppv_percent=(ppv_ori-ppv_boot)/ppv_ori;
RUN;

/*NPV*/

DATA npv (KEEP=op_npv npv_ori npv_boot npv_percent);
	SET optimism_croc;
	npv_percent=(npv_ori-npv_boot)/npv_ori;
RUN;

/*PSEP*/

DATA psep (KEEP=op_psep psep_ori psep_boot psep_percent);
	SET optimism_croc;
	psep_percent=(psep_ori-psep_boot)/psep_ori;
RUN;

/*Ergebnisse ausgeben*/

PROC PRINT DATA= sens NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR sens_ori sens_boot op_sens sens_percent; 
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4 "ROC- Kenngren zum Cutpoint des Ausgangsmodells (&prob_original)";;
	LABEL 	sens_ori		='Sensitivitt O' 
			sens_boot 		='Sensitivitt B'
			op_sens 		='Sensitivitt OP'
			sens_percent	='%';
RUN;

PROC PRINT DATA= spez NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR spez_ori spez_boot op_spez spez_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	spez_ori 		='Spezifitt O'
			spez_boot 		='Spezifitt B'
			op_spez			='Spezifitt OP'
			spez_percent	='%';
RUN;

PROC PRINT DATA= youden NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR maxyouden_ori youden_boot op_youden youden_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	maxyouden_ori 	='Max. YoudenIndex O'
			youden_boot		='YoudenIndex B'
			op_youden		='YoudenIndex OP'
			youden_percent	='%';
RUN;

PROC PRINT DATA= ppv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR ppv_ori ppv_boot op_ppv ppv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	ppv_ori			='Positive Korrektheit O'
			ppv_boot 		='Positive Korrektheit B'
			op_ppv 			='Positive Korrektheit OP'
			ppv_percent		='%';
RUN;

PROC PRINT DATA= npv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  npv_ori npv_boot op_npv npv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	npv_ori 		='Negative Korrektheit O'
			npv_boot 		='Negative Korrektheit B'
			op_npv			='Negative Korrektheit OP'
			npv_percent		='%';
RUN;

PROC PRINT DATA= psep NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR psep_ori psep_boot op_psep psep_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	psep_ori		='Separation Index PSEP O'
			psep_boot		='Separation Index PSEP B'
			op_psep			='Separation Index PSEP OP'
			psep_percent	='%';
RUN;

/*Zwischendatenstze lschen*/

PROC DATASETS NOLIST; 
	DELETE sens spez youden result_roc_cmean result_roc_cboot_mean result_roc_cboot_mean1
		   psep npv ppv optimism_croc;
RUN; 
QUIT;

/*ROC- Kenngren zum neuen maximalen Youdenindex*/

/*Zunchst Mittelwerte bestimmen;*/

PROC MEANS NOPRINT DATA=result_roc_ymean FW=8 MAXDEC=2; 
	OUTPUT OUT=result_roc_yboot_mean;
RUN;

/*Da nur mean interessiert, restliche Statistiken lschen;*/

DATA result_roc_yboot_mean1;
	SET result_roc_yboot_mean;
	IF _STAT_='N' 	THEN DELETE;
	IF _STAT_='MIN' THEN DELETE;
	IF _STAT_='MAX' THEN DELETE;
	IF _STAT_='STD' THEN DELETE;
	dummy=1;
RUN;

/*Mit Ergebnisdatei des Ausgangsmodells zusammenmergen*/

DATA optimism_yroc;
	MERGE result_roc_yboot_mean1 result_roc_ori1;
	BY dummy;
RUN;

/*Differenzen berechnen*/

DATA optimism_yroc (DROP=dummy _Type_ _Freq_ _Stat_  _prob_);
	SET optimism_yroc;
	op_sens=sens_ori-_sensit_;
	op_spez=spez_ori-spez;
	op_youden=maxyouden_ori-maxyouden;
	op_ppv=ppv_ori-ppv;
	op_npv=npv_ori-npv;
	op_psep=psep_ori-psep;
RUN;

/*Ergebnisdatei optimism_yroc fr schneren Output aufsplitten*/

/*Sensitivitt*/

DATA sens (KEEP=op_sens sens_ori _sensit_ sens_percent);
	SET optimism_yroc;
	sens_percent=(sens_ori-_sensit_)/sens_ori;
RUN;

/*Spezifitt*/

DATA spez (KEEP=op_spez spez_ori spez spez_percent);
	SET optimism_yroc;
	spez_percent=(spez_ori-spez)/spez_ori;
RUN;

/*YoudenIndex*/

DATA youden (KEEP=op_youden maxyouden_ori maxyouden youden_percent);
	SET optimism_yroc;
	youden_percent=(maxyouden_ori-maxyouden)/maxyouden_ori;
RUN;

/*PPV*/

DATA ppv (KEEP=op_ppv ppv_ori ppv ppv_percent);
	SET optimism_yroc;
	ppv_percent=(ppv_ori-ppv)/ppv_ori;
RUN;

/*NPV*/

DATA npv (KEEP=op_npv npv_ori npv npv_percent);
	SET optimism_yroc;
	npv_percent=(npv_ori-npv)/npv_ori;
RUN;

/*PSEP*/

DATA psep (KEEP=op_psep psep_ori psep psep_percent);
	SET optimism_yroc;
	psep_percent=(psep_ori-psep)/psep_ori;
RUN;

/*Ergebnisse ausgeben*/

PROC PRINT DATA= sens NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR sens_ori _sensit_ op_sens sens_percent; 
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4 "ROC- Kenngren zum neuen maximalen Youdenindex";;
	LABEL 	sens_ori		='Sensitivitt O' 
			_sensit_ 		='Sensitivitt B'
			op_sens 		='Sensitivitt OP'
			sens_percent	='%';
RUN;

PROC PRINT DATA= spez NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR spez_ori spez op_spez spez_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	spez_ori 		='Spezifitt O'
			spez 			='Spezifitt B'
			op_spez			='Spezifitt OP'
			spez_percent	='%';
RUN;

PROC PRINT DATA= youden NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR maxyouden_ori maxyouden op_youden youden_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	maxyouden_ori 	='Max. YoudenIndex O'
			maxyouden		='YoudenIndex B'
			op_youden		='YoudenIndex OP'
			youden_percent	='%';
RUN;

PROC PRINT DATA= ppv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR ppv_ori ppv op_ppv ppv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	ppv_ori			='Positive Korrektheit O'
			ppv 			='Positive Korrektheit B'
			op_ppv 			='Positive Korrektheit OP'
			ppv_percent		='%';
RUN;

PROC PRINT DATA= npv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  npv_ori npv op_npv npv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	npv_ori 		='Negative Korrektheit O'
			npv 			='Negative Korrektheit B'
			op_npv			='Negative Korrektheit OP'
			npv_percent		='%';
RUN;

PROC PRINT DATA= psep NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR psep_ori psep  op_psep psep_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	psep_ori		='Separation Index PSEP O'
			psep 			='Separation Index PSEP B'
			op_psep			='Separation Index PSEP OP'
			psep_percent	='%';
RUN;

/*Zwischendatenstze lschen*/

PROC DATASETS NOLIST; 
	DELETE sens spez youden result_roc_ymean result_roc_yboot_mean result_roc_yboot_mean1
		   psep npv ppv optimism_yroc;
RUN; 
QUIT;


%END;

/*(6) Hufigkeiten der Hauptzielgre*/

/*----------------------------------------------------------------------------------*/

/*Proc Means ber die Datei: work.freqsBootGesamt*/

PROC SORT DATA=freqsBootGesamt;
	BY &resp_var;
RUN;

PROC MEANS DATA=freqsBootGesamt FW=8 MAXDEC=0 MEAN MIN MAX NOPRINT; 
	BY &resp_var; 
	OUTPUT OUT= freqsBootGesamt mean=MEAN min=MIN max=MAX;
RUN;

/*(6.1) im Originaldatensatz*/

PROC PRINT DATA=freqsori NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4 "Hufigkeiten der Hauptzielgre (&resp_var) im Originaldatensatz";
	VAR &resp_var frequency;
	LABEL &resp_var ="&resp_var"
		  frequency	='Hufigkeit';
RUN;

/*(6.2) in den Bootstrap- Samples (Durchschnitt)*/

PROC PRINT DATA=freqsBootGesamt NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4 "Durchschnittliche Hufigkeiten der Hauptzielgre (&resp_var) in den Bootstrap- Samples";
	VAR &resp_var mean min max;
	LABEL &resp_var ="&resp_var"
		  mean		='Mittelwert'
		  min		='Minimum'
		  max		='Maximum';
RUN;

PROC DATASETS NOLIST; 
	DELETE	meansestimates meansestimates1 out_bootx;
RUN;
QUIT;

%ende:

PROC DATASETS NOLIST; 
		DELETE boot freqsBootGesamt freqsori boot_all
			   result_roc_ori1 result_roc_ori2 boot_est;
RUN;
QUIT;

%MEND BOOT_MEAN_VALIDATION;
