/************************************************************************************/
/*																				  	*/
/*	NAME:		boot_model_validation.mac.sas										*/
/*  TITLE: 		Bootstrap- Validierung (Bootstrap- Modelle nach Harrell)			*/
/*  KEYS: 		bootstrap, validation, logistic regression, roc analysis 			*/
/*	VERSION:	SAS 8.2																*/
/* 	PRODUCT:	BASE, STAT															*/
/*  PROCS:		LOGISTIC, FREQ, MEANS												*/
/*																					*/
/*																					*/
/*	Verzeichnis/Dateiname:			F:\Prognose\boot_model_validation.mac.sas		*/
/*  Autoren:						R. Muche, C. Ring, I. Assfalg, C. Ziegler		*/
/*	Programmierer:					Ina Assfalg/ Christoph Ziegler					*/
/*	Datum der Ersterstellung:		9/2002											*/
/*  Aktuelle Makro- Version:		12/2003											*/
/*																					*/
/*	Kurzbeschreibung:				Allgemeines Makro zur internen Validierung von 	*/
/*							    	logistischen Regressionsmodellen mittels 		*/
/*									Bootstrap (Bootstrap-Modelle nach Harrell)  	*/
/*																					*/
/*	Hilfs-/Untermakro von:			PM_BOOTSTRAP_VALIDATION.MAC.SAS					*/
/*																					*/
/*  Genauere Beschreibung siehe:	PM_BOOTSTRAP_VALIDATION.MAC.SAS					*/
/*																					*/
/*  Verwendete externe Untermakros:	numberobs.mac.sas								*/
/*									pm_logreg.mac.sas								*/
/*									pm_roc.mac.sas									*/
/*																					*/
/*	Bentigte Dateien:				Datensatz (Parameter 'data=')					*/
/*																					*/
/*																					*/
/*  Erzeugte Dateien:				-----											*/
/*																					*/
/*  Programmnderungen 				Sandra Mller, 04.05.2016, Makrovariable		*/
/*  und -ergnzungen 				binwidth hinzugefgt							*/
/*  (Name, Datum, Beschreibung):     												*/
/*   																   			    */
/*																					*/
/************************************************************************************/

%MACRO BOOT_MODEL_VALIDATION(		data				= ,
									resp_var			= , 
									event				= 1,
									xvar				= ,
									cvar				= ,
									ref					= ,
									miss 				= 0,
									anzahl_samples		= 200, 
									random				= 0,
									macro_path			= ,
									binwidth			= 0
	            			  );

/*(1) Externe Dateien (Untermakros)(%include) einbinden	*/

FILENAME y &macro_path;

%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('pm_logreg.mac.sas');
%INCLUDE y ('pm_roc.mac.sas');

/* Wird auch im Untermakro %numberobs bentigt*/

%GLOBAL nnnn1;
%LET datensatz=&data;

/*Unterdrckt OUTPUT:*/

ODS SELECT NONE;


/*(2) Erzeugen der Bootstrap-Samples:	*/

/*(2.1) Makro zur Bestimmung der Anzahl der Beobachtungen des Ausgangsdatensatzes (%numberobs)*/
/*(Anzahl der Beobachtungen im bergebenen Datensatz ausgeben)*/

%NUMBEROBS (datensatz=&data);

/*(2.2) Aufruf der PROC LOGISTIC  (%logreg) fr den Originaldatensatz (OD)*/
/* -> Kenngren bestimmen (fr Ausgangsmodell)*/

%PM_LOGREG ( 	data			= &data,
				resp_var		= &resp_var,
				event			= &event,
				xvar			= &xvar,
				cvar			= &cvar,
				ref				= &ref,
				fxvar			= ,
				fcvar			= ,
				miss			= &miss,
				sel				= NONE,
				sel_details		= 1,
				slstay			= 0.05,
				slentry			= 0.05,
				clodds			= 1,
				alpha			= 0.05,
				rsquare			= 1,
				ctable			= 1,
				ctable_pprob	= ,
				out_pred		= work.gof_ori,
				pred			= 1,
				predprobs_art	=INDIVIDUAL,
				roc				= 1,
				out_roc			= work.roc_ori,
				est				= 0,
				out_est			= ,
				inest_flag		= 0,
				inest_data		= ,
				boot_data		= ,
				c_wert			= 1,
				c_data			= work.association_ori,
				test			= ,
				mi				= 0,
				out_mi			= work.out_mi_log,
				macro_path		= &macro_path,
				binwidth		= &binwidth
		);

/*Event-Rate der Ausgangsdatei bestimmen*/

DATA eventOri (KEEP=&resp_var);
	SET &data;
RUN; 

/*Hufigkeitsauszhlung von Event -> in Extra Datei*/

ODS LISTING CLOSE;
ODS OUTPUT OneWayFreqs=freqsOri;
	PROC FREQ DATA=eventOri;
	RUN;
ODS OUTPUT CLOSE;
ODS LISTING;

/*ROC-Analyse (OD) mit Makro %ROC*/

%PM_ROC			(
				 data				=&data,
				 resp_var			=&resp_var,
				 event				=&event,
				 data_roc			=work.roc_ori,
				 data_pred			=work.gof_ori,
				 data_asso			=work.association_ori,
				 validation			=BOOTSTRAP,
				 praevalenz			= ,
				 all_cp				=0,
			     cp					=,
				 cibinom			=0,
				 rocdat				=0,
				 auroc				=0,
				 printit			=0,
				 alpha_cl			=0.95,
				 ext				=tiffp,
				 winkel				=1,
				 point				=1,
				 path				='c:\temp',
				 mi					=0,
				 out_mi				=work.out_mi_roc,
				 macro_path			=&macro_path,
				 binwidth			=&binwidth
			    );

DM "clear log;";

/*ROC- / Sonstige Kenngren des Originaldatensatzes zusammenfassen*/

DATA result_roc_ori1;
	SET result;
	RENAME maxyouden 	= maxyouden_ori;
	RENAME _PROB_ 		= prob_ori;
	RENAME _SENSIT_		= sens_ori;
	RENAME spez 		= spez_ori;
	RENAME ppv 			= ppv_ori;
	RENAME npv 			= npv_ori;
	RENAME psep 		= psep_ori;
	dummy				= 1;
RUN;

DATA result_roc_ori2;
	SET index_ori;
	dummy				= 1;
RUN;

/*Cutpoint des Ausgangsdatensatzes zur spteren Verwendung in einer Makrovariable */
/*zwischenspeichern*/

DATA result_roc_ori1;
	SET result_roc_ori1;
	call symput('prob_original',prob_ori);
RUN;

/*Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE result roc_ori gof_ori association_ori index_ori eventori;
RUN;
QUIT;

DATA boot;
	SET &data;
RUN;

/*Nullinitialisierung der Dateien die innerhalb der Schleife angelegt werden*/

DATA result_roc_boot; 		SET _NULL_;
DATA result_roc_NewModel; 	SET _NULL_;
DATA freqsBootGesamt; 		SET _NULL_;

/*(2.3) Schleife zur Erzeugung der Bootstrap-Samples	*/

/*Schleife*/

%NUMBEROBS (datensatz=&data);

%LET samples_nnnn1=&nnnn1;
%LET boot=boot;

/* Gesamten Bootstrap-Datensatz erzeugen */
PROC SURVEYSELECT DATA=&boot OUT=boot_all
	seed=&random
	method=urs 
	samprate=1 
	outhits 
	rep=&anzahl_samples
	NOPRINT; 
RUN;



%DO kw=1 %TO &anzahl_samples;
	DATA boot&kw (DROP=Replicate);
	SET boot_all (WHERE= (Replicate=&kw));
	RUN;


/*(2.4) Aufruf der PROC LOGISTIC  (%logreg)	fr jeden Boostrap-Sample		 */
/*		--> Kenngren bestimmen (fr jedes Bootstrap-Modell)				 */
/* 	    --> Bootstrap-Modelle (BM): insgesamt soviel BM wie Bootstrap-Samples*/

%PM_LOGREG ( 	data			= boot&kw,
				resp_var		= &resp_var,
				event			= &event,
				xvar			= &xvar,
				cvar			= &cvar,
				ref				= &ref,
				fxvar			= ,
				fcvar			= ,
				miss			= &miss,
				sel				= NONE,
				sel_details		= 1,
				slstay			= 0.05,
				slentry			= 0.05,
				clodds			= 1,
				alpha			= 0.05,
				rsquare			= 1,
				ctable			= 1,
				ctable_pprob	= ,
				out_pred		= work.out_gof&kw,
				pred			= 1,
				predprobs_art	= INDIVIDUAL,
				roc				= 1,
				out_roc			= work.out_roc&kw,
				est				= 1,
				out_est			= work.boot_est&kw,
				inest_flag		= 0,
				inest_data		= ,
				boot_data		= ,
				c_wert			= 1,
				c_data			= work.association_boot&kw,
				test			=,
				mi				= 0,
				out_mi			= work.out_mi_log,
				macro_path		= &macro_path,
				binwidth		= &binwidth
		);

/*Event-Rate pro Sample bestimmen*/

DATA eventBoot&kw (KEEP=&resp_var);
	SET boot&kw;
RUN; 
		
ODS LISTING CLOSE;
ODS OUTPUT OneWayFreqs=freqsBoot&kw;
	PROC FREQ DATA=eventBoot&kw;
	RUN;
ODS OUTPUT CLOSE;
ODS LISTING;

/*Event-Raten aller Samples in einer Datei zusammengefasst*/

DATA freqsBootGesamt;
	SET freqsBootGesamt freqsBoot&kw;
RUN;

/*ROC-Analyse mit Makro %ROC (Fr jedes Ausgangsmodell, angewandt auf die BS);*/
/*Sens., Spez. usw. zum Cutpoint des OD ermittlen*/

%PM_ROC			(
				 data				=boot&kw,
				 resp_var			=&resp_var,
				 event				=&event,
				 data_roc			=work.out_roc&kw,
				 data_pred			=work.out_gof&kw,
				 data_asso			=work.association_boot&kw,
				 validation			=BOOTSTRAP,
				 praevalenz			= ,
				 all_cp				=0,
			     cp					=&prob_original,
				 cibinom			=0,
				 rocdat				=0,
				 auroc				=0,
				 printit			=0,
				 alpha_cl			=0.95,
				 ext				=tiffp,
				 winkel				=1,
				 point				=1,
				 path				='c:\temp',
				 mi					=0,
				 out_mi				=work.out_mi_roc,
				 macro_path			=&macro_path,
				 binwidth			=&binwidth
			    );

DM "clear log;";

/*Sensitivitt usw. pro BM zum angegebenen cp (hier: cp aus OD) */
/*in einer Datei zusammenfassen. Wird bei jedem Schleifendurchlauf untern angehngt.*/
/*Datei "result2" ist die Ergebnisdatei aus o.g. Makro*/

/*sowie: Variablen umbenennen*/

DATA result_roc_boot1 (DROP=_Pos_ _Neg_ _Falpos_ _Falneg_);
	SET result2;
	RENAME next_cutpoint 	= prob_boot;
	RENAME _SENSIT_ 		= sens_boot;
	RENAME spez 			= spez_boot;
	RENAME youden			= youden_boot;
	RENAME ppv 				= ppv_boot;
	RENAME npv 				= npv_boot;
	RENAME psep 			= psep_boot;
	dummy					= 1;
RUN;

DATA result_roc_boot2;
	SET index_ori;
	RENAME Max_resca_ori	= max_resca_boot;
	RENAME R_Square_ori		= r_square_boot;
	RENAME brier_ori		= brier_boot;
	RENAME slope_ori		= slope_boot;
	RENAME intercept_ori	= intercept_boot;
	RENAME emax_ori			= emax_boot;
	RENAME som_ori			= som_boot;
	RENAME auc_ori			= auc_boot;
	dummy					= 1;
RUN;


/*Lscht die temporren Zwischendateien (Analyse der Boostrap-Samples)*/

PROC DATASETS NOLIST; 
	DELETE  out_roc&kw out_gof&kw result2 result index_ori eventBoot&kw freqsBoot&kw; 
	DELETE  association_boot&kw;
RUN; 
QUIT;

/*(2.5) Anwenden jedes Boostrap-Modells auf den Ausgangsdatensatz ("Simple Bootstrap"):*/
/*	    Aufruf der PROC LOGISTIC (%logreg) fr den Ausgangsdatensatz				   */
/*				-> Kenngren bestimmen (fr jedes "neue" Modell)                      */

/*Anwenden des jeweiligen Bootstrap-Modells auf den Ausgangsdatensatz: */
/*(BM auf OD): */
/*--> Ausfhren der Logistischen Regression fr OD*/
/*--> "neue" BM = newModel*/
/*--> insgesamt soviel "neue" BM wie Bootstrap-Samples*/

%PM_LOGREG ( 	data			= &data,
				resp_var		= &resp_var,
				event			= &event,
				xvar			= &xvar,
				cvar			= &cvar,
				ref				= &ref,
				fxvar			= ,
				fcvar			= ,
				miss			= &miss,
				sel				= NONE,
				sel_details		= 1,
				slstay			= 0.05,
				slentry			= 0.05,
				clodds			= 1,
				alpha			= 0.05,
				rsquare			= 1,
				ctable			= 1,
				ctable_pprob	= ,
				out_pred		= work.out_gof_NewModel&kw,
				pred			= 1,
				predprobs_art	=INDIVIDUAL,
				roc				= 1,
				out_roc			= work.roc_newModel&kw,
				est				= 0,
				out_est			= ,
				inest_flag		= 2,
				inest_data		= work.boot_est&kw,
				boot_data		= ,
				c_wert			= 1,
				c_data			= work.association_NewModel&kw,
				test			=,
				mi				=0,
				out_mi			= work.out_mi_log,
				macro_path		= &macro_path,
				binwidth		= &binwidth
		);

/*ROC-Analyse mit Makro %ROC (pro NewModel);*/
/*Sens., Spez. usw. zum Cutpoint des OD  ermittlen*/

%PM_ROC			(
				 data				=&data,
				 resp_var			=&resp_var,
				 event				=&event,
				 data_roc			=work.roc_newModel&kw,
				 data_pred			=work.out_gof_NewModel&kw,
				 data_asso			=work.association_NewModel&kw,
				 validation			=BOOTSTRAP,
				 praevalenz			= ,
				 all_cp				=0,
			     cp					=&prob_original,
				 cibinom			=0,
				 rocdat				=0,
				 auroc				=0,
				 printit			=0,
				 alpha_cl			=0.95,
				 ext				=tiffp,
				 winkel				=1,
				 point				=1,
				 path				='c:\temp',
				 mi					=0,
				 out_mi				=work.out_mi_roc,
				 macro_path			=&macro_path,
				 binwidth			=&binwidth
			    );

DM "clear log;";

/*Sensitivitt usw. pro newModel zum angegebenen cp (hier: cp aus OD) */
/*in einer Datei zusammenfassen. Wird bei jedem Schleifendurchlauf unten angehngt.*/
/*Datei "result2" ist die Ergebnisdatei aus o.g. Makro;*/

/*sowie: Variablen umbenennen*/

DATA result_roc_NewModel1 (DROP=_Pos_ _Neg_ _Falpos_ _Falneg_);
	SET result2;
	RENAME next_cutpoint 	= prob_NewModel;
	RENAME _SENSIT_ 		= sens_NewModel;
	RENAME spez 			= spez_NewModel;
	RENAME youden			= youden_NewModel;
	RENAME ppv 				= ppv_NewModel;
	RENAME npv 				= npv_NewModel;
	RENAME psep				= psep_NewModel;
	dummy					= 1;
RUN;

DATA result_roc_newModel2;
	SET index_ori;
	RENAME Max_resca_ori	= max_resca_NewModel;
	RENAME R_Square_ori		= r_square_NewModel;
	RENAME brier_ori		= brier_NewModel;
	RENAME slope_ori		= slope_NewModel;
	RENAME intercept_ori	= intercept_NewModel;
	RENAME emax_ori			= emax_NewModel;
	RENAME som_ori			= som_NewModel;
	RENAME auc_ori			= auc_NewModel;
	dummy					= 1;
RUN;

/*Lscht die temporren Zwischendateien (Analyse der NewModles-Samples)*/

PROC DATASETS NOLIST; 
	DELETE  roc_newModel&kw boot&kw association_NewModel&kw
			boot_est&kw out_gof_NewModel&kw result result2 index_ori; 
RUN; 
QUIT;

/*Ende Bootstrap- Schleife*/

%END;

/*Lscht die temporren Zwischendateien*/

PROC DATASETS NOLIST; 
	DELETE boot result_roc_NewModel result_roc_boot; 
RUN; 
QUIT;

/*----------------------------------------------------------------------------------*/

/*(3) Optimismus der Kenngren nach Harrell (AUC, SomersD, Brier-Score, Emax) bestimmen:*/
/*    Aus den Ergebnissen jeweils die Mittelwerte und anschlieend die Differenzen bilden*/

/*Mittelwert bilden (Datei: result_roc_boot2)*/

PROC MEANS NOPRINT DATA=result_roc_boot2 FW=8 MAXDEC=2 ; 
	OUTPUT OUT=index_bootMean;
RUN;

/*Ausgabedatei auf die jeweiligen Mittelwerte beschrnken*/

DATA index_bootMean1;
	SET index_bootMean;
	IF _STAT_='N' THEN DELETE;
	IF _STAT_='MIN' THEN DELETE;
	IF _STAT_='MAX' THEN DELETE;
	IF _STAT_='STD' THEN DELETE;
	dummy=1;
RUN;

/*----------------------------------------------------------------------------------*/

/*Mittelwert bilden (Datei: result_roc_newModel2)*/

PROC MEANS NOPRINT DATA=result_roc_newModel2 FW=8 MAXDEC=2 ; 
	OUTPUT OUT=index_newModelMean;
RUN;

/*Ausgabedatei auf die jeweiligen Mittelwerte beschrnken*/

DATA index_newModelMean1;
	SET index_newModelMean;
	IF _STAT_='N' THEN DELETE;
	IF _STAT_='MIN' THEN DELETE;
	IF _STAT_='MAX' THEN DELETE;
	IF _STAT_='STD' THEN DELETE;
	dummy=1;
RUN;

/*----------------------------------------------------------------------------------*/

/*Beide Ergebnisdatein zusammenmergen*/

DATA index_total;
	MERGE index_bootMean1 index_newModelMean1;
	BY dummy;
RUN;

/*-----------------------------------------------------------------------------*/

/*(3.1) Differenz zwischen Boostrap-Samples und NewModel-Samples zur Schtzung des */
/*	  Optimismus*/


DATA index_total;
	SET index_total;
	slope_boot		=1;
	intercept_boot	=0;
	emax_boot		=0;
RUN;

/*Simple Bootstrap: Differenz zwischen BM und NewModel zur Schtzung des Optimismus */
/*(Variable dif_ z.B. dif_auc)*/

DATA index_total1;
	SET index_total;
	dif_auc			=abs(auc_boot-auc_newModel);
	dif_som			=abs(som_boot-som_newModel);
	dif_brier		=abs(brier_boot-brier_newModel);
	dif_slope		=abs(slope_boot-slope_newModel);
	dif_intercept	=abs(intercept_boot-intercept_newModel);
	dif_emax		=abs(emax_boot-emax_newModel);
	dif_r_square	=abs(r_square_boot-r_square_newModel);
	dif_max_resca	=abs(max_resca_boot-max_resca_newModel);
RUN;

/*Datei mit Ergebnisdatei des Ausgangsdatensatzes zusammenmergen*/

DATA index_total2;
	MERGE result_roc_ori2 index_total1;
	BY dummy;
RUN;

/*-----------------------------------------------------------------------------*/

/*(3.2) Differenz zwischen der in (3.1) berechneten Differenz und der Ausgangsdatei */
/*	    zur Bias-Korrektur des Optimismus ("Enhanced Bootstrap")				    */

DATA index_total2;
	SET index_total2;
	slope_ori		=1;
	intercept_ori	=0;
	emax_ori		=0;
RUN;

/*Enhanced Bootstrap: Differenz zwischen OD und der gerade berechneter Differenz */
/*(Variable "dif_") zur Bias-Korrektur des Optimismus (Variable d_ z.B. d_auc)*/

DATA index_total3;
		SET index_total2;
		d_auc		=abs(auc_ori-dif_auc);
		d_som		=abs(som_ori-dif_som);
		d_brier		=abs(brier_ori-dif_brier);
		d_slope		=abs(slope_ori-dif_slope);
		d_intercept	=abs(intercept_ori-dif_intercept);
		d_emax		=abs(emax_ori-dif_emax);
		d_r_square	=abs(r_square_ori-dif_r_square);
		d_max_resca	=abs(max_resca_ori-dif_max_resca);
RUN;

/*Ergebnisdatei index_total3 fr schneren Output aufsplitten*/

/*AUC*/

DATA auc (KEEP=auc_ori auc_boot auc_newModel dif_auc d_auc auc_percent auc_diff);
	SET index_total3;
	auc_diff=auc_ori-d_auc;
	auc_percent=(auc_ori-d_auc)/auc_ori;
RUN;

/*SomersD*/

DATA som (KEEP=som_ori som_boot som_newModel dif_som d_som som_percent som_diff);
	SET index_total3;
	som_diff=som_ori-d_som;
	som_percent=(som_ori-d_som)/som_ori;
RUN;

/*Brier Score*/

DATA brier (KEEP=brier_ori brier_boot brier_newModel dif_brier d_brier brier_percent brier_diff);
	SET index_total3;
	brier_diff=brier_ori-d_brier;
	brier_percent=(brier_ori-d_brier)/brier_ori;
RUN;

/*Slope*/

DATA slope (KEEP=slope_ori slope_boot slope_newModel dif_slope d_slope slope_percent slope_diff);
	SET index_total3;
	slope_diff=slope_ori-d_slope;
	slope_percent=(slope_ori-d_slope)/slope_ori;
RUN;

/*Intercept*/

DATA intercept (KEEP=intercept_ori intercept_boot intercept_newModel dif_intercept d_intercept intercept_percent intercept_diff);
	SET index_total3;
	intercept_diff=intercept_ori-d_intercept;
	intercept_percent=(intercept_ori-d_intercept)/intercept_ori;
RUN;

/*EMax*/

DATA emax (KEEP=emax_ori emax_boot emax_newModel dif_emax d_emax emax_percent emax_diff);
	SET index_total3;
	emax_diff=emax_ori-d_emax;
	emax_percent=(emax_ori-d_emax)/emax_ori;
RUN;

/*R-Square*/

DATA rsquare (KEEP=r_square_ori r_square_boot r_square_NewModel dif_r_square d_r_square r_square_percent r_square_diff);
	SET index_total3;
	r_square_diff=r_square_ori-d_r_square;
	r_square_percent=(r_square_ori-d_r_square)/r_square_ori;
RUN;

/*MaxRescaled R-Square*/

DATA maxresca (KEEP=max_resca_ori max_resca_boot max_resca_NewModel dif_max_resca d_max_resca max_resca_percent max_resca_diff);
	SET index_total3;
	max_resca_diff=max_resca_ori-d_max_resca;
	max_resca_percent=(max_resca_ori-d_max_resca)/max_resca_ori;
RUN;

/*Aktiviert OUTPUT wieder*/

ODS SELECT ALL;

/*++++++++++++++++++++*/

/*Standardoutput ausgeben*/

/*Fallzahl bestimmen:*/

%IF &miss=1 %THEN %DO;

%NUMBEROBS(datensatz=&datensatz);

%LET fallzahl=&nnnn1;

%END;

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/
%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls miss=0 angegeben wurde*/


		DATA work.fallzahl;
		SET  &data;
		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

		RUN;

%LET fallzahldata=fallzahl;

%NUMBEROBS(datensatz=&fallzahldata);

%LET fallzahl=&nnnn1;

%END;

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatz       ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

PROC DATASETS NOLIST;
	DELETE standardoutput fallzahl;
RUN;
QUIT;

/*++++++++++++++++++++*/

/*Ergebnisse ausgeben*/

PROC PRINT DATA= auc NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR auc_ori auc_boot auc_newModel dif_auc d_auc auc_diff auc_percent; 
    TITLE1 "Bootstrap-Zusammenfassung Variante 'Bootstrap-Modelle nach Harrell'";
	TITLE2 "(Enhanced und Simple Bootstrap)      Anzahl Samples: &anzahl_samples";
	TITLE3 'O=Original / B=Bootstrap / NM=NewModel / DIF=Bootstrap-NewModel / D=Original-DIF';
	TITLE4 'DIF=simple Bootstrap / D=enhanced Bootstrap / OPT=Original-D';
	TITLE5 'Optimismus und Bias-Korrektur der Accuracy-Indizes nach Harrell';
	LABEL 	auc_ori			='AUC O' 
			auc_boot 		='AUC B'
			auc_newModel 	='AUC NM'
			dif_auc 		='AUC DIF'
			d_auc			='AUC D'
			auc_diff 		='AUC OPT'
			auc_percent		='%';
RUN;

PROC PRINT DATA= som NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR som_ori som_boot som_newModel dif_som d_som som_diff som_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL 	som_ori 		='SomersD O'
			som_boot 		='SomersD B'
			som_newModel 	='SomersD NM'
			dif_som 		='SomersD DIF'
			d_som			='SomersD D'
			som_diff 		='SomersD OPT'
			som_percent		='%';
RUN;

PROC PRINT DATA= brier NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  brier_ori brier_boot brier_newModel dif_brier d_brier brier_diff brier_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	brier_ori 		='Brier Score O'
			brier_boot 		='Brier Score B'
			brier_newModel 	='Brier Score NM'
			dif_brier 		='Brier Score DIF'
			d_brier			='Brier Score D'
			brier_diff 		='Brier Score OPT'
			brier_percent	='%';
RUN;

PROC PRINT DATA= intercept NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  intercept_ori intercept_boot intercept_newModel dif_intercept d_intercept intercept_diff intercept_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	intercept_ori 		='Intercept O'
			intercept_boot 		='Intercept B'
			intercept_newModel 	='Intercept NM'
			dif_intercept 		='Intercept DIF'
			d_intercept			='Intercept D'
			intercept_diff 		='Intercept OPT'
			intercept_percent	='%';
RUN;

PROC PRINT DATA= slope NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  slope_ori slope_boot slope_newModel dif_slope d_slope slope_diff slope_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	slope_ori 		='Slope O'
			slope_boot 		='Slope B'
			slope_newModel 	='Slope NM'
			dif_slope 		='Slope DIF'
			d_slope			='Slope D'
			slope_diff 		='Slope OPT'
			slope_percent	='%';
RUN;

PROC PRINT DATA= emax NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR emax_ori emax_boot emax_newModel dif_emax d_emax emax_diff emax_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	emax_ori 		='Emax O'
			emax_boot 		='Emax B'
			emax_newModel 	='Emax NM'
			dif_emax 		='Emax DIF'
			d_emax			='Emax D'
			emax_diff		='Emax OPT'
			emax_percent	='%';
RUN;

PROC PRINT DATA= rsquare NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR r_square_ori r_square_boot r_square_NewModel dif_r_square d_r_square r_square_diff r_square_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	r_square_ori 		='R-Square O'
			r_square_boot 		='R-Square B'
			r_square_NewModel 	='R-Square NM'
			dif_r_square 		='R-Square DIF'
			d_r_square			='R-Square D'
			r_square_diff 		='R-Square OPT'
			r_square_percent	='%';
RUN;

PROC PRINT DATA= maxresca NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR max_resca_ori max_resca_boot max_resca_NewModel dif_max_resca d_max_resca max_resca_diff max_resca_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL 	max_resca_ori 		='Max-Rescaled R-Square O'
			max_resca_boot 		='Max-Rescaled R-Square B'
			max_resca_NewModel 	='Max-Rescaled R-Square NM'
			dif_max_resca 		='Max-Rescaled R-Square DIF'
			d_max_resca			='Max-Rescaled R-Square D'
			max_resca_diff		='Max-Rescaled R-Square OPT'
			max_resca_percent	='%';
RUN;

/*(4) Optimismus zustzlich fr ROC-Kenngren (Sensitivitt, Spezifitt, Youden-Index zum*/
/*	  Cutpoint des Ausgangsmodells) nach demselben Vorgehen bestimmen:*/

/*Optimismus fr ROC-Kenngren bestimmen: */
/*(ROC-Kenngren jeweils zum Cutpoint des Ausgangsmodells berechnet)*/

/*Mittelwert bilden (Datei: result_roc_boot1);*/

PROC MEANS NOPRINT DATA=result_roc_boot1 FW=8 MAXDEC=2 ; 
	OUTPUT OUT=result_roc_bootMean;
RUN;

/*Ausgabedatei auf die jeweiligen Mittelwerte beschrnken*/

DATA result_roc_bootMean1;
	SET result_roc_bootMean;
	IF _STAT_='N' THEN DELETE;
	IF _STAT_='MIN' THEN DELETE;
	IF _STAT_='MAX' THEN DELETE;
	IF _STAT_='STD' THEN DELETE;
	dummy=1;
RUN;

/*-----------------------------------------------------------------------------*/

/*Mittelwert bilden (Datei result_roc_NewModel1)*/

PROC MEANS NOPRINT DATA=result_roc_NewModel1 FW=8 MAXDEC=2 ; 
	OUTPUT OUT=result_roc_NewModelMean;
RUN;

/*Ausgabedatei auf die jeweiligen Mittelwerte beschrnken*/

DATA result_roc_NewModelMean1;
	SET result_roc_NewModelMean;
	IF _STAT_='N' THEN DELETE;
	IF _STAT_='MIN' THEN DELETE;
	IF _STAT_='MAX' THEN DELETE;
	IF _STAT_='STD' THEN DELETE;
	dummy=1;
RUN;

/*-----------------------------------------------------------------------------*/

/*Die gerade ermittelten Mittelwerte der zwei Dateien zusammenmergen:*/

DATA result_roc;
	MERGE result_roc_bootMean1 result_roc_NewModelMean1;
	BY dummy;
RUN;

/*Diese Datei mit ROC-Ergebnisdatei des OD zusammenmergen*/

DATA result_roc_gesamt (DROP=_TYPE_ _FREQ_ _STAT_ dummy);
	MERGE result_roc_ori1 result_roc;
	BY dummy;
RUN;

/*-----------------------------------------------------------------------------*/

/*(4.1) Differenz zwischen Boostrap-Samples und NewModel-Samples zur Schtzung des */
/*		Optimismus*/

/*(4.2) Differenz zwischen der in (4.1) berechneten Differenz und der Ausgangsdatei */
/*		zur Bias-Korrektur des Optimismus ("Enhanced Bootstrap")*/

/*Simple Bootstrap + Enhanced Bootstrap in einem Schritt:*/
/*Simple Bootstrap: Differenz zwischen BM und NewModel zur Schtzung des Optimismus*/
/*= Variable dif_ (z.B. dif_sens)*/
/*Enhanced Bootstrap: Differenz zwischen OD und der gerade berechneter Differenz zur */
/*					Bias-Korrektur des Optimismus */
/*					Variable d_ z.B. d_sens*/

DATA result_roc_gesamt1;
	SET result_roc_gesamt;
	dif_sens		=abs(sens_newModel-sens_boot);
	dif_spez		=abs(spez_newModel-spez_boot);
	dif_youden		=abs(youden_newModel-youden_boot);
	dif_ppv			=abs(ppv_newModel-ppv_boot);
	dif_npv			=abs(npv_newModel-npv_boot);
	dif_psep		=abs(psep_newModel-psep_boot);
	d_sens			=abs(dif_sens-sens_ori);
	d_spez			=abs(dif_spez-spez_ori);
	d_youden		=abs(dif_youden-maxyouden_ori);
	d_ppv			=abs(dif_ppv-ppv_ori);
	d_npv			=abs(dif_npv-npv_ori);
	d_psep			=abs(dif_psep-psep_ori);
RUN;

/*-----------------------------------------------------------------------------*/

/*Ergebnisdatei result_roc_gesamt1 fr schneren Output aufsplitten*/

/*Sensitivitt*/

DATA sens (KEEP=sens_ori sens_boot sens_newModel dif_sens d_sens sens_diff sens_percent);
	SET result_roc_gesamt1;
	sens_diff=sens_ori-d_sens;
	sens_percent=(sens_ori-d_sens)/sens_ori;
RUN;

/*Spezifitt*/

DATA spez (KEEP=spez_ori spez_boot spez_newModel dif_spez d_spez spez_diff spez_percent);
	SET result_roc_gesamt1;
	spez_diff=spez_ori-d_spez;
	spez_percent=(spez_ori-d_spez)/spez_ori;
RUN;

/*YoudenIndex*/

DATA youden (KEEP=maxyouden_ori youden_boot youden_newModel dif_youden d_youden youden_diff youden_percent);
	SET result_roc_gesamt1;
	youden_diff=maxyouden_ori-d_youden;
	youden_percent=(maxyouden_ori-d_youden)/maxyouden_ori;
RUN;

/*PPV*/

DATA ppv (KEEP=ppv_ori ppv_boot ppv_newModel dif_ppv d_ppv ppv_diff ppv_percent);
	SET result_roc_gesamt1;
	ppv_diff=ppv_ori-d_ppv;
	ppv_percent=(ppv_ori-d_ppv)/ppv_ori;
RUN;

/*NPV*/

DATA npv (KEEP=npv_ori npv_boot npv_newModel dif_npv d_npv npv_diff npv_percent);
	SET result_roc_gesamt1;
	npv_diff=npv_ori-d_npv;
	npv_percent=(npv_ori-d_npv)/npv_ori;
RUN;

/*PSEP*/

DATA psep (KEEP=psep_ori psep_boot psep_newModel dif_psep d_psep psep_diff psep_percent);
	SET result_roc_gesamt1;
	psep_diff=psep_ori-d_psep;
	psep_percent=(psep_ori-d_psep)/psep_ori;
RUN;
		
/*Ergebnisse ausgeben*/

PROC PRINT DATA= sens NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR sens_ori sens_boot sens_newModel dif_sens d_sens sens_diff sens_percent; 
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4 'Optimismus und Bias-Korrektur der ROC-Kenngren';
	TITLE5 "(bei Cutpoint (cp= &prob_original) des Ausgangsmodells)";
	LABEL 	sens_ori		='Sensitivitt O' 
			sens_boot		='Sensitivitt B'
			sens_newModel 	='Sensitivitt NM'
			dif_sens 		='Sensitivitt DIF'
			d_sens			='Sensitivitt D'
			sens_diff 		='Sensitivitt OPT'
			sens_percent	='%';
RUN;

PROC PRINT DATA= spez NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR spez_ori spez_boot spez_newModel dif_spez d_spez spez_diff spez_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL 	spez_ori 		='Spezifitt O'
			spez_boot 		='Spezifitt B'
			spez_newModel 	='Spezifitt NM'
			dif_spez 		='Spezifitt DIF'
			d_spez			='Spezifitt D'
			spez_diff 		='Spezifitt OPT'
			spez_percent	='%';
RUN;

PROC PRINT DATA= youden NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR maxyouden_ori youden_boot youden_newModel dif_youden d_youden youden_diff youden_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	maxyouden_ori 	='Max. YoudenIndex O'
			youden_boot		='YoudenIndex B'
			youden_newModel ='YoudenIndex NM'
			dif_youden 		='YoudenIndex DIF'
			d_youden		='YoudenIndex D'
			youden_diff 	='YoudenIndex OPT'
			youden_percent	='%';
RUN;

PROC PRINT DATA= ppv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR ppv_ori ppv_boot ppv_newModel dif_ppv d_ppv ppv_diff ppv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	ppv_ori			='Positive Korrektheit O'
			ppv_boot 		='Positive Korrektheit B'
			ppv_newModel 	='Positive Korrektheit NM'
			dif_ppv 		='Positive Korrektheit DIF'
			d_ppv			='Positive Korrektheit D'
			ppv_diff 		='Positive Korrektheit OPT'
			ppv_percent		='%';
RUN;

PROC PRINT DATA= npv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  npv_ori npv_boot npv_newModel dif_npv d_npv npv_diff npv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL	npv_ori 		='Negative Korrektheit O'
			npv_boot 		='Negative Korrektheit B'
			npv_newModel 	='Negative Korrektheit NM'
			dif_npv 		='Negative Korrektheit DIF'
			d_npv			='Negative Korrektheit D'
			npv_diff 		='Negative Korrektheit OPT'
			npv_percent		='%';
RUN;

PROC PRINT DATA= psep NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR psep_ori psep_boot psep_newModel dif_psep d_psep psep_diff psep_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	LABEL 	psep_ori		='Separation Index PSEP O'
			psep_boot 		='Separation Index PSEP B'
			psep_newModel 	='Separation Index PSEP NM'
			dif_psep 		='Separation Index PSEP DIF'
			d_psep			='Separation Index PSEP D'
			psep_diff 		='Separation Index PSEP OPT'
			psep_percent	='%';
RUN;

/*----------------------------------------------------------------------------------*/

/*(5) Hufigkeiten der Hauptzielgre*/
		
/*Proc Means ber die Datei: work.freqsBootGesamt*/

PROC SORT DATA=freqsBootGesamt;
	BY &resp_var;
RUN;

PROC MEANS DATA=freqsBootGesamt FW=8 MAXDEC=0 MEAN MIN MAX NOPRINT; 
	BY &resp_var; 
	OUTPUT OUT= freqsBootGesamt mean=MEAN min=MIN max=MAX;
RUN;

/*(5.1) im Originaldatensatz*/

PROC PRINT DATA=freqsori NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5 "Hufigkeiten der Hauptzielgre (&resp_var) im Originaldatensatz";
	VAR &resp_var frequency;
	LABEL &resp_var ="&resp_var"
		  frequency	='Hufigkeit';
RUN;

/*(5.2) in den Bootstrap- Samples (Durchschnitt)*/

PROC PRINT DATA=freqsBootGesamt NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5 "Durchschnittliche Hufigkeiten der Hauptzielgre (&resp_var) in den Bootstrap- Samples";
	VAR &resp_var mean min max;
	LABEL &resp_var ="&resp_var"
		  mean		='Mittelwert'
		  min		='Minimum'
		  max		='Maximum';
RUN;

PROC DATASETS NOLIST; 
	DELETE freqsBootGesamt freqsori boot_all
		   result_roc_boot1 result_roc_boot2 result_roc_newmodel1 result_roc_newmodel2
		   result_roc_ori1 result_roc_ori2
		   result_roc_gesamt result_roc
		   result_roc_bootMean1 result_roc_NewModelMean1
		   result_roc_NewModelMean
		   result_roc_bootMean
		   index_total2 index_total1 index_total index_bootmean
		   index_bootmean1 index_newmodelmean index_newmodelmean1
		   auc som emax brier maxresca rsquare index_total3
		   ppv npv psep result_roc_gesamt1 sens spez youden slope intercept
	; 
RUN; 
QUIT;

%MEND BOOT_MODEL_VALIDATION;
