/************************************************************************************/
/*																				  	*/
/*	NAME:		cibinom_daly.mac.sas												*/
/*  TITLE: 		Berechnung eines punktweisen Konfidenzintervalls			 		*/
/*  KEYS: 		roc analysis, roc curves, confidence limit						    */
/*	VERSION:	SAS ???																*/
/* 	PRODUCT:	BASE, STAT															*/
/*  PROCS:		-----																*/
/*																					*/
/*																					*/
/*	Verzeichnis/Dateiname:			F:\Prognose\cibinom_daly.mac.sas				*/
/*  Autoren:						L. Daly											*/
/*	Programmierer:					Leslie Daly										*/
/*	Datum der Ersterstellung:		3/2001											*/
/*  Aktuelle Makro- Version:		3/2001											*/
/*																					*/
/*  Kurzbeschreibung:				Makro zur Berechnung eines punktweisen			*/
/*									Konfidenzintervalls								*/
/*																					*/
/*	Hilfs-/Untermakro von:			PM_ROC.MAC.SAS									*/
/*																					*/
/*  Genauere Beschreibung siehe:	Makrobeschreibung des Autors					*/
/*									Daly L., Simple SAS macros for the       		*/
/*                      			calculation of exact binomial and        		*/
/*                      			Poisson confidence limits' Comput. Biol. 		*/
/*                      			Med. (1992) 22:351-361                   		*/
/*																					*/
/*  Verwendete externe Untermakros:	-----											*/
/*																					*/
/*	Bentigte Dateien:				Datensatz (Parameter 'inset=')					*/
/*																					*/
/*  Erzeugte Dateien:				work._tmp_ 										*/
/*									Ergebnisdatensatz (Parameter 'outset=')		 	*/
/*																					*/
/*  Programmnderungen 																*/
/*  und -ergnzungen 																*/
/*  (Name, Datum, Beschreibung):     												*/
/*   																   			    */
/*																					*/
/************************************************************************************/

%MACRO CIBINOM;

  data _tmp_(drop = _alpha _add_);
    set &inset;

    *** set alpha to 0.05 by default if no variable specified ***;
    %if &alpha = %then %do; 
      _alpha = 0.95; 
      alpha=0.05; 
    %end;
	%else %do;
      _alpha = 1-&alpha;
	%end;

	*** no CI to calculate ***;
    IF ((&N) EQ 0) OR (NOT(0 LE (&R) LE (&N))) OR (NOT(0 LT (_alpha) LT 1)) THEN DO;
       &P = .;
       &EXLCL = .;
       &EXUCL = .;
       &ASLCL = .;
       &ASUCL = .;
    END;

    ELSE DO;
      &P = (&R)/(&N);

      *** calculate asymptotic two-sided CI ***;
	  _add_ = probit(1-((1-_alpha)/2)) * sqrt(&p*(1-&p)/&n);
      &ASLCL = &p - _add_;
      &ASUCL = &p + _add_;

      *** calculate exact CI ***;
      IF (&R) EQ 0 THEN DO;
         &EXLCL = 0;
         &EXUCL = 1 - 10**(LOG10((1 - (_alpha))/2)/(&N));
      END;

      ELSE IF (&R) EQ (&N) THEN DO;
         &EXLCL = 10**(LOG10((1 - (_alpha))/2)/(&N));
         &EXUCL = 1;
      END;

      ELSE DO;
         &EXLCL = 1 - BETAINV(((1 + (_alpha))/2),((&N) + 1 - (&R)),(&R));
         &EXUCL = BETAINV(((1 + (_alpha))/2),((&R) + 1),((&N) - (&R)));
      END;
    END;
  run;

  *** save result ***;
  data &outset;
    set _tmp_;
  run;
%MEND CIBINOM;


/****************************************************************
*                                                               *
*   CIBINOM: A SAS MACRO FOR EXACT BINOMIAL CONFIDENCE LIMITS   *
*                                                               *
*   This SAS Macro calculates the exact confidence limits       *
*   for a Binomial proportion.                                  *
*                                                               *
*   ALGORITHM:         Daly L., Simple SAS macros for the       *
*                      calculation of exact binomial and        *
*                      Poisson confidence limits' Comput. Biol. *
*                      Med. (1992) 22:351-361                   *
*                                                               *
*   AUTHOR:            Leslie Daly,                             *
*                      Lecturer in Medical Statistics,          *
*                      Department of Public Health Medicine and *
*                      Epidemioilogy,                           *
*                      University College Dublin,               *
*                      Earlsfort Terrace,  Dublin 2, IRELAND    *
*                      Email:  LDALY@IVEAGH.UCD.IE              *
*                      Tel:  +(353)-1-7067315 or                *	
*                            +(353)-1-2693244 ext 7315          *
*                      Fax:    +(353)-1-7067407                 *
*                                                               *
*   PLATFORMS:         Tested on VM/CMS, SAS Versions 5.18 and  *
*                      6.06.01 and on MS-DOS, SAS Version 6.03. *
*                                                               *
*   FUNCTIONS CALLED:  LOG10                                    *
*                      BETAINV                                  *
*                                                               *
*   USAGE:             %CIBINOM(CL,N,R,P,PL,PU)                 *
*                                                               *
*   INPUT PARAMETERS:  CL - Confidence level as a proportion    *
*                           (ie for a 95% confidence level      *
*                           CL should be 0.95)                  *
*                      N  - Total sample size                   *
*                      R  - Number with characteristic          *
*                                                               *
*   OUTPUT PARAMETERS: P  - Observed proportion (R/N)           *
*                      PL - Lower confidence limit              *
*                      PU - Upper confidence limit              *
*                                                               *
*   USAGE NOTES:       Missing values for the output parameters *
*                      are generated if (i) N is equal to zero, *
*                      or if (ii) R is less than zero or        *
*                      greater than N or if (iii) CL is not     *
*                      between 0.0 and 1.0.                     *
*                      SW (30.3.2001): has to be used/called    *
*                      within a data step                       *
*                                                               *
  CHANGES: SW, 30MAR2001: PL to EXLCL, PU to EXLCL, 1-CL to alpha
  		   SW, 30MAR2001: usage NOT within a data step, add data=
           SW, 30MAR2001: add asymptotic CI using normal approx
           SW, 30MAR2001: set defaults CL = 0.95, N=N, 
                          EXLCL=EXLCL, EXUCL=EXUCL,
						  ASLCL=ASLCL, ASUCL=ASUCL
*****************************************************************/
