/************************************************************************************/
/*																				  	*/
/*	NAME:		gof_pulkstenis.mac.sas												*/
/*  TITLE: 		Analyse der Gte eines logistischen Regressionsmodells mittels 		*/
/*			 	globalen Goodness-Of-Fit-Tests										*/
/*  KEYS: 		goodness-of-fit, logistic regression, sparse data, 				    */
/*			 	standard pearson-test, standard deviance-test						*/
/*	VERSION:	SAS ???																*/
/* 	PRODUCT:	BASE, STAT															*/
/*  PROCS:		LOGISTIC, FREQ, MEANS												*/
/*																					*/
/*																					*/
/*	Verzeichnis/Dateiname:			F:\Prognose\gof_pulkstenis.mac.sas				*/
/*  Autoren:						E. Pulkstenis, T. Robinson						*/
/*	Programmierer:					Erik Pulkstenis									*/
/*	Datum der Ersterstellung:		4/2003											*/
/*  Aktuelle Makro- Version:		4/2003											*/
/*																					*/
/*  Kurzbeschreibung:				Makro zur Analyse der Gte eines logistischen 	*/
/*									Regressionsmodells mittels globalen				*/
/*									Goodness-Of-Fit-Tests							*/
/*																					*/
/*	Hilfs-/Untermakro von:			PM_GOF.MAC.SAS									*/
/*																					*/
/*  Genauere Beschreibung siehe:	Makrobeschreibung des Autors					*/
/*									Pulkstenis E, Robinson T. Two goodness-of-fit	*/
/*									tests for logistic regression models with 		*/
/*									continuous covariates. 							*/
/*									Statistics in Medicine 2002; 21:79-93 			*/
/*																					*/
/*  Verwendete externe Untermakros:	-----											*/
/*																					*/
/*	Bentigte Dateien:				Datensatz (Parameter 'dset=')					*/
/*																					*/
/*  Erzeugte Dateien:				work.expect work.final work.hats work.hold1 	*/
/*									work.hold2 work.holder work.phats work.chis 	*/
/*									work.crosstab work.devs work.display 			*/
/*									work.display1 work.display2						*/
/*																					*/
/*  Programmnderungen 																*/
/*  und -ergnzungen 																*/
/*  (Name, Datum, Beschreibung):     												*/
/*   																   			    */
/*																					*/
/************************************************************************************/

%macro goodness(dset=,vars=,event=,ref=,contvar=,k=,y=,alpha=);

/* dset:     SAS data set                                                */
/* vars:     categorical covariates in model                             */
/* contvar:  continuous covariates in model                              */
/* k:        number of categorical terms in model                        */
/*           (Main effects + Categorical Interactions) 
             not counting the intercept                                   */
/* y:        binary response (coded 0/1 or Y/N)                           */
/* alpha:    desired significance level                                   */

 
data source;
set &dset;               /* remove records with missing Y values */
if &y=. then delete;
run;

/*------------------*/
/*C. Ziegler 10/2003*/

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&vars,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&vars,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The cvar contains &count variables.;

/*Anzahl der Referenzen ermitteln*/

%GLOBAL count_ref var_ref;
%LET count_ref=1;
%LET var_ref=%QSCAN(&ref,&count_ref,*);
%DO %WHILE(&var_ref ne);
     %LET count_ref=%EVAL(&count_ref+1);
     %LET var_ref=%QSCAN(&ref,&count_ref,*);
%END;
%LET count_ref=%EVAL(&count_ref-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The refs contains &count_ref variables.;

/*------------------*/

proc logistic data=source descending ;

/*------------------*/
/*C. Ziegler 10/2003*/
/*Falls class- Variablen angegeben wurden:*/

%IF &count ne 0 %THEN %DO;

/*   PROC LOGISTIC CLASS- Statement ohne dazugehrigen Referenzkategorien, falls keine */
/*   angegeben wurden*/

%IF &count_ref=0 %THEN
	%DO;
		CLASS

	%DO i=1 %TO &count;
		%LET var_liste=%SCAN(&vars,&i,%STR( ));
      
		&var_liste (PARAM=REF REF=FIRST)

	%END;
     	;
%END;

%ELSE
	%DO;

/*PROC LOGISTIC CLASS- Statement mit dazugehrigen Referenzkategorien*/

		CLASS
	%DO i=1 %TO &count;
   		%LET var_liste=%SCAN(&vars,&i,%STR( ));
   		%LET ref_liste=%SCAN(&ref,&i,*);

   		&var_liste (PARAM=REF REF=&ref_liste)
	%END;
   		;
%END;

%END;
/*------------------*/

title3' ';
title5' ';
model &y

/*------------------*/
/*C. Ziegler 10/2003*/
(

%IF &event=1 %THEN
	%DO;
		event='1'
%END;

%IF &event=0 %THEN
	%DO;
		event='0'
%END;

)

/*------------------*/

=&vars &contvar   / lackfit;
output out=phats p=phats;
run;

data hats;
set phats;
if phats=. then delete;
keep &vars &Y phats;
run;

/* sort within the cross-classification of the
   design by the phats */

proc sort data=hats; by &vars phats;
run;

proc freq data=hats noprint  ;
tables &vars  / list out=crosstab;
by &vars;
run;

data crosstab;
set crosstab;
do index=1 to count;
output;
end;
run;

data hats;
merge hats crosstab;
covar=1;
retain covar;
if index > count/2+.5 then covar=covar+1;
run;

proc freq data=hats noprint;
tables &vars covar / out=hold1;
by &vars covar;
run;

proc freq data=hold1 noprint;
tables percent / out=hold2;
run;

data hold2; set hold2; I=count; do pattern=1 to count; output; end; drop percent count; run;

data hold1;
merge hold1 hold2;
drop count percent;
run;

data hats; merge hats hold1; by &vars covar;
df=I-&k-2;
drop count percent index covar; run;


proc sort; by &vars; run;


proc freq noprint ;
tables df*pattern*&Y  /    out=crosstab sparse;
by &vars;
run;


proc sort data=crosstab; by pattern &Y; run;

proc freq data=crosstab noprint  ;
tables pattern*&Y / out=holder sparse;
run;

data holder; set holder; keep pattern &Y; run;

proc sort; by pattern &Y; run;

data crosstab; merge crosstab holder; by pattern &Y; run;

data crosstab; merge crosstab hold1; by pattern; drop covar I;
df=I-&k-2;
if count=. then count=0;
if percent=. then percent=0;
run;


proc sort data=crosstab; by pattern; run;


proc print data=crosstab;
title3'crosstab data set';
run cancel  ;


proc sort data=hats; by df pattern ; run;

proc means data=hats sum n noprint ;
var phats;
by df pattern ;
output  out=expect sum=Expected N=N;
run;

data expect;
set expect;
keep df pattern  Expected N;
run;

proc sort data=expect; by df pattern ; run;

proc sort data=crosstab; by df pattern ; run;

data crosstab; merge crosstab expect;
by df pattern ;
run;

data crosstab; set crosstab ;
by pattern;
observed=count;
if first.pattern=1 then Expected=N-Expected;
chi=(count-Expected)*(count-Expected)/Expected;
if observed ne 0 then dev=2*count*log((count/Expected)); else dev=0;
drop count percent n;
run;

proc print data=crosstab;
title3'Observed vs Expected Counts';
run cancel;

data display1; set crosstab; by pattern; if first.pattern=0 then delete;
expect0=expected;
observe0=observed;
chi0=chi;
dev0=dev;
drop expected observed chi dev;
run;

data display2; set crosstab; by pattern; if first.pattern=1 then delete;
expect1=expected;
observe1=observed;
chi1=chi;
dev1=dev;
drop expected observed chi dev;
run;

data display;
merge display1 display2;
dev=dev0+dev1;
chi=chi0+chi1;
attrib expect0 expect1 label='Expected Count' format=6.2
       observe0 observe1 label='Observed Count'
           chi label='X*^2 Contibution' format=6.2
           dev label='D*^2 Contribution' format=6.2;
run;

proc print data=display noobs;
title3'Covariate Pattern Classification';
var &vars pattern;
run;

proc print data=display noobs label;
title3'Table of Observed and Expected Frequencies';
title5"--- Response Y = 0/N --- --- Response Y = 1/Y ---                   ";
var pattern expect0 observe0 expect1 observe1 chi dev;
run;

proc means data=crosstab sum noprint;
title3' ';
var chi;
by df;
output  out=chis sum=csquare;
run;

data chis;
set chis;
cpvalue=1-probchi(csquare,df);
if cpvalue <=&alpha then choicepc='Reject';
if cpvalue > &alpha then choicepc='No Rej';
keep csquare df cpvalue choicepc;
run;

proc means data=crosstab sum noprint ;
title3' ';
var dev;
by df;
output  out=devs sum=dsquare;
run;

data devs;
set devs;
dpvalue=1-probchi(dsquare,df);
if dpvalue <=&alpha then choiced='Reject';
if dpvalue > &alpha then choiced='No Rej';
keep dsquare df dpvalue choiced;
run;

data final; merge devs chis;
attrib df label='Degrees of Freedom'
       dpvalue label='Modified Deviance P-value'
       dsquare label='Modified Deviance Statistic'
       cpvalue label='Modified Pearson Chi-square P-value'
       csquare label='Modified Pearson Chi-square Statistic'
       choiced label='Modified Deviance Result'
       choicepc label='Modified Pearson Chi-square Result';
run;

proc print noobs label;
var df csquare cpvalue choicepc dsquare dpvalue choiced;
run;
%mend goodness;



/*options ls=95  ps=80 nofmterr;*/
/*libname home  'INSERT PATHNAME HERE';*/
 
 
/* This program implements goodness-of-fit procedures corresponding to the
   following Reference:

   Pulkstenis E, Robinson T. Two goodness-of-fit tests for logistic regression models
   with continuous covariates. Statistics in Medicine 2002; 21:79-93 

   Author: Erik Pulkstenis
           erik_pulkstenis@hgsi.com
   Date: 23APR2003

   This software can be freely used for non-commercial purposes and can be freely 
   distributed. The author is willing to provide assistance where required.     */

/* The data set 'sample.sd2'   contains 2 categorical variables X1, X2
                                      1 continuous variable Z
                                      1 interaction X2Z
                                      1 binary response Y */

/* The program is provided "as is", without warranty of any kind, either
   expressed or implied, including, but not limited to, the implied
   warranties of merchantability and fitness for a particular purpose. 
   It may or may not perform as intended in any situation. The user assumes
   all responsibility for its results */

/* Fit full model */
/*%goodness(vars=x1 x2 ,contvar=z x2z ,k=2,dset=home.sample,y=y,alpha=0.05);*/
/* Omit significant interaction term */
/*%goodness(vars=x1 x2 ,contvar=z ,k=2,dset=home.sample,y=y,alpha=0.05);*/
 