/************************************************************************************/
/*						        													*/
/*     NAME: pm_bootstrap_validation.mac.sas	         							*/
/*    TITLE: Validierung von logistischen Regressionsmodellen mittels 				*/
/*			 Bootstrap (Bootstrap-Modelle nach Harrell/ Bootstrap- Variante 		*/
/*			 'Mittleres Modell')													*/
/*     KEYS: bootstrap, validation, logistic regression, roc analysis			    */
/*  VERSION: SAS 8.2												      			*/
/*  PRODUCT: BASE, STAT							        							*/
/*    PROCS: LOGISTIC									        					*/
/*															         				*/
/*																					*/
/*  Verzeichnis/Dateiname:			F:\Prognose\pm_bootstrap_validation.mac.sas	    */
/*  Autoren:						R. Muche, C. Ring, I. Assfalg, C. Ziegler		*/
/*  Programmierer:					Christoph Ziegler/ Ina Assfalg					*/
/*  Datum der Ersterstellung:		9/2002											*/
/*  Aktuelle Makro-Version:			12/2003											*/
/*																					*/
/*  Kurzbeschreibung:				Makro zur internen Validierung von logistischen */
/*									Regressionsmodellen mittels	Bootstrap			*/
/*									(Bootstrap-Modelle nach Harrell/ 				*/
/* 					 				Bootstrap- Variante 'Mittleres Modell')			*/
/*																					*/
/*  Verwendete externe Untermakros:	boot_model_validation.mac.sas 					*/
/*									boot_mean_validation.mac.sas					*/
/*									numchar.mac.sas									*/
/*									numberobs.mac.sas								*/
/*									exist.mac.sas									*/
/*									exist_var.mac.sas								*/
/*									ref.mac.sas										*/
/*									error1.sas	error2.sas	error3.sas	error4.sas	*/
/*									error5.sas	error6.sas	error7.sas	error8.sas	*/
/*									error9.sas	error11.sas	error24.sas error52.sas	*/
/*									error53.sas error60.sas	error66.sas				*/
/*																					*/
/*  Bentigte Dateien:				Datensatz (Parameter 'data=')			   		*/
/*															           				*/
/*																					*/
/*  Erzeugte Dateien:				-----		 									*/
/*  								        									   	*/
/*  						 										        		*/
/*																					*/
/*  Programmnderungen				Sandra Mller, 04.05.2016, Makrovariable		*/
/*  und -ergnzungen 				binwidth hinzugefgt							*/ 
/*  (Name, Datum, Beschreibung):											 		*/
/*  																				*/
/*        																			*/
/************************************************************************************/
/*

 TITLE
 -----
 pm_bootstrap_validation.mac.sas

 SAS- Makro zur internen Validierung von logistischen Regressionsmodellen mittels 
 Bootstrap (Bootstrap-Modelle nach Harrell/ Bootstrap- Variante 'Mittleres Modell')	
	
 BACKGROUND
 ----------

 Der Begriff Bootstrap leitet sich von der Redewendung "to pull oneself up by ones 
 bootstrap" (sich selbst an den eigenen Stiefelschlaufen hochziehen) ab und spielt auf 
 die "Abenteuer des Baron von Mnchhausen" von Rudolf Erich Raspe an. Hinter der Idee 
 des Bootstraps verbirgt sich ein Ziehen mit Zurcklegen. Es wird durch zuflliges 
 Ziehen mit Zurcklegen aus einem Datensatz ein neuer Datensatz gleicher Fallzahl gezogen 
 [davison1997, efron1993]. Dadurch erhlt man einem dem Ausgangsdatensatz hnlichen 
 Datensatz mit gleichen statistischen Eigenschaften, in dem die Beobachtungen des 
 Ausgangsdatensatzes keinmal, einmal oder mehrmals vorkommen knnen. Dabei ist festgelegt, 
 dass alle Beobachtungen die gleiche Wahrscheinlichkeit 1/n haben, um gezogen zu werden. 
 Die Idee ist nun, durch mehrfache Erzeugung solcher Bootstrap-Samples und Wiederholung 
 der Auswertungen vom Originaldatensatz eine Schtzung der empirischen Verteilung ber die 
 interessierende Statistik zu bekommen, anhand derer dann u.a. Varianzschtzungen fr die 
 Parameter bestimmt werden knnen. 

 Fr die Modellvalidierung auf Grundlage der logistischen Regression bedeutet dies nun, 
 anhand vieler Bootstrap-Samples die Prognosekennzahlen zu bestimmen und daraus den 
 validierten Schtzer ermitteln zu knnen. Assfalg [assfalg2003] stellt folgende 
 berlegungen zur Anzahl der Bootstrap-Stichproben zusammen: "Geht die Anzahl der 
 Bootstrap-Samples gegen unendlich, nhert sich die Bootstrap-Stichprobenverteilung der 
 interessierenden Statistik der wahren Verteilung an [efron1993]. ... Wieviel 
 Bootstrap-Samples mindestens erzeugt werden sollen um diesen Effekt zu erreichen, wird 
 in der Literatur unterschiedlich festgelegt. Efron und Tibshirani treffen hierfr eine 
 Unterscheidung welche statistische Kenngre geschtzt werden soll. Sie empfehlen z.B. 
 mindestens 50 bis 200 Bootstrap-Samples zur Schtzung des Standardfehlers und mindestens 
 1000 Bootstrap-Samples zur Schtzung von Konfidenzintervallen [efron1993]. Auch Harrell 
 trifft eine hnliche Einteilung [harrell2001], was durch Sauerbrei [sauerbrei1999] 
 ebenfalls besttigt wird."   

 Auf Grund des eher mehrhundertfachen Durchfhrens dieses Vorgehens (Stichprobenziehung 
 und Auswertung) gehrt der Bootstrap zu den Resampling-Verfahren. Die prinzipielle Idee, 
 auf der das "Bootstrapping" basiert reicht mindestens 2 Jahrhunderte zurck, fand jedoch 
 in der praktischen Anwendung wegen der rechenintensiven Anforderungen zunchst wenig 
 Beachtung [assfalg2003]. Erst durch die technischen Entwicklungen bei den heutigen 
 Computern kann das Verfahren allgemein eingesetzt werden. Dabei ist eine direkte 
 Implementierung des Bootstraps wie auch die Kreuzvalidierung in der Statistiksoftware 
 SAS noch nicht erfolgt, so dass eine Programmierung zur Durchfhrung dieser 
 Validierungsverfahren notwendig ist. 

 Die Vor- und Nachteile des Bootstraps knne folgendermaen zusammengefasst werden:
 - Der Hauptvorteil dieses Verfahrens gegenber den anderen Validierungsverfahren liegt 
   (unter anderem) an der relativ stabilen und varianzarmen Schtzung der Modellparameter
   [harrell2001].
 - Beim Bootstrap wird die Validierung auf dem gesamten Datensatz (n Beobachtungen) 
   durchgefhrt. Es mssen nicht wie beim Data-Splitting und bei der Kreuzvalidierung 
   Beobachtungen bei der Wiederholung der Modellanpassung herausgenommen werden.
 - Bootstrap kann auf jede beliebige, einfache oder komplizierte Statistik angewendet 
   werden [efron1983]
 - Es mssen keine Verteilungsannahmen gemacht werden, da die Kenngren aus der 
   empirischen Verteilung geschtzt werden.
 - Die Gte der Bootstrap-Schtzer hngt von der Datenqualitt der zugrundeliegenden 
   Stichprobe ab.
 - Bei ausreierempfindlichen Statistiken kann das Ergebnis verflscht werden, wenn einige 
   Ausreier im Originaldatensatz vorhanden sind [mooney1993]. 
 - Ein Kritikpunkt ist oft die unter Umstnden hohe Rechenzeit bei groen Datenstzen. 

 In der Diplomarbeit von Assfalg [assfalg2003] knnen die Grundlagen, verschiedene 
 Varianten und eine Umsetzung als SAS-Makro nachgelesen werden. Als weitere bersichts-
 arbeiten eignen sich die Lehrbcher von Efron und Tibshirani [efron1993] und Davison 
 und Hinkley [davison1997], in denen neben der allgemeinen Herleitung dieses Resampling-
 verfahrens auch die Anwendung auf den Vorhersagefehler in Regressionsmodellen enthalten 
 ist.  

 Simple und Enhanced Bootstrap
 Mit dem Bootstrap-Vorgehen wurde u.a. von Efron und Tibshirani [efron1993] ein Vorgehen 
 beschrieben, das fast unverzerrte Schtzer fr die Modellperformanz in der Zukunft 
 schtzen kann, ohne dass Beobachtungen zur Schtzung zurckgehalten werden wie bei der 
 Kreuzvalidierung. Dabei wird meist vom intuitiven Ansatz des "simple Bootstrap" 
 ausgegangen, bei dem in jedem Bootstrap-Sample die Regressionskoeffizienten neu geschtzt 
 werden und dies so erzeugte Modell auf den Oroginaldatensatz (OD) zur Schtzung der 
 Prognosekennzahlen angewendet wird. Durch eine Mittelung dieser meist mehreren hundert 
 Schtzer ergibt sich dann der simple Bootstrap-Schtzer nach Modellvalidierung.

 Efron und Tibshirani zeigen in Ihrem Buch, dass eine etwas komplexere Variante, der 
 enhanced Bootstrap-Schtzer, bessere Eigenschaften bzgl. des Bias hat. Bei dieser 
 Variante wird nicht der Schtzer direkt durch Mittelwertbildung der Bootstrap-Ergebnisse 
 bestimmt, sondern man schtzt hier einen sogenannten Optimismus, den Schtzer fr den 
 gesuchten Bias. Nach Schtzung des Optimismus kann dann durch Subtraktion vom Prognose-
 ergebnis nach Reklassifikation der enhanced Bootstrap-Schtzer ermittelt werden. Das 
 allgemeine Vorgehen kann nach Efron und Tibshirani [efron1993] und Harrell [harrell2001] 
 folgendermaen zusammengefasst werden:	
 1. Berechnung der Prognosegte am Originaldatensatz (OD) (Gte 1.)
 2. B Bootstrap-Samples durch Ziehen mit Zurcklegen aus Datensatz erzeugen
 3. In jedem Bootstrap-Sample Refit des Modells
 4. Anwendung der Bootstrap-Modelle auf OD und Berechnung der Prognosegte (Gte 4.), 
    Mittelwert ber Gtemae (simple Bootstrap)
 5. Anwendung Bootstrap-Modelle auf Bootstrap-Samples und Berechnung Prognosegte (Gte 5.)
 6. Optimismus = Mittelwert der Differenzen der beiden Prognosegten fr die Anwendung auf 
    OD und auf Bootstrap-Sample (Gte 5. - Gte 4.)
 7. Enhanced Bootstrap = Gte 1. - Optimismus

 Mittleres Modell 
 Eine etwas andere Herangehensweise beschreiben u.a. Altman und Andersen [altman1989] sowie 
 Johnson [johnson1996]. Auch hier werden ausgehend vom Originaldatensatz die Bootstrap-
 Samples erzeugt und fr jedes Sample das Bootstrap-Modell neu angepasst, in dem die 
 Regressionskoeffizienten geschtzt werden. Im Unterschied zum enhanced Bootstrap werden 
 jetzt allerdings erst diese Regressionskoeffizienten durch Mittelwertbildung zusammen-
 gefasst, so dass ein sogenanntes "mittleres Modell" entsteht. Dieses wird dann auf den 
 Originaldatensatz angewendet und nach ROC-Analyse die so validierten Prognosegten 
 ermittelt [assfalg2003]. 

 Erzeugt werden mit diesem Vorgehen u.a. Bootstrap-Schtzer fr die Regressionskoeffizienten. 
 Dieses eine so neu geschtzte Modell kann als validierte Variante des Ausgangsmodells 
 gelten. In Bezug auf die Korrektur des beroptimismus des Ausgangsmodells wird hier aller-
 dings weniger erwartet als beim enhanced Bootstrap [assfalg2003]. Als weitere Variante kann 
 das mittlere Modell nicht nur auf den Ausgangsdatensatz sondern auch auf jeden Bootstrap-
 Sample angewendet werden, die Prognosegten bestimmt und durch Mittelwertbildung zusammen-
 gefasst werden. Diese Variante ist ebenfalls im Makro enthalten (s.u.). 


 Weitere Verfahren  
 Es werden in der Literatur noch weitere Varianten angegeben, so z.B. der .632- und der 
 .632+ -Bootstrap. Diese beiden Verfahren sind allerdings bisher noch nicht in das Makro 
 implementiert worden. 

 

 DESCRIPTION
 -----------

 Das Makro "PM_BOOTSTRAP_VALIDATION.MAC.SAS" ermglicht die Auswahl zweier
 verschiedener Bootstrap-Validierungsvarianten, die jeweils als eigene 
 Programmmodule separat ausgelagert vorliegen und deren programmtechnische
 Funktionsweisen im Folgenden erlutert sind.

 Bootstrap-Modelle nach Harrell (simple und enhanced Bootstrap):

- Zuerst werden mittels "PM_LOGREG.MAC.SAS" (ohne Variablenselektion) und 
   "PM_ROC.MAC.SAS" die Prognosegtekenngren des bergebenen Originaldatensatzes ermittelt.
 - Anschliessend wird in einer Schleife durch zuflliges Ziehen mit Zurcklegen 
   (SAS-Zufallsfunktion RANUNI) je ein Bootstrap-Sample gezogen.
 - Auf dieses Bootstrap-Sample wird "PM_LOGREG.MAC.SAS" (ohne Variablenselektion)
   angewandt, das heit ein neues Modell entwickelt und anschliessend dazu die
   Prognosegtekenngren bestimmt (Makro "PM_ROC.MAC.SAS").
 - Im Output sind diese Kenngren mit "Bootstrap" gekennzeichnet.
 - Ferner wird innerhalb der Schleife jedes einzelne Bootstrap-Sample auf den
   Originaldatensatz angewandt ("PM_LOGREG.MAC.SAS" mit Option INEST=).
 - Auch hierzu werden mittels "PM_ROC.MAC.SAS" die entsprechenden Prognosegtekenn-
   gren ermittelt. 
 - Im Output sind diese Kenngren mit "NewModel" gekennzeichnet.
 - Dieses Vorgehen wird so oft wie unter "anzahl_samples=" angegeben wiederholt.
 - Nach dem Ende der Schleife werden jegliche Kenngren gemittelt. 
 - Der sogenannte "simple bootstrap" wird anschliessend durch die Differenz 
   zwischen "bootstrap" und "NewModel" berechnet, der sogenannte "enhanced 
   bootstrap" durch die Differenz zwischen Originalkenngren und "simple 
   bootstrap".
 - Abschliessend werden im Output die Prognosegten der Bootstrap-Validierung denen 
   des Originaldatensatzes gegenbergestellt sowie die Differenz und die prozentuale 
   Abweichung vom Ausgangswert zustzlich ausgegeben. 
 - Neben den Cutpoint-unabhngigen Prognosegtekenngren werden die Cutpoint-
   abhngigen Prognosegtemae zum Cutpoint des Ausgangsmodells ausgegeben.
 - Zustzlich werden im Output darber hinaus die Hufigkeiten der Hauptzielgre
   (event=0/event=1) sowie die durchschnittlichen Hufigkeiten der Hauptzielgre
   in den Bootstrap-Samples zusammen mit dem Minimum und dem Maximum mit ausgegeben.


 Bootstrap-Variante 'Mittleres Modell':

 - In einem ersten Schritt werden mittels "PM_LOGREG.MAC.SAS" (ohne Variablen-
   selektion) und "PM_ROC.MAC.SAS" die Prognosegtekenngren des bergebenen
   Originaldatensatzes ermittelt.
 - Anschliessend wird in einer Schleife durch zuflliges Ziehen mit Zurcklegen 
   (SAS-Zufallsfunktion RANUNI) je ein Bootstrap-Sample gezogen.
 - Auf dieses Bootstrap-Sample wird "PM_LOGREG.MAC.SAS" (ohne Variablenselektion)
   angewandt, das heit ein neues Modell entwickelt und die Regressionskoeffizienten 
   abgespeichert.
 - Dieses Vorgehen wird so oft wie unter "anzahl_samples=" angegeben wiederholt.
 - Am Ende der Schleife werden die Regressionskoeffizienten aller gezogenen 
   Bootstrap-Samples gemittelt.
 - Dieses gemittelte Modell wird anschliessend auf den Ausgangsdatensatz angewandt
   ("PM_LOGREG.MAC.SAS" mit Option INEST=), falls der Parameter "mean_flag" auf den
   Wert 1 gesetzt wurde.
 - Wurde jedoch dem Parameter "mean_flag" der Wert 2 bergeben, so wird das mittlere
   Modell auf jeden einzelnen Bootstrap-Sample angewendet ("PM_LOGREG.MAC.SAS" mit 
   Option INEST=). 
 - Die Bestimmung der Prognosegtekenngren erfolgt danach mit Hilfe des
   Makros "PM_ROC.MAC.SAS".
 - Wurde das mittlere Modell auf jeden einzelnen Bootstrap-Sample angewendet, so
   werden alle Prognosegtekenngren gemittelt.
 - Abschliessend werden im Output die Prognosegten der Bootstrap-Validierung denen 
   des Originaldatensatzes gegenbergestellt sowie die Differenz und die prozentuale 
   Abweichung vom Ausgangswert zustzlich ausgegeben. 
 - Neben den Cutpoint-unabhngigen Prognosegtekenngren werden die Cutpoint-
   abhngigen Prognosegtemae sowohl zum Cutpoint des Ausgangsmodells als auch
   zum neuen Cutpoint, welcher nach maximalem Youden-Index ausgewhlt wird,
   ausgegeben.
 - Zustzlich werden im Output darber hinaus die Hufigkeiten der Hauptzielgre
   (event=0/event=1) sowie die durchschnittlichen Hufigkeiten der Hauptzielgre
   in den Bootstrap-Samples zusammen mit dem Minimum und dem Maximum mit ausgegeben.



 LITERATURE
 ----------

 - SAS Online Doc zur Prozedur PROC LOGISTIC
 - Altman, DG, Andersen PK (1989): Bootstrap investigation of the stability of a 
                   Cox regression model. Statist. Med. 8, 771-783
 - Assfalg I (2003): Die Bootstrap- Methode zur internen Validierung von 
				   Prognosemodellen. Diplomarbeit, FH Ulm
 - Boudreaux D, Cranford K (1995): Simple Random Sampling and Subsetting Strategies 
				   Using SAS Software, Observations, Third Quarter, pp. 34-40	
 - Davison AC, Hinkley DV (1997): Bootstrap methods and their application, 
				   Cambridge University Press, USA, 1997
 - Efron B, Tibshirani R J (1993): An Introduction to the Bootstrap. 
				   Chapman & Hall, NewYork 
 - Harrell F E (2001): Regression Modeling Strategies, Springer Verlag New York
 - Johnson P (1996): The bootstrap estimate of model parameters in logistic regression 
                   - a SAS macro.  http://lib.stat.cmu.edu/general/bootlogit 
                   (aufgerufen am 20.11.2003)
 - Mooney CZ, Duval RD (1993): Bootstrapping: a nonparametric approach to statistical 
                   inference. SAGE, Newbery Park
 - Sauerbrei W (1999): The use of resampling methods to simplify regression models in 
                   medical statistics. Applied Statistics 48, 313-329
 - Ziegler C (2003): Ein SAS-Makropaket zur Entwicklung und Validierung von 
				   Prognosemodellen auf Basis der logistischen Regression. 
				   Diplomarbeit, FH Ulm 


 SYNTAX
 ------
 [Makroaufruf und Beschreibung der zu bergebenden Parameter]

 %PM_BOOTSTRAP_VALIDATION(		
   *data			= ,		Datensatz
   *resp_var		= , 	Zielgre
	event			= 1,	Event- Kategorie der binren Zielgre (0/1)
   *xvar			= ,		stetige Variable(n)
   *cvar			= ,		kategorielle Variable(n)
	ref				= ,		Referenzkategorien fr kategorische Variablen
	miss 			= 0,	Missing Values ausschliessen (ja/ nein)
	boot_type		= 1,	Auswahl der Bootstrap-Variante
	anzahl_samples	= 200,  Anzahl der Bootstrap- Samples
	mean_flag		= 1,    Anwendung des mittleren Modells (Variante 'Mittlere Modelle')
	random			= 0,	Auswahl der Zufallszahlenerzeugungsart
   *macro_path		= ,		Pfad zur Makrosammlung
	binwidth		= 0		Bin-Breite
  );

  Mit (*) gekennzeichnete Parameter sind beim Makroaufruf zwingend erforderlich.
  Bereits gesetzte Parameter stellen die jeweilige Standardeinstellung dar.

   data:		    Parameter zur Angabe des zu analysierenden Datensatzes. Wurde 
					beispielsweise vor dem Aufruf des Makros ein Datensatz mit dem 
					Namen "dat300" angelegt, lautet der korrekte Aufruf: "data=dat300". 
					Zustzlich kann auch die dazugehrige SAS-Library bergeben werden 
					(Beispiel: "data=work.dat300"). Die bergabe dieses Parameters ist 
					zwingend erforderlich.
 
   resp_var:		"resp_var" steht fr "Response Variable", also fr die Zielgre. 
					Dieser bergabeparameter ist ebenfalls zwingend erforderlich. 
					Ferner wird eine binre Datenstruktur (0/1) der Zielgre bentigt. 
					Die Zielgre muss mit "0" und "1" kodiert sein. Sollte dies nicht 
					der Fall, ist eine vorherige Umkodierung erforderlich. Heit 
					beispielsweise die Zielgre "tod" lautet der Aufruf wie folgt: 
					"resp_var=tod".

   event:			Je nach dem welche Kategorie der binren Zielgre untersucht 
					werden soll, kann diese mit Hilfe dieses Parameters bergeben werden. 
					Soll beispielsweise die Zielgre "tod" analysiert werden und steht 
					"1" fr tot sowie "0" fr lebendig, lautet der korrekte Aufruf 
					"event=1". Dies ist zugleich die Standardeinstellung, da blicherweise
					"1" fr das Eintreten eines bestimmten Ereignisses steht.

   xvar:			Dieser Parameter ist zur Angabe der stetigen Variablen 
					(Einflussgren). Allgemein gesagt sollten hier alle Variablen 
					genannt werden, die nicht in das CLASS-Statement von PROC LOGISTIC 
					aufgenommen werden sollen, also beispielsweise auch quasi-stetige 
					Variablen. Werden mehrere stetige oder quasi-stetige Variablen 
					bergeben, so mssen diese durch ein Blank (Leerzeichen) getrennt 
					werden. Beispiel: "xvar=alter bmi".

   cvar:			"cvar" steht fr "categorical variables" und ist fr die bergabe 
					aller kategorischen sowie aller dichotomen Variablen verantwortlich. 
					Diese werden innerhalb der LOGISTIC-Prozedur in das CLASS-Statement 
					aufgenommen und werden automatisch Dummy-kodiert. Werden mehrere 
					kategorische Variablen bergeben, mssen diese getrennt durch ein 
					Leerzeichen (Blank) bergeben werden (Beispiel: "cvar=diagnose msnr").
					Wurden dem Parameter "cvar" dichotome Variablen bergeben, so werden 
					diese zwar in das CLASS-Statement von PROC LOGISTIC aufgenommen, 
					jedoch keine Dummy-Kodierung durchgefhrt, da dichotome Variablen nur 
					zwei Klassen besitzen.

   ref:				Mittels dieses Parameters, der fr "reference" steht, ist es 
					mglich, pro bergebener kategorischer Variable eine sogenannte 
					Referenzkategorie anzugeben. Werden keine Referenzkategorien 
					angegeben, wird der Standard bei jeder Variablen innerhalb des 
					CLASS-Statements von PROC LOGISTIC auf REF=FIRST gesetzt. Dies 
					bedeutet, dass die erste (sortierte) Kategorie jeder bergebenen 
					Variablen als Referenzkategorie gewhlt wird. Werden 
					Referenzkategorien bergeben, so muss deren Anzahl mit der Anzahl 
					der bergebenen kategorischen Variablen bereinstimmen. SAS bietet 
					standardmig die Kategorien FIRST und LAST an. Zustzlich ist es 
					mit Hilfe dieses Parameters mglich, beliebige Kategorien anzugeben. 
					Das Trennungszeichen ist der Stern (*). Eigens gewhlte 
					Referenzkategorien mssen zustzlich in Hochkommata eingeschlossen 
					werden. Die SAS-Standardkategorien FIRST / LAST nicht. Die ntige 
					Form lautet beispielsweise: "ref='1'*FIRST*'4'". Der dazugehrige 
					"cvar"-Parameter: "cvar=icdklass3 msnr_num beginn_b_num". Von der 
					bergebenen Variablen "icdklass" wird die Kategorie "1" als 
					Referenzkategorie gewhlt. Von der Variablen "msnr_num" die erste 
					(sortierte), unabhngig was diese fr einen Wert besitzt und von der 
					Variablen "beginn_b_num" die Kategorie "4". 

   miss:			Dieser Parameter legt fest, ob Beobachtungen mit fehlenden Werten 
					(missing values) in den unter "xvar", "cvar" und "resp_var" 
					bergebenen Variablen vor der Analyse aus dem Datensatz eliminiert 
					werden sollen (0) oder nicht (1). Der Standardwert dieses Parameters 
					ist 0 (Complete Case-Analyse). Hierbei ist zu beachten, dass fehlende 
					Werte im Datensatz durch einen Punkt (.) reprsentiert sein mssen.

   boot_type:		Der Parameter 'boot_type' dient der Auswahl der Bootstrap- 
					Variante, welche ausgefhrt werden soll.
					1 (Standard)  : "boot_model" ('boot_model_validation.mac.sas')
									 = Bootstrap- Modelle nach Harrell
					2 (Alternativ): "boot_mean"  ('boot_mean_validation.mac.sas')
									 = Bootstrap- Variante 'Mittleres Modell'

   anzahl_samples:  'anzahl_samples' definiert die Anzahl der Bootstrap-Samples 
					(z.B. 20 | 200 | usw.). Standard=200.

   mean_flag:	    Dieser Parameter zeigt nur in Zusammenhang mit 'boot_type=2'
					Wirkung (Variante 'Mittleres Modell'). Er bestimmt die 
					Anwendung des Mittleren Modells (Default: 'mean_flag=1').
				    1 (Standard)  -> Anwenden auf den Ausgangsdatensatz
				    2 (Alternativ)-> Anwenden auf jeden Bootstrap-Sample

   random:			Der Parameter 'random' erlaubt eine Auswahl, wie die Zufallszahlen 
					erzeugt werden sollen. Der default- Wert ist 'random=0'.

					0:	
					Hierbei wird das Systemdatum und die aktuelle Uhrzeit 
					verwendet, was zur Folge hat, dass Zufallszahlen bei jedem
					Schleifendurchlauf und bei jeder Wiederholung immer 
					unterschiedlich sind.

					positive Intergerzahl:	
					Erzeugt eine gleiche Folge von Zufallszahlen bei jedem Durchlauf 
					der Schleife und bei jeder Wiederholung.

					Laufvariable "&k":	
					Es wird eine postitive Integerzahl verwendet, die jedoch bei 
					jedem Durchlauf der Schleife gleich, aber bei jeder Wiederholung 
					anders ist. Dieses Vorgehen ermglicht eine Kontrolle.
					
   macro_path: 		Aufgrund der Tatsache, dass innerhalb des Makros diverse 
					Untermakros aufgerufen werden, ist es zwingend ntig, den Pfad 
					zu dem Verzeichnis, wo die komplette Makrosammlung abgelegt ist,
					anzugeben. Der Pfad selber muss in Hochkommata eingeschlossen 
					sein. Beispiel: 'f:\makros'. 

   binwidth:		Der Parameter "binwidth" gibt die Bin-Breite fr die Schtzung der 
					Assoziationsstatistik an. Der Wert muss dabei zwischen 0 und kleiner
					1 liegen. SAS verwendet als	Standard 0.002, ab Version 9.4 wird bei 
					binrem Endpunkt und weniger als 5 Millionen Beobachtungen jedoch 
					auf Binnen verzichtet. Der Parameter erleichtert daher den Vergleich
					der Ergebnisse unterschiedlicher SAS Versionen.

  VALIDATION
  ----------

  Validiert durch: Stephanie Heiss
 				   University of Ulm 
                   Department of Biometry and Medical Documentation
                   89075 Ulm
  
  Validiert am:    06. Oktober 2003

  Validierungsprogramm::      PM_BOOTSTRAP_VALIDATION_VAL.SAS
  Validierungsdokumentation:  PM_BOOTSTRAP_VALIDATION_VAL.DOC
   
  Validierungsvorgehen:
  - Das Testen der Makros wird durch eine von der Programmierung unabhngige 
    Person durchgefhrt.
  - Fr die Durchfhrung der Validierung wird auf einen in der Literatur 
    genutzten Datensatz zurckgegriffen:  LBW (Low Birth Weight), beschrieben
    in Hosmer, Lemeshow: Applied logistic regression [hosmer2000]
  - Der Datensatz wird fr verschiedene Testlufe verndert bzw. erweitert, 
    damit "die Testdaten neben den normalen Angaben auch Grenzwerte, ungltige
    Angaben, Fehlerbedingungen und Sonderflle beinhalten." [beiwanger2003]
  - Fehlerkontrolle:
    Hierbei werden nur die im LOG-Fenster auftretenden Fehler bzw. Warnungen 
    und offensichtlichen Fehler im OUTPUT betrachtet. Fr jeden Parameter des 
    Makros werden mehrere Durchlufe mit verschiedenen Wertangaben durchgefhrt, 
    wobei auch ungltige Werte angegeben werden, um das Verhalten des Makros
    bei fehlerhaften Aufrufen zu untersuchen. Gefundene Fehler werden 
    dokumentiert und dem Programmierer zur Verfgung gestellt. Nach der Korrektur 
    aller Fehler werden die Testlufe wiederholt, um sicherzustellen, dass 
    wirklich alle Fehler abgefangen wurden und dass keine neuen Fehler entstanden 
    sind.
  - Ergebniskontrolle:
    Um die Richtigkeit der Ergebnisse zu berprfen, werden soweit mglich, die
    Rechenbeispiele und bungsaufgaben aus Hosmer/Lemeshow durch vergleichbare
    Makroaufrufe nachgerechnet. Die Ergebnisse werden denen aus dem Begleitbuch 
    Hosmer/Lemeshow/Cook [hosmer2001] gegenbergestellt und direkt verglichen.
  - Alle Validierungen werden dokumentiert.

  Literatur: 
  - Beiwanger S (2003): Validierung von SAS-Makros. Diplomarbeit, FH Ulm
  - Hosmer DW, Lemeshow S (2000): Applied logistic regression 2nd Ed. 
           J. Wiley & Sons, New York
  - Hosmer DW, Lemeshow S, Cook ED (2001): Solutions manual to accompany
           applied logistic regression 2nd. Ed. J. Wiley & Sons, New York


  AUTHOR
  ------
  R. Muche / C. Ring / I. Assfalg / C. Ziegler
  University of Ulm
  Department of Biometry and Medical Documentation
  89075 Ulm

  The author does not overtake any responsibility for the correctness
  of results. It must not be modified without removing the heading lines 
  containing the authors name.

----------------------------------------------------------------------------------*/

%MACRO PM_BOOTSTRAP_VALIDATION(		data				= ,
									resp_var			= , 
									event				= 1,
									xvar				= ,
									cvar				= ,
									ref					= ,
									miss 				= 0,
									boot_type			= 1,
									anzahl_samples		= 200, 
									mean_flag			= 1,
									random				= 0,
									macro_path			= ,
									binwidth			= 0
							 );

/*globale Variable des Untermakros 'numberobs.mac.sas'*/

%GLOBAL nnnn1;

/*Prfungen / Referenzierungen / Standardbelegungen*/

/*Wichtigster Parameter 'macro_path' berprfen*/
/*bergabe erfolgt?*/

%IF &macro_path= %THEN %DO;

	%WINDOW error color=grey                                                     
     #5  @28 'Fehler!' attr=highlight                    
             color=blue                                                 
     #7  @28 "Dem zwingend erforderlichen Parameter 'macro_path' wurde kein Wert" 
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"	 
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';               
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Hilfsprogramm welches berprft, ob dem Parameter 'macro_path' ein korrekter Pfad*/
/*bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/

/*Anzahl der Zeichen im String 'macro_path' bestimmen:*/

%LET path_length=%LENGTH(&macro_path);

%LET infile=&macro_path;

%IF %bquote(&infile) NE %THEN %DO;
     %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
      %IF %bquote(&char1) ne %STR(%')
         %THEN %LET command='NOK';
         %ELSE %LET command='OK';

	 %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
      %IF %bquote(&char2) ne %STR(%')
         %THEN %LET command2='NOK';
         %ELSE %LET command2='OK';
%END;

/*falls die Syntax nicht stimmt, Anzeige eines Hinweisfensters*/

%IF &command='NOK' OR &command2='NOK' %THEN %DO;

	%WINDOW error color=grey                
	 #5  @28 'Fehler!' attr=highlight                                            
             color=blue                                                           
     #7  @28 "Dem Parameter 'macro_path' wurde ein syntaktisch falscher Wert" 
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"	 
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                       
    %DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Test, ob das unter 'macro_path' angegebene Verzeichnis existiert*/

%LET dir=&macro_path;

%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
       %PUT Directory does not exist;
%END;
%ELSE %DO;
       %PUT Directory exists;
       %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;

	%WINDOW error color=grey                                    
	 #5  @28 'Fehler!' attr=highlight                           
             color=blue                                           
	 #7  @28 "Das unter dem Parameter 'macro_path' angegebene Verzeichnis existiert"
	 #8  @28 "nicht- Bentigte Untermakros knnen nicht referenziert werden"
	 #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';       
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Falls bestimmte Parameter ohne Wert bergeben wurden, diese mit Standard belegen*/

%IF &event= %THEN %DO;
	%LET event=1;
%END;

%IF &miss= %THEN %DO;
	%LET miss=0;
%END;

%IF &boot_type= %THEN %DO;
	%LET boot_type=1;
%END;

%IF &anzahl_samples= %THEN %DO;
	%LET anzahl_samples=200;
%END;

%IF &mean_flag= %THEN %DO;
	%LET mean_flag=1;
%END;

%IF &random= %THEN %DO;
	%LET random=0;
%END;

/*Externe Makros, die innerhalb des Makros 'PM_BOOTSTRAP_VALIDATION.MAC.SAS'*/
/*verwendet werden (Untermakros) einbinden (%include)*/

FILENAME y &macro_path;

%INCLUDE y ('boot_model_validation.mac.sas');
%INCLUDE y ('boot_mean_validation.mac.sas');
%INCLUDE y ('numchar.mac.sas');
%INCLUDE y ('exist.mac.sas');
%INCLUDE y ('exist_var.mac.sas');
%INCLUDE y ('ref.mac.sas');
%INCLUDE y ('numberobs.mac.sas');

/*Test, ob bei bestimmten Parametern anstatt numerischen Werten*/
/*character- Werte bergeben wurden*/

/*berprfung erfolgt mit Hilfe des Untermakros 'numchar.mac.sas'*/

%NUMCHAR(parameterliste=&event &miss &random &anzahl_samples &mean_flag &boot_type &binwidth);

%IF &numcharok='NOK' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error8.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Parameter, bei denen keine negativen Werte erlaubt sind, berprfen*/

%IF &anzahl_samples <=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error9.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%IF &binwidth <0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error66.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfen, ob der angegebene Datensatz (Parameter 'data=') auch wirklich existiert*/
/*(mit Hilfe des Hilfs-/Untermakros 'exist.mac.sas')*/

%EXIST(dsn=&data);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error4.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Datenquelle (Parameter 'data=' UND response-Variable*/
/*(Zielgre / Parameter 'resp_var=') angegeben wurde*/

%IF &data= or &resp_var= %THEN %DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

	%INCLUDE y ('error1.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob berhaupt Variablen (Parameter 'xvar=' / 'cvar=') angegeben wurden*/

%IF &xvar= and &cvar= %THEN %DO;

/*wenn keine Variablen angegeben wurden, Anzeige eines Fensters mit*/ 
/*Fehlermeldung/ Hinweis*/

	%INCLUDE  y ('error2.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Prfung, ob alle bergebenen Variablen numerisch sind*/

/*Erzeugung eines temporren Zwischendatensatzes, welcher lediglich die*/
/*bergebenen Variablen enthlt*/

DATA test3;
	SET &data;
	KEEP &xvar &resp_var &cvar;
RUN;

/*mittels der Prozedur PROC CONTENTS kann der Typ einer Variablen bestimmt werden*/

/*Unterdrckung der Ausgabe von PROC CONTENTS*/

PROC CONTENTS DATA=test3 NOPRINT OUT=testcontent;
RUN;

/*lediglich die Variable 'type' der Ergebnisdatei behalten,*/
/*da nur diese von Interesse ist*/

DATA testcontent;
	SET testcontent;
	KEEP type;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=testcontent);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test4=&nnnn1;

/*Prfung ob Variablen vom Typ 'character' vorhanden sind*/

DATA test4;
	SET testcontent;
	IF type=2 THEN DELETE;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test4);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test5=&nnnn1;

/*falls Variablen bergeben wurden, die nicht numerisch sind --> Fehlermeldung*/

%IF &test4 ne &test5 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error60.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE test4 testcontent test3;
RUN;
QUIT;

/*berprfung, ob alle bergebenen Variablen im bergebenen Datensatz*/
/*(Parameter 'data' vorhanden sind)*/

/*Zusammenfassung aller Variablen in einer neuen Makro- Variablen*/

%LET all_variables= &resp_var &xvar &cvar;

/*Prfung erfolgt mit Hilfe des Untermakros 'exist_var.mac.sas'*/

%EXIST_VAR(dsn=&data, var=&all_variables);

/*falls irgendwelche Variablen nicht existieren, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error6.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob die Zielgre (Parameter 'resp_var')*/
/*2 Kategorien (Ausprgungen) besitzt*/

/*missing values ausschliessen*/

DATA freq2;
	SET &data;
	IF &resp_var ne .;
RUN;

/*Anzahl der Ausprgungen mit Hilfe der Prozedur PROC FREQ ermitteln*/

/*Ausgabe unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnis als ODS- Datei zur weiteren Verarbeitung ausgeben*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=freq2;
	TABLES &resp_var;
RUN;

/*ODS- Output schliessen*/

ODS OUTPUT CLOSE;

/*ODS- Ausgabe wieder aktivieren*/

ODS LISTING;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen (= # Ausprgungen der Zielgre)*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq freq2;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

/*falls Zielgre mehr als zwei Ausprgungen besitz--> Fehlermeldung*/

%IF &event_check > 2 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error5.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Parameter 'miss' 0 oder 1 ist*/

%IF &miss ne 1 %THEN %DO;
	%IF &miss ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error7.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'event' 0 oder 1 ist*/

%IF &event ne 1 %THEN %DO;
	%IF &event ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error7.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'boot_type' 1 oder 2 ist*/

%IF &boot_type ne 1 %THEN %DO;
	%IF &boot_type ne 2 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error52.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'mean_flag' 1 oder 2 ist*/

%IF &mean_flag ne 1 %THEN %DO;
	%IF &mean_flag ne 2 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error53.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*Anzahl der angegebenen kategoriellen Variablen ermitteln:*/

%GLOBAL c_count c_var;
%LET c_count=1;
%LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%DO %WHILE(&c_var ne);
     %LET c_count=%EVAL(&c_count+1);
     %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%END;
%LET c_count=%EVAL(&c_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/

%PUT The cvar contains &c_count variables.;

/*Anzahl der Referenzen ermitteln*/

%GLOBAL ref_count var_ref;
%LET ref_count=1;
%LET var_ref=%QSCAN(&ref,&ref_count,*);
%DO %WHILE(&var_ref ne);
     %LET ref_count=%EVAL(&ref_count+1);
     %LET var_ref=%QSCAN(&ref,&ref_count,*);
%END;
%LET ref_count=%EVAL(&ref_count-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The refs contains &ref_count variables.;

%IF &ref_count >0 %THEN %DO;
	%IF &c_count ne &ref_count %THEN %DO;

/*falls Anzahl der kategoriellen Variablen grer Null ist und
/* nicht mit der Anzahl der  angegebenen*/
/*Referenzkategorien bereinstimmt, Anzeige eines Fensters*/
/*mit Fehlermeldung/ Hinweis*/

		%INCLUDE y ('error3.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

   	%END;
%END;


/*berprfung, ob etwaige bergebene Referenzkategorien vorhanden sind bzw.*/
/*korrekt bergeben wurden*/

%IF &cvar ne %THEN %DO;
	%IF &ref_count ne 0 %THEN %DO;

	%REF(data=&data, resp_var=&resp_var, event=&event, cvar=&cvar, ref=&ref);

	%IF &correct='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error11.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;

%END;
%END;	

/*berprfen ob eine Variable nach Entfernung aller Beobachtungen mit missing*/
/*values nur noch eine Ausprgung besitzt*/

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values*/

DATA work.category_test;
	SET  &data;

	%DO i=1 %TO &count_x;
		%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
		IF &xvar_liste=. THEN DELETE;
	%END;

	%DO i=1 %TO &count;
		%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
		IF &cvar_liste=. THEN DELETE;
	%END;

		IF &resp_var=. THEN DELETE;

RUN;

/*Prfen ob 'resp_var' /'xvar'/ 'cvar' nur 1 Ausprgung besitzen*/

%LET category_test=0;
%LET category_test_fail=;

/*event*/

/*Anzahl der Ausprgungen mittels PROC FREQ bestimmen*/

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &resp_var;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

%IF &event_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &resp_var;

%END;

/*xvar*/

%IF &xvar ne %THEN %DO;

%DO uhgu=1 %TO &count_x;

%LET xvar_liste=%SCAN(&xvar,&uhgu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &xvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET xvar_check=&nnnn1;

%IF &xvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &xvar_liste;

%END;

%END;
%END;

/*cvar*/

%IF &cvar ne %THEN %DO;

%DO uhfu=1 %TO &count;

%LET cvar_liste=%SCAN(&cvar,&uhfu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &cvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET cvar_check=&nnnn1;

%IF &cvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &cvar_liste;

%END;

%END;
%END;

PROC DATASETS NOLIST;
	DELETE category_test;
RUN;
QUIT;

%IF &category_test=1 %THEN %DO;

	DATA category_test;
		var="&category_test_fail";
	RUN;

	PROC PRINT DATA=category_test NOOBS LABEL;
	TITLE1 "Folgende Variablen besitzen nach Entfernung aller";
	TITLE2 "missing values lediglich eine Ausprgung:";
		VAR var;
			LABEL var='Variable';
	RUN;

	PROC DATASETS NOLIST;
		DELETE category_test;
	RUN;
	QUIT;

	%INCLUDE y ('error24.sas');
	%GOTO ende;

%END;

%IF &binwidth >=1 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error66.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%IF &binwidth >0.01 %THEN %DO;

/*Anzeige eines Warnhinweises*/

	DATA hint;
		hint="Bei einem Wert von 0 wird kein Binning angewandt. In manchen SAS Versionen wird der Standard 0.002 verwendet.";
	RUN;
	PROC PRINT DATA=hint NOOBS LABEL HEADING=H WIDTH=UNIFORM;
		VAR   hint;
		TITLE1 "Dem Parameter 'binwidth' wurde ein Wert grer 0.01 bergeben.";
		LABEL hint = "Sinnvoll ist ein Wert zwischen 0 und 0.01."; 
	RUN;
%END;

/*ENDE Prfungen / Referenzierungen / Standardbelegungen*/

/*-------------------------------------------------------------------------------*/

/*Auswahl der Bootstrap-Methode.*/
/*Model (Simple/Enhanced) (1) = Standard*/

%IF &boot_type=1 %THEN %DO;

%BOOT_MODEL_VALIDATION(		data				= &data,
							resp_var			= &resp_var, 
							event				= &event,
							xvar				= &xvar,
							cvar				= &cvar,
							ref					= &ref,
							miss 				= &miss,
							anzahl_samples		= &anzahl_samples, 
							random				= &random,
							macro_path			= &macro_path,
							binwidth			= &binwidth
	           		  );

%END;

/*Auswahl der Bootstrap-Methode.*/
/*Mean (Mittleres Modell) (2) = Alternativ*/

%IF &boot_type=2 %THEN %DO;

%BOOT_MEAN_VALIDATION (		data				= &data,
							resp_var			= &resp_var, 
							event				= &event,
							xvar				= &xvar,
							cvar				= &cvar,
							ref					= &ref,
							miss 				= &miss,
							random				= &random,
							anzahl_samples		= &anzahl_samples,
							mean_flag			= &mean_flag,
							macro_path			= &macro_path,
							binwidth			= &binwidth
			    	  );

%END;

%ende:

%MEND PM_BOOTSTRAP_VALIDATION;
