/************************************************************************************/
/*						        													*/
/*     NAME: pm_crossvalidation.mac.sas	         									*/
/*    TITLE: Validierung von logistischen Regressionsmodellen mittels 				*/
/*			 Crossvalidierung (K-Fold /Jackknife (approximative/ genau)/ Adjustiert)*/
/*     KEYS: crossvalidation, k-fold-crossvalidation, jackknife-crossvalidation	    */
/*			 adjusted crossvalidation, validation, logistic regression, roc analysis*/
/*  VERSION: SAS 8.2												      			*/
/*  PRODUCT: BASE, STAT							        							*/
/*    PROCS: LOGISTIC									        					*/
/*															         				*/
/*																					*/
/*  Verzeichnis/Dateiname:			F:\Prognose\pm_crossvalidation.mac.sas	        */
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*  Programmierer:					Christoph Ziegler								*/
/*  Datum der Ersterstellung:		3/2003											*/
/*  Aktuelle Makro-Version:			12/2003											*/
/*																					*/
/*  Kurzbeschreibung:				Makro zur Validierung von logistischen 			*/
/*									Regressionsmodellen mittels						*/
/*									K-Fold-Crossvalidierung,						*/
/* 					 				Jackknife-Crossvalidierung 						*/
/*									(approximativ/genau)							*/
/*									sowie adjustierter Crossvalidierung				*/
/*																					*/
/*  Verwendete externe Untermakros:	jackknife_crossvalidation.mac.sas 				*/
/*									k_fold_crossvalidation.mac.sas					*/
/*									adjusted_crossvalidation.mac.sas				*/
/*									numchar.mac.sas									*/
/*									exist.mac.sas									*/
/*									exist_var.mac.sas								*/
/*									numberobs.mac.sas								*/
/*									ref.mac.sas										*/
/*									error1.sas	error2.sas	error3.sas	error4.sas	*/
/*									error5.sas	error6.sas	error7.sas	error8.sas	*/
/*									error9.sas	error11.sas	error24.sas error51.sas */
/*									error55.sas error57.sas	error60.sas	error66.sas	*/
/*																					*/
/*  Bentigte Dateien:				Datensatz (Parameter 'data=')			   		*/
/*															           				*/
/*																					*/
/*  Erzeugte Dateien:				-----		 									*/
/*  								        									   	*/
/*  						 										        		*/
/*																					*/
/*  Programmnderungen				Sandra Mller, 04.05.2016, Makrovariable		*/
/*  und -ergnzungen 				binwidth hinzugefgt							*/ 
/*  (Name, Datum, Beschreibung):											 		*/
/*  																				*/
/*        																			*/
/************************************************************************************/
/*

 TITLE
 -----
 pm_crossvalidation.mac.sas

 SAS Makro zur Validierung von logistischen Regressionsmodellen mittels 
 Crossvalidierung (K-Fold /Jackknife (approximative/ genau)/ Adjustiert)

 BACKGROUND
 ----------

 Die Kreuzvalidierung (engl. crossvalidation) ist eine Erweiterung / Verallgemeinerung des 
 Data-Splitting, um deren Nachteile zu umgehen. Dabei wird der Datensatz mehrfach in K 
 Gruppen unterteilt und die Auswertungen (Modellentwicklung und Validierung) jeweils auf 
 diesen Untergruppen durchgefhrt [harrell2001]. Auf Grund des mehrfachen Durchfhrens 
 dieses Vorgehens gehrt die Kreuzvalidierung zu den Resamplingverfahren, bei denen durch 
 Stichprobenziehungen Informationen ber das Verhalten und die Verteilung von statistischen 
 Verfahren untersucht werden knnen. Diese computerintensiven Verfahren sind schon seit 
 lngerem bekannt (z.B. Verffentlichungen von Efron aus dem Jahre 1979 [efron 1979, 
 efron1979a]) knnen aber erst durch die technischen Entwicklungen bei den heutigen 
 Computern allgemein eingesetzt werden. Dabei ist eine direkte Implementierung dieser 
 Verfahren (im wesentlichen die Kreuzvalidierung und der Bootstrap) in der Statistik-
 software noch nicht erfolgt, so dass eine Programmierung in den vorliegenden SAS-Makros 
 zur Durchfhrung dieser Validierungsverfahren notwendig ist. 

 Vorausgesetzt wird im Folgenden, dass die Struktur des Modells nach der Modellentwicklung, 
 d.h. welche Variablen in das Modell aufgenommen werden, festgehalten wird und nicht mehr 
 vernderbar ist. Die Resamplingverfahren Kreuzvalidierung und Bootstrap knnen aber auch 
 zur Variablenselektion bei der Modellentwicklung und zur Strukturuntersuchung des Modells 
 herangezogen werden. Durch geeignete Nutzung der entwickelten SAS-Makros ist dieses 
 Vorgehen auch mit den hier vorgestellten Makros mglich, eine Beschreibung dieses Aspektes 
 wre allerdings so umfangreich, dass er hier nicht dargelegt werden kann. Als Hintergrund-
 literatur dazu sind vor allem die Arbeiten von Sauerbrei und Kollegen aus der Universitt 
 Freiburg zu nennen, die diese Auswertungssituation vor allem fr das Proportional Hazard 
 Modell untersucht haben [sauerbrei1992, sauerbrei1999, schumacher1997, schumacher2001] und 
 eine Verffentlichung unserer Arbeitsgruppe zum logistischen Regressionsmodell [ring2003]. 
 Als allgemeine Hintergrundliteratur zur Schtzung der Vorhersagefehler durch Kreuz-
 validierung sind das Kapitel 17 des Buches von Efron und Tibshirani [efron1993] und das 
 Kapitel 5.2 des Buches von Harrell [harrell2001] zu empfehlen.

 K-Fold Crossvalidation
 Die sogenannte "K-fold Crossvalidation" basiert auf einer Stichprobenziehung ohne 
 Zurcklegen: Der Datensatz wird dabei mit einem Zufallsverfahren (gleichverteilte 
 Zufallszahlen) in K Untergruppen aufgeteilt. Eine dieser Untergruppen wird dann aus dem 
 Datensatz herausgenommen und fungiert im Sinne des Data-Splitting als Testdatensatz. Der 
 Datensatz der restlichen K-1 Untergruppen ist dann der Trainingsdatensatz, an dem die 
 Regressionskoeffizienten des Modells bestimmt werden. Dies Modell wird dann auf den 
 Testdatensatz angewendet und mit der ROC-Analyse die Prognosekenngren bestimmt. Dieses 
 Vorgehen wird fr alle K Gruppen durchgefhrt, so dass anschlieend jeweils K Kenngren 
 vorliegen, die durch Mittelwertbildung zusammengefasst werden [efron1993, harrell2001] 
 und die kreuzvalidierten Prognosekenngren darstellen. Diese Auswertung hngt allerdings 
 noch von der konkreten Zufallsziehung in die K Gruppen ab. Um diese Abhngigkeit zu 
 verringern wird die gesamte Auswertung mit jeweils anderer Zufallsziehung x mal wiederholt. 
 Typische Werte fr x bei der Kreuzvalidierung sind 20 bis 50 Wiederholungen [harrell2001]. 
 Die Zusammenfassung ber die x*K Auswertungen ergibt dann den kreuzvalidierten Schtzer 
 fr die Prognosegte. 

 Fr die Auswahl der Anzahl der Gruppen (K) geben Davison und Hinkley [davison1997] an, 
 das Minimum von 10 und Wurzel(n) zu benutzen, wobei beide Werte aus pragmatischen Grnden 
 gewhlt sind. Die Anzahl der Gruppen sollte nicht 10 bersteigen, um nicht zu viele Modelle 
 rechnen zu mssen, Wurzel(n) um eine gewisse Anzahl Beobachtungen in den Untergruppen zu 
 garantieren. Die Erhhung der Anzahl Gruppen verringert die Varianz der Schtzer, aller-
 dings auf Kosten einer Erhhung des Bias [davison1997], so dass eine vernnftige 
 Abschtzung der Anzahl Gruppen notwendig ist.   

 Fr die Programmierung sind in der Konsequenz neben den entsprechenden Stichproben-
 ziehungen und der dafr notwendigen Dateibearbeitungen in der Regel sehr viele 
 logistische Modellierungen und ROC-Analysen (x*K) notwendig. Dabei wird hier auf die 
 SAS-Makros PM_LOGREG.MAC.SAS, PM_ROC.MAC.SAS zurckgegriffen.

 Adjusted Crossvalidation
 Davison und Hinkley [davison1997] geben auf den Seiten 294 bis 295 eine adjustierte 
 Variante fr den kreuzvalidierten Schtzer an, um den Bias zu korrigieren. Bentigt werden 
 dazu der kreuzvalidierte Schtzer fr eine Kenngre, der Schtzwert nach Reklassifikation 
 am Originaldatensatz, der Anteil der K-ten Gruppe sowie der Schtzwert des Modells auf 
 Basis von k-1 Gruppen angewandt auf den Originaldatensatz (Formel siehe unter DESCRIPTION). 
 Der adjustierte Schtzer hat einen kleineren Bias und ist nicht mit viel mehr Aufwand zu 
 berechnen als der kreuzvalidierte Schtzer, da die meisten Formelbestandteile sowieso 
 berechnet werden. Davison und Hinkley geben auf Seite 295 in [davison1997] einen 
 Algorithmus zur Berechnung an, welcher der Makro-Programmierung zu Grunde liegt.  

 Jack-Knife - Crossvalidation
 Eine extreme Form der Kreuzvalidierung wird als Jack-Knife oder auch "leaving one out" 
 bezeichnet. Beim Jack-Knife Vorgehen wird K=n gesetzt, d.h. jede Beoabchtung im Datensatz 
 bildet eine Untergruppe [davison1997, efron1993, harrell2001]. Somit wird jedes mal eine 
 Beobachtung aus dem Datensatz entfernt, auf Grundlage der verbliebenen n-1 Beobachtungen 
 werden die Regressionskoeffizienten geschtzt und mit diesem Modell fr die ausgelassene 
 Beobachtung die Prognose (Wahrscheinlichkeit fr den Outcome) geschtzt. Dieses Vorgehen 
 wird fr alle Beobachtungen durchgefhrt, so dass am Ende dieses Prozesses sogenannte 
 Jack-Knife predicted values fr jede Beobachtung existieren. ber diese Werte wird dann 
 eine ROC-Analyse durchgefhrt, welche die Prognosegten ausgibt. 

 Wie schon oben beschrieben, ist die Anzahl K der Untergruppen vorsichtig zu whlen. Eine 
 Erhhung von K fhrt zu einer Varianzreduktion auf Kosten des Bias. Bei der Wahl von K=n 
 beim Jack-Knife ist die Gefahr eines relevanten Bias in der Schtzung 
 der Prognosegte recht gro, so dass eine Validierung mit dem Jack-Knife-Schtzer nicht 
 allgemein empfohlen werden kann. Im Programmpaket ist er einerseits aus historischen 
 Grnden enthalten, da diese Validierung schon sehr lange genutzt und beschrieben wird und 
 andererseits, da es Parallelen zum Bootstrap gibt [efron1979]. 

 SAS gibt in der Prozedur PROC LOGISTIC unter der Option PREDPROBS=X Jack-Knife kreuz-
 validierte Schtzer fr die Prognosewahrscheinlichkeiten aus. Diese beruhen auf einer 
 asymptotischen Formel, so dass keine Schleife ber alle n durchlaufen werden muss. 
 Dies resultiert in sehr viel geringeren Programmlaufzeiten. Fr eine exakte Berechnung 
 ist allerdings zustzlich die Schleifen-Lsung ber die n Modellberechnungen implementiert 
 worden (Parameter exact=1). 

 Probleme mit der Kreuzvalidierung
 Folgende Probleme bei der Nutzung der Kreuzvalidierung werden in der Literatur (z.B. 
 Harrell S. 93-94 [harrel2001]) genannt:
 - Es werden wie beim Data-Splitting die Modellschtzungen nicht auf der vollen Information 
   der n Beobachtungen durchgefhrt (jeweils n-(n/k) Beobachtungen). Der Effekt ist nicht 
   so gro, aber offensichtlich. Dadurch wird vom Grundsatz her nicht unbedingt das Modell, 
   welches auf der Basis von n Beobachtungen entwickelt wurde, validiert. 
 - Die Auswertung hngt von der Wahl von K ab, so dass eine gewisse Subjektivitt in die 
   Ergebnisse gelangen kann.
 - Die Anzahl notwendiger Wiederholungen (>20) fhrt zu sehr vielen Modellschtzungen und 
   ROC-Analysen, die in einer langen Laufzeit des Programms mit von der Anzahl aufwendigen 
   Berechnungen resultieren.   
 - Ein weiteres, bei relativ kleinen Fallzahlen oft vorkommendes Problem ist das der schon 
   mehrfach angesprochenen Quasi-Complete-Separation. Dies trifft speziell die K-fold- und 
   Adjusted-Kreuzvalidierung, da hier logistische Regressionsmodelle auf einem bzgl. der 
   Fallzahl entsprechend kleineren Datensatz als dem Originaldatensatz berechnet werden. 
   Mit fallender Fallzahl und gleichbleibender Modellkomplexitt steigt aber die Wahrschein-
   lichkeit fr die Separation. Dann werden statt mit einer PROC LOGISTIC die Regressions-
   koeffizienten mit dem FL-Makro [heinze2002] geschtzt, welches wesentlich lngere 
   Laufzeiten zur Folge hat. Fr den seltenen Fall, dass dieses Makro abbricht werden 
   dann wieder die nicht validen Modellparameter zur Bestimmung der kreuzvalidierten 
   Prognosegten aus PROC LOGISTIC genutzt, allerdings versehen mit einem entsprechenden 
   Hinweis.  

 

 DESCRIPTION
 -----------

 Im Rahmen des Makros wurden insgesamt drei verschiedene Crossvalidierungsvarianten 
 implementiert, wobei jede Variante in einem eigenen Makro ausgelagert vorliegt und
 diese mit Hilfe des Parameters "crossval_type=" ausgewhlt werden knnen. 
 Innerhalb der Programme einer jeden Variante kommen im wesentlichen die Makros
 "PM_LOGREG.MAC.SAS" (logistische Regression) und "PM_ROC.MAC.SAS" (ROC-Analyse) 
 zum Einsatz. Die Beschreibung des jeweiligen Programmablaufs jeder Variante folgt
 im Anschlu.

 Jackknife-Crossvalidierung (approximativ):

 - In einem ersten Schritt werden mittels "PM_LOGREG.MAC.SAS" (ohne Variablen-
   selektion) und "PM_ROC.MAC.SAS" die Prognosegtekenngren des Originaldaten-
   satzes ermittelt.
 - Anschliessend folgt die approximative Jackknife-Crossvalidierung mit Hilfe der
   PROC LOGISTIC-Option PREDPROBS=X. Dazu wird das Makro "PM_LOGREG.MAC.SAS" mit
   dem Parameter "predprobs_art=X" aufgerufen. 
 - Falls dem Parameter "roc_auto" der Wert 0 bergeben wurde, so werden die 
   cutpoint-abhngigen ROC- Kenngren aus einer manuell generierten Tabelle 
   berechnet, weshalb dem Makro "PM_ROC.MAC.SAS" fr diesen Fall das Hilfsmakro
   "RECLASS.MAC.SAS" vorgeschaltet ist, welches diese Aufgabe bernimmt. Ist eine
   manuelle Berechnung nicht erwnscht, so werden die Prognosegtekenngren auf
   Basis der automatisch generierten Tabelle aus "PM_LOGREG.MAC.SAS" ("out_roc=")
   berechnet.
 - Im Output werden die Prognosegten der approximativen Jackknife-Crossvalidierung
   denen des Originaldatensatzes gegenbergestellt sowie die Differenz und die 
   prozentuale Abweichung vom Ausgangswert zustzlich ausgegeben. 

 Jackknife-Crossvalidierung (genau):

 - Wurde mittels dem Parameter "exact=1" zustzlich eine genaue 
   Jackknife-Crossvalidierung angefordert, so wird diese im Anschlu durchgefhrt,
   ist jedoch sehr rechenintensiv und kann durchaus einige Zeit in Anspruch nehmen.
   Dabei wird je eine Beobachtung bei der Modellentwicklung nicht bercksichtigt,
   das heit deren Zielgre auf missing gesetzt, wobei PROC LOGISTIC die vorher-
   gesagte Wahrscheinlichkeit fr genau diese Beobachtung automatisch schtzt.
 - Innerhalb des Makros "PM_CROSSVALIDATION.MAC.SAS" wird dies innerhalb einer
   Schleife ber alle Beobachtungen des Datensatzes durchgefhrt, wobei bei jedem
   Schleifendurchlauf das Makro "PM_LOGREG.MAC.SAS" aufgerufen wird.
 - Die Bestimmung der Prognosegtekenngren erfolgt nach der Schleife mit Hilfe des
   Makros "PM_ROC.MAC.SAS". Auch hierbei wird die Einstellung des Parameters
   "roc_auto" bercksichtigt und gegebenenfalls die cutpoint-abhngigen ROC- 
   Kenngren aus einer manuell generierten Tabelle berechnet (siehe oben).
 - Abschliessend werden im Output ebenfalls die Prognosegten der genauen 
   Jackknife-Crossvalidierung denen des Originaldatensatzes gegenbergestellt 
   sowie die Differenz und die prozentuale Abweichung vom Ausgangswert zustzlich 
   ausgegeben. 

 K-fold-Crossvalidierung:

 - Bei dieser Crossvalidierungsvariante werden in einem ersten Schritt ebenfalls
   mittels "PM_LOGREG.MAC.SAS" (ohne Variablenselektion) und "PM_ROC.MAC.SAS" 
   die Prognosegtekenngren des Originaldatensatzes ermittelt.
 - Anschliessend folgt die zufllige Aufteilung (RANUNI-Funktion) des Datensatzes 
   in eine bestimmte Anzahl (Parameter "gruppen=") von Untergruppen. 
 - Es folgen zwei Schleifen, eine ber die Anzahl der durchzufhrenden 
   k-fold-Crossvalidierungen (Parameter "anzahl=") und eine ber die Anzahl der
   Gruppen (Parameter "gruppen=").
 - Innerhalb der Schleifen wird jeweils "PM_LOGREG.MAC.SAS" mit dem jeweiligen 
   Datensatz ohne die Gruppe i aufgerufen, das heit je ein Modell entwickelt, 
   wobei die Regressionskoeffizienten mittels OUTEST ausgegeben werden. Dieses 
   entwickelte Modell wird im Anschluss an den Daten der Gruppe i validiert. Dazu 
   wird "PM_LOGREG.MAC.SAS" mit der Option INEST aufgerufen. Die Bestimmung der
   Prognosegtekenngren erfolgt mit dem Makro "PM_ROC.MAC.SAS". 
 - Dies wird fr alle Gruppen und so hufig wie unter "anzahl=" angegeben durch-
   gefhrt
 - Jegliche Prognosegtekenngren werden am Ende gemittelt und im Output denen des 
   Originaldatensatzes gegenbergestellt. Ferner wird zustzlich die Differenz und 
   die prozentuale Abweichung vom Ausgangswert ausgegeben. 

 Adjustierte Crossvalidierung:

 - Auch bei der adjustierten Crossvalidierung werden ebenfalls zuerst mittels 
   "PM_LOGREG.MAC.SAS" (ohne Variablenselektion) und "PM_ROC.MAC.SAS" 
   die Prognosegtekenngren des Originaldatensatzes ermittelt.
 - Anschliessend folgt wie bei der K-Fold-Crossvalidierung die zufllige Aufteilung 
   (RANUNI-Funktion) des bergebenen Datensatzes in eine bestimmte Anzahl (Parameter 
   "gruppen=") von Untergruppen. 
 - Dabei werden zustzlich die Anteile der jeweiligen Gruppenstrke bestimmt.
 - Es folgen zwei Schleifen, eine ber die Anzahl der durchzufhrenden 
   adjustierten Crossvalidierungen (Parameter "anzahl=") und eine ber die Anzahl 
   der Gruppen (Parameter "gruppen=").
 - Innerhalb der Schleifen wird wie bei der K-Fold-Crossvalidierung ebenfalls jeweils 
   "PM_LOGREG.MAC.SAS" mit dem jeweiligen Datensatz ohne die Gruppe i aufgerufen, das 
   heit je ein Modell entwickelt, wobei die Regressionskoeffizienten mittels OUTEST 
   ausgegeben werden. Dieses entwickelte Modell wird im Anschluss an den Daten der 
   Gruppe i validiert. Dazu wird "PM_LOGREG.MAC.SAS" mit der Option INEST aufgerufen. 
   Die Bestimmung der Prognosegtekenngren erfolgt mit dem Makro "PM_ROC.MAC.SAS". 
 - Zustzlich wird das jeweilige entwickelte Modell ohne die Gruppe i bei jedem
   Schleifendurchlauf auch auf den vollen Originaldatensatz angewandt (Makro 
   "PM_LOGREG.MAC.SAS", Option INEST=).
 - Die Prognosegtekenngren dieses Schrittes werden mittels dem Makro 
   "PM_ROC.MAC.SAS" bestimmt.
 - Nach den beiden Schleifen ber die Anzahl der durchgefhrten adjustierten 
   Crossvalidierungen (Parameter "anzahl=") und der ber die Anzahl der Gruppen 
   (Parameter "gruppen=") liegen alle ntigen Ergebnisse vor, um die Adjustierung
   durchzufhren.
 - Die adjustierten Schtzungen aller Prognosegtekenngren errechnen sich nach der
   Formel von Davison & Hinkley [Davison, 1997] folgendermaen:
   Adjustierter Schtzer = K-Fold-Crossvalidierungsschtzer + Originalschtzung - 
   Summe [i=1 bis k] (Anteil der Gruppe i * Schtzer der Anwendung des Modells ohne 
   die jeweilige Gruppe i auf den Originaldatensatz).
 - Jegliche Prognosegtekenngren werden am Ende ebenfalls gemittelt und im Output 
   denen des Originaldatensatzes gegenbergestellt. Ferner wird auch bei dieser 
   Variante zustzlich die Differenz und die prozentuale Abweichung vom Ausgangswert 
   mit ausgegeben. 
 
 LITERATURE
 ----------
 - SAS Online Doc zur Prozedur PROC LOGISTIC
 - Davison AC, Hinkley DV (1997): Bootstrap methods and their application, 
				   Cambridge University Press, USA, 1997
 - Efron B (1979): Bootstrap methods: another look at the jackknife.   
                   Ann. Statist. 7, 1-26 
 - Efron B (1979a): Computers and the theory of statistics: thinking the unthinkable.  
                   SIAM Review 21, 460-480
 - Efron B, Tibshirani RJ (1993): An introduction to the bootstrap.  
                   Chapman & Hall, New York 
 - Harrell FE Jr (2001): Regression modeling strategies. 
				   Springer, New York
 - Heinze G, Schemper M (2002): A solution to the problem of separation in logistic 
                   regression.  Statist. Med. 21, 2409-2419
 - Ring C, Muche R (2003): Bootstrap- und Crossvalidierung eines logistischen 
                   Regressionsmodells unter Einbeziehung des Variablenselektionsprozesses.  
                   Informatik, Biometrie und Epidemiologie in Medizin und Biologie 34, 214-216
 - Sauerbrei W, Schumacher M (1992): A bootstrap resampling procedure for model building: 
                   application to the Cox regression model.  
                   Statist. Med. 11, 2093-2109
 - Sauerbrei W (1999): The use of resampling methods to simplify regression models in 
                   medical statistics. Applied Statistics 48, 313-329
 - Schumacher M, Hollnder N, Sauerbrei W (1997): Resampling and cross-validation 
                   techniques: a tool to reduce bias caused by model building?  
                   Statist. Med. 16, 2813-2827 
 - Schumacher M, Hollnder N, Schwarzer G, Sauerbrei W (2001): Prognostic factor studies. 
                   In: Crowley (Hrsg.): Handbook of statistics in clinical oncology. 
                   Marcel Dekker, New York  321-378 
 - Ziegler C (2003): Ein SAS-Makropaket zur Entwicklung und Validierung von 
					 Prognosemodellen auf Basis der logistischen Regression. 
					 Diplomarbeit, FH Ulm 

 SYNTAX
 ------
 [Makroaufruf und Beschreibung der zu bergebenden Parameter]

 %PM_CROSSVALIDATION (	
   *data			= ,		Datensatz
   *resp_var		= , 	Zielgre
	event			= 1,	Event- Kategorie der binren Zielgre (0/1)
   *xvar			= ,		stetige Variable(n)
   *cvar			= ,		kategorielle Variable(n)
	ref				= ,		Referenzkategorien fr kategorische Variablen
	miss			= 0,	Missing Values ausschliessen (ja/ nein)
	crossval_type	= 1,	Auswahl der Crossvalidierungsart
	gruppen			= 10,	Anzahl der Untergruppen (K-Fold/ Adjusted)
	anzahl			= 50,	Anzahl der Durchlufe (K-Fold/ Adjusted)
	exact			= 0,	Jackknife- Validierung approximativ/ genau (0/1)
	roc_auto		= 1,	Cutpoint-abhngige ROC- Kenngren manuell/ automatisch (0/1)
   *macro_path		= 		Pfad zur Makrosammlung
	binwidth		= 0		Bin-Breite
  );

  Mit (*) gekennzeichnete Parameter sind beim Makroaufruf zwingend erforderlich.
  Bereits gesetzte Parameter stellen die jeweilige Standardeinstellung dar.

   data:		    Parameter zur Angabe des zu analysierenden Datensatzes. Wurde 
					beispielsweise vor dem Aufruf des Makros ein Datensatz mit dem 
					Namen "dat300" angelegt, lautet der korrekte Aufruf: "data=dat300". 
					Zustzlich kann auch die dazugehrige SAS-Library bergeben werden 
					(Beispiel: "data=work.dat300"). Die bergabe dieses Parameters ist 
					zwingend erforderlich.
 
   resp_var:		"resp_var" steht fr "Response Variable", also fr die Zielgre. 
					Dieser bergabeparameter ist ebenfalls zwingend erforderlich. 
					Ferner wird eine binre Datenstruktur (0/1) der Zielgre bentigt. 
					Die Zielgre muss mit "0" und "1" kodiert sein. Sollte dies nicht 
					der Fall, ist eine vorherige Umkodierung erforderlich. Heit 
					beispielsweise die Zielgre "tod" lautet der Aufruf wie folgt: 
					"resp_var=tod".

   event:			Je nach dem welche Kategorie der binren Zielgre untersucht 
					werden soll, kann diese mit Hilfe dieses Parameters bergeben werden. 
					Soll beispielsweise die Zielgre "tod" analysiert werden und steht 
					"1" fr tot sowie "0" fr lebendig, lautet der korrekte Aufruf 
					"event=1". Dies ist zugleich die Standardeinstellung, da blicherweise
					"1" fr das Eintreten eines bestimmten Ereignisses steht.

   xvar:			Dieser Parameter ist zur Angabe der stetigen Variablen 
					(Einflussgren). Allgemein gesagt sollten hier alle Variablen 
					genannt werden, die nicht in das CLASS-Statement von PROC LOGISTIC 
					aufgenommen werden sollen, also beispielsweise auch quasi-stetige 
					Variablen. Werden mehrere stetige oder quasi-stetige Variablen 
					bergeben, so mssen diese durch ein Blank (Leerzeichen) getrennt 
					werden. Beispiel: "xvar=alter bmi".

   cvar:			"cvar" steht fr "categorical variables" und ist fr die bergabe 
					aller kategorischen sowie aller dichotomen Variablen verantwortlich. 
					Diese werden innerhalb der LOGISTIC-Prozedur in das CLASS-Statement 
					aufgenommen und werden automatisch Dummy-kodiert. Werden mehrere 
					kategorische Variablen bergeben, mssen diese getrennt durch ein 
					Leerzeichen (Blank) bergeben werden (Beispiel: "cvar=diagnose msnr").
					Wurden dem Parameter "cvar" dichotome Variablen bergeben, so werden 
					diese zwar in das CLASS-Statement von PROC LOGISTIC aufgenommen, 
					jedoch keine Dummy-Kodierung durchgefhrt, da dichotome Variablen nur 
					zwei Klassen besitzen.

   ref:				Mittels dieses Parameters, der fr "reference" steht, ist es 
					mglich, pro bergebener kategorischer Variable eine sogenannte 
					Referenzkategorie anzugeben. Werden keine Referenzkategorien 
					angegeben, wird der Standard bei jeder Variablen innerhalb des 
					CLASS-Statements von PROC LOGISTIC auf REF=FIRST gesetzt. Dies 
					bedeutet, dass die erste (sortierte) Kategorie jeder bergebenen 
					Variablen als Referenzkategorie gewhlt wird. Werden 
					Referenzkategorien bergeben, so muss deren Anzahl mit der Anzahl 
					der bergebenen kategorischen Variablen bereinstimmen. SAS bietet 
					standardmig die Kategorien FIRST und LAST an. Zustzlich ist es 
					mit Hilfe dieses Parameters mglich, beliebige Kategorien anzugeben. 
					Das Trennungszeichen ist der Stern (*). Eigens gewhlte 
					Referenzkategorien mssen zustzlich in Hochkommata eingeschlossen 
					werden. Die SAS-Standardkategorien FIRST / LAST nicht. Die ntige 
					Form lautet beispielsweise: "ref='1'*FIRST*'4'". Der dazugehrige 
					"cvar"-Parameter: "cvar=icdklass3 msnr_num beginn_b_num". Von der 
					bergebenen Variablen "icdklass" wird die Kategorie "1" als 
					Referenzkategorie gewhlt. Von der Variablen "msnr_num" die erste 
					(sortierte), unabhngig was diese fr einen Wert besitzt und von der 
					Variablen "beginn_b_num" die Kategorie "4". 

   miss:			Dieser Parameter legt fest, ob Beobachtungen mit fehlenden Werten 
					(missing values) in den unter "xvar", "cvar" und "resp_var" 
					bergebenen Variablen vor der Analyse aus dem Datensatz eliminiert 
					werden sollen (0) oder nicht (1). Der Standardwert dieses Parameters 
					ist 0 (Complete Case-Analyse). Hierbei ist zu beachten, dass fehlende 
					Werte im Datensatz durch einen Punkt (.) reprsentiert sein mssen.

   crossval_type:	Der Parameter 'crossval_type' steht fr "Crossvalidation Type" 
					und bestimmt, welche Art der Crossvalidierung durchgefhrt 
					werden soll:
				 	crossval_type= 1 (Standard)  : K-Fold-Crossvalidation
				 	crossval_type= 2 (Alternativ): Jackknife-Crossvalidation
				 	crossval_type= 3 (Alternativ): Adjusted-Crossvalidation

   gruppen:			Dieser Parameter ist lediglich fr die K-Fold-Crossvalidierung
					und die adjustierte Crossvalidierung relevant. Er dient der
					Festlegung, in wie viele Gruppen der Datensatz unterteilt werden 
					soll. Standard=10, das heit, dass der Datensatz in 10 Untergruppen 
					unterteilt wird.
				 	Wird beispielsweise als Parameter 10 bergeben, werden 9 Gruppen 
					fr die Entwicklung des Modells herangezogen und die letzte Gruppe 
					zur Validierung verwendet. Dabei wird mittels einer Schleife jede 
					der 10 Gruppen einmal zur Validierung herangezogen und die anderen 
					Gruppen jeweils zur Modellbildung verwendet. Kenngren werden 
					gemittelt. 

   anzahl:			Dieser Parameter legt fest, wie oft die K-Fold-Crossvalidierung 
					bzw. die adjustierte Crossvalidierung durchgefhrt werden soll.
				    Kenngren werden auch hierbei gemittelt. Der Standardwert ist 50.
					Wird das Makro beispielsweise mit den Parametern 'crossval_type=1',
					'gruppen=10' und 'anzahl=50' aufgerufen, so findet die K-Fold-
					Crossvalidierung 50 mal mit je 10 Gruppen statt.

   exact:			Der Parameter 'exact' ist fr die Jackknife- Crossvalidierung von
					Bedeutung und legt fest, ob eine genaue Berechnung mittels 
					"Leaving-One-Out" durchgefhrt werden soll (1) oder nicht (0). Der
					Standardwert ist 0, d.h. es wird nur die approximative Berechnung
				    mittels der SAS PROC LOGISTIC OPTION PREDPROBS=X durchgefhrt.
					Bei der Jackknife- Crossvalidierung wird je eine Beobachtung bei der 
					Modellentwicklung nicht bercksichtigt. An dieser wird das entwickelte 
					Modell danach validiert. Diese Prozedur wird fr jede Beobachtung 
					durchgefhrt.

   roc_auto:		Auch dieser Parameter ist lediglich fr die Jackknife- 
					Crossvalidierung relevant und legt fest, ob cutpoint- abhngige 
					ROC- Kenngren aus der automatisch von SAS generierten Tabelle 
					(PROC LOGISTIC OPTION "OUTROC=") oder aus einer manuell generierten 
					Tabelle berechnet werden sollen. Standard=1 (automatische Tabelle). 
					0= manuelle Tabelle (Achtung: viel Rechenzeit!).
					
   macro_path: 		Aufgrund der Tatsache, dass innerhalb des Makros diverse 
					Untermakros aufgerufen werden, ist es zwingend ntig, den Pfad 
					zu dem Verzeichnis, wo die komplette Makrosammlung abgelegt ist,
					anzugeben. Der Pfad selber muss in Hochkommata eingeschlossen 
					sein. Beispiel: 'f:\makros'. 

   binwidth:		Der Parameter "binwidth" gibt die Bin-Breite fr die Schtzung der 
					Assoziationsstatistik an. Der Wert muss dabei zwischen 0 und kleiner
					1 liegen. SAS verwendet als	Standard 0.002, ab Version 9.4 wird bei 
					binrem Endpunkt und weniger als 5 Millionen Beobachtungen jedoch 
					auf Binnen verzichtet. Der Parameter erleichtert daher den Vergleich
					der Ergebnisse unterschiedlicher SAS Versionen.

  VALIDATION
  ----------

  Validiert durch: Stephanie Heiss
 				   University of Ulm 
                   Department of Biometry and Medical Documentation
                   89075 Ulm
  
  Validiert am:    06. Oktober 2003

  Validierungsprogramm::      PM_CROSSVALIDATION_VAL.SAS
  Validierungsdokumentation:  PM_CROSSVALIDATION_VAL.DOC
   
  Validierungsvorgehen:
  - Das Testen der Makros wird durch eine von der Programmierung unabhngige 
    Person durchgefhrt.
  - Fr die Durchfhrung der Validierung wird auf einen in der Literatur 
    genutzten Datensatz zurckgegriffen:  LBW (Low Birth Weight), beschrieben
    in Hosmer, Lemeshow: Applied logistic regression [hosmer2000]
  - Der Datensatz wird fr verschiedene Testlufe verndert bzw. erweitert, 
    damit "die Testdaten neben den normalen Angaben auch Grenzwerte, ungltige
    Angaben, Fehlerbedingungen und Sonderflle beinhalten." [beiwanger2003]
  - Fehlerkontrolle:
    Hierbei werden nur die im LOG-Fenster auftretenden Fehler bzw. Warnungen 
    und offensichtlichen Fehler im OUTPUT betrachtet. Fr jeden Parameter des 
    Makros werden mehrere Durchlufe mit verschiedenen Wertangaben durchgefhrt, 
    wobei auch ungltige Werte angegeben werden, um das Verhalten des Makros
    bei fehlerhaften Aufrufen zu untersuchen. Gefundene Fehler werden 
    dokumentiert und dem Programmierer zur Verfgung gestellt. Nach der Korrektur 
    aller Fehler werden die Testlufe wiederholt, um sicherzustellen, dass 
    wirklich alle Fehler abgefangen wurden und dass keine neuen Fehler entstanden 
    sind.
  - Ergebniskontrolle:
    Um die Richtigkeit der Ergebnisse zu berprfen, werden soweit mglich, die
    Rechenbeispiele und bungsaufgaben aus Hosmer/Lemeshow durch vergleichbare
    Makroaufrufe nachgerechnet. Die Ergebnisse werden denen aus dem Begleitbuch 
    Hosmer/Lemeshow/Cook [hosmer2001] gegenbergestellt und direkt verglichen.
  - Alle Validierungen werden dokumentiert.

  Literatur: 
  - Beiwanger S (2003): Validierung von SAS-Makros. Diplomarbeit, FH Ulm
  - Hosmer DW, Lemeshow S (2000): Applied logistic regression 2nd Ed. 
           J. Wiley & Sons, New York
  - Hosmer DW, Lemeshow S, Cook ED (2001): Solutions manual to accompany
           applied logistic regression 2nd. Ed. J. Wiley & Sons, New York



  AUTHOR
  ------
  R. Muche / C. Ring / C. Ziegler
  University of Ulm
  Department of Biometry and Medical Documentation
  89075 Ulm

  The author does not overtake any responsibility for the correctness
  of results. It must not be modified without removing the heading lines 
  containing the authors name.

----------------------------------------------------------------------------------*/

%MACRO PM_CROSSVALIDATION (	data				= ,
							resp_var			= , 
							event				= 1,
							xvar				= ,
							cvar				= ,
							ref					= ,
							miss				= 0,
							crossval_type		= 1,
							gruppen				= 10,
							anzahl				= 50,
							exact				= 0,
							roc_auto			= 1,
							macro_path			= ,
							binwidth			= 0
						 );

/*globale Variable des Untermakros 'numberobs.mac.sas'*/

%GLOBAL nnnn1;

/*Prfungen / Referenzierungen / Standardbelegungen*/

/*Wichtigster Parameter 'macro_path' berprfen*/
/*bergabe erfolgt?*/

%IF &macro_path= %THEN %DO;

	%WINDOW error color=grey                                    
     #5  @28 'Fehler!' attr=highlight                
             color=blue                                                                 
     #7  @28 "Dem zwingend erforderlichen Parameter 'macro_path' wurde kein Wert" 
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"	
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';               
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Hilfsprogramm welches berprft, ob dem Parameter 'macro_path' ein korrekter Pfad*/
/*bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/

/*Anzahl der Zeichen im String 'macro_path' bestimmen:*/

%LET path_length=%LENGTH(&macro_path);

%LET infile=&macro_path;

%IF %bquote(&infile) NE %THEN %DO;
     %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
      %IF %bquote(&char1) ne %STR(%')
         %THEN %LET command='NOK';
         %ELSE %LET command='OK';

	 %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
      %IF %bquote(&char2) ne %STR(%')
         %THEN %LET command2='NOK';
         %ELSE %LET command2='OK';
%END;

/*falls die Syntax nicht stimmt, Anzeige eines Hinweisfensters*/

%IF &command='NOK' OR &command2='NOK' %THEN %DO;

	%WINDOW error color=grey               
	 #5  @28 'Fehler!' attr=highlight                                          
             color=blue                                                      
     #7  @28 "Dem Parameter 'macro_path' wurde ein syntaktisch falscher Wert" 
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"	
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';     
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Test, ob das unter 'macro_path' angegebene Verzeichnis existiert*/

%LET dir=&macro_path;

%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
       %PUT Directory does not exist;
%END;
%ELSE %DO;
       %PUT Directory exists;
       %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;

	%WINDOW error color=grey                                                   
	 #5  @28 'Fehler!' attr=highlight                           
             color=blue                                                      
	 #7  @28 "Das unter dem Parameter 'macro_path' angegebene Verzeichnis existiert"
	 #8  @28 "nicht- Bentigte Untermakros knnen nicht referenziert werden"
	 #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';         
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Falls bestimmte Parameter ohne Wert bergeben wurden, diese mit Standard belegen*/

%IF &event= %THEN %DO;
	%LET event=1;
%END;

%IF &miss= %THEN %DO;
	%LET miss=0;
%END;

%IF &crossval_type= %THEN %DO;
	%LET crossval_type=1;
%END;

%IF &gruppen= %THEN %DO;
	%LET gruppen=10;
%END;

%IF &anzahl= %THEN %DO;
	%LET anzahl=50;
%END;

%IF &exact= %THEN %DO;
	%LET exact=0;
%END;

%IF &roc_auto= %THEN %DO;
	%LET roc_auto=1;
%END;

/*Externe Makros, die innerhalb des Makros 'PM_CROSSVALIDATION.MAC.SAS'*/
/*verwendet werden (Untermakros) einbinden (%include)*/

FILENAME y &macro_path;

%INCLUDE y ('jackknife_crossvalidation.mac.sas');
%INCLUDE y ('k_fold_crossvalidation.mac.sas');
%INCLUDE y ('adjusted_crossvalidation.mac.sas');
%INCLUDE y ('exist_var.mac.sas');
%INCLUDE y ('exist.mac.sas');
%INCLUDE y ('numchar.mac.sas');
%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('ref.mac.sas');

/*Test, ob bei bestimmten Parametern anstatt numerischen Werten*/
/*character- Werte bergeben wurden*/

/*berprfung erfolgt mit Hilfe des Untermakros 'numchar.mac.sas'*/

%NUMCHAR(parameterliste=&event &miss &crossval_type &gruppen &roc_auto &exact &anzahl &binwidth);

%IF &numcharok='NOK' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error8.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Parameter, bei denen keine negativen Werte erlaubt sind, berprfen*/

%IF &anzahl <=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error9.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%IF &binwidth <0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error66.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%IF &gruppen <=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error9.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%IF &gruppen =1 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%WINDOW error color=grey                                                   
	 #5  @28 'Fehler!' attr=highlight                           
             color=blue                                                      
	 #7  @28 "Dem Parameter 'gruppen' wurde der Wert 1 bergeben!"
	 #8  @28 "Dies ist jedoch nicht mglich!"
	 #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';         
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfen, ob der angegebene Datensatz (Parameter 'data=') auch wirklich existiert*/
/*(mit Hilfe des Hilfs-/Untermakros 'exist.mac.sas')*/

%EXIST(dsn=&data);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error4.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Datenquelle (Parameter 'data=' UND response-Variable*/
/*(Zielgre / Parameter 'resp_var=') angegeben wurde*/

%IF &data= or &resp_var= %THEN %DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

	%INCLUDE y ('error1.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob berhaupt Variablen (Parameter 'xvar=' / 'cvar=') angegeben wurden*/

%IF &xvar= and &cvar= %THEN %DO;

/*wenn keine Variablen angegeben wurden, Anzeige eines Fensters mit*/ 
/*Fehlermeldung/ Hinweis*/

	%INCLUDE  y ('error2.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Prfung, ob alle bergebenen Variablen numerisch sind*/

/*Erzeugung eines temporren Zwischendatensatzes, welcher lediglich die*/
/*bergebenen Variablen enthlt*/

DATA test3;
	SET &data;
	KEEP &xvar &resp_var &cvar;
RUN;

/*mittels der Prozedur PROC CONTENTS kann der Typ einer Variablen bestimmt werden*/

/*Unterdrckung der Ausgabe von PROC CONTENTS*/

PROC CONTENTS DATA=test3 NOPRINT OUT=testcontent;
RUN;

/*lediglich die Variable 'type' der Ergebnisdatei behalten,*/
/*da nur diese von Interesse ist*/

DATA testcontent;
	SET testcontent;
	KEEP type;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=testcontent);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test4=&nnnn1;

/*Prfung ob Variablen vom Typ 'character' vorhanden sind*/

DATA test4;
	SET testcontent;
	IF type=2 THEN DELETE;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test4);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test5=&nnnn1;

/*falls Variablen bergeben wurden, die nicht numerisch sind --> Fehlermeldung*/

%IF &test4 ne &test5 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error60.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE test4 testcontent test3;
RUN;
QUIT;

/*berprfung, ob alle bergebenen Variablen im bergebenen Datensatz*/
/*(Parameter 'data' vorhanden sind)*/

/*Zusammenfassung aller Variablen in einer neuen Makro- Variablen*/

%LET all_variables= &resp_var &xvar &cvar;

/*Prfung erfolgt mit Hilfe des Untermakros 'exist_var.mac.sas'*/

%EXIST_VAR(dsn=&data, var=&all_variables);

/*falls irgendwelche Variablen nicht existieren, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error6.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob die Zielgre (Parameter 'resp_var')*/
/*2 Kategorien (Ausprgungen) besitzt*/

/*missing values ausschliessen*/

DATA freq2;
	SET &data;
	IF &resp_var ne .;
RUN;

/*Anzahl der Ausprgungen mit Hilfe der Prozedur PROC FREQ ermitteln*/

/*Ausgabe unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnis als ODS- Datei zur weiteren Verarbeitung ausgeben*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=freq2;
	TABLES &resp_var;
RUN;

/*ODS- Output schliessen*/

ODS OUTPUT CLOSE;

/*ODS- Ausgabe wieder aktivieren*/

ODS LISTING;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen (= # Ausprgungen der Zielgre)*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq freq2;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

/*falls Zielgre mehr als zwei Ausprgungen besitz--> Fehlermeldung*/

%IF &event_check > 2 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE  y ('error5.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Parameter 'miss' 0 oder 1 ist*/

%IF &miss ne 1 %THEN %DO;
	%IF &miss ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error7.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'event' 0 oder 1 ist*/

%IF &event ne 1 %THEN %DO;
	%IF &event ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error7.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'crossval_type' 1 oder 2 oder 3 ist*/

%IF &crossval_type ne 1 %THEN %DO;
	%IF &crossval_type ne 2 %THEN %DO;
		%IF &crossval_type ne 3 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

			%INCLUDE y ('error55.sas');

/*Sprungmarke im Fehlerfall*/

			%GOTO ende;

		%END;
	%END;
%END;

/*berprfung, ob Parameter 'exact' 0 oder 1 ist*/

%IF &exact ne 1 %THEN %DO;
	%IF &exact ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error56.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'roc_auto' 0 oder 1 ist*/

%IF &roc_auto ne 1 %THEN %DO;
	%IF &roc_auto ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error56.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*Anzahl der angegebenen kategoriellen Variablen ermitteln:*/

%GLOBAL c_count c_var;
%LET c_count=1;
%LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%DO %WHILE(&c_var ne);
     %LET c_count=%EVAL(&c_count+1);
     %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%END;
%LET c_count=%EVAL(&c_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/

%PUT The cvar contains &c_count variables.;

/*Anzahl der Referenzen ermitteln*/

%GLOBAL ref_count var_ref;
%LET ref_count=1;
%LET var_ref=%QSCAN(&ref,&ref_count,*);
%DO %WHILE(&var_ref ne);
     %LET ref_count=%EVAL(&ref_count+1);
     %LET var_ref=%QSCAN(&ref,&ref_count,*);
%END;
%LET ref_count=%EVAL(&ref_count-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The refs contains &ref_count variables.;

%IF &ref_count >0 %THEN %DO;
		%IF &c_count ne &ref_count %THEN %DO;

/*falls Anzahl der kategoriellen Variablen grer Null ist und
/* nicht mit der Anzahl der  angegebenen*/
/*Referenzkategorien bereinstimmt, Anzeige eines Fensters*/
/*mit Fehlermeldung/ Hinweis*/

			%INCLUDE y ('error3.sas');

/*Sprungmarke im Fehlerfall*/

			%GOTO ende;

   		%END;
%END;

/*berprfung, ob etwaige bergebene Referenzkategorien vorhanden sind bzw.*/
/*korrekt bergeben wurden*/

%IF &cvar ne %THEN %DO;
	%IF &ref_count ne 0 %THEN %DO;

	%REF(data=&data, resp_var=&resp_var, event=&event, cvar=&cvar, ref=&ref);

	%IF &correct='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error11.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;

%END;
%END;	

/*Prfen, ob angegebene Gruppenzahl > # Beobachtungen im DS ist*/

%NUMBEROBS(datensatz=&data);

%LET groupcheck=&nnnn1;

%IF &gruppen > &groupcheck %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error57.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfen ob eine Variable nach Entfernung aller Beobachtungen mit missing*/
/*values nur noch eine Ausprgung besitzt*/

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values*/

DATA work.category_test;
	SET  &data;

	%DO i=1 %TO &count_x;
		%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
		IF &xvar_liste=. THEN DELETE;
	%END;

	%DO i=1 %TO &count;
		%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
		IF &cvar_liste=. THEN DELETE;
	%END;

		IF &resp_var=. THEN DELETE;

RUN;

/*Prfen ob 'resp_var' /'xvar'/ 'cvar' nur 1 Ausprgung besitzen*/

%LET category_test=0;
%LET category_test_fail=;

/*event*/

/*Anzahl der Ausprgungen mittels PROC FREQ bestimmen*/

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &resp_var;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

%IF &event_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &resp_var;

%END;

/*xvar*/

%IF &xvar ne %THEN %DO;

%DO uhgu=1 %TO &count_x;

%LET xvar_liste=%SCAN(&xvar,&uhgu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &xvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET xvar_check=&nnnn1;

%IF &xvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &xvar_liste;

%END;

%END;
%END;

/*cvar*/

%IF &cvar ne %THEN %DO;

%DO uhfu=1 %TO &count;

%LET cvar_liste=%SCAN(&cvar,&uhfu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &cvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET cvar_check=&nnnn1;

%IF &cvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &cvar_liste;

%END;

%END;
%END;

PROC DATASETS NOLIST;
	DELETE category_test;
RUN;
QUIT;

%IF &category_test=1 %THEN %DO;

	DATA category_test;
		var="&category_test_fail";
	RUN;

	PROC PRINT DATA=category_test NOOBS LABEL;
	TITLE1 "Folgende Variablen besitzen nach Entfernung aller";
	TITLE2 "missing values lediglich eine Ausprgung:";
		VAR var;
			LABEL var='Variable';
	RUN;

	PROC DATASETS NOLIST;
		DELETE category_test;
	RUN;
	QUIT;

	%INCLUDE y ('error24.sas');
	%GOTO ende;

%END;

%IF &binwidth >=1 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error66.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%IF &binwidth >0.01 %THEN %DO;

/*Anzeige eines Warnhinweises*/

	DATA hint;
		hint="Bei einem Wert von 0 wird kein Binning angewandt. In manchen SAS Versionen wird der Standard 0.002 verwendet.";
	RUN;
	PROC PRINT DATA=hint NOOBS LABEL HEADING=H WIDTH=UNIFORM;
		VAR   hint;
		TITLE1 "Dem Parameter 'binwidth' wurde ein Wert grer 0.01 bergeben.";
		LABEL hint = "Sinnvoll ist ein Wert zwischen 0 und 0.01."; 
	RUN;
%END;

/*ENDE Prfungen / Referenzierungen / Standardbelegungen*/

/*-------------------------------------------------------------------------------*/

/*Auswahl der Crossvalidierungsmethode.*/
/*K-Fold (1) = Standard*/

%IF &crossval_type=1 %THEN %DO;

%K_FOLD_CROSSVALIDATION	(		data				= &data,
								resp_var			= &resp_var, 
								event				= &event,
								xvar				= &xvar,
								cvar				= &cvar,
								ref					= &ref,
								miss 				= &miss,
								gruppen				= &gruppen,
								anzahl				= &anzahl,
								macro_path			= &macro_path,
								binwidth			= &binwidth
	       					 );

%END;

/*Auswahl der Crossvalidierungsmethode*/
/*Jackknife (2)*/

%IF &crossval_type=2 %THEN %DO;

%JACKKNIFE_CROSSVALIDATION(		data				= &data,
								resp_var			= &resp_var, 
								event				= &event,
								xvar				= &xvar,
								cvar				= &cvar,
								ref					= &ref,
								miss 				= &miss,
								exact				= &exact,
								roc_auto			= &roc_auto,
								macro_path			= &macro_path,
								binwidth			= &binwidth
	       					 );

%END;

/*Auswahl der Crossvalidierungsmethode*/
/*Adjusted (3)*/

%IF &crossval_type=3 %THEN %DO;

%ADJUSTED_CROSSVALIDATION(		data				= &data,
								resp_var			= &resp_var, 
								event				= &event,
								xvar				= &xvar,
								cvar				= &cvar,
								ref					= &ref,
								miss 				= &miss,
								gruppen				= &gruppen,
								anzahl				= &anzahl,
								macro_path			= &macro_path,
								binwidth			= &binwidth
	       					 );

%END;

%ende:

%MEND PM_CROSSVALIDATION;
