/************************************************************************************/
/*						        													*/
/*     NAME: pm_datasplitting.mac.sas	   				      						*/
/*    TITLE: Zufllige Aufteilung eines Datensatzes in zwei Gruppen auf Basis 		*/
/*			 eines gewnschten Anteils von Beobachtungen je Gruppe (Datasplitting)	*/
/*     KEYS: external validation, datasplitting									 	*/
/*  VERSION: SAS 8.2												      			*/
/*  PRODUCT: BASE, STAT									        					*/
/*    PROCS: Zufallsfunktion RANUNI										      		*/
/*															         				*/
/*																					*/
/*  Verzeichnis/Dateiname:			F:\Prognose\pm_datasplitting.mac.sas		    */
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*  Programmierer:					Christoph Ziegler								*/
/*  Datum der Ersterstellung:		7/2003											*/
/*  Aktuelle Makro-Version:			12/2003											*/
/*																					*/
/*  Kurzbeschreibung:				Makro zur zuflligen Aufteilung eines 			*/
/*									Datensatzes in zwei Gruppen auf Basis 			*/
/*									eines gewnschten Anteils von Beobachtungen    	*/
/*									je Gruppe (Datasplitting)						*/
/*																					*/
/*  Verwendete externe Untermakros:	numberobs.mac.sas								*/
/*									numchar.mac.sas									*/
/*									exist.mac.sas									*/
/*									error1.sas	error4.sas	error8.sas				*/
/*									error9.sas	error16.sas	error17.sas				*/
/*																					*/
/*  Bentigte Dateien:				Datensatz (Parameter 'data=')			   		*/
/*															           				*/
/*																					*/
/*  Erzeugte Dateien:				work.splitdata1 (Parameter 'name_data1=')		*/
/*					  				work.splitdata2	(Parameter 'name_data2=')		*/
/*  (bei Standardeinstellung;														*/
/*	 Dateinamen frei whlbar)														*/
/*  									            								*/
/*  Programmnderungen																*/
/*  und -ergnzungen 																*/
/*  (Name, Datum, Beschreibung):											 		*/
/*  																				*/
/*        																			*/
/************************************************************************************/
/*

 TITLE
 -----
 pm_datasplitting.mac.sas

 SAS- Makro zur zuflligen Aufteilung eines Datensatzes in zwei Gruppen auf Basis
 eines gewnschten Anteils von Beobachtungen je Gruppe.

 BACKGROUND
 ----------
 Durch die zufllige Aufteilung eines vorliegenden Datensatzes in zwei Teildatenstze 
 kann die Situation der externen (nach Altman und Royston [altman2000]: temporalen) 
 Validierung ermglicht werden. Ein Teildatensatz, Harrell bezeichnet ihn in [harrell2001] 
 als "training (model development) sample", wird dann fr die Modellentwicklung inklusive 
 Reklassifikation und Bestimmung der Prognosegte (ROC-Analyse) genutzt, der zweite 
 Teildatensatz ("test (model validation) sample") steht dann fr eine externe Validierung 
 zur Verfgung (s. PM_EXTERNAL_VALIDATION.MAC.SAS).

 Die Aufteilung wird mit einem Zufallsmechanismus gesteuert, der den Originaldatensatz auf 
 der Basis eines vorgegebenen Prozentsatzes teilt. Gngige Prozentstze fr den Anteil des 
 Trainingsdatensatzes sind 50%, 66% oder 70%. Der jeweilige Restdatensatz bildet dann den 
 Testdatensatz. 

 Die Vorteile des Data-Splitting sind erstens, dass der Testdatensatz sicher dieselbe 
 Struktur wie der Trainingsdatensatz hat und zweitens, dass der Testdatensatz aus der 
 selben Grundgesamtheit wie der Modellierungsdatensatz stammt, da er durch eine 
 Zufallsstichprobenziehung aus dem Originaldatensatz gewonnen wurde. Somit kann eine 
 externe (temporale) Validierung problemlos erfolgen. 
 Leider berwiegen bei der Anwendung dieses Validierungsverfahrens die Nachteile. Unter 
 anderen beschreiben Harrell [harrell2001], Altman und Royston [altman2000] sowie Efron und 
 Tibshirani [efron1993] folgende Probleme:
 - Data-Splitting reduziert die Fallzahl in den Datenstzen (sowohl im Trainings- als auch 
   im Testdatensatz). Damit geht die bessere Schtzung der Prognosegte (unbiased) zu 
   Lasten der Modellentwicklung (grere Streuung der Regressionskoeffizienten bis hin zur 
   Unmglichkeit einer Modellierung).
 - Die Schtzer der Prognosegte werden ungenauer.
 - Das Ergebnis hngt von der konkreten Zufallsstichprobe ab.
 - Mit der Methode des Data-Splitting kann nicht ein endgltiges Prognosemodell validiert 
   werden sondern nur das jenige, welches auf einem Teildatensatz entwickelt wurde. 
 - Data-Splitting erfordert, dass die Aufteilung vor Beginn der gesamten Analyse vorgenommen 
   wird. 


 DESCRIPTION
 -----------
 Das Makro PM_DATASPLITTING.MAC.SAS trennt einen Datensatz in zwei Teil-Datenstze mit dem 
 angegebenen Prozentsatz. Dabei wird die SAS-Funktion RANUNI verwendet, die eine zufllige 
 Reihenfolge der Beobachtungen im Datensatz definiert, der dann anhand des entsprechenden 
 Perzentils aufgeteilt wird. Mit den so erzeugten Datenstzen knnen zur Modellentwicklung 
 anhand des Trainingsdatensatzes die Makros des Makropaketes direkt benutzt werden. Der 
 Testdatensatz muss dann mit dem Makro fr die externe Validierung 
 (PM_EXTERNAL_VALIDATION.MAC.SAS) in die Modellvalidierung eingebunden werden. Somit stehen 
 fr alle Auswertungsschritte nach der Aufteilung des Datensatzes entsprechende SAS-Makros 
 zur Verfgung. 

 LITERATURE
 ----------
 - SAS Online Doc zu der Funktion RANUNI (Zufallsfunktion)
 - Altman DG, Royston P (2000):  What do we mean by validating a prognostic model?
                Statistics in Medicine, 19, 453 - 473
 - Efron B, Tibshirani RJ (1993):  An introduction to the bootstrap.
                Chapman & Hall, New York
 - Harrell, FE (2001): Regression Modeling Strategies. Springer, New York
 - Ziegler C (2003): Ein SAS-Makropaket zur Entwicklung und Validierung von 
					 Prognosemodellen auf Basis der logistischen Regression. 
					 Diplomarbeit, FH Ulm 

 SYNTAX
 ------
 [Makroaufruf und Beschreibung der zu bergebenden Parameter]

 %PM_DATASPLITTING(	
   *data 		 = ,				Datensatz
	name_data1   = splitdata1,		Name fr den Datensatz Nr.1 
	name_data2   = splitdata2,		Name fr den Datensatz Nr.2
	percent	     = 50,				Aufteilungskriterium in Prozent
   *macro_path   =					Pfad zur Makrosammlung
  );

  Mit (*) gekennzeichnete Parameter sind beim Makroaufruf zwingend erforderlich.
  Bereits gesetzte Parameter stellen die jeweilige Standardeinstellung dar.

   data:			Zur Angabe des zu analysierenden Datensatzes. 
					Wurde beispielsweise vor dem Aufruf des Makros ein Datensatz mit
					dem Namen "dat300" angelegt, lautet der korrekte Aufruf: 
					data=dat300. Zustzlich kann auch die dazugehrige SAS- Library
					bergeben werden (Beispiel: data=work.dat300). 
					Die bergabe dieses Parameters ist zwingend erforderlich.

   name_data1:		Zur Angabe des spteren Datensatzes Nr.1 nach der Aufteilung des
					Originaldatensatzes. Standard= work.splitdata1.

   name_data2:		Zur Angabe des spteren Datensatzes Nr.2 nach der Aufteilung des
					Originaldatensatzes. Standard= work.splitdata2.

   percent:			Zur Angabe einer Prozentzahl von Beobachtungen, die vom 
					Originaldatensatz (Parameter 'data=') abgetrennt werden sollen. 
					Standard=50. Wird diesem Parameter beispielsweise der Wert 20 
					bergeben, so enthlt das sptere Dataset1 (Parameter 'name_data1=')
					80% der Beobachtungen und das sptere Dataset2 (Parameter 
					'name_data2=') 20% der Beobachtungen vom Originaldatensatz.
					
   macro_path: 		Aufgrund der Tatsache, dass innerhalb des Makros diverse 
					Untermakros aufgerufen werden, ist es zwingend ntig, den Pfad 
					zu dem Verzeichnis, wo die komplette Makrosammlung abgelegt ist,
					anzugeben. Der Pfad selber muss in Hochkommata eingeschlossen 
					sein. Beispiel: 'f:\makros'. 

  VALIDATION
  ----------

  Validiert durch: Stephanie Heiss
 				   University of Ulm 
                   Department of Biometry and Medical Documentation
                   89075 Ulm
  
  Validiert am:    12. August 2003

  Validierungsprogramm::      PM_DATASPLITTING_VAL.SAS
  Validierungsdokumentation:  PM_DATASPLITTING_VAL.DOC
   
  Validierungsvorgehen:
  - Das Testen der Makros wird durch eine von der Programmierung unabhngige 
    Person durchgefhrt.
  - Fr die Durchfhrung der Validierung wird auf einen in der Literatur 
    genutzten Datensatz zurckgegriffen:  LBW (Low Birth Weight), beschrieben
    in Hosmer, Lemeshow: Applied logistic regression [hosmer2000]
  - Der Datensatz wird fr verschiedene Testlufe verndert bzw. erweitert, 
    damit "die Testdaten neben den normalen Angaben auch Grenzwerte, ungltige
    Angaben, Fehlerbedingungen und Sonderflle beinhalten." [beiwanger2003]
  - Fehlerkontrolle:
    Hierbei werden nur die im LOG-Fenster auftretenden Fehler bzw. Warnungen 
    und offensichtlichen Fehler im OUTPUT betrachtet. Fr jeden Parameter des 
    Makros werden mehrere Durchlufe mit verschiedenen Wertangaben durchgefhrt, 
    wobei auch ungltige Werte angegeben werden, um das Verhalten des Makros
    bei fehlerhaften Aufrufen zu untersuchen. Gefundene Fehler werden 
    dokumentiert und dem Programmierer zur Verfgung gestellt. Nach der Korrektur 
    aller Fehler werden die Testlufe wiederholt, um sicherzustellen, dass 
    wirklich alle Fehler abgefangen wurden und dass keine neuen Fehler entstanden 
    sind.
  - Ergebniskontrolle:
    Um die Richtigkeit der Ergebnisse zu berprfen, werden soweit mglich, die
    Rechenbeispiele und bungsaufgaben aus Hosmer/Lemeshow durch vergleichbare
    Makroaufrufe nachgerechnet. Die Ergebnisse werden denen aus dem Begleitbuch 
    Hosmer/Lemeshow/Cook [hosmer2001] gegenbergestellt und direkt verglichen.
  - Alle Validierungen werden dokumentiert.

  Literatur: 
  - Beiwanger S (2003): Validierung von SAS-Makros. Diplomarbeit, FH Ulm
  - Hosmer DW, Lemeshow S (2000): Applied logistic regression 2nd Ed. 
           J. Wiley & Sons, New York
  - Hosmer DW, Lemeshow S, Cook ED (2001): Solutions manual to accompany
           applied logistic regression 2nd. Ed. J. Wiley & Sons, New York



  AUTHOR
  ------
  R. Muche / C. Ring / C. Ziegler
  University of Ulm
  Department of Biometry and Medical Documentation
  89075 Ulm

  The author does not overtake any responsibility for the correctness
  of results. It must not be modified without removing the heading lines 
  containing the authors name.

----------------------------------------------------------------------------------*/

%MACRO PM_DATASPLITTING (	data 		 = ,
					  		name_data1 	 = splitdata1,
					  		name_data2 	 = splitdata2,
					  		percent	     = 50,
					  		macro_path   =
				    	);

/*Prfungen / Referenzierungen / Standardbelegungen*/

/*Wichtigster Parameter 'macro_path' berprfen*/
/*bergabe erfolgt?*/

%IF &macro_path= %THEN %DO;

	%WINDOW error color=grey                                                        
      #5  @28 'Fehler!' attr=highlight                                        
               color=blue                                                     
      #7  @28 "Dem zwingend erforderlichen Parameter 'macro_path' wurde kein Wert"
	  #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
      #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                  
    %DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Hilfsprogramm welches berprft, ob dem Parameter 'macro_path' ein korrekter Pfad*/
/*bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/

/*Anzahl der Zeichen im String 'macro_path' bestimmen:*/

%LET path_length=%LENGTH(&macro_path);

%LET infile=&macro_path;

%IF %bquote(&infile) NE %THEN %DO;
     %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
      %IF %bquote(&char1) ne %STR(%')
         %THEN %LET command='NOK';
         %ELSE %LET command='OK';

	 %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
      %IF %bquote(&char2) ne %STR(%')
         %THEN %LET command2='NOK';
         %ELSE %LET command2='OK';
%END;

/*falls die Syntax nicht stimmt, Anzeige eines Hinweisfensters*/

%IF &command='NOK' OR &command2='NOK' %THEN %DO;

	%WINDOW error color=grey      
     #5  @28 'Fehler!' attr=highlight 
              color=blue  
     #7  @28 "Dem Parameter 'macro_path' wurde ein syntaktisch falscher Wert" 
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';    
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Test, ob das unter 'macro_path' angegebene Verzeichnis existiert*/

%LET dir=&macro_path;

%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
       %PUT Directory does not exist;
%END;
%ELSE %DO;
       %PUT Directory exists;
       %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;

	%WINDOW error color=grey       
     #5  @28 'Fehler!' attr=highlight  
              color=blue                  
     #7  @28 "Das unter dem Parameter 'macro_path' angegebene Verzeichnis existiert"
	 #8  @28 "nicht- Bentigte Untermakros knnen nicht referenziert werden"
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';  
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Falls bestimmte Parameter ohne Wert bergeben wurden, diese mit Standard belegen*/

%IF &name_data1= %THEN %DO;
	%LET name_data1=splitdata1;
%END;

%IF &name_data2= %THEN %DO;
	%LET name_data2=splitdata2;
%END;

%IF &percent= %THEN %DO;
	%LET percent=50;
%END;

/*Externe Makros, die innerhalb des Makros 'PM_DATASPLITTING.MAC.SAS'*/
/*verwendet werden (Untermakros) einbinden (%include)*/

FILENAME y &macro_path;

%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('numchar.mac.sas');
%INCLUDE y ('exist.mac.sas');

/*globale Variable des Untermakros 'numberobs.mac.sas'*/

%GLOBAL nnnn1;

/*Test, ob bei bestimmten Parametern anstatt numerischen Werten*/
/*character- Werte bergeben wurden*/

/*berprfung erfolgt mit Hilfe des Untermakros 'numchar.mac.sas'*/

%NUMCHAR(parameterliste=&percent);

%IF &numcharok='NOK' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error8.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Parameter, bei denen keine negativen Werte erlaubt sind, berprfen*/

%IF &percent <0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error9.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfen, ob der angegebene Datensatz (Parameter 'data=') auch wirklich existiert*/
/*(mit Hilfe des Hilfs-/Untermakros 'exist.mac.sas')*/

%EXIST(dsn=&data);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error4.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Datenquelle angegeben wurde*/

%IF &data=  %THEN %DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

	%INCLUDE y ('error1.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Libnames der Zieldateien vorhanden*/
/*sind falls welche angegeben wurden:*/

%IF &name_data1 ne %THEN %DO;

	%LET length=%LENGTH(&name_data1);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&name_data1,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&name_data1,1,.);
%LET file=%QSCAN(&name_data1,2,.);

DATA &lib..test;
	test='Test';
RUN;

%EXIST(dsn=&lib..test);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE test;
RUN;
QUIT;

%IF &exist ='NO' %THEN %DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

	%INCLUDE  y ('error16.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%END;

%END;

%IF &name_data2 ne %THEN %DO;

	%LET length=%LENGTH(&name_data2);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&name_data2,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&name_data2,1,.);
%LET file=%QSCAN(&name_data2,2,.);

DATA &lib..test;
	test='Test';
RUN;

%EXIST(dsn=&lib..test);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE test;
RUN;
QUIT;

%IF &exist ='NO' %THEN %DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

	%INCLUDE y ('error16.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%END;

%END;

/*berprfen, ob Parameter 'percent' zwischen 0 und 100 ist*/

%IF &percent >100 %THEN %DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

	%INCLUDE y ('error17.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*ENDE Prfungen / Referenzierungen / Standardbelegungen*/

/*-------------------------------------------------------------------------------*/

/*Standardoutput ausgeben*/

%LET datensatz=&data;

/*Fallzahl bestimmen:*/

%NUMBEROBS(datensatz=&data);

%LET fallzahl=&nnnn1;

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatz       ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

PROC DATASETS NOLIST;
	DELETE standardoutput;
RUN;
QUIT;

/*Ende Standardoutput*/
/*-------------------------------------------------------------------------------*/

/*Anzahl Beobachtungen bestimmen, die vom Originaldatensatz abgetrennt werden sollen*/
/*(in Abhngigkeit der angegebenen Prozentzahl)*/

%LET split_count=%SYSEVALF((&fallzahl/100)*&percent);
%PUT &split_count;

/*runden*/

%LET split_count=%SYSEVALF(&split_count, floor);
%PUT &split_count;

/*jeweilige Gruppenstrken bestimmen*/

%LET data1_count=%SYSEVALF(&fallzahl-&split_count);
%LET data2_count=&split_count;

/*Gruppenanzahl*/

%LET gruppen=2;

%PUT Ergebnis: Gruppenzahl = &gruppen . Gruppenstrken: Dataset1=&data1_count  Dataset2=&data2_count;

/*Gruppenstrke zuweisen*/

%LET gruppe1= %EVAL(&data1_count);
%LET gruppe2= %EVAL(&data2_count);

/*Gruppenstrke in %*/

%LET gruppe1_percent=%SYSEVALF(&fallzahl/100);
%LET gruppe2_percent=%SYSEVALF(&fallzahl/100);

%LET gruppe1_percent=%SYSEVALF(&data1_count/&gruppe1_percent);
%LET gruppe2_percent=%SYSEVALF(&data2_count/&gruppe2_percent);

/*Prozentzahlen auf eine Stellen nach dem Komma runden*/

DATA round;
	percent_1=&gruppe1_percent;
	percent_2=&gruppe2_percent;
	call symput('percent_1',trim(left(put(percent_1,BEST5.))));
	call symput('percent_2',trim(left(put(percent_2,BEST5.))));
RUN;

/*Datensatz in 2 Gruppen aufteilen */

/*neuer Zwischendatensatz anlegen, neue Beobachtungsnummer einfhren*/

DATA groups;
	SET &data;
	obs_vwxyz=_N_;
RUN;

/*Ergebnisdatei null-initialisieren*/

DATA ergebnis; SET _NULL_;

/*Schleife ber jede Gruppe (Gruppenanzahl)*/

%DO h=1 %TO &gruppen;

/*Funktion fr Ziehen ohne Zurcklegen*/

DATA group&h (drop=i j count);
	count=0;
	array sel_obs(&&gruppe&h) _TEMPORARY_;
    DO i=1 TO &&gruppe&h;	    
    	redo: select=CEIL(RANUNI(0)*total);
		DO j=1 TO count;
			IF sel_obs(j)=select THEN GOTO redo;
		END;
		SET groups POINT=select NOBS=total;
		count=count+1;
		sel_obs(count)=select;
		OUTPUT;
	END;
	STOP;
RUN;

/*Jede Gruppendatei nach der neue Beobachtungsnummer sortieren*/

PROC SORT DATA=group&h;
	BY obs_vwxyz;
RUN;

/*Speichern, welche Beobachtungen ausgewhlt wurden*/

DATA group&h;
	SET group&h;
	CALL SYMPUT('names'||left(_n_), obs_vwxyz);
RUN;

/*Diese lschen, damit sie in der nchsten Gruppe nicht mehr enthalten sein knnen*/

DATA groups;
	SET groups group&h;
	
	%DO g=1 %TO &&gruppe&h;

	IF obs_vwxyz = &&names&g THEN DELETE;

	%END;

RUN;

/*Neue Spalte einfhren, die die Gruppennummer enthlt*/

DATA group&h;
	SET group&h;
	groupnumber=&h;
RUN;

/*An Ergebnisdatei anhngen*/

DATA ergebnis;
	SET ergebnis group&h;
RUN;

/*Ergebnisdatei wieder sortieren*/

PROC SORT DATA=ergebnis;
	BY obs_vwxyz;
RUN;
	
/*einzelne Gruppen-Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE group&h;
RUN;
QUIT;

/*Log- Fenster lschen (Schutz vor berlauf)*/

DM "clear log;";

%END;

/*Ergebnisdatensatz nach den zufllig zugewiesen Gruppennummern aufteilen*/
/*und in entsprechendes Verzeichnis abspeichern*/

DATA &name_data1 (DROP=obs_vwxyz);
	SET ergebnis;
	IF groupnumber ne 1 THEN DELETE;
RUN;

DATA &name_data2 (DROP=obs_vwxyz);
	SET ergebnis;
	IF groupnumber ne 2 THEN DELETE;
RUN;

/*Zusammenfassung des Datasplittingvorgangs im Output- Fenster ausgeben*/

DATA ergebnisoutput;
	ergebnis="";
RUN;

PROC PRINT DATA=ergebnisoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   ergebnis;

    TITLE1 "Zusammenfassung des Datasplitting:";
	TITLE2 ;
	TITLE3 "Es wurden &percent % der Beobachtungen";
	TITLE4 "vom bergebenen Datensatz (&datensatz) abgetrennt";
	TITLE5 ;
	TITLE6 "Datensatz1: &name_data1 (&data1_count Beobachtungen / &percent_1 %)";
	TITLE7 "Datensatz2: &name_data2 (&data2_count Beobachtungen / &percent_2 %)";

	LABEL ergebnis = '_________________________________________________________';
RUN;

/*temporre Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE ergebnis groups ergebnisoutput round;
RUN;
QUIT;

%ende:

%MEND PM_DATASPLITTING;