/************************************************************************************/
/*						        													*/
/*     NAME: pm_external_validation.mac.sas	   			      						*/
/*    TITLE: Externe Validierung eines logistischen Regressionsmodells anhand 		*/
/*			 eines zweiten Datensatzes												*/
/*     KEYS: external validation, datasplitting, logistic regression, roc analysis 	*/
/*  VERSION: SAS 8.2												      			*/
/*  PRODUCT: BASE, STAT									        					*/
/*    PROCS: LOGISTIC											        			*/
/*															         				*/
/*																					*/
/*  Verzeichnis/Dateiname:			F:\Prognose\pm_external_validation.mac.sas	    */
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*  Programmierer:					Christoph Ziegler								*/
/*  Datum der Ersterstellung:		7/2003											*/
/*  Aktuelle Makro-Version:			12/2003											*/
/*																					*/
/*  Kurzbeschreibung:				Makro zur externen Validierung eines			*/
/*									logistischen Regressionsmodells anhand 			*/
/*									eines zweiten Datensatzes						*/
/*																					*/
/*  Verwendete externe Untermakros:	numberobs.mac.sas								*/
/*									pm_logreg.mac.sas								*/
/*									pm_roc.mac.sas									*/
/*									numchar.mac.sas									*/
/*									exist.mac.sas									*/
/*									exist_var.mac.sas								*/
/*									ref.mac.sas										*/
/*									error1.sas	error2.sas	error3.sas	error4.sas	*/
/*									error5.sas	error6.sas	error7.sas	error8.sas	*/
/*									error11.sas	error24.sas error49.sas	error50.sas */
/*									error51.sas error60.sas	error66.sas				*/
/*																					*/
/*  Bentigte Dateien:				Datensatz 1 (Parameter 'data=')			   		*/
/*									Datensatz 2 (Parameter 'validate_data=')		*/
/*															           				*/
/*																					*/
/*  Erzeugte Dateien:				-----										 	*/
/*  									            								*/
/*  Programmnderungen				Sandra Mller, 04.05.2016, Makrovariable		*/
/*  und -ergnzungen 				binwidth hinzugefgt							*/ 
/*  (Name, Datum, Beschreibung):											 		*/
/*  																				*/
/*        																			*/
/************************************************************************************/
/*

 TITLE
 -----
 pm_external_validation.mac.sas

 SAS- Makro zur externen Validierung eines logistischen Regressionsmodells anhand
 eines zweiten Datensatzes	

 BACKGROUND
 ----------
 Bei der Externen Validierung wird ein entwickeltes Prognosemodell auf einen zweiten, 
 unabhngigen Datensatz angewandt, wobei die erzielten Prognosegten anschlieend zur 
 Beurteilung des Modells genutzt werden. Somit erhlt man den geforderten Eindruck des 
 Prognosefehlers, der durch die gleichzeitige Nutzung des Originaldatensatzes fr 
 Modellentwicklung und Prognosegtebestimmung eingetreten ist. Nach Harrell [harrell2001] 
 ist die strengste externe Validierung dann gegeben, wenn das Modell auf Daten angewendet 
 wird, welche in einem anderen Land und zu einem anderen Zeitpunkt erhoben wurden. Dieses 
 Vorgehen wird aber in der Praxis selten angewendet. Altman und Royston [altman2000] 
 unterscheiden hier zwischen einer temporalen Validierung an einem zweiten Datensatz aus 
 der selben Quelle wie der Originaldatensatz und zwischen der externen Validierung an 
 einem Datensatz aus vllig anderer Quelle. Beide schreiben dazu, dass die Generalisier-
 barkeit und bertragbarkeit der Ergebnisse eigentlich nur so geprft werden kann. 

 Die externe Validierung ist genau die Validierung, die zur untersuchten Fragestellung 
 fhrt: Wie gut ist ein Modell bei Anwendung auf neue Daten? 
 Dementsprechend ist dieses Vorgehen, wann immer mglich allen anderen Validierungs-
 methoden vorzuziehen. Gleichzeitig untersucht man bei Durchfhrung einer externen 
 Validierung auch die bertragbarkeit der Ergebnisse auf andere Datenstze (z.B. aus 
 anderen Kliniken), die nicht immer gegeben sein muss. 

 Der Hauptnachteil der externen Validierung ist die Notwendigkeit, einen zweiten, 
 strukturgleichen Datensatz zur Verfgung haben zu mssen. Sehr oft wird schon aus Kosten- 
 und Aufwandgrnden dieser zweite Datensatz nicht zu erhalten sein. Gegenber dem spteren 
 Anwendungsfall muss hier auch die Zielgre jeweils bekannt sein, um die Prognosegte 
 anhand dieses zweiten Datensatzes bestimmen zu knnen. 


 DESCRIPTION
 -----------
 Im Makro PM_EXTERNAL_VALIDATION.MAC.SAS wird hauptschlich das Makro fr die logistische 
 Regression (PM_LOGREG.MAC.SAS) sowie das Makro fr die ROC-Analyse (PM_ROC.MAC.SAS) 
 aufgerufen und zur Bestimmung der Prognosegte eines festen Modells auf Basis neuer Daten
 genutzt. Folgendes Vorgehen wird Makro-intern durchgefhrt:
 - Das Modell mit seinen Regressionskoeffizienten wird anhand des Originaldatensatzes 
   (logistische Regression ohne Variablenselektion) entwickelt. Anschlieend wird die 
   Prognosegte nach Reklassifikation am selben Datensatz bestimmt. Diese wird im Output 
   des Makros ausgegeben und muss nicht gesondert mit dem ROC-Makro erzeugt werden. 
 - Die Regressionskoeffizienten werden anschlieend mit INEST= und MAXITER=0 einer zweiten 
   logistischen Regression bergeben, die dieses Modell auf den Validierungsdatensatz 
   anwendet und die Ergebnisdateien fr die ROC-Analyse erzeugt. Nach der folgenden 
   ROC-Analyse werden die Prognosegten denen des Originaldatensatzes im Output 
   gegenbergestellt, wobei zustzlich die Differenz und die prozentuale Abweichung vom 
   Ausgangswert ausgegeben werden. 
 - Damit das Makro korrekt ablaufen kann mssen die beiden Datenstze strukturgleich sein,
   d.h. die Variablennamen und deren Formatierung mssen bereinstimmen. 
 - Im Falle einer Datenseparation wird bei der Analyse des Originaldatensatzes das 
   FL-Makro von Heinze [heinze2002] eingesetzt, um die Regressionskoeffizienten zu 
   bestimmen. 
 - Fr die Berechnung der validierten Prognosegten macht eine Unterscheidung nach 
   temporaler und externe Validierung (s. Altmann und Royston [altman2000]) hierbei keinen 
   Sinn, da diese sich nur inhaltlich durch die Quelle des Validierungsdatensatzes, jedoch 
   nicht durch unterschiedliche Auswertungen unterscheiden. Deshalb wird hier im Makro 
   nicht zwischen beiden Versionen unterschieden.  


 LITERATURE
 ----------
 - SAS Online Doc zu der Prozedur PROC LOGISTIC
 - Altman DG, Royston P (2000):  What do we mean by validating a prognostic model?
                Statistics in Medicine, 19, 453 - 473
 - Harrell, FE (2001): Regression Modeling Strategies. Springer, New York
 - Heinze G, Schemper, M (2002): A Solution to the Problem of Separation in logistic 
				regression. Statistics in Medicine, 21, 2409 - 2419.
 - Ziegler C (2003): Ein SAS-Makropaket zur Entwicklung und Validierung von 
				Prognosemodellen auf Basis der logistischen Regression. 
				Diplomarbeit, FH Ulm 

 SYNTAX
 ------
 [Makroaufruf und Beschreibung der zu bergebenden Parameter]

 %PM_EXTERNAL_VALIDATION(	
   *data 	       = ,				Datensatz
   *resp_var	   = , 				Zielgre
	event		   = 1,				Event- Kategorie der binren Zielgre (0/1)
   *xvar		   = ,				stetige Variable(n)
   *cvar		   = ,				kategorielle Variable(n)
	ref		       = ,				Referenzkategorien fr kategorische Variablen
	miss		   = 0,				Missing Values ausschliessen (ja/ nein)
   *validate_data  = ,				Datensatz an dem das Modell validiert werden soll
   *macro_path     = ,				Pfad zur Makrosammlung
	binwidth	   = 0				Bin-Breite
  );

  Mit (*) gekennzeichnete Parameter sind beim Makroaufruf zwingend erforderlich.
  Bereits gesetzte Parameter stellen die jeweilige Standardeinstellung dar.

   data:		    Parameter zur Angabe des zu analysierenden Datensatzes. Wurde 
					beispielsweise vor dem Aufruf des Makros ein Datensatz mit dem 
					Namen "dat300" angelegt, lautet der korrekte Aufruf: "data=dat300". 
					Zustzlich kann auch die dazugehrige SAS-Library bergeben werden 
					(Beispiel: "data=work.dat300"). Die bergabe dieses Parameters ist 
					zwingend erforderlich.
 
   resp_var:		"resp_var" steht fr "Response Variable", also fr die Zielgre. 
					Dieser bergabeparameter ist ebenfalls zwingend erforderlich. 
					Ferner wird eine binre Datenstruktur (0/1) der Zielgre bentigt. 
					Die Zielgre muss mit "0" und "1" kodiert sein. Sollte dies nicht 
					der Fall, ist eine vorherige Umkodierung erforderlich. Heit 
					beispielsweise die Zielgre "tod" lautet der Aufruf wie folgt: 
					"resp_var=tod".

   event:			Je nach dem welche Kategorie der binren Zielgre untersucht 
					werden soll, kann diese mit Hilfe dieses Parameters bergeben werden. 
					Soll beispielsweise die Zielgre "tod" analysiert werden und steht 
					"1" fr tot sowie "0" fr lebendig, lautet der korrekte Aufruf 
					"event=1". Dies ist zugleich die Standardeinstellung, da blicherweise
					"1" fr das Eintreten eines bestimmten Ereignisses steht.

   xvar:			Dieser Parameter ist zur Angabe der stetigen Variablen 
					(Einflussgren). Allgemein gesagt sollten hier alle Variablen 
					genannt werden, die nicht in das CLASS-Statement von PROC LOGISTIC 
					aufgenommen werden sollen, also beispielsweise auch quasi-stetige 
					Variablen. Werden mehrere stetige oder quasi-stetige Variablen 
					bergeben, so mssen diese durch ein Blank (Leerzeichen) getrennt 
					werden. Beispiel: "xvar=alter bmi".

   cvar:			"cvar" steht fr "categorical variables" und ist fr die bergabe 
					aller kategorischen sowie aller dichotomen Variablen verantwortlich. 
					Diese werden innerhalb der LOGISTIC-Prozedur in das CLASS-Statement 
					aufgenommen und werden automatisch Dummy-kodiert. Werden mehrere 
					kategorische Variablen bergeben, mssen diese getrennt durch ein 
					Leerzeichen (Blank) bergeben werden (Beispiel: "cvar=diagnose msnr").
					Wurden dem Parameter "cvar" dichotome Variablen bergeben, so werden 
					diese zwar in das CLASS-Statement von PROC LOGISTIC aufgenommen, 
					jedoch keine Dummy-Kodierung durchgefhrt, da dichotome Variablen nur 
					zwei Klassen besitzen.

   ref:				Mittels dieses Parameters, der fr "reference" steht, ist es 
					mglich, pro bergebener kategorischer Variable eine sogenannte 
					Referenzkategorie anzugeben. Werden keine Referenzkategorien 
					angegeben, wird der Standard bei jeder Variablen innerhalb des 
					CLASS-Statements von PROC LOGISTIC auf REF=FIRST gesetzt. Dies 
					bedeutet, dass die erste (sortierte) Kategorie jeder bergebenen 
					Variablen als Referenzkategorie gewhlt wird. Werden 
					Referenzkategorien bergeben, so muss deren Anzahl mit der Anzahl 
					der bergebenen kategorischen Variablen bereinstimmen. SAS bietet 
					standardmig die Kategorien FIRST und LAST an. Zustzlich ist es 
					mit Hilfe dieses Parameters mglich, beliebige Kategorien anzugeben. 
					Das Trennungszeichen ist der Stern (*). Eigens gewhlte 
					Referenzkategorien mssen zustzlich in Hochkommata eingeschlossen 
					werden. Die SAS-Standardkategorien FIRST / LAST nicht. Die ntige 
					Form lautet beispielsweise: "ref='1'*FIRST*'4'". Der dazugehrige 
					"cvar"-Parameter: "cvar=icdklass3 msnr_num beginn_b_num". Von der 
					bergebenen Variablen "icdklass" wird die Kategorie "1" als 
					Referenzkategorie gewhlt. Von der Variablen "msnr_num" die erste 
					(sortierte), unabhngig was diese fr einen Wert besitzt und von der 
					Variablen "beginn_b_num" die Kategorie "4". 

   miss:			Dieser Parameter legt fest, ob Beobachtungen mit fehlenden Werten 
					(missing values) in den unter "xvar", "cvar" und "resp_var" 
					bergebenen Variablen vor der Analyse aus dem Datensatz eliminiert 
					werden sollen (0) oder nicht (1). Der Standardwert dieses Parameters 
					ist 0 (Complete Case-Analyse). Hierbei ist zu beachten, dass fehlende 
					Werte im Datensatz durch einen Punkt (.) reprsentiert sein mssen.

   validate_data:	Dieser Parameter dient zur bergabe eines zweiten Datensatzes,
					an dem das entwickelte Modell (Datensatz #1, Parameter 'data=') 
					validiert wird. Diesem Parameter mu ein gleich strukturierter
					Datensatz wie der unter dem Parameter 'data=' angegebene bergeben
					werden. Diese Konvention wird beispielsweise mittels vorherigem 
					Datasplitting erfllt, wobei ein Datensatz auf zwei Datenstze
					aufgeteilt wird. Wurde beispielsweise vor dem Aufruf des Makros ein 
					Datensatz mit dem Namen "vali_data" angelegt, lautet der korrekte 
					Aufruf: 'validate_data= vali_data'. Zustzlich kann auch die 
					dazugehrige SAS- Library bergeben werden 
					(Beispiel: validate_data= work.vali_data). Die bergabe dieses 
					Parameters ist zwingend erforderlich.
					
   macro_path: 		Aufgrund der Tatsache, dass innerhalb des Makros diverse 
					Untermakros aufgerufen werden, ist es zwingend ntig, den Pfad 
					zu dem Verzeichnis, wo die komplette Makrosammlung abgelegt ist,
					anzugeben. Der Pfad selber muss in Hochkommata eingeschlossen 
					sein. Beispiel: 'f:\makros'. 

   binwidth:		Der Parameter "binwidth" gibt die Bin-Breite fr die Schtzung der 
					Assoziationsstatistik an. Der Wert muss dabei zwischen 0 und kleiner
					1 liegen. SAS verwendet als	Standard 0.002, ab Version 9.4 wird bei 
					binrem Endpunkt und weniger als 5 Millionen Beobachtungen jedoch 
					auf Binnen verzichtet. Der Parameter erleichtert daher den Vergleich
					der Ergebnisse unterschiedlicher SAS Versionen.

  VALIDATION
  ----------

  Validiert durch: Stephanie Heiss
 				   University of Ulm 
                   Department of Biometry and Medical Documentation
                   89075 Ulm
  
  Validiert am:    29. September 2003

  Validierungsprogramm::      PM_EXTERNAL_VALIDATION_VAL.SAS
  Validierungsdokumentation:  PM_EXTERNAL_VALIDATION_VAL.DOC
   
  Validierungsvorgehen:
  - Das Testen der Makros wird durch eine von der Programmierung unabhngige 
    Person durchgefhrt.
  - Fr die Durchfhrung der Validierung wird auf einen in der Literatur 
    genutzten Datensatz zurckgegriffen:  LBW (Low Birth Weight), beschrieben
    in Hosmer, Lemeshow: Applied logistic regression [hosmer2000]
  - Der Datensatz wird fr verschiedene Testlufe verndert bzw. erweitert, 
    damit "die Testdaten neben den normalen Angaben auch Grenzwerte, ungltige
    Angaben, Fehlerbedingungen und Sonderflle beinhalten." [beiwanger2003]
  - Fehlerkontrolle:
    Hierbei werden nur die im LOG-Fenster auftretenden Fehler bzw. Warnungen 
    und offensichtlichen Fehler im OUTPUT betrachtet. Fr jeden Parameter des 
    Makros werden mehrere Durchlufe mit verschiedenen Wertangaben durchgefhrt, 
    wobei auch ungltige Werte angegeben werden, um das Verhalten des Makros
    bei fehlerhaften Aufrufen zu untersuchen. Gefundene Fehler werden 
    dokumentiert und dem Programmierer zur Verfgung gestellt. Nach der Korrektur 
    aller Fehler werden die Testlufe wiederholt, um sicherzustellen, dass 
    wirklich alle Fehler abgefangen wurden und dass keine neuen Fehler entstanden 
    sind.
  - Ergebniskontrolle:
    Um die Richtigkeit der Ergebnisse zu berprfen, werden soweit mglich, die
    Rechenbeispiele und bungsaufgaben aus Hosmer/Lemeshow durch vergleichbare
    Makroaufrufe nachgerechnet. Die Ergebnisse werden denen aus dem Begleitbuch 
    Hosmer/Lemeshow/Cook [hosmer2001] gegenbergestellt und direkt verglichen.
  - Alle Validierungen werden dokumentiert.

  Literatur: 
  - Beiwanger S (2003): Validierung von SAS-Makros. Diplomarbeit, FH Ulm
  - Hosmer DW, Lemeshow S (2000): Applied logistic regression 2nd Ed. 
           J. Wiley & Sons, New York
  - Hosmer DW, Lemeshow S, Cook ED (2001): Solutions manual to accompany
           applied logistic regression 2nd. Ed. J. Wiley & Sons, New York



  AUTHOR
  ------
  R. Muche / C. Ring / C. Ziegler
  University of Ulm
  Department of Biometry and Medical Documentation
  89075 Ulm

  The author does not overtake any responsibility for the correctness
  of results. It must not be modified without removing the heading lines 
  containing the authors name.

----------------------------------------------------------------------------------*/

%MACRO PM_EXTERNAL_VALIDATION (	data 	       = ,
							 	resp_var	   = , 
					         	event		   = 1,
					  		 	xvar		   = ,
				      		 	cvar		   = ,
					   		 	ref		   	   = ,
					   		 	miss		   = 0,
							 	validate_data  = ,
					   		 	macro_path     = ,
								binwidth	   = 0
			                 );

/*Prfungen / Referenzierungen / Standardbelegungen*/

/*Wichtigster Parameter 'macro_path' berprfen*/
/*bergabe erfolgt?*/

%IF &macro_path= %THEN %DO;

	%WINDOW error color=grey                                                         
      #5  @28 'Fehler!' attr=highlight                              
              color=blue                                                
      #7  @28 "Dem zwingend erforderlichen Parameter 'macro_path' wurde kein Wert" 
	  #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
      #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';            
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Hilfsprogramm, welches berprft, ob dem Parameter "macro_path" ein korrekter Pfad*/
/*bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/

/*Anzahl der Zeichen im String 'macro_path' bestimmen:*/

%LET path_length=%LENGTH(&macro_path);

%LET infile=&macro_path;

%IF %bquote(&infile) NE %THEN %DO;
     %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
      %IF %bquote(&char1) ne %STR(%')
         %THEN %LET command='NOK';
         %ELSE %LET command='OK';

	 %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
      %IF %bquote(&char2) ne %STR(%')
         %THEN %LET command2='NOK';
         %ELSE %LET command2='OK';
%END;

/*falls die Syntax nicht stimmt, Anzeige eines Hinweisfensters*/

%IF &command='NOK' OR &command2='NOK' %THEN %DO;

	%WINDOW error color=grey                
	 #5  @28 'Fehler!' attr=highlight                          
             color=blue                                          
     #7  @28 "Dem Parameter 'macro_path' wurde ein syntaktisch falscher Wert" 
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Test, ob das angegebene Verzeichnis existiert*/

%LET dir=&macro_path;

%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
       %PUT Directory does not exist;
%END;
%ELSE %DO;
       %PUT Directory exists;
       %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;

	%WINDOW error color=grey                                                        
	 #5  @28 'Fehler!' attr=highlight                                     
             color=blue                                                        
	 #7  @28 "Das unter dem Parameter 'macro_path' angegebene Verzeichnis existiert"
	 #8  @28 "nicht- Bentigte Untermakros knnen nicht referenziert werden"
	 #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';    
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Falls bestimmte Parameter ohne Wert bergeben wurden, diese mit Standard belegen*/

%IF &event= %THEN %DO;
	%LET event=1;
%END;

%IF &miss= %THEN %DO;
	%LET miss=0;
%END;

/*Externe Makros, die innerhalb des Makros 'PM_EXTERNAL_VALIDATION.MAC.SAS'*/
/*verwendet werden (Untermakros) einbinden (%include)*/

FILENAME y &macro_path;

%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('pm_logreg.mac.sas');
%INCLUDE y ('pm_roc.mac.sas');
%INCLUDE y ('numchar.mac.sas');
%INCLUDE y ('exist.mac.sas');
%INCLUDE y ('exist_var.mac.sas');
%INCLUDE y ('ref.mac.sas');

/*globale Variable des Untermakros 'numberobs.mac.sas'*/

%GLOBAL nnnn1;

/*Test, ob bei bestimmten Parametern anstatt numerischen Werten*/
/*character- Werte bergeben wurden*/

/*berprfung erfolgt mit Hilfe des Untermakros 'numchar.mac.sas'*/

%NUMCHAR(parameterliste=&event &miss &binwidth);

%IF &numcharok='NOK' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error8.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfen, ob der angegebene Datensatz (Parameter 'data=') auch wirklich existiert*/
/*(mit Hilfe des Hilfs-/Untermakros 'exist.mac.sas')*/

%EXIST(dsn=&data);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error4.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Datenquelle (Parameter 'data=' UND response-Variable*/
/*(Zielgre / Parameter 'resp_var=') angegeben wurde*/

%IF &data= or &resp_var= %THEN %DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

	%INCLUDE y ('error1.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob berhaupt Variablen (Parameter 'xvar=' / 'cvar=') angegeben wurden*/

%IF &xvar= and &cvar= %THEN %DO;

/*wenn keine Variablen angegeben wurden, Anzeige eines Fensters mit*/ 
/*Fehlermeldung/ Hinweis*/

	%INCLUDE y ('error2.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Prfung, ob alle bergebenen Variablen numerisch sind*/

/*Erzeugung eines temporren Zwischendatensatzes, welcher lediglich die*/
/*bergebenen Variablen enthlt*/

DATA test3;
	SET &data;
	KEEP &xvar &resp_var &cvar;
RUN;

/*mittels der Prozedur PROC CONTENTS kann der Typ einer Variablen bestimmt werden*/

/*Unterdrckung der Ausgabe von PROC CONTENTS*/

PROC CONTENTS DATA=test3 NOPRINT OUT=testcontent;
RUN;

/*lediglich die Variable 'type' der Ergebnisdatei behalten,*/
/*da nur diese von Interesse ist*/

DATA testcontent;
	SET testcontent;
	KEEP type;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=testcontent);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test4=&nnnn1;

/*Prfung ob Variablen vom Typ 'character' vorhanden sind*/

DATA test4;
	SET testcontent;
	IF type=2 THEN DELETE;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test4);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test5=&nnnn1;

/*falls Variablen bergeben wurden, die nicht numerisch sind --> Fehlermeldung*/

%IF &test4 ne &test5 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error60.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE test4 testcontent test3;
RUN;
QUIT;

/*berprfung, ob alle bergebenen Variablen im bergebenen Datensatz*/
/*(Parameter 'data' vorhanden sind)*/

/*Zusammenfassung aller Variablen in einer neuen Makro- Variablen*/

%LET all_variables= &resp_var &xvar &cvar;

/*Prfung erfolgt mit Hilfe des Untermakros 'exist_var.mac.sas'*/

%EXIST_VAR(dsn=&data, var=&all_variables);

/*falls irgendwelche Variablen nicht existieren, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error6.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob die Zielgre (Parameter 'resp_var')*/
/*2 Kategorien (Ausprgungen) besitzt*/

/*missing values ausschliessen*/

DATA freq2;
	SET &data;
	IF &resp_var ne .;
RUN;

/*Anzahl der Ausprgungen mit Hilfe der Prozedur PROC FREQ ermitteln*/

/*Ausgabe unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnis als ODS- Datei zur weiteren Verarbeitung ausgeben*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=freq2;
	TABLES &resp_var;
RUN;

/*ODS- Output schliessen*/

ODS OUTPUT CLOSE;

/*ODS- Ausgabe wieder aktivieren*/

ODS LISTING;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen (= # Ausprgungen der Zielgre)*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq freq2;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

/*falls Zielgre mehr als zwei Ausprgungen besitz--> Fehlermeldung*/

%IF &event_check > 2 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error5.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Parameter 'miss' 0 oder 1 ist*/

%IF &miss ne 1 %THEN %DO;
	%IF &miss ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error7.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'event' 0 oder 1 ist*/

%IF &event ne 1 %THEN %DO;
	%IF &event ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error7.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*Anzahl der angegebenen kategoriellen Variablen ermitteln:*/

%GLOBAL c_count c_var;
%LET c_count=1;
%LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%DO %WHILE(&c_var ne);
     %LET c_count=%EVAL(&c_count+1);
     %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%END;
%LET c_count=%EVAL(&c_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/

%PUT The cvar contains &c_count variables.;

/*Anzahl der Referenzen ermitteln*/

%GLOBAL ref_count var_ref;
%LET ref_count=1;
%LET var_ref=%QSCAN(&ref,&ref_count,*);
%DO %WHILE(&var_ref ne);
     %LET ref_count=%EVAL(&ref_count+1);
     %LET var_ref=%QSCAN(&ref,&ref_count,*);
%END;
%LET ref_count=%EVAL(&ref_count-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The refs contains &ref_count variables.;

%IF &ref_count >0 %THEN %DO;
	%IF &c_count ne &ref_count %THEN %DO;

/*falls Anzahl der kategoriellen Variablen grer Null ist und
/* nicht mit der Anzahl der  angegebenen*/
/*Referenzkategorien bereinstimmt, Anzeige eines Fensters*/
/*mit Fehlermeldung/ Hinweis*/

		%INCLUDE y ('error3.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

   	%END;
%END;

/*berprfung, ob etwaige bergebene Referenzkategorien vorhanden sind bzw.*/
/*korrekt bergeben wurden*/

%IF &cvar ne %THEN %DO;
	%IF &ref_count ne 0 %THEN %DO;

	%REF(data=&data, resp_var=&resp_var, event=&event, cvar=&cvar, ref=&ref);

	%IF &correct='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE  y ('error11.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;

%END;
%END;	

/*2. Datensatz 'validate_data' berprfen*/

/*berprfung,ob Parameter 'validate_data=' angegeben wurde*/

%IF &validate_data= %THEN %DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

	%WINDOW error color=grey                                                        
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Dem Parameter 'validate_data' (Datensatz)"  
		   #8  @28 "wurde kein Wert bergeben!"   
		   #9  @28 "Dieser Parameter ist jedoch zwingend erforderlich."  
           #14 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';         
	%DISPLAY error;
/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfen, ob der angegebene Datensatz auch wirklich existiert*/

%EXIST(dsn=&validate_data);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error49.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob alle bergebenen Variablen im bergebenen Datensatz*/
/*(Parameter 'data' vorhanden sind)*/

/*Zusammenfassung aller Variablen in einer neuen Makro- Variablen*/

%LET all_variables= &resp_var &xvar &cvar;

/*Prfung erfolgt mit Hilfe des Untermakros 'exist_var.mac.sas'*/

%EXIST_VAR(dsn=&validate_data, var=&all_variables);

/*falls irgendwelche Variablen nicht existieren, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE  y ('error50.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob etwaige bergebene Referenzkategorien vorhanden sind bzw.*/
/*korrekt bergeben wurden*/

%IF &cvar ne %THEN %DO;
	%IF &ref_count ne 0 %THEN %DO;

	%REF(data=&validate_data, resp_var=&resp_var, event=&event, cvar=&cvar, ref=&ref);

	%IF &correct='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error51.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;

%END;
%END;	

/*berprfen ob eine Variable nach Entfernung aller Beobachtungen mit missing*/
/*values nur noch eine Ausprgung besitzt*/

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values*/

DATA work.category_test;
	SET  &data;

	%DO i=1 %TO &count_x;
		%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
		IF &xvar_liste=. THEN DELETE;
	%END;

	%DO i=1 %TO &count;
		%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
		IF &cvar_liste=. THEN DELETE;
	%END;

		IF &resp_var=. THEN DELETE;

RUN;

/*Prfen ob 'resp_var' /'xvar'/ 'cvar' nur 1 Ausprgung besitzen*/

%LET category_test=0;
%LET category_test_fail=;

/*event*/

/*Anzahl der Ausprgungen mittels PROC FREQ bestimmen*/

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &resp_var;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

%IF &event_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &resp_var;

%END;

/*xvar*/

%IF &xvar ne %THEN %DO;

%DO uhgu=1 %TO &count_x;

%LET xvar_liste=%SCAN(&xvar,&uhgu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &xvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET xvar_check=&nnnn1;

%IF &xvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &xvar_liste;

%END;

%END;
%END;

/*cvar*/

%IF &cvar ne %THEN %DO;

%DO uhfu=1 %TO &count;

%LET cvar_liste=%SCAN(&cvar,&uhfu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &cvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET cvar_check=&nnnn1;

%IF &cvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &cvar_liste;

%END;

%END;
%END;

PROC DATASETS NOLIST;
	DELETE category_test;
RUN;
QUIT;

%IF &category_test=1 %THEN %DO;

	DATA category_test;
		var="&category_test_fail";
	RUN;

	PROC PRINT DATA=category_test NOOBS LABEL;
	TITLE1 "Folgende Variablen besitzen nach Entfernung aller";
	TITLE2 "missing values lediglich eine Ausprgung:";
		VAR var;
			LABEL var='Variable';
	RUN;

	PROC DATASETS NOLIST;
		DELETE category_test;
	RUN;
	QUIT;

	%INCLUDE y ('error24.sas');
	%GOTO ende;

%END;

%IF &binwidth <0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error66.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;


%IF &binwidth >=1 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error66.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%IF &binwidth >0.01 %THEN %DO;

/*Anzeige eines Warnhinweises*/

	DATA hint;
		hint="Bei einem Wert von 0 wird kein Binning angewandt. In manchen SAS Versionen wird der Standard 0.002 verwendet.";
	RUN;
	PROC PRINT DATA=hint NOOBS LABEL HEADING=H WIDTH=UNIFORM;
		VAR   hint;
		TITLE1 "Dem Parameter 'binwidth' wurde ein Wert grer 0.01 bergeben.";
		LABEL hint = "Sinnvoll ist ein Wert zwischen 0 und 0.01."; 
	RUN;
%END;

/*ENDE Prfungen / Referenzierungen / Standardbelegungen*/

/*Proc Logistic ber DS1*/

/*Unterdrckt OUTPUT:*/

ODS SELECT NONE;

%PM_LOGREG ( 	data			= &data,
				resp_var		= &resp_var,
				event			= &event,
				xvar			= &xvar,
				cvar			= &cvar,
				ref				= &ref,
				fxvar			= ,
				fcvar			= ,
				miss			= &miss,
				sel				= NONE,
				sel_details		= 1,
				slstay			= 0.05,
				slentry			= 0.05,
				clodds			= 0,
				alpha			= 0.05,
				rsquare			= 1,
				ctable			= 1,
				ctable_pprob	= ,
				out_pred		= work.gof_ori,
				pred			= 1,
				predprobs_art	= INDIVIDUAL,
				roc				= 1,
				out_roc			= work.roc_ori,
				est				= 1,
				out_est			= work.est_ori,
				inest_flag		= 0,
				inest_data		= ,
				boot_data		= ,
				c_wert			= 1,
				c_data			= work.association_ori,
				test			= ,
				mi				= 0,
				out_mi			= work.out_mi_log,
				macro_path		= &macro_path,
				binwidth		= &binwidth
		  );

/*ROC-Analyse (DS1) mit Makro 'PM_ROC.MAC.SAS'*/

%PM_ROC			(
				 data				= &data,
				 resp_var			= &resp_var,
				 event				= &event,
				 data_roc			= work.roc_ori,
				 data_pred			= work.gof_ori,
				 data_asso			= work.association_ori,
				 validation			= NONE,
				 praevalenz			= ,
				 all_cp				= 0,
			     cp					= ,
				 cibinom			= 0,
				 rocdat				= 0,
				 auroc				= 0,
				 printit			= 0,
				 alpha_cl			= 0.95,
				 ext				= tiffp,
				 winkel				= 1,
				 point				= 1,
				 path				= 'c:\temp',
				 mi					= 0,
				 out_mi				= work.out_mi_roc,
				 macro_path			= &macro_path,
				 binwidth			= &binwidth
			    );

DM "clear log;";

/*ROC- / Sonstige Kenngren des Originaldatensatzes zusammenfassen*/

DATA result_roc_ori1;
	SET result;
	RENAME maxyouden 	= maxyouden_ori;
	RENAME _PROB_ 		= prob_ori;
	RENAME _SENSIT_		= sens_ori;
	RENAME spez 		= spez_ori;
	RENAME ppv 			= ppv_ori;
	RENAME npv 			= npv_ori;
	RENAME psep 		= psep_ori;
	dummy				= 1;
RUN;

DATA result_roc_ori2;
	SET index_ori;
	dummy				= 1;
RUN;

/*Cutpoint des Ausgangsdatensatzes zur spteren Verwendung*/
/*in einer Makrovariable zwischenspeichern*/

DATA result_roc_ori1;
	SET result_roc_ori1;
	call symput('prob_original',prob_ori);
RUN;

/*Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE result roc_ori gof_ori association_ori index_ori;
RUN;
QUIT;

/*Aufruf der PROC LOGISTIC ('PM_LOGREG.MAC.SAS') fr DS2 */

%PM_LOGREG ( 	data			= &validate_data,
				resp_var		= &resp_var,
				event			= &event,
				xvar			= &xvar,
				cvar			= &cvar,
				ref				= &ref,
				fxvar			= ,
				fcvar			= ,
				miss			= &miss,
				sel				= NONE,
				sel_details		= 1,
				slstay			= 0.05,
				slentry			= 0.05,
				clodds			= 0,
				alpha			= 0.05,
				rsquare			= 1,
				ctable			= 1,
				ctable_pprob	= ,
				out_pred		= work.out_gof,
				pred			= 1,
				predprobs_art	= INDIVIDUAL,
				roc				= 1,
				out_roc			= work.out_roc,
				est				= 1,
				out_est			= work.out_est,
				inest_flag		= 2,
				inest_data		= work.est_ori,
				boot_data		= ,
				c_wert			= 1,
				c_data			= work.out_association,
				test			= ,
				mi				= 0,
				out_mi			= work.out_mi_log,
				macro_path		= &macro_path,
				binwidth		= &binwidth
		);

/*ROC-Analyse mit Makro 'PM_ROC.MAC.SAS' (DS2) */

%PM_ROC			(
				 data				= &validate_data,
				 resp_var			= &resp_var,
				 event				= &event,
				 data_roc			= work.out_roc,
				 data_pred			= work.out_gof,
				 data_asso			= work.out_association,
				 validation			= NONE,
				 praevalenz			= ,
				 all_cp				= 0,
			     cp					= &prob_original,
				 cibinom			= 0,
				 rocdat				= 0,
				 auroc				= 0,
				 printit			= 0,
				 alpha_cl			= 0.95,
				 ext				= tiffp,
				 winkel				= 1,
				 point				= 1,
				 path				= 'c:\temp',
				 mi					= 0,
				 out_mi				= work.out_mi_roc,
				 macro_path			= &macro_path,
				 binwidth			= &binwidth
			    );

DM "clear log;";

ODS SELECT ALL;

/*ROC- / Sonstige Kenngren zusammenfassen*/

DATA result_roc_ext1;
	SET result;
	RENAME maxyouden 	= maxyouden_ext;
	RENAME _PROB_ 		= prob_ext;
	RENAME _SENSIT_		= sens_ext;
	RENAME spez 		= spez_ext;
	RENAME ppv 			= ppv_ext;
	RENAME npv 			= npv_ext;
	RENAME psep 		= psep_ext;
	dummy				= 1;
RUN;

DATA result_roc_ext2;
	SET index_ori;
	RENAME	brier_ori		= brier_ext;
	RENAME  max_resca_ori	= max_resca_ext;
	RENAME  r_square_ori	= rsquare_ext;
	RENAME	auc_ori			= auc_ext;
	RENAME  som_ori			= som_ext;
	dummy					= 1;
RUN;

DATA result_roc_ext3 (DROP=_POS_ _NEG_ _FALNEG_ _FALPOS_);
	SET result2;
	RENAME	_SENSIT_		= sensit_ext_cp;
	RENAME	spez			= spez_ext_cp;			
	RENAME  youden 			= youden_ext_cp;
	RENAME  ppv				= ppv_ext_cp;
	RENAME	npv				= npv_ext_cp;
	RENAME  psep			= psep_ext_cp;
	dummy					= 1;
RUN;

/*Ergebnisdateien zusammenfhren:*/
/*1. Cutpoint-unabhngige Prognosegtemae:*/

DATA gesamt1;
	MERGE result_roc_ext2 result_roc_ori2;
	BY dummy;
RUN;

/*2. Cutpoint-abhngige Prognosegtemae:*/

DATA gesamt2;
	MERGE result_roc_ext1 result_roc_ori1;
	BY dummy;
RUN;

/*3. Prognosegtemae beim Cutpoint des Ausgangsmodells:*/

DATA gesamt3;
	MERGE result_roc_ext3 result_roc_ori1;
	BY dummy;
RUN;

/*Ergebnisdatei gesamt1 fr schneren Output aufsplitten*/

/*AUC*/

DATA auc (KEEP=auc_ori auc_ext auc_diff auc_percent);
	SET gesamt1;
		auc_diff=auc_ori-auc_ext;
		auc_percent=(auc_ori-auc_ext)/auc_ori;
RUN;

/*SomersD*/

DATA som (KEEP=som_ori som_ext som_diff som_percent);
	SET gesamt1;
			som_diff=som_ori-som_ext;
			som_percent=(som_ori-som_ext)/som_ori;
RUN;

/*Brier Score*/

DATA brier (KEEP=brier_ori brier_ext brier_diff brier_percent);
	SET gesamt1;
			brier_diff=brier_ori-brier_ext;
			brier_percent=(brier_ori-brier_ext)/brier_ori;
RUN;

/*R-Square*/

DATA rsquare (KEEP=r_square_ori rsquare_ext rsquare_diff rsquare_percent);
	SET gesamt1;
			rsquare_diff=r_square_ori-rsquare_ext;
			rsquare_percent=(r_square_ori-rsquare_ext)/r_square_ori;
RUN;

/*MaxRescaled R-Square*/

DATA maxresca (KEEP=max_resca_ori max_resca_ext max_resca_diff max_resca_percent);
	SET gesamt1;
			max_resca_diff=max_resca_ori-max_resca_ext;
			max_resca_percent=(max_resca_ori-max_resca_ext)/max_resca_ori;
RUN;

/*Ergebnisse (gesamt1)ausgeben*/

/*-------------------------------------------------------------------------------*/

/*Standardoutput ausgeben*/

%LET datensatz1=&data;
%LET datensatz2=&validate_data;

/*Fallzahl bestimmen:*/

%IF &miss=1 %THEN %DO;

%NUMBEROBS(datensatz=&data);

%LET fallzahl1=&nnnn1;

%NUMBEROBS(datensatz=&validate_data);

%LET fallzahl2=&nnnn1;

%END;

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls miss=0 angegeben wurde. DS1*/

DATA work.fallzahl1;
	SET  &data;
	%DO i=1 %TO &count_x;
		%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
		IF &xvar_liste=. THEN DELETE;
	%END;

	%DO i=1 %TO &count;
		%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
		IF &cvar_liste=. THEN DELETE;
	%END;

		IF &resp_var=. THEN DELETE;

RUN;

%LET fallzahldata1=fallzahl1;

%NUMBEROBS(datensatz=&fallzahldata1);

%LET fallzahl1=&nnnn1;

/*Elimination aller missing values falls miss=0 angegeben wurde. DS2*/

DATA work.fallzahl2;
	SET  &validate_data;
	%DO i=1 %TO &count_x;
		%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
		IF &xvar_liste=. THEN DELETE;
	%END;

	%DO i=1 %TO &count;
		%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
		IF &cvar_liste=. THEN DELETE;
	%END;

		IF &resp_var=. THEN DELETE;

RUN;

%LET fallzahldata2=fallzahl2;

%NUMBEROBS(datensatz=&fallzahldata2);

%LET fallzahl2=&nnnn1;

%END;

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierte Datenstze: &datensatz1 / &datensatz2       ";
	TITLE7 "Fallzahl Dataset1:      &fallzahl1       ";
	TITLE8 "Fallzahl Dataset2:      &fallzahl2       ";
	TITLE9 "Datum:                  &sysday &sysdate ";
	TITLE10 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

PROC DATASETS NOLIST;
	DELETE standardoutput fallzahl1 fallzahl2;
RUN;
QUIT;

/*Ende Standardoutput*/
/*-------------------------------------------------------------------------------*/

PROC PRINT DATA= auc NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR auc_ori auc_ext auc_diff auc_percent; 
    TITLE1 "Externe Validierung, Zusammenfassung:";
	TITLE3 "O=Original (&data) / EXT=Extern validiert (&validate_data)";
	TITLE4 'D=Differenz(Original-EXT)';
	TITLE5 'Korrektur der cutpoint-unabhngigen Prognosegtemae';
	LABEL 	auc_ori			='AUC O' 
			auc_ext 		='AUC EXT'
			auc_diff 		='AUC D'
			auc_percent		='%';
RUN;

PROC PRINT DATA= som NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR som_ori som_ext som_diff som_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	som_ori 		='SomersD O'
			som_ext 		='SomersD EXT'
			som_diff 		='SomersD D'
			som_percent		='%';
RUN;

PROC PRINT DATA= brier NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  brier_ori brier_ext brier_diff brier_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	brier_ori 		='Brier Score O'
			brier_ext 		='Brier Score EXT'
			brier_diff 		='Brier Score D'
			brier_percent	='%';
RUN;

PROC PRINT DATA= rsquare NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR r_square_ori rsquare_ext rsquare_diff rsquare_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	r_square_ori 		='R-Square O'
			rsquare_ext 		='R-Square EXT'
			rsquare_diff		='R-Square D'
			rsquare_percent		='%';
RUN;

PROC PRINT DATA= maxresca NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR max_resca_ori max_resca_ext max_resca_diff max_resca_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	max_resca_ori 		='Max-Rescaled R-Square O'
			max_resca_ext 		='Max-Rescaled R-Square EXT'
			max_resca_diff 		='Max-Rescaled R-Square D' 
			max_resca_percent	='%';
RUN;

/*Ergebnisdatei gesamt3 fr schneren Output aufsplitten*/

/*Sensitivitt*/

DATA sens (KEEP=sens_ori sensit_ext_cp sens_diff sens_percent);
	SET gesamt3;
		sens_diff=sens_ori-sensit_ext_cp;
		sens_percent=(sens_ori-sensit_ext_cp)/sens_ori;
RUN;

/*Spezifitt*/

DATA spez (KEEP=spez_ori spez_ext_cp spez_diff spez_percent);
	SET gesamt3;
		spez_diff=spez_ori-spez_ext_cp;
		spez_percent=(spez_ori-spez_ext_cp)/spez_ori;
RUN;

/*YoudenIndex*/

DATA youden (KEEP=maxyouden_ori youden_ext_cp youden_diff youden_percent);
	SET gesamt3;
		youden_diff=maxyouden_ori-youden_ext_cp;
		youden_percent=(maxyouden_ori-youden_ext_cp)/maxyouden_ori;
RUN;

/*PPV*/

DATA ppv (KEEP=ppv_ori ppv_ext_cp ppv_diff ppv_percent);
	SET gesamt3;
		ppv_diff=ppv_ori-ppv_ext_cp;
		ppv_percent=(ppv_ori-ppv_ext_cp)/ppv_ori;
RUN;

/*NPV*/

DATA npv (KEEP=npv_ori npv_ext_cp npv_diff npv_percent);
	SET gesamt3;
		npv_diff=npv_ori-npv_ext_cp;
		npv_percent=(npv_ori-npv_ext_cp)/npv_ori;
RUN;

/*PSEP*/

DATA psep (KEEP=psep_ori psep_ext_cp psep_diff psep_percent);
	SET gesamt3;
		psep_diff=psep_ori-psep_ext_cp;
		psep_percent=(psep_ori-psep_ext_cp)/psep_ori;
RUN;

/*Ergebnisse (gesamt3) ausgeben*/

PROC PRINT DATA= sens NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR sens_ori sensit_ext_cp sens_diff sens_percent; 
	TITLE1;
	TITLE2;
	TITLE3 'Korrektur der cutpoint-abhngigen Prognosegtemae';
	TITLE4 "(bei Cutpoint des Ausgangsmodells (cp=&prob_original)";
	LABEL 	sens_ori		='Sensitivitt O' 
			sensit_ext_cp	='Sensitivitt EXT'
			sens_diff 		='Sensitivitt D'
			sens_percent	='%';
RUN;

PROC PRINT DATA= spez NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR spez_ori spez_ext_cp spez_diff spez_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	spez_ori 		='Spezifitt O'
			spez_ext_cp 	='Spezifitt EXT'
			spez_diff 		='Spezifitt D' 
			spez_percent	='%';
RUN;

PROC PRINT DATA= youden NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR maxyouden_ori youden_ext_cp youden_diff youden_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	maxyouden_ori 	='Max. YoudenIndex O'
			youden_ext_cp	='YoudenIndex EXT'
			youden_diff 	='YoudenIndex D' 
			youden_percent	='%';
RUN;

PROC PRINT DATA= ppv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR ppv_ori ppv_ext_cp ppv_diff ppv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	ppv_ori			='Positive Korrektheit O'
			ppv_ext_cp 	    ='Positive Korrektheit EXT'
			ppv_diff 		='Positive Korrektheit D' 
			ppv_percent		='%';
RUN;

PROC PRINT DATA= npv NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR  npv_ori npv_ext_cp npv_diff npv_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL	npv_ori 		='Negative Korrektheit O'
			npv_ext_cp 	    ='Negative Korrektheit EXT'
			npv_diff 		='Negative Korrektheit D'
			npv_percent		='%';
RUN;

PROC PRINT DATA= psep NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR psep_ori psep_ext_cp psep_diff psep_percent;
	TITLE1;
	TITLE2;
	TITLE3;
	TITLE4;
	LABEL 	psep_ori		='Separation Index PSEP O'
			psep_ext_cp 	='Separation Index PSEP EXT'
			psep_diff 		='Separation Index PSEP D'
			psep_percent	='%';
RUN;
		
/*Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE result2 result index_ori est_ori gesamt2
		   auc som maxresca rsquare  brier out_association out_est out_gof
		   out_roc result_roc_ext1 result_roc_ext2 result_roc_ext3
		   result_roc_ori1 result_roc_ori2 
		   sens spez youden ppv npv psep gesamt3 gesamt1;
RUN;
QUIT;

%ende:

%MEND PM_EXTERNAL_VALIDATION;
