/************************************************************************************/
/*						        													*/
/*     NAME: pm_gof.mac.sas	   			      										*/
/*    TITLE: Analyse der Gte eines logistischen Regressionsmodells mittels 		*/
/*			 globalen Goodness-Of-Fit-Tests											*/
/*     KEYS: goodness-of-fit, logistic regression, sparse data, 				    */
/*			 hosmer & lemeshow-test, standard pearson-test, standard deviance-test	*/
/*			 osius-test, mccullagh-test, farrington-test, information matrix (im)   */
/*			 -test, residual sum of squares (rss)- test								*/
/*  VERSION: SAS 8.2												      			*/
/*  PRODUCT: BASE, STAT									        					*/
/*    PROCS: LOGISTIC, FREQ										        			*/
/*															         				*/
/*																					*/
/*  Verzeichnis/Dateiname:			F:\Prognose\pm_gof.mac.sas	    		    	*/
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*  Programmierer:					Christoph Ziegler								*/
/*  Datum der Ersterstellung:		1/2003											*/
/*  Aktuelle Makro-Version:			12/2003											*/
/*																					*/
/*  Kurzbeschreibung:				Makro zur Analyse der Gte eines logistischen 	*/
/*									Regressionsmodells mittels globalen				*/
/*									Goodness-Of-Fit-Tests							*/
/*																					*/
/*  Verwendete externe Untermakros:	numberobs.mac.sas 								*/
/*									goflogit_kuss.mac.sas							*/
/*									dummy_friendly.mac.sas							*/
/*									gof_pulkstenis.mac.sas							*/
/*									fl_heinze.mac.sas								*/
/*									numchar.mac.sas									*/
/*									exist.mac.sas									*/
/*									exist_var.mac.sas								*/
/*									ref.mac.sas										*/
/*									error1.sas	error2.sas	error3.sas	error4.sas	*/
/*									error5.sas	error6.sas	error7.sas	error8.sas	*/
/*									error11.sas	error24.sas error48.sas	error60.sas	*/
/*																					*/
/*  Bentigte Dateien:				Datensatz (Parameter 'data=')			   		*/
/*															           				*/
/*																					*/
/*  Erzeugte Dateien:				-----										 	*/
/*  									            								*/
/*  Programmnderungen																*/
/*  und -ergnzungen 																*/
/*  (Name, Datum, Beschreibung):											 		*/
/*  																				*/
/*        																			*/
/************************************************************************************/
/*

 TITLE
 -----
 pm_gof.mac.sas

 SAS- Makro zur Analyse der Gte eines logistischen Regressionsmodells 
 mittels globalen Goodness-Of-Fit-Tests.			


 BACKGROUND
 ----------
 Nach einer Modellentwicklung soll anschliessend nun untersucht werden, wie gut das so 
 entstandene Modell zu den beobachteten Daten passt. Die Idee, die dahinter steckt ist 
 wie bei der Untersuchung einflussreicher Beobachtungen der Vergleich der beobachteten 
 Werte y mit den aus dem Modell geschtzten Werten. Dabei interessiert hier nicht die 
 einzelne Beobachtung, sondern es soll ein globales Anpassungsma (Goodness-of-Fit - 
 Statistik) zur Beurteilung hergenommen werden. 
 Hosmer und Lemeshow [hosmer2000] definieren ein passendes Modell so, dass 1. die Abstnde 
 der Modellvorhersage von den Beobachtungen gering sind und dass 2. die Verteilung dieser 
 Abstnde unsystematisch und klein im Vergleich zur Fehlerstruktur des Modells ist. Im 
 wesentlichen wird in der Praxis auf der Basis von "overall-measures" (globale Anpassungs-
 mae) ein statistischer Test mit der Hypothese, dass es kein Anpassungsproblem des Modells 
 an die Daten gibt, angepasst. Bei einem signifikanten Testergebnis muss man somit von 
 einem Anpassungsproblem (Lack of Fit) ausgehen. Von der testtheoretischen Seite her gibt 
 es aber das Dilemma [kuss2002], dass ein nicht signifikantes Testresultat nicht angibt, 
 dass das Modell passt, sondern dass der Lack-of-fit u.U. nicht gro genug war, das Modell 
 abzulehnen. Deshalb sollten die Testergebnisse eher mit Vorsicht zur Kenntnis genommen 
 werden.

 Anpassungsgte / Goodness-of-fit
 Speziell bei der logistischen Regression sind die blichen Verfahren fr den Anpassungs-
 test von Regressionsmodellen, die u.a. auf den Pearson-Residuen bzw. Devianzen basieren, 
 nicht akzeptabel. Beide Teststatistiken sind asymptotisch Chi-Quadrat-verteilt, wenn die 
 Anzahl der Beobachtungen pro Merkmalskombination ber alle im Modell enthaltene Variablen 
 gro genug ist. Da bei steigender Zahl von Variablen und dabei eventuell stetigen 
 Variablen diese Annahme sicher nicht erfllt ist, kann das Ergebnis dieser zwei Standard-
 tests nicht interpretiert werden, da die Verteilung nicht einer Chi-Quadrat-Verteilung 
 folgt.
 Die groe Anzahl an Merkmalskombinationen bzw. die geringe Anzahl Beobachtungen pro 
 Merkmalskombination wird als Sparseness bezeichnet. Es gibt einige Testverfahren, die 
 versuchen, mit dieser sehr oft anzutreffenden Situation umzugehen. Hosmer und Lemeshow 
 [hosmer2000] geben einen speziellen Test an, der auf einer Gruppierung der Beobachtungen 
 anhand der geschtzten Wahrscheinlichkeiten beruht (Hosmer-Lemeshow-Test). Dieser 
 allgemein blich eingesetzte Test fr die Anpassungsuntersuchung hat ebenfalls einige 
 Probleme. Oliver Kuss diskutiert diese und gibt in einer Arbeit ber GOF-Tests fr die 
 logistische Regression [kuss2002] 5 verschiedene Alternativen an: 
 - Osius-Test
 - McCullagh-Test
 - Farrington-Test
 - IM-Test
 - RSS-Test
 Kuss zeigt, dass es fr jede Situation einen besseren Test als den Hosmer-Lemeshow-Test 
 gibt, jedoch leider keinen insgesamt besten Test. Fr die meisten Anwendungssituationen 
 wird nach Simulationsuntersuchungen von Kuss der Osius-Test und/oder der Farrington-Test 
 am besten anzuwenden sein. Kuss hat zu dieser Arbeit ein SAS-Makro entwickelt [kuss2001], 
 welches in das hier beschriebene SAS-Makro eingebunden wurde.
 Auf Basis der Standardtests Pearson-Test und Devianz-Test wurden 2002 von Pulkstenis 
 und Robinson jeweils Varianten vorgeschlagen, die wie die brigen, von Kuss zitierten 
 Tests die Sparseness (speziell bei stetigen Einflussgren) bercksichtigen 
 [pulkstenis2002]. Da von diesen Autoren ebenfalls ein SAS-Makro zur Verfgung gestellt 
 wird, wurde dieses ebenfalls in das Makro PM_GOF.MAC.SAS integriert. 
 Somit sind zur Betrachtung der Anpassungsgte insgesamt 10 statistische Tests fr die 
 logistische Regression verfgbar. Allen Tests ist gemein, dass sie eine geringe Power 
 besitzen, ein Anpassungsproblem zu entdecken, speziell bei Sparseness. Daher ist sicher 
 die bersicht ber die Bandbreite der Ergebnisse interessant und es wird bei der Ausgabe 
 keine Selektion bzgl. der Verfahren vorgenommen. Fr die Routineuntersuchung der Anpassung
 im Prozess einer Modellentwicklung mit dem logistischen Regressionsmodell ist die Unter-
 suchung der Anpassungsgte anhand solcher globaler Mae sicher ausreichend. Je nach 
 Konsequenz des Prognosemodells sollte eine genauere Betrachtung, speziell auf Basis der 
 individuellen Anpassungen durchgefhrt werden. Bei Anpassungsproblemen (signifikante 
 Testergebnisse) hilft dies Analyseergebnis nicht direkt weiter, da es keine Hinweise gibt, 
 welche Modellkomponenten das Problem verursachen. Genaue Residuenanalysen verbunden mit 
 genauer Kenntnis des Datensatzes und der Variablen knnen dann nur weiterhelfen. 


 DESCRIPTION
 -----------
 Im Makro sind die angesprochenen Methoden der Anpassungsuntersuchung sowie die zwei 
 externen SAS-Makros folgendermaen implementiert worden:
 - Zuerst wird bei der Anwendung des Makros das auszuwertende Modell dokumentiert und die 
   Anzahl der Merkmalskombinationen ausgewertet. Bei einer groen Anzahl von Merkmals-
   kombinationen (Sparseness) sind die Ergebnisse der beiden Standard-Tests (Person-, 
   Devianz-Test) nicht sinnvoll zu interpretieren.  
 - Dann wurde das SAS-Makro von Kuss (GOFLOGIT_KUSS.MAC.SAS [kuss2001]), in das Makro 
   integriert. Das Makro von Kuss kann nicht direkt genutzt werden, da es fr die 
   SAS-Version 6 geschrieben wurde und noch kein CLASS-Statement enthlt. Deshalb wird mit 
   dem Makro von Friendly (DUMMY-FRIENDLY.MAC.SAS) automatisch eine Dummy-Codierung der 
   kategoriellen Variablen erzeugt. Weiterhin bentigt das Kuss-Makro die Daten in der 
   events/trial Formulierung fr PROC LOGISTIC. Darin wird fr jede auftretende 
   Merkmalskombination die Anzahl der Beobachtungen bergeben. Das Makro PM_GOF.MAC.SAS 
   setzt die Daten in diese Formulierung mit Hilfe der PROC FREQ-Option SPARSE um. Dieser 
   Konvertierungsschritt dauert leider etwas, wenn die Anzahl Merkmalskombinationen und 
   die Anzahl Beobachtungen nicht gering sind. 
 - Der Output des Makros von Kuss kann einmal mit einem blichen PROC LOGISTIC - Output 
   oder ohne angefordert werden. Da meist vorher das Makro PM_LOGREG.MAC.SAS ausgefhrt 
   wurde, kann durch die Einstellung des Parameters logistic=0 einiges an doppeltem 
   Output eingespart werden.
 - Das Makro von Kuss gibt die p-Werte der beiden Standard-Tests (Pearson-, Devianz-Test) 
   sowie die der zustzlichen 5 angegeben Tests (Osius-, McCullagh-, Farrington-, IM-, 
   RSS-Test) aus. 
 - Zustzlich wird standardmig durch das Makro der Hosmer-Lemeshow-Test (Option LACKFIT) 
   und die Angabe des R2 als Model-Fit-Statistik (Option RSQUARE) ausgegeben. 
 - Als letztes ist noch das SAS-Makro von Pulkstenis und Robinson (GOF_PULKSTENIS.MAC.SAS 
   [pulkstenis2002]) integriert worden. 
 - Anmerkung: Sollten keine kategoriellen Variablen unter 'cvar' angegeben worden sein,
   so ist die Ausfhrung dieses Makros (GOF_PULKSTENIS.MAC.SAS) nicht mglich!


 LITERATURE
 ----------
 - SAS Online Doc zu den Prozeduren PROC LOGISTIC, PROC FREQ
 - Hosmer DW, Lemeshow S (2000): Applied logistic regression (2nd Ed.).  
                     John Wiley and Sons Inc., New York
 - Kuss O (2001): A SAS/IML macro for goodness-of-fit testing in logistic regression 
                     models with sparse data.  Proceedings of the 26th Annual SAS Users 
                     Group International Conference (SUGI)   Paper 265-26
 - Kuss O (2002): Global goodness-of-fit tests in logistic regression with sparse data.  
                     Stat. Med. 21: 3789-3801
 - Pulkstenis E, Robinson TJ (2002): Two goodness-of-fit tests for logistic regression 
                     models with continuous covariates.  Stat. Med. 21: 79-93
 - Ziegler C (2003): Ein SAS-Makropaket zur Entwicklung und Validierung von 
					 Prognosemodellen auf Basis der logistischen Regression. 
					 Diplomarbeit, FH Ulm 

 Literatur zum Makro "GOFLOGIT_KUSS.MAC.SAS":
 - Copas JB (1989): Unweighted Sum of Squares Test for Proportions. 
                    Appl Statist, 38, 71-80.
 - Farrington CP (1996): On Assessing Goodness of Fit of Generalized Linear Models 
                    to Sparse Data. J R Statist Soc B 58 (2), 349-360.
 - Hosmer DW, Lemeshow S (1980): Goodness of fit tests for the multiple logistic
                    regression model. Commun Statist - Theor Meth 9 (10), 1043-1069.
 - Hosmer DW et al. (1997): A comparison of goodness-of-fit tests for the logistic 
                    regression model. Statistics in Medicine 16, 965-980.
 - McCullagh P (1985a): Sparse data and conditional tests. Bulletin of the International 
                    Statistics Institute, Proceedings of the 45th Session of ISI 
                    (Amsterdam), Invited Paper, 28, 1-10.
 - McCullagh P (1985b): On the Asymptotic Distribution of Pearsons Statistic in Linear 
                    Exponential-Family Models. Int Stat Rev, 53, 61-67.
 - McCullagh P, Nelder JA (1986): Generalized Linear Models. Chapman & Hall.
 - Orme C (1988): The calculation of the information matrix test for binary data models.
                    The Manchester School 54 (4), 370-376.
 - Osius G, Rojek D (1992): Normal Goodness-of-Fit Tests for Multinomial Models With 
                     Large Degrees of Freedom. JASA, 87, 1145-1152.
 - White H (1982): Maximum Likelihood Estimation of Misspecified Models. 
                     Econometrica, 50, 1-25.

 Literatur zum Makro "GOF_PULKSTENIS.MAC.SAS":
 - Pulkstenis E, Robinson TJ (2002): Two goodness-of-fit tests for logistic regression 
                     models with continuous covariates.  Stat. Med. 21: 79-93


 SYNTAX
 ------
 [Makroaufruf und Beschreibung der zu bergebenden Parameter]

 %PM_GOF(	
   *data		= ,					Datensatz
   *resp_var	= ,					Zielgre
	event		= 1,				Event- Kategorie der binren Zielgre (0/1)
   *xvar		= ,					stetige Variable(n)
   *cvar		= ,					kategorielle Variable(n)
	ref			= ,					Referenzkategorien fr kategorische Variablen
	logistic	= 1,				Zustzliche PROC LOGISTIC- Ausgabe (ja/nein)
   *macro_path	=					Pfad zur Makrosammlung
  );

  Mit (*) gekennzeichnete Parameter sind beim Makroaufruf zwingend erforderlich.
  Bereits gesetzte Parameter stellen die jeweilige Standardeinstellung dar.

   data:		    Parameter zur Angabe des zu analysierenden Datensatzes. Wurde 
					beispielsweise vor dem Aufruf des Makros ein Datensatz mit dem 
					Namen "dat300" angelegt, lautet der korrekte Aufruf: "data=dat300". 
					Zustzlich kann auch die dazugehrige SAS-Library bergeben werden 
					(Beispiel: "data=work.dat300"). Die bergabe dieses Parameters ist 
					zwingend erforderlich.
 
   resp_var:		"resp_var" steht fr "Response Variable", also fr die Zielgre. 
					Dieser bergabeparameter ist ebenfalls zwingend erforderlich. 
					Ferner wird eine binre Datenstruktur (0/1) der Zielgre bentigt. 
					Die Zielgre muss mit "0" und "1" kodiert sein. Sollte dies nicht 
					der Fall, ist eine vorherige Umkodierung erforderlich. Heit 
					beispielsweise die Zielgre "tod" lautet der Aufruf wie folgt: 
					"resp_var=tod".

   event:			Je nach dem welche Kategorie der binren Zielgre untersucht 
					werden soll, kann diese mit Hilfe dieses Parameters bergeben werden. 
					Soll beispielsweise die Zielgre "tod" analysiert werden und steht 
					"1" fr tot sowie "0" fr lebendig, lautet der korrekte Aufruf 
					"event=1". Dies ist zugleich die Standardeinstellung, da blicherweise
					"1" fr das Eintreten eines bestimmten Ereignisses steht.

   xvar:			Dieser Parameter ist zur Angabe der stetigen Variablen 
					(Einflussgren). Allgemein gesagt sollten hier alle Variablen 
					genannt werden, die nicht in das CLASS-Statement von PROC LOGISTIC 
					aufgenommen werden sollen, also beispielsweise auch quasi-stetige 
					Variablen. Werden mehrere stetige oder quasi-stetige Variablen 
					bergeben, so mssen diese durch ein Blank (Leerzeichen) getrennt 
					werden. Beispiel: "xvar=alter bmi".

   cvar:			"cvar" steht fr "categorical variables" und ist fr die bergabe 
					aller kategorischen sowie aller dichotomen Variablen verantwortlich. 
					Diese werden innerhalb der LOGISTIC-Prozedur in das CLASS-Statement 
					aufgenommen und werden automatisch Dummy-kodiert. Werden mehrere 
					kategorische Variablen bergeben, mssen diese getrennt durch ein 
					Leerzeichen (Blank) bergeben werden (Beispiel: "cvar=diagnose msnr").
					Wurden dem Parameter "cvar" dichotome Variablen bergeben, so werden 
					diese zwar in das CLASS-Statement von PROC LOGISTIC aufgenommen, 
					jedoch keine Dummy-Kodierung durchgefhrt, da dichotome Variablen nur 
					zwei Klassen besitzen.

   ref:				Mittels dieses Parameters, der fr "reference" steht, ist es 
					mglich, pro bergebener kategorischer Variable eine sogenannte 
					Referenzkategorie anzugeben. Werden keine Referenzkategorien 
					angegeben, wird der Standard bei jeder Variablen innerhalb des 
					CLASS-Statements von PROC LOGISTIC auf REF=FIRST gesetzt. Dies 
					bedeutet, dass die erste (sortierte) Kategorie jeder bergebenen 
					Variablen als Referenzkategorie gewhlt wird. Werden 
					Referenzkategorien bergeben, so muss deren Anzahl mit der Anzahl 
					der bergebenen kategorischen Variablen bereinstimmen. SAS bietet 
					standardmig die Kategorien FIRST und LAST an. Zustzlich ist es 
					mit Hilfe dieses Parameters mglich, beliebige Kategorien anzugeben. 
					Das Trennungszeichen ist der Stern (*). Eigens gewhlte 
					Referenzkategorien mssen zustzlich in Hochkommata eingeschlossen 
					werden. Die SAS-Standardkategorien FIRST / LAST nicht. Die ntige 
					Form lautet beispielsweise: "ref='1'*FIRST*'4'". Der dazugehrige 
					"cvar"-Parameter: "cvar=icdklass3 msnr_num beginn_b_num". Von der 
					bergebenen Variablen "icdklass" wird die Kategorie "1" als 
					Referenzkategorie gewhlt. Von der Variablen "msnr_num" die erste 
					(sortierte), unabhngig was diese fr einen Wert besitzt und von der 
					Variablen "beginn_b_num" die Kategorie "4". 

   logistic:		Dieser Parameter ermglicht die Auswahl, ob das verwendete Untermakro
					"GOFLOGIT_KUSS.MAC.SAS" eine PROC LOGISTIC- Ausgabe erzeugen soll (1)
					oder nicht (0). Die Standardeinstellung dieses Parameters ist 
					"logistic=1".
					
   macro_path: 		Aufgrund der Tatsache, dass innerhalb des Makros diverse 
					Untermakros aufgerufen werden, ist es zwingend ntig, den Pfad 
					zu dem Verzeichnis, wo die komplette Makrosammlung abgelegt ist,
					anzugeben. Der Pfad selber muss in Hochkommata eingeschlossen 
					sein. Beispiel: 'f:\makros'. 

  VALIDATION
  ----------

  Validiert durch: Stephanie Heiss
 				   University of Ulm 
                   Department of Biometry and Medical Documentation
                   89075 Ulm
  
  Validiert am:    23. September 2003

  Validierungsprogramm::      PM_GOF_VAL.SAS
  Validierungsdokumentation:  PM_GOF_VAL.DOC
   
  Validierungsvorgehen:
  - Das Testen der Makros wird durch eine von der Programmierung unabhngige 
    Person durchgefhrt.
  - Fr die Durchfhrung der Validierung wird auf einen in der Literatur 
    genutzten Datensatz zurckgegriffen:  LBW (Low Birth Weight), beschrieben
    in Hosmer, Lemeshow: Applied logistic regression [hosmer2000]
  - Der Datensatz wird fr verschiedene Testlufe verndert bzw. erweitert, 
    damit "die Testdaten neben den normalen Angaben auch Grenzwerte, ungltige
    Angaben, Fehlerbedingungen und Sonderflle beinhalten." [beiwanger2003]
  - Fehlerkontrolle:
    Hierbei werden nur die im LOG-Fenster auftretenden Fehler bzw. Warnungen 
    und offensichtlichen Fehler im OUTPUT betrachtet. Fr jeden Parameter des 
    Makros werden mehrere Durchlufe mit verschiedenen Wertangaben durchgefhrt, 
    wobei auch ungltige Werte angegeben werden, um das Verhalten des Makros
    bei fehlerhaften Aufrufen zu untersuchen. Gefundene Fehler werden 
    dokumentiert und dem Programmierer zur Verfgung gestellt. Nach der Korrektur 
    aller Fehler werden die Testlufe wiederholt, um sicherzustellen, dass 
    wirklich alle Fehler abgefangen wurden und dass keine neuen Fehler entstanden 
    sind.
  - Ergebniskontrolle:
    Um die Richtigkeit der Ergebnisse zu berprfen, werden soweit mglich, die
    Rechenbeispiele und bungsaufgaben aus Hosmer/Lemeshow durch vergleichbare
    Makroaufrufe nachgerechnet. Die Ergebnisse werden denen aus dem Begleitbuch 
    Hosmer/Lemeshow/Cook [hosmer2001] gegenbergestellt und direkt verglichen.
  - Alle Validierungen werden dokumentiert.

  Literatur: 
  - Beiwanger S (2003): Validierung von SAS-Makros. Diplomarbeit, FH Ulm
  - Hosmer DW, Lemeshow S (2000): Applied logistic regression 2nd Ed. 
           J. Wiley & Sons, New York
  - Hosmer DW, Lemeshow S, Cook ED (2001): Solutions manual to accompany
           applied logistic regression 2nd. Ed. J. Wiley & Sons, New York



  AUTHOR
  ------
  R. Muche / C. Ring / C. Ziegler
  University of Ulm
  Department of Biometry and Medical Documentation
  89075 Ulm

  The author does not overtake any responsibility for the correctness
  of results. It must not be modified without removing the heading lines 
  containing the authors name.

----------------------------------------------------------------------------------*/


%MACRO PM_GOF(	data		= ,
				resp_var	= ,
				event		= 1,
				xvar		= ,
				cvar		= ,
				ref			= ,
				logistic	= 1,
				macro_path	= 
 			 );

/*Prfungen / Referenzierungen / Standardbelegungen*/

/*Wichtigster Parameter 'macro_path' berprfen*/
/*bergabe erfolgt?*/

%IF &macro_path= %THEN %DO;

	%WINDOW error color=grey                                                      
       #5  @28 'Fehler!' attr=highlight                                       
                color=blue                                           
       #7  @28 "Dem zwingend erforderlichen Parameter 'macro_path' wurde kein Wert"
	   #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
       #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';
    %DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Hilfsprogramm welches berprft, ob dem Parameter 'macro_path' ein korrekter Pfad*/
/*bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/

/*Anzahl der Zeichen im String 'macro_path' bestimmen:*/

%LET path_length=%LENGTH(&macro_path);

%LET infile=&macro_path;

%IF %bquote(&infile) NE %THEN %DO;
     %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
      %IF %bquote(&char1) ne %STR(%')
         %THEN %LET command='NOK';
         %ELSE %LET command='OK';

	 %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
      %IF %bquote(&char2) ne %STR(%')
         %THEN %LET command2='NOK';
         %ELSE %LET command2='OK';
%END;

%IF &command='NOK' OR &command2='NOK' %THEN %DO;

	%WINDOW error color=grey
	 #5  @28 'Fehler!' attr=highlight  
              color=blue  
     #7  @28 "Dem Parameter 'macro_path' wurde ein syntaktisch falscher Wert" 
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                     
    %DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Test, ob das unter 'macro_path' angegebene Verzeichnis existiert*/

%LET dir=&macro_path;

%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
       %PUT Directory does not exist;
%END;
%ELSE %DO;
       %PUT Directory exists;
       %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;

	%WINDOW error color=grey                                               
	 #5  @28 'Fehler!' attr=highlight                          
             color=blue                                             
	 #7  @28 "Das unter dem Parameter 'macro_path' angegebene Verzeichnis existiert"
	 #8  @28 "nicht- Bentigte Untermakros knnen nicht referenziert werden"
	 #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';           
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Falls bestimmte Parameter ohne Wert bergeben wurden, diese mit Standard belegen*/

%IF &event= %THEN %DO;
	%LET event=1;
%END;

%IF &logistic= %THEN %DO;
	%LET logistic=1;
%END;

/*Externe Makros, die innerhalb des Makros 'PM_GOF.MAC.SAS'*/
/*verwendet werden (Untermakros) einbinden (%include)*/

FILENAME y &macro_path;

%INCLUDE y ('goflogit_kuss.mac.sas');
%INCLUDE y ('dummy_friendly.mac.sas');
%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('gof_pulkstenis.mac.sas');
%INCLUDE y ('fl_heinze.mac.sas');
%INCLUDE y ('numchar.mac.sas');
%INCLUDE y ('exist.mac.sas');
%INCLUDE y ('exist_var.mac.sas');
%INCLUDE y ('ref.mac.sas');

%GLOBAL nnnn1;

/*Test, ob bei bestimmten Parametern anstatt numerischen Werten*/
/*character- Werte bergeben wurden*/

/*berprfung erfolgt mit Hilfe des Untermakros 'numchar.mac.sas'*/

%NUMCHAR(parameterliste=&event &logistic);

%IF &numcharok='NOK' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error8.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfen, ob der angegebene Datensatz (Parameter 'data=') auch wirklich existiert*/
/*(mit Hilfe des Hilfs-/Untermakros 'exist.mac.sas')*/

%EXIST(dsn=&data);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error4.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Datenquelle (Parameter 'data=' UND response-Variable*/
/*(Zielgre / Parameter 'resp_var=') angegeben wurde*/

%IF &data= OR &resp_var= %THEN %DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

		%INCLUDE y ('error1.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

/*berprfung, ob berhaupt Variablen (Parameter 'xvar=' / 'cvar=') angegeben wurden*/

%IF &xvar= AND &cvar= %THEN %DO;

/*wenn keine Variablen angegeben wurden, Anzeige eines Fensters mit*/ 
/*Fehlermeldung/ Hinweis*/

		%INCLUDE y ('error2.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

/*Prfung, ob alle bergebenen Variablen numerisch sind*/

/*Erzeugung eines temporren Zwischendatensatzes, welcher lediglich die*/
/*bergebenen Variablen enthlt*/

DATA test3;
	SET &data;
	KEEP &xvar &resp_var &cvar;
RUN;

/*mittels der Prozedur PROC CONTENTS kann der Typ einer Variablen bestimmt werden*/

/*Unterdrckung der Ausgabe von PROC CONTENTS*/

PROC CONTENTS DATA=test3 NOPRINT OUT=testcontent;
RUN;

/*lediglich die Variable 'type' der Ergebnisdatei behalten,*/
/*da nur diese von Interesse ist*/

DATA testcontent;
	SET testcontent;
	KEEP type;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=testcontent);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test4=&nnnn1;

/*Prfung ob Variablen vom Typ 'character' vorhanden sind*/

DATA test4;
	SET testcontent;
	IF type=2 THEN DELETE;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test4);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test5=&nnnn1;

/*falls Variablen bergeben wurden, die nicht numerisch sind --> Fehlermeldung*/

%IF &test4 ne &test5 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error60.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE test4 testcontent test3;
RUN;
QUIT;


/*berprfung, ob alle bergebenen Variablen im bergebenen Datensatz*/
/*(Parameter 'data' vorhanden sind)*/

/*Zusammenfassung aller Variablen in einer neuen Makro- Variablen*/

%LET all_variables= &resp_var &xvar &cvar;

/*Prfung erfolgt mit Hilfe des Untermakros 'exist_var.mac.sas'*/

%EXIST_VAR(dsn=&data, var=&all_variables);

/*falls irgendwelche Variablen nicht existieren, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error6.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob die Zielgre (Parameter 'resp_var')*/
/*2 Kategorien (Ausprgungen) besitzt*/

/*missing values ausschliessen*/

DATA freq2;
	SET &data;
	IF &resp_var ne .;
RUN;

/*Anzahl der Ausprgungen mit Hilfe der Prozedur PROC FREQ ermitteln*/

/*Ausgabe unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnis als ODS- Datei zur weiteren Verarbeitung ausgeben*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=freq2;
	TABLES &resp_var;
RUN;

/*ODS- Output schliessen*/

ODS OUTPUT CLOSE;

/*ODS- Ausgabe wieder aktivieren*/

ODS LISTING;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen (= # Ausprgungen der Zielgre)*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq freq2;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

/*falls Zielgre mehr als zwei Ausprgungen besitz--> Fehlermeldung*/

%IF &event_check > 2 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error5.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

/*berprfung, ob Parameter 'event' 0 oder 1 ist*/

%IF &event ne 1  %THEN %DO;
	%IF &event ne 0  %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error7.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'logistic' 0 oder 1 ist*/

%IF &logistic ne 1  %THEN %DO;
	%IF &logistic ne 0  %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error48.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count_c varc;
%LET count_c=1;
%LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%DO %WHILE(&varc ne);
     %LET count_c=%EVAL(&count_c+1);
     %LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%END;
%LET count_c=%EVAL(&count_c-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The cvar contains &count_c variables.;

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The xvar contains &count_x variables.;

/*Anzahl der Referenzen ermitteln*/

%GLOBAL count_ref var_ref;
%LET count_ref=1;
%LET var_ref=%QSCAN(&ref,&count_ref,*);
%DO %WHILE(&var_ref ne);
     %LET count_ref=%EVAL(&count_ref+1);
     %LET var_ref=%QSCAN(&ref,&count_ref,*);
%END;
%LET count_ref=%EVAL(&count_ref-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The refs contains &count_ref variables.;

%IF &count_ref >0 %THEN %DO;
	%IF &count_c ne &count_ref %THEN %DO;

/*falls Anzahl der kategoriellen Variablen grer Null ist 
/*und nicht mit der Anzahl der angegebenen*/
/*Referenzkategorien bereinstimmt,*/
/*Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

		%INCLUDE y ('error3.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

   	%END;
%END;

/*berprfung, ob etwaige bergebene Referenzkategorien vorhanden sind bzw.*/
/*korrekt bergeben wurden*/

%IF &cvar ne %THEN %DO;
	%IF &count_ref ne 0 %THEN %DO;

	%REF(data=&data, resp_var=&resp_var, event=&event, cvar=&cvar, ref=&ref);

	%IF &correct='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error11.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;

%END;
%END;	

%LET datensatz=&data;

/*Elimination aller missing values (standardmig)*/

DATA work.nomiss;
	SET &data;

		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count_c;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

		%LET data=nomiss; 
RUN;

/*berprfen ob eine Variable nach Entfernung aller Beobachtungen mit missing*/
/*values nur noch eine Ausprgung besitzt*/

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values*/

DATA work.category_test;
	SET  &data;

	%DO i=1 %TO &count_x;
		%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
		IF &xvar_liste=. THEN DELETE;
	%END;

	%DO i=1 %TO &count;
		%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
		IF &cvar_liste=. THEN DELETE;
	%END;

		IF &resp_var=. THEN DELETE;

RUN;

/*Prfen ob 'resp_var' /'xvar'/ 'cvar' nur 1 Ausprgung besitzen*/

%LET category_test=0;
%LET category_test_fail=;

/*event*/

/*Anzahl der Ausprgungen mittels PROC FREQ bestimmen*/

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &resp_var;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

%IF &event_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &resp_var;

%END;

/*xvar*/

%IF &xvar ne %THEN %DO;

%DO uhgu=1 %TO &count_x;

%LET xvar_liste=%SCAN(&xvar,&uhgu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &xvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET xvar_check=&nnnn1;

%IF &xvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &xvar_liste;

%END;

%END;
%END;

/*cvar*/

%IF &cvar ne %THEN %DO;

%DO uhfu=1 %TO &count;

%LET cvar_liste=%SCAN(&cvar,&uhfu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &cvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET cvar_check=&nnnn1;

%IF &cvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &cvar_liste;

%END;

%END;
%END;

PROC DATASETS NOLIST;
	DELETE category_test;
RUN;
QUIT;

%IF &category_test=1 %THEN %DO;

	DATA category_test;
		var="&category_test_fail";
	RUN;

	PROC PRINT DATA=category_test NOOBS LABEL;
	TITLE1 "Folgende Variablen besitzen nach Entfernung aller";
	TITLE2 "missing values lediglich eine Ausprgung:";
		VAR var;
			LABEL var='Variable';
	RUN;

	PROC DATASETS NOLIST;
		DELETE category_test;
	RUN;
	QUIT;

	%INCLUDE y ('error24.sas');
	%GOTO ende;

%END;

/*ENDE Prfungen / Referenzierungen / Standardbelegungen*/

/*-------------------------------------------------------------------------------*/

/*Standardoutput ausgeben*/

/*Fallzahl bestimmen:*/

%NUMBEROBS(datensatz=nomiss);

%LET fallzahl=&nnnn1;

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatz       ";
	TITLE7 "Fallzahl:               &fallzahl (Missing Values automatisch eliminiert)";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

PROC DATASETS NOLIST;
	DELETE standardoutput;
RUN;
QUIT;

/*Ende Standardoutput*/
/*-------------------------------------------------------------------------------*/

/*Anzahl der Beobachtungen im Datensatz ermitteln*/

%NUMBEROBS(datensatz=&data);

/*stetige und kategorielle Variablen zusammenfassen*/

%LET var_string=&cvar &xvar;

/*Anzahl der stetigen und der kategoriellen Variablen addieren*/

%LET count_all=%EVAL(&count_c+&count_x);

/*Datensatz fr PROC FREQ nach ALLEN Variablen sortieren*/

PROC SORT DATA=&data;
	BY &var_string;
RUN;

/*Variablen fr BY- Statement von PROC FREQ bestimmen (eine weniger)*/

%LET var_string2=;

%DO i=1 %TO &count_all-1;
	%LET var=%SCAN(&var_string,&i,%STR( ));
	%LET var_string2=&var_string2 &var;
%END;

/*Alle mglichen Kombinationen mittels PROC FREQ auszhlen (Option SPARSE)*/

PROC FREQ DATA=&data NOPRINT;
	BY &var_string2;
	TABLES &var_string   /  SPARSE OUT=test2;
RUN;

/*Kombinationen, die nicht vorkommen, lschen*/

DATA test2 (DROP=percent);
	SET test2;
	IF count=0 THEN DELETE;
RUN;

DATA test;
	SET &data;
RUN;

/*Mgliche Kombinationen mit Originaldatensatz zusammenfhren*/

DATA gesamt;
	MERGE test test2;
	BY &var_string;
RUN;

/*Kombinationen die fters als ein Mal vorkommen selektieren*/

DATA weiter;
	SET gesamt;
	IF count=1 THEN DELETE;
	IF &resp_var ne &event THEN DELETE;
RUN;

/*Test ob Zwischendatei keine Beobachtungen enthlt*/

%NUMBEROBS(datensatz=weiter);

%LET anz_weiter=&nnnn1;

%IF &anz_weiter=0 %THEN %DO;

DATA weiter;
	SET gesamt;
	IF count=1 THEN DELETE;
/*	IF &resp_var ne &event THEN DELETE;*/
RUN;

%END;

/*Kombinationen, die fters als ein Mal vorkommen*/
/*und event=0 besitzen selektieren*/

DATA weiterx;
	SET gesamt;
	IF count=1 THEN DELETE;
RUN;

DATA weiterx;
	SET weiterx;
	IF &resp_var >0 THEN DELETE;
RUN;

/*Sortieren und doppelte Beobachtungen eliminieren (Option NODUPKEY)*/

PROC SORT DATA= weiterx NODUPKEY;
	BY &var_string;
RUN;

PROC SORT DATA= weiter NODUPKEY OUT=weitery;
	BY &var_string;
RUN;

/*Test ob Zwischendateien keine Beobachtungen enthalten*/
/*Ist dies der Fall, kein PROC COMPARE*/

%NUMBEROBS(datensatz=weiterx);

%LET anz_weiterx=&nnnn1;

%NUMBEROBS(datensatz=weitery);

%LET anz_weitery=&nnnn1;

%IF &anz_weiterx ne 0 AND &anz_weitery ne 0 %THEN %DO;

/*Vergleich, ob etwaige Beobachtungen schon in den Kombinationen,*/
/*die fters vorkommen, enthalten sind und eine event- Rate >0 haben*/

PROC COMPARE OUT=x OUTDIF OUTBASE BASE=weiterx COMPARE=weitery NOPRINT ;
	ID &var_string;
RUN;

/*Falls Doppelte vorhanden sind: Eliminieren.*/
/*Erkennung durch doppelte Kombinationen*/

PROC FREQ DATA=x NOPRINT;
	BY &var_string2  ;
	TABLES &var_string  /  SPARSE OUT=xyz;
RUN;

/*Ergebnisdatei anpassen*/

DATA xyz (DROP=percent count);
	SET xyz;
	IF count>1 THEN DELETE;
	event_zaehler=0;
RUN;

/*Datei die den event_zaehler enthlt mit der Datei*/
/*die die Kombinationsanzahl enthlt zusammenfhren*/

DATA xxxy;
	MERGE xyz weiterx;
	BY &var_string;
	IF event_zaehler=. THEN DELETE;
RUN;

%END;

PROC FREQ DATA=weiter NOPRINT;
	BY &var_string;
	TABLES &var_string &resp_var  /  SPARSE OUT=ende;
RUN;

DATA ende2 (DROP=percent &resp_var);
	SET ende;
	IF count=0 THEN DELETE;
	RENAME count=event_zaehler;
RUN;

%IF &anz_weiter=0 %THEN %DO;

DATA ende2;
	SET ende2;
	IF event_zaehler ne 0 THEN event_zaehler=0;
RUN;

%END;

DATA neu;
	SET gesamt;
	IF count>1 THEN DELETE;
	RENAME &resp_var=event_zaehler;
RUN;

DATA xxxx;
	MERGE ende2 test2;
	BY &var_string;
	IF event_zaehler=. THEN DELETE;
RUN;

/*Ergebnisse der drei Zwischenschritte zusammenfhren*/

DATA ergebnis (KEEP= &var_string count event_zaehler);
	MERGE xxxx neu;
	BY &var_string;
	LABEL count		   ='Haeufigkeit der Kombination'
	  	  event_zaehler="Haeufigkeit &resp_var=&event";
RUN;

%IF &anz_weiterx ne 0 AND &anz_weitery ne 0 %THEN %DO;

DATA ergebnis_ende (KEEP= &var_string count event_zaehler);
	MERGE xxxy ergebnis;
		BY &var_string;
		LABEL count		   ='Haeufigkeit der Kombination'
	  	  	  event_zaehler="Haeufigkeit &resp_var=&event";
RUN;

%END;

%ELSE %DO;

DATA ergebnis_ende;
	SET ergebnis;
RUN;

%END;

/*Dummy- Kodierung (nicht automatisch via CLASS- Statement bei PROC LOGISTIC*/
/*---> Kuss- Makro GOFLOGIT in SAS- Version 6! (Option noch nicht vorhanden)*/

%LET all_variables=;

/*Referenzen zu jeweiliger Variable zuordnen*/

%DO m=1 %TO &count_c;
	%LET var=%SCAN(&cvar,&m,%STR( ));
	%LET var_ref=%SCAN(&ref,&m,*);

/*Ausprgungen der jeweiligen kategoriellen Variable ermitteln*/

PROC FREQ DATA=ergebnis_ende NOPRINT;
	TABLES &var /  SPARSE OUT=&var;
RUN;

/*Ausgabedatensatz von PROC FREQ anpassen*/

DATA &var (DROP=percent count);
	SET &var;
	IF &var=. THEN DELETE;
	IF count=0 THEN DELETE;
RUN;

/*Anzahl der mglichen Ausprgungen je kategorieller Variable ermitteln*/
	
%NUMBEROBS(datensatz=&var);

%LET count_var=&nnnn1;

		%LET underscore=_;

/*Falls keine Referenzkategorie angegeben wurde, die erste nehmen*/
	
			%IF &count_ref=0 %THEN %DO;
				%LET var_ref=_FIRST_;
			%END;

/*Referenzkategorie "FIRST" fr bergabe an Dummy- Makro anpassen ("_FIRST_")*/

			%IF &var_ref=FIRST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

/*Referenzkategorie "LAST" fr bergabe an Dummy- Makro anpassen ("_LAST_")*/

	        %IF &var_ref=LAST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

			/*jeweilige Basiskategorie lschen*/

				DATA var2;
					SET &var end=_eof_;
					%IF 		&var_ref = _FIRST_ | &var_ref = LOW 
					%THEN 		%STR( IF _n_ = 1 THEN DELETE;);
					%ELSE %IF 	&var_ref = _LAST_ | &var_ref = HIGH
					%THEN 		%STR( IF _eof_ THEN DELETE;);
					%ELSE 		%STR(IF &var = &var_ref THEN DELETE;);
				RUN;

			/*Anzahl Kategorien der jeweiligen Variable ermitteln*/

				%NUMBEROBS(datensatz=var2);

	/*mgliche Ausprgungen pro Variable als Makro- Variable zwischenspeichern*/

			%LET count_var2=&nnnn1;

			DATA var2;
				SET var2;
				CALL SYMPUT('auspraeg'||left(_n_), trim(left(put(&var,BEST8.))));
			RUN;

/*Aufruf des Dummy- Kodierungs- Makros*/

	ODS LISTING CLOSE;

	%DUMMY( 
 	 		 data		= work.ergebnis_ende, /* name of input dataset        */
   			 out		= work.ergebnis_ende, /* name of output dataset       */
  			 var		= &var,         	  /* variable(s) to be dummied    */
  			 base		= &var_ref, /* base category                          */
   			 prefix 	= ,    		/* prefix for dummy variable names        */
   			 format 	= ,      	/* format used to categorize variable     */
  			 name  		= VAL,  	/* VAL: variable names are D_value        */
  			 fullrank	= 1     	/* Eliminate dummy for baseline category? */
   		  );

   ODS LISTING;


/*Falls fr eine Variable Dummies generiert wurden, Originalvariable lschen*/

DATA ergebnis_ende (DROP=&var);
	SET ergebnis_ende;

/*zustzliche Umbenennung (Anpassung an automatische Ausgabe der Dummy- Variablen*/
/*von PROC LOGISTIC)*/

		%DO o=1 %TO &count_var2;

		RENAME &var._&&auspraeg&o = &var&&auspraeg&o;

		%END;

RUN;

/*Zwischendatensatz lschen*/

PROC DATASETS NOLIST;
	DELETE &var var2;
RUN;
QUIT;

%END;

/*Mittels PROC CONTENTS die Variablen, die fr das GOFLOGIT- Makro selektieren*/
/*COUNT und EVENT_ZAEHLER entfernen*/

	PROC CONTENTS DATA=ergebnis_ende NOPRINT OUT=ergebnis2 (KEEP=name);
	RUN;

	DATA ergebnis2;
		SET ergebnis2;
			IF name='COUNT' 		  THEN DELETE;
			IF name='event_zaehler'   THEN DELETE;
	RUN;

	%NUMBEROBS(datensatz=ergebnis2);

	%LET count_ergebnis2=&nnnn1;

/*Variablennamen als Makro-Variablen speichern*/

	DATA ergebnis2;
		SET ergebnis2;
		CALL SYMPUT('y'||left(_N_), name);
	RUN;

/*Variablennamen fr GOFLOGIT- Makro zusammenfassen*/

	%DO s=1 %TO &count_ergebnis2;

	%LET all_variables=&all_variables &&y&s;
	%END;

	DATA null;
	RUN;

	PROC PRINT DATA=null;
	TITLE1 "Goodness Of Fit- Analyse";
	TITLE2;
	TITLE3;
	TITLE4;
	TITLE5;
	TITLE6;
	TITLE7;
	TITLE8;
	TITLE9;
	TITLE10;
	RUN;

DATA goflogit_data;
	SET ergebnis_ende;
RUN;
QUIT;

/*PROC LOGISTIC um Hosmer Lemeshow- Test & R-Quadrat zu erhalten*/

%IF &logistic=0 %THEN %DO;

ODS OUTPUT LackFitChiSq=LackFitChiSq;
ODS OUTPUT LackFitPartition=LackFitPartition;

%END;

ODS LISTING CLOSE;
ODS OUTPUT RSquare=RSquare;

ODS OUTPUT ConvergenceStatus=cs;

PROC LOGISTIC DATA=&data NAMELEN=32 OUTEST=fl_fail_est;

/*PROC LOGISTIC CLASS- Statement ohne dazugehrigen*/
/*Referenzkategorien, falls keine  angegeben wurden*/

%IF &count_ref=0 %THEN
	%DO;
		CLASS

	%DO i=1 %TO &count_c;
		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
      
		&var_liste (PARAM=REF REF=FIRST)

	%END;
     	;
%END;

%ELSE
	%DO;

/*PROC LOGISTIC CLASS- Statement mit dazugehrigen Referenzkategorien*/

		CLASS
	%DO i=1 %TO &count_c;
   		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
   		%LET ref_liste=%SCAN(&ref,&i,*);

   		&var_liste (PARAM=REF REF=&ref_liste)
	%END;
   		;
%END;

MODEL &resp_var(

%IF &event=1 %THEN
	%DO;
		event='1'
%END;

%IF &event=0 %THEN
	%DO;
		event='0'
%END;

)

= &cvar &xvar/ SELECTION=NONE 

%IF &logistic=0 %THEN %DO;

	LACKFIT 

%END;

	RSQUARE;

RUN;

ODS OUTPUT CLOSE;
ODS LISTING;

/*(Quasi)- Separation von PROC LOGISTIC #1 immer berprfen*/

%NUMBEROBS(datensatz=cs);

%LET cs_check=&nnnn1;

DATA est_check;
	SET cs;
	IF reason ne 'Quasi-complete separation of data points detected.';
	IF reason ne 'Complete separation of data points detected.';
RUN;

/*Test ob es berhaupt Estimates ohne Warnung gibt:*/

%NUMBEROBS (datensatz=est_check);

%LET est_count=&nnnn1;

PROC DATASETS NOLIST;
	DELETE est_check cs ;
RUN;
QUIT;

%GLOBAL status_separation;

%IF &est_count ne &cs_check %THEN %DO;

%LET status_separation=1;

%END;
%ELSE %DO;

%LET status_separation=0;

%END;

/*Falls (quasi-) separation aufgetreten ist:*/
/*Korrektur:*/

%IF &status_separation=1 %THEN %DO;

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count_c varc;
%LET count_c=1;
%LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%DO %WHILE(&varc ne);
     %LET count_c=%EVAL(&count_c+1);
     %LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%END;
%LET count_c=%EVAL(&count_c-1);

/*Anzahl ausgeben im LOG- Fenster*/
%PUT The cvar contains &count_c variables.;

/*Dummy- Kodierung (nicht automatisch via CLASS- Statement bei PROC LOGISTIC*/
/*---> Heinze- Makro FL kennt keine kategorischen (klassierten) Variablen*/

%LET all_variables=;

DATA ergebnis_ende (KEEP=&resp_var &cvar &xvar);
	SET &data;
RUN;

/*Referenzen zu jeweiliger Variable zuordnen*/

%DO m=1 %TO &count_c;
	%LET var=%SCAN(&cvar,&m,%STR( ));
	%LET var_ref=%SCAN(&ref,&m,*);

/*Ausprgungen der jeweiligen kategoriellen Variable ermitteln*/

	PROC FREQ DATA=&data NOPRINT;
		TABLES &var /  SPARSE OUT=&var;
	RUN;

/*Ausgabedatensatz von PROC FREQ anpassen*/

	DATA &var (DROP=percent count);
		SET &var;
		IF &var=. THEN DELETE;
		IF count=0 THEN DELETE;
	RUN;

/*Anzahl der mglichen Ausprgungen je kategorieller Variable ermitteln*/
	
	%NUMBEROBS(datensatz=&var);

	%LET count_var=&nnnn1;

		%LET underscore=_;

	/*Falls keine Referenzkategorie angegeben wurde, die erste nehmen*/
	
			%IF &count_ref=0 %THEN %DO;
				%LET var_ref=_FIRST_;
			%END;

	/*Referenzkategorie "FIRST" fr bergabe an Dummy- Makro anpassen ("_FIRST_")*/

			%IF &var_ref=FIRST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

	/*Referenzkategorie "LAST" fr bergabe an Dummy- Makro anpassen ("_LAST_")*/

	        %IF &var_ref=LAST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

			/*jeweilige Basiskategorie lschen*/

				DATA var2;
					SET &var end=_eof_;
					%IF 		&var_ref = _FIRST_ | &var_ref = LOW 
					%THEN 		%STR( IF _n_ = 1 THEN DELETE;);
					%ELSE %IF 	&var_ref = _LAST_ | &var_ref = HIGH
					%THEN 		%STR( IF _eof_ THEN DELETE;);
					%ELSE 		%STR(IF &var = &var_ref THEN DELETE;);
				RUN;

	/*Anzahl Kategorien der jeweiligen Variable ermitteln*/

				%NUMBEROBS(datensatz=var2);

	/*mgliche Ausprgungen pro Variable als Makro- Variable zwischenspeichern*/

			%LET count_var2=&nnnn1;

			DATA var2;
				SET var2;
				CALL SYMPUT('auspraeg'||left(_n_), trim(left(put(&var,BEST8.))));
			RUN;

/*Aufruf des Dummy- Kodierungs- Makros*/

	ODS LISTING CLOSE;

%DUMMY( 
 	 	data	= work.ergebnis_ende,  /* name of input dataset                  */
   		out		= work.ergebnis_ende,  /* name of output dataset                 */
  		var		= &var,         	   /* variable(s) to be dummied              */
  		base	= &var_ref,     	   /* base category                          */
   		prefix 	= ,    				   /* prefix for dummy variable names        */
   		format 	= ,      			   /* format used to categorize variable     */
  		name  	= VAL,  			   /* VAL: variable names are D_value        */
  		fullrank= 1     			   /* Eliminate dummy for baseline category? */
     );

   ODS LISTING;


/*Falls fr eine Variable Dummies generiert wurden, Originalvariable lschen*/

	DATA ergebnis_ende (DROP=&var);
		SET ergebnis_ende;

		/*zustzliche Umbenennung (Anpassung an automatische*/
		/*Ausgabe der Dummy- Variablen von PROC LOGISTIC)*/

		%DO o=1 %TO &count_var2;

		RENAME &var._&&auspraeg&o = &var&&auspraeg&o;

		%END;
	RUN;


/*Zwischendatensatz lschen*/

	PROC DATASETS NOLIST;
		DELETE &var var2;
	RUN;
	QUIT;


%END;

/*Mittels PROC CONTENTS die Variablen, die fr das FL- Makro selektieren*/
/*ohne Event (resp_var)*/

	DATA ergebnis_ende2 (DROP=&resp_var);
		SET ergebnis_ende;
	RUN;

	PROC CONTENTS DATA=ergebnis_ende2 NOPRINT OUT=ergebnis2 (KEEP=name);
	RUN;

	%NUMBEROBS(datensatz=ergebnis2);

	%LET count_ergebnis2=&nnnn1;

/*Variablennamen als Makro-Variablen speichern*/

	DATA ergebnis2;
		SET ergebnis2;
		CALL SYMPUT('y'||left(_N_), name);
	RUN;

/*Variablennamen fr FL- Makro zusammenfassen*/

	%DO s=1 %TO &count_ergebnis2;

	%LET all_variables=&all_variables &&y&s;
	%PUT &all_variables;
	%END;


/*Aufruf des FL- Makros*/

%FL		(	data			= ergebnis_ende, 
			y				= &resp_var, 
			varlist			= &all_variables, 
			outest			= outest_fl, 
			print			= 0
        );

/*berflssige Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE ergebnis_ende _work _outtab _offset_ _maxby ergebnis2;
	DELETE _fby _estwot_ _cats_ var2 ergebnis_ende2;
RUN;
QUIT;

/*Datei mit den Regressionskoeffizienten aus dem FL- Makro*/
/*fr INEST- Statement bei PROC LOGISTIC vorbereiten*/

DATA inest_fl (DROP=_rby_ _penlik_ _it_ _resp_ _noresp_ _by_ _total_ _code_);
	SET outest_fl;
	IF _code_ ne -1 THEN DELETE;
	RENAME INTERCEP=Intercept;
RUN;

/*Prfen, ob FL- Makro korrekt abgelaufen ist*/

%NUMBEROBS(datensatz=inest_fl);

%LET fl_fail_check=&nnnn1;

%IF &fl_fail_check=0 %THEN %DO;

%LET fl_false=1;

%END;

%ELSE %DO;

%LET fl_false=0;

%END;

/*Warnung ausgeben, dass Firth- Prozedur benutzt wurde*/

DATA note;
	note="";
RUN;

PROC PRINT DATA=note NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Warnung (PROC LOGISTIC):               ";
	TITLE4 ;
	TITLE5 "There is possibly a quasi-complete separation of data "; 
	TITLE6 "points. The maximum likelihood estimate may not exist.";
	TITLE7 "Validity of the model fit is questionable.";
	TITLE8 ;
	TITLE9 'Es wurde deshalb eine automatische Korrektur mittels der';
	TITLE10 "Firth- Prozedur (Makro 'FL_HEINZE.MAC.SAS') durchgefhrt!";

	
	LABEL note = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
RUN;

%IF &logistic=1 %THEN %DO;
DATA note2;
	note2="";
RUN;

PROC PRINT DATA=note2 NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note2;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Aufgrund der Korrektur der Regressionskoeffizienten wurde";
	TITLE4 "PROC LOGISTIC mit den Optionen INEST=neue (korrigierte)";
	TITLE5 "Regressionskoeffizienten sowie MAXITER=0 aufgerufen."; 
	TITLE6 "Dadurch erscheint nun unter Umstnden folgende Meldung";
	TITLE7 "im Output, welche ignoriert werden kann:";
	TITLE8 "'Iteration limit reached without convergence'";
	TITLE9 ;
	TITLE10;

	LABEL note2 = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
RUN;

%END;
%IF &logistic=0 %THEN %DO;
DATA note3;
	note3="";
RUN;

PROC PRINT DATA=note3 NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note3;

    TITLE1 ;
	TITLE2 ;
	TITLE3 ;
	TITLE4 ;
	TITLE5 ; 
	TITLE6 ;
	TITLE7 ;
	TITLE8 ;
	TITLE9 ;
	TITLE10;

	LABEL note3 = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
RUN;
%END;

PROC DATASETS NOLIST;
	DELETE note

%IF &logistic=1 %THEN %DO;

note2

%END;
%ELSE %DO;

note3

%END;
;
RUN;
QUIT;


%IF &fl_false=1 %THEN %DO;

/*Hinweis ausgeben, dass Firth- Prozedur (Korrektur) fehlgeschlagen ist*/

DATA note2;
	note2="";
RUN;

PROC PRINT DATA=note2 NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note2;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Warnung:               ";
	TITLE4 "Die Korrektur der Regressionskoeffizienten mittels der";
	TITLE5 "Firth- Prozedur (Makro 'FL_HEINZE.MAC.SAS') ist"; 
	TITLE6 "fehlgeschlagen! Evtl. wurden zu viele Koeffizienten in";
	TITLE7 "Bezug auf die Anzahl der Beobachtungen geschtzt. In der";
	TITLE8 "weiteren Analyse werden die Koeffizienten des letzten";
	TITLE9 "ML-Iterationsschrittes aus PROC LOGISTIC verwendet.";
	TITLE10 "Die Validitt dieses Modells ist fraglich!";
	
	LABEL note2 = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";

RUN;

/*temporre Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE note2;
RUN;
QUIT;

%END;

/*PROC LOGISTIC um Hosmer Lemeshow- Test & R-Quadrat zu erhalten*/

%IF &logistic=0 %THEN %DO;

ODS OUTPUT LackFitChiSq=LackFitChiSq;
ODS OUTPUT LackFitPartition=LackFitPartition;

%END;

ODS LISTING CLOSE;
ODS OUTPUT RSquare=RSquare;

PROC LOGISTIC DATA=&data 

%IF &fl_false=0 %THEN %DO;

INEST=inest_fl 

%END;
%ELSE %DO;

INEST=fl_fail_est

%END;

NAMELEN=32;

/*PROC LOGISTIC CLASS- Statement ohne dazugehrigen Referenzkategorien,*/
/*falls keine angegeben wurden*/

%IF &count_ref=0 %THEN
	%DO;
		CLASS

	%DO i=1 %TO &count_c;
		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
      
		&var_liste (PARAM=REF REF=FIRST)

	%END;
     	;
%END;

%ELSE
	%DO;

/*PROC LOGISTIC CLASS- Statement mit dazugehrigen Referenzkategorien*/

		CLASS
	%DO i=1 %TO &count_c;
   		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
   		%LET ref_liste=%SCAN(&ref,&i,*);

   		&var_liste (PARAM=REF REF=&ref_liste)
	%END;
   		;
%END;

MODEL &resp_var(

%IF &event=1 %THEN
	%DO;
		event='1'
%END;

%IF &event=0 %THEN
	%DO;
		event='0'
%END;

)

= &cvar &xvar/ SELECTION=NONE MAXITER=0

%IF &logistic=0 %THEN %DO;

	LACKFIT 

%END;

	RSQUARE;

RUN;

ODS OUTPUT CLOSE;
ODS LISTING;


PROC DATASETS NOLIST;
	DELETE outest_fl inest_fl;
RUN;
QUIT;

%END;

/*Aufruf GOFLOGIT-Makro Kuss*/

/*Entweder mit oder ohne PROC LOGISTIC*/

	%IF &logistic=1 %THEN %DO;

		%LET logistic_flag=ON;

	%END;

	%IF &logistic=0 %THEN %DO;

		%LET logistic_flag=OFF;

	%END;

%GOFLOGIT(	data		= work.goflogit_data,
		 	y			= event_zaehler,
		 	m			= count,
            xlist		= &all_variables,
			logistic	= &logistic_flag
         );

%IF &logistic=0 %THEN %DO;

/*Hosmer Lemeshow- Test & R-Quadrat ausgeben*/

PROC PRINT DATA=Lackfitpartition NOOBS LABEL;
	TITLE1 'Partition for the Hosmer and Lemeshow Test';
	TITLE2 ;
	TITLE3 ;
	TITLE4 ;
	TITLE5 ; 
	TITLE6 ;
	TITLE7 ;
	TITLE8 ;
	TITLE9 ;
	TITLE10;
		 LABEL 		EventsObserved			='Event Observed'
     	 			EventsExpected			='Event Expected'
	 				NoneventsObserved		='Nonevent Observed'
	 	 			NoneventsExpected		='Nonevent Expected'
;
RUN;

PROC PRINT DATA=Lackfitchisq NOOBS LABEL;
	TITLE1 'Hosmer and Lemeshow Goodness-of-Fit Test';
    TITLE2 ;
	TITLE3 ;
	TITLE4 ;
	TITLE5 ; 
	TITLE6 ;
	TITLE7 ;
	TITLE8 ;
	TITLE9 ;
	TITLE10;
	 LABEL 		ChiSq			='Chi-Square'
     			DF				='Degrees Of Freedom'
				ProbChiSq		='P > Chisq'
;
RUN;

%END;

DATA RSquare (DROP=nValue1 nValue2);
	SET RSquare;
RUN;

PROC PRINT DATA=RSquare NOOBS LABEL;
	TITLE1 'Model Fit Statistics';
	TITLE2 ;
	TITLE3 ;
	TITLE4 ;
	TITLE5 ; 
	TITLE6 ;
	TITLE7 ;
	TITLE8 ;
	TITLE9 ;
	TITLE10;
	LABEL Label1	='______'
    	  Label2	='______'
	 	  cValue1	='______'
	 	  cValue2	='______'
;
RUN;

/*dummy- codierung*/

/*Output des Makros nicht komplett ausgeben*/

ODS LISTING CLOSE;

%IF &cvar ne %THEN %DO;

/*Makro von Pulkstenis & Robinson aufrufen*/

%GOODNESS(	vars	= &cvar,
			ref		= &ref,
			event	= &event,
			contvar = &xvar,
			k		= &count_c,
			dset	= &data,
			y		= &resp_var,
			alpha	= 0.05
         );

ODS LISTING;

/*Datei "final" des %GOODNESS- makros enthlt die gewnschten Daten*/

PROC PRINT DATA=work.final NOOBS LABEL;
TITLE1 'Pulkstenis & Robinson GOF-Output, p=0.05';
RUN;

/*Zwischendateien von Pulkstenis & Robinson- Makro lschen*/

PROC DATASETS NOLIST;
	DELETE expect final hats hold1 hold2 holder phats
		   chis crosstab devs display display1 display2
;
RUN;
QUIT;

%END;

/*Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE ende gesamt neu nomiss test test2 weiter xxxx weiterx weitery;
	DELETE ende2 ergebnis rsquare null  goflogit_data fl_fail_est;

	%IF &status_separation ne 1 %THEN %DO;

	DELETE 

	%IF &cvar ne %THEN %DO;

	_cats_ source

	%END;

	ergebnis2  ergebnis_ende;

	%END;

	%IF &logistic=0 %THEN %DO;

	DELETE lackfitpartition lackfitchisq;

	%END;

	%IF &anz_weiterx ne 0 AND &anz_weitery ne 0 %THEN %DO;

	DELETE xxxy x xyz;

	%END;

RUN;
QUIT;

%ende:
%MEND PM_GOF;