/************************************************************************************/
/*						        													*/
/*     NAME: pm_influence.mac.sas	         										*/
/*    TITLE: Influence- Diagnostik		        									*/
/*     KEYS: influence, extreme observations, residues, pearson			        	*/
/*  VERSION: SAS 8.2												      			*/
/*  PRODUCT: BASE, STAT, GRAPH							        					*/
/*    PROCS: LOGISTIC, GPLOT							        					*/
/*															         				*/
/*																					*/
/*  Verzeichnis/Dateiname:			F:\Prognose\pm_influence.mac.sas	        	*/
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*  Programmierer:					Christoph Ziegler								*/
/*  Datum der Ersterstellung:		11/2002											*/
/*  Aktuelle Makro-Version:			12/2003											*/
/*																					*/
/*  Kurzbeschreibung:				Makro zur Detektion etwaiger einflussreicher	*/
/*							    	Beobachtungen (Influence Diagnostic)			*/
/*																					*/
/*  Verwendete externe Untermakros:	numberobs.mac.sas								*/
/*									exist.mac.sas 									*/
/*									exist_var.mac.sas								*/
/*									numchar.mac.sas									*/
/*									ref.mac.sas										*/
/*									fl_heinze.mac.sas								*/
/*									dummy_friendly.mac.sas							*/
/*									words.mac.sas									*/
/*									error1.sas	error2.sas	error3.sas				*/
/*									error4.sas	error5.sas	error6.sas				*/
/*									error7.sas	error8.sas	error9.sas				*/
/*									error10.sas	error11.sas	error12.sas				*/
/*									error13.sas	error14.sas	error15.sas	error24.sas	*/
/*									error59.sas error60.sas error61.sas				*/
/*																					*/
/*  Bentigte Dateien:				Datensatz (Parameter 'data=')			   		*/
/*															           				*/
/*																					*/
/*  Erzeugte Dateien:				'path'/gplot - 'path'/gplotx 					*/
/*  								(falls Parameter 'graphics=1')	            	*/
/*  (Dateinamen nicht frei whlbar) (x ist Anzahl einflussreicher Beobachtungen)    */
/*																					*/
/*  Programmnderungen																*/ 
/*  und -ergnzungen 																*/
/*  (Name, Datum, Beschreibung):											 		*/
/*  																				*/
/*        																			*/
/************************************************************************************/
/*

 TITLE
 -----
 pm_influence.mac.sas

 SAS Makro zur Detektion etwaiger einflussreicher Beobachtungen (Influence Diagnostic) 

 BACKGROUND
 ----------
 Einflussreiche Beobachtungen knnen die Modellanpassung und Schtzung der 
 Regressionskoeffizienten stark beeinflussen. Bezogen auf die Schtzung der 
 Regressionskoeffizienten und auf die Gte des Modells ist es nicht wnschenswert, 
 dass einige wenige Beobachtungen die Ergebnisse in Bezug auf die Modellstruktur bzw. 
 Prognoseergebnisse verndern knnen. Harrell gibt in seinem Buch [harrell2001] einige 
 mgliche Erklrungen fr solche extremen Beobachtungen an:
 - Zu wenige Beobachtungen im Datensatz gegenber der Komplexitt des Modells,
 - Dateneingabefehler,
 - Extreme Werte in einigen Variablen, auch wenn die Werte als korrekt berprft wurden,
 - Beobachtungen, bei denen ein groer Unterschied zwischen beobachtetem und durch das 
   Modell vorhergesagtem Wert vorliegen. 
 Die ersten beiden Aspekte lassen sich leicht berprfen und ggf. durch sorgfltige 
 Datenkontrolle und Auswahl der mglichen Einflussgren korrigieren. Beobachtungen, 
 die durch extreme Merkmalsausprgungen auffallen, knnten atypische Beobachtungen sein, 
 fr die das Modell nicht gelten soll. Dann knnen diese entfernt werden. Die Gltigkeit 
 des Modells wird allerdings dadurch beschrnkt. Harrell weist darauf hin, dass solche 
 einflussreichen Beobachtungen im "wahren Leben" vorkommen knnen und die Identifikation 
 nicht automatisch zur Elimination der Beobachtung aus dem Datensatz fhren sollte 
 zugunsten einer besseren Modellanpassung. Sollten Beobachtungen identifiziert werden, 
 sollte sehr genau darber nachgedacht werden, warum diese Beobachtungen sich anders 
 verhalten als die brigen Beobachtungen. 

 Meist fallen einflussreiche Beobachtungen aber nicht unbedingt durch ein oder zwei 
 hervorstechende Merkmalsausprgungen auf, sondern deren Einfluss ist im mehr-
 dimensionalen Raum der Variablen im Modell zu verstehen. Die Identifikation solcher 
 Beobachtungen gelingt mit Methoden der Ausreier- und / oder Residualanalyse. Dabei 
 geht in die statistischen Kenngren der jeweilige Unterschied zwischen beobachtetem 
 und mit dem Modell vorhergesagten Wert ein. Hosmer und Lemeshow [hosmer2000] geben 
 einige Kenngren an, die die Informationen ber eine mehrdimensionale Beobachtung 
 auf jeweils einen Wert reduzieren. Diese Kenngren stammen aus der Regressions-
 diagnostik des linearen Modells und sind von Pregibon auf die logistische Regression 
 bertragen worden [pregibon1981]. Pregibon unterscheidet u.a. Kenngren, die auf der 
 Vernderung der Parameterschtzungen basieren (DIFBETA) und Kenngren auf der Basis 
 von Modell-Fit-Statistiken (Pearson- bzw. Deviance (DIFCHISQ, DIFDEV)), wobei hier 
 nicht die Gte des Modells sondern die Vernderung der Kenngren durch das Weglassen 
 einer Beobachtung im Vordergrund steht. 

 Die Identifizierung der einflussreichen Beobachtungen wird anhand einer Residuenanalyse 
 durchgefhrt. Die am hufigsten benutzten Kenngren basieren auf den Pearson- und 
 Deviance-Residuen [allison1999, harrell2001, hosmer2000, pregibon1981]. Diese sind 
 definiert als Beitrag jeder Beobachtung zur Pearson-Chi-Quadrat-Statistik oder zur 
 Deviance-Statistik. Die fr die logistische Regression speziell angepassten Residuen 
 (siehe [hosmer2000]) knnen genutzt werden, die verschiedenen Statistiken zur 
 Beurteilung des Einflusses einzelner Beobachtungen zu berechnen. Hosmer und Lemeshow 
 betrachten hauptschlich folgende Gren:
 - DIFCHISQ:	Vernderung der Pearson-Statistik bei Entfernung eines Patienten 
 - DIFDEV:		Vernderung der Deviance bei Entfernung eines Patienten 
 - DIFBETA:		Vernderung der Parameterschtzung (global) bei Entfernung eines 
 			    Patienten, berechnet analog zur Statistik nach Cook fr die lineare 
 			    Regression  
 Sie empfehlen, diese Differenzen gegen die geschtzten Wahrscheinlichkeiten aufzutragen 
 und an beiden Enden des Wertebereiches nach aufflligen Werten zu suchen. Eine 
 beispielhafte Grafik, die im Makro bernommen wurde, wird von Hosmer und Lemeshow in 
 ihrem Buch auf Seite 180 gezeigt, in der DIFCHISQ gegen die geschtzten Wahrschein-
 lichkeiten aufgetragen wird, wobei jede Beobachtung noch gewichtet wird bzgl. des 
 Einflusses auf die Parameterschtzung (DIFBETA). So lassen sich in der Grafik bzgl. 
 dieser zwei Kriterien einflussreiche Beobachtungen identifizieren. Genaue Grenzen, 
 ab denen in diesen Differenzen eine Beobachtung als einflussreich gewertet wird, 
 werden allerdings nur fr DIFCHISQ mit > 4 angegeben, wobei diese Grenze abgeleitet 
 wird vom 95%-Perzentil der Chi-Quadrat-Verteilung. Diese Grenze erscheint unter Bezug 
 der Vielzahl der Vergleiche (Adjustierung) sehr gering und wir raten, diese Grenze 
 wesentlich liberaler zu whlen (10-100, je nach Anwendung).

 Da die Modellierung von diesen einflussreichen Beobachtungen abhngt, sollten bei der 
 Modellentwicklung zumindest diese Beobachtungen identifiziert werden [armitage2002,
 concato1993]. Auch wenn einflussreiche Beobachtungen identifiziert werden knnen, 
 sollten diese aber nicht automatisch aus der Modellierung herausgenommen werden. 
 Eventuell zeigen sie an, dass fr bestimmte Beobachtungen (Patienten) die Modellierung 
 nicht passt. Diese Beobachtungen stellen eventuell ein spezielles Kollektiv dar oder 
 es lsst sich anhand dieser Beobachtungen noch einmal Sinn und Zweck des zu 
 entwickelnden Modells diskutieren.  


 DESCRIPTION
 -----------
 Im Makro sind die angesprochenen Methoden der Identifizierung einflussreicher 
 Beobachtungen folgendermaen implementiert worden:
 - In PROC LOGISTIC knnen ber das OUTPUT-Statement die Pearson- und Devianz-Residuen 
   (RESCHI bzw. RESDEV), DIFCHISQ, DIFDEV sowie die Diagonalelemente (H) der Hat-Matrix 
   fr jede Beobachtung ausgelesen werden. DIFBETA wird dann ber die standardisierten 
   Pearson-Residuen: RESCHISTD = RESCHI / SQRT(1-H) bestimmt nach [pregibon1981] als
   DIFBETA = (RESCHISTD**2)*H/(1-H)	
 - Sollte die ML-Schtzung der Regressionskoeffizienten auf Grund einer Datenseparation
   nicht konvergieren wird das FL-Makro von Heinze [heinze2002] zur Schtzung der 
   Koeffizienten genutzt. Diese werden anschliessend PROC LOGISTIC bergeben und 
   es werden mit dieser Prozedur die Kenngren so bestimmt.
 - Eine Alternative zur Erzeugung dieser Kenngren wre die Nutzung der Option 
   INFLUENCE im MODEL-Statement von PROC LOGISTIC mit anschlieender Verarbeitung der 
   ODS-Tabelle INFLUENCE. Dieses Vorgehen wurde hier nicht angewandt. 
 - Im Makro werden diese Kenngren inklusive der Beobachtungsidentifikation und der 
   geschtzten Wahrscheinlichkeit fr das Auftreten des Ereignisses ausgegeben fr alle 
   Beobachtungen mit einem DIFCHISQ grer einer vom Benutzer festgelegten Grenze. Dabei 
   werden die Beobachtungen nach DIFCHISQ sortiert angezeigt. Anschlieend wird schritt-
   weise die Beobachtung mit der grten Vernderung der Person-Residuen aus dem 
   Datensatz (temporr) eliminiert und die Prozedur neu gestartet. So erhlt man 
   sukzessive eine Auflistung der einflussreichsten Beobachtungen. Die Beobachtung mit 
   dem hchsten DIFCHISQ-Wert nach der Elimination wird ebenfalls angezeigt, um einen 
   Eindruck zu bekommen, wie relevant die Wahl der Grenze ist. 
 - Optional knnen Grafiken ausgegeben werden, die nach Hosmer und Lemeshow [hosmer2000]
   fr jeden Eliminationsschritt DIFCHISQ gegen die geschtzten Wahrscheinlichkeiten 
   auftragen, wobei die Information DIFBETA durch Nutzung der Option BUBBLE in PROC 
   GPLOT anhand der Flche der Kreise mit bercksichtigt wird. Die Grafiken knnen in 
   verschiedenen Grafikformaten in ein Verzeichnis gespeichert werden, ein Anzeigen auf 
   dem Bildschirm ist nicht vorgesehen. 
 
 LITERATURE
 ----------
 - SAS Online Doc zu den Prozeduren PROC LOGISTIC, PROC GPLOT
 - Allison PD (1999):  Logistic regression using the SAS System. 
                     SAS Institute Inc., Cary NC 
 - Armitage P, Berry G, Matthews JNS (2002): Statistical methods in medical research.
                     Blackwell Science Ltd., London 
 - Concato J, Feinstein AR, Holford TR (1993): The risk of determining risk with 
                     multivariable models. Annals of Internal Medicine 118: 201-210
 - Harrell FE JR (2001):  Regression modeling strategies.  Springer, New York 
 - Heinze G, Schemper M (2002): A solution to the problem of separation in logistic 
                     regression. Statistics in Medicine 21: 2409-2419
 - Hosmer DW, Lemeshow S (2000): Applied logistic regression 2nd Ed. J. Wiley & Sons,
                     New York
 - Pregibon D (1981): Logistic regression diagnostics. Annals of Statistics 9: 705-724
 - Ziegler C (2003): Ein SAS-Makropaket zur Entwicklung und Validierung von 
      				 Prognosemodellen auf Basis der logistischen Regression. 
				     Diplomarbeit, FH Ulm 

 SYNTAX
 ------
 [Makroaufruf und Beschreibung der zu bergebenden Parameter]

 %PM_INFLUENCE (  	
   *data		= ,			  Datensatz
   *resp_var	= ,			  Zielgre
	event		= 1,		  Event- Kategorie der binren Zielgre (0/1)
   *xvar		= ,			  stetige Variable(n)
   *cvar		= ,			  kategorielle Variable(n)
	ref			= ,			  Referenzkategorien fr kategorische Variablen
	miss		= 0,		  Missing Values ausschliessen (ja/ nein)
   *obs_id		= ,			  Variablenname der Beobachtungsnummern im Datensatz
	difchisq	= 100,		  Kriterium fr die Suche nach einflussreichen Beobachtungen
	graphics	= 0,		  Ausgabe der Grafiken (ja/nein)
	ext			= tiffp,	  Format fr Grafikdateien
	path		= 'c:\temp',  Pfad fr Grafikausgaben
   *macro_path  =			  Pfad zur Makrosammlung
  );

  Mit (*) gekennzeichnete Parameter sind beim Makroaufruf zwingend erforderlich.
  Bereits gesetzte Parameter stellen die jeweilige Standardeinstellung dar.

   data:		    Parameter zur Angabe des zu analysierenden Datensatzes. Wurde 
					beispielsweise vor dem Aufruf des Makros ein Datensatz mit dem 
					Namen "dat300" angelegt, lautet der korrekte Aufruf: "data=dat300". 
					Zustzlich kann auch die dazugehrige SAS-Library bergeben werden 
					(Beispiel: "data=work.dat300"). Die bergabe dieses Parameters ist 
					zwingend erforderlich.
 
   resp_var:		"resp_var" steht fr "Response Variable", also fr die Zielgre. 
					Dieser bergabeparameter ist ebenfalls zwingend erforderlich. 
					Ferner wird eine binre Datenstruktur (0/1) der Zielgre bentigt. 
					Die Zielgre muss mit "0" und "1" kodiert sein. Sollte dies nicht 
					der Fall, ist eine vorherige Umkodierung erforderlich. Heit 
					beispielsweise die Zielgre "tod" lautet der Aufruf wie folgt: 
					"resp_var=tod".

   event:			Je nach dem welche Kategorie der binren Zielgre untersucht 
					werden soll, kann diese mit Hilfe dieses Parameters bergeben werden. 
					Soll beispielsweise die Zielgre "tod" analysiert werden und steht 
					"1" fr tot sowie "0" fr lebendig, lautet der korrekte Aufruf 
					"event=1". Dies ist zugleich die Standardeinstellung, da blicherweise
					"1" fr das Eintreten eines bestimmten Ereignisses steht.

   xvar:			Dieser Parameter ist zur Angabe der stetigen Variablen 
					(Einflussgren). Allgemein gesagt sollten hier alle Variablen 
					genannt werden, die nicht in das CLASS-Statement von PROC LOGISTIC 
					aufgenommen werden sollen, also beispielsweise auch quasi-stetige 
					Variablen. Werden mehrere stetige oder quasi-stetige Variablen 
					bergeben, so mssen diese durch ein Blank (Leerzeichen) getrennt 
					werden. Beispiel: "xvar=alter bmi".

   cvar:			"cvar" steht fr "categorical variables" und ist fr die bergabe 
					aller kategorischen sowie aller dichotomen Variablen verantwortlich. 
					Diese werden innerhalb der LOGISTIC-Prozedur in das CLASS-Statement 
					aufgenommen und werden automatisch Dummy-kodiert. Werden mehrere 
					kategorische Variablen bergeben, mssen diese getrennt durch ein 
					Leerzeichen (Blank) bergeben werden (Beispiel: "cvar=diagnose msnr").
					Wurden dem Parameter "cvar" dichotome Variablen bergeben, so werden 
					diese zwar in das CLASS-Statement von PROC LOGISTIC aufgenommen, 
					jedoch keine Dummy-Kodierung durchgefhrt, da dichotome Variablen nur 
					zwei Klassen besitzen.

   ref:				Mittels dieses Parameters, der fr "reference" steht, ist es 
					mglich, pro bergebener kategorischer Variable eine sogenannte 
					Referenzkategorie anzugeben. Werden keine Referenzkategorien 
					angegeben, wird der Standard bei jeder Variablen innerhalb des 
					CLASS-Statements von PROC LOGISTIC auf REF=FIRST gesetzt. Dies 
					bedeutet, dass die erste (sortierte) Kategorie jeder bergebenen 
					Variablen als Referenzkategorie gewhlt wird. Werden 
					Referenzkategorien bergeben, so muss deren Anzahl mit der Anzahl 
					der bergebenen kategorischen Variablen bereinstimmen. SAS bietet 
					standardmig die Kategorien FIRST und LAST an. Zustzlich ist es 
					mit Hilfe dieses Parameters mglich, beliebige Kategorien anzugeben. 
					Das Trennungszeichen ist der Stern (*). Eigens gewhlte 
					Referenzkategorien mssen zustzlich in Hochkommata eingeschlossen 
					werden. Die SAS-Standardkategorien FIRST / LAST nicht. Die ntige 
					Form lautet beispielsweise: "ref='1'*FIRST*'4'". Der dazugehrige 
					"cvar"-Parameter: "cvar=icdklass3 msnr_num beginn_b_num". Von der 
					bergebenen Variablen "icdklass" wird die Kategorie "1" als 
					Referenzkategorie gewhlt. Von der Variablen "msnr_num" die erste 
					(sortierte), unabhngig was diese fr einen Wert besitzt und von der 
					Variablen "beginn_b_num" die Kategorie "4". 

   miss:			Dieser Parameter legt fest, ob Beobachtungen mit fehlenden Werten 
					(missing values) in den unter "xvar", "cvar" und "resp_var" 
					bergebenen Variablen vor der Analyse aus dem Datensatz eliminiert 
					werden sollen (0) oder nicht (1). Der Standardwert dieses Parameters 
					ist 0 (Complete Case-Analyse). Hierbei ist zu beachten, dass fehlende 
					Werte im Datensatz durch einen Punkt (.) reprsentiert sein mssen.

   obs_id:			Dieser zwingend erforderliche Parameter dient der bergabe des
					Variablennamens der Beobachtungsnummern im angegebenen Datensatz.
				    Dieser Variablenname wird bentigt, um Beobachtungen eindeutig zu
				 	identifizieren. 	

   difchisq:		Der Parameter 'difchisq' erlaubt die Angabe eines benutzer-
					spezifischen Kriteriums fr die Suche nach einflussreichen
					Beobachtungen im angegebenen Datensatz. 'difchisq' steht fr
					die Vernderung der Pearson-Chi-Quadrat-Statistik bei Entfernung
                    einer Beobachtung. Die Standardeinstellung ist difchisq=100. 

   graphics:		Der optionale Parameter 'graphics' dient der Auswahl, ob 
					zustzliche Grafiken erzeugt werden sollen (1) oder nicht (0/ 
					Standard). Wird der Parameter auf den Wert 1 gesetzt, so wird
					zu jedem Schritt der Influence Diagnostik eine Grafik in das
					unter dem Parameter 'path' angegebene Verzeichnis ausgegeben.
					Dabei werden die DIFCHISQ- Werte gegen die vorhergesagten 
					Wahrscheinlichkeiten (mit DIFBETA im Bubble-Statement) auf-
                    getragen. Das Grafikformat wird ber den Parameter 'ext' bestimmt.

   ext:				Der Parameter 'ext' (Extension) dient der Bestimmung des 
					Grafikformates, falls Grafiken angefordert wurden (Parameter
					'graphics=1'). Die Standardeinstellung ist'ext=tiffp', das
					heit, dass alle Grafiken im TIFF- Format ausgegeben werden.
					Mgliche Angaben sind:
					GIF 			GIF format 										
				 	GIF160 			GIF format (160 by 120 pixels) 					
					GIF260 			GIF format (260 by 195 pixels) 					
					GIF373 			GIF format (373 by 280 pixels) 					
					GIF570 			GIF format (570 by 480 pixels) 					
					GIF733 			GIF format (733 by 550 pixels) 					
				   	GIFANIM 		GIF animation driver 							
			 	 	IMGGIF 			GIF format 										
				 	IMGJPEG 		JPEG format 									
				 	JPEG 			JPEG format 									
				 	TIFFP 			TIFF format 									
				 	IMGTIFB 		TIFF format (black and white, uncompressed)		
				 	IMGTIFG3 		TIFF format (black and white, CCITT Group 3) 	
				 	IMGTIFG4 		TIFF format (black and white, CCITT Group 4) 	
				 	IMGTIFP 		TIFF format (256 colors, uncompressed) 			
				 	BMP 			BMP format (256 colors) (Windows host driver) 
				 	BMP20 			BMP 2.0 file format (256 colors) 				
				 	IMGBMP			BMP format 										
				 	IMGPAINT 		BMP format (MS Windows Paintbrush) 				
				 	WMF 			Windows Metafile format 						
				 	EMFDRV 			Windows Enhanced Metafile format 				
				 	PDF 			PDF format 										
				 	PDFC 			PDF format (color) 		

   path:			Dieser Parameter erlaubt die benutzerspezifische bergabe des
					Zielverzeichnisses fr die erzeugten Grafiken, falls der
					Parameter 'graphics' auf den Wert 1 gesetzt wurde. Die
					Standardeinstellung ist path='c:\temp'.
					
   macro_path: 		Aufgrund der Tatsache, dass innerhalb des Makros diverse 
					Untermakros aufgerufen werden, ist es zwingend ntig, den Pfad 
					zu dem Verzeichnis, wo die komplette Makrosammlung abgelegt ist,
					anzugeben. Der Pfad selber muss in Hochkommata eingeschlossen 
					sein. Beispiel: 'f:\makros'. 

  VALIDATION
  ----------
  Validiert durch: Stephanie Heiss
 				   University of Ulm 
                   Department of Biometry and Medical Documentation
                   89075 Ulm
  
  Validiert am:    24. November 2003

  Validierungsprogramm::      PM_INFLUENCE_VAL.SAS
  Validierungsdokumentation:  PM_INFLUENCE_VAL.DOC
   
  Validierungsvorgehen:
  - Das Testen der Makros wird durch eine von der Programmierung unabhngige 
    Person durchgefhrt.
  - Fr die Durchfhrung der Validierung wird auf einen in der Literatur 
    genutzten Datensatz zurckgegriffen:  LBW (Low Birth Weight), beschrieben
    in Hosmer, Lemeshow: Applied logistic regression [hosmer2000]
  - Der Datensatz wird fr verschiedene Testlufe verndert bzw. erweitert, 
    damit "die Testdaten neben den normalen Angaben auch Grenzwerte, ungltige
    Angaben, Fehlerbedingungen und Sonderflle beinhalten." [beiwanger2003]
  - Fehlerkontrolle:
    Hierbei werden nur die im LOG-Fenster auftretenden Fehler bzw. Warnungen 
    und offensichtlichen Fehler im OUTPUT betrachtet. Fr jeden Parameter des 
    Makros werden mehrere Durchlufe mit verschiedenen Wertangaben durchgefhrt, 
    wobei auch ungltige Werte angegeben werden, um das Verhalten des Makros
    bei fehlerhaften Aufrufen zu untersuchen. Gefundene Fehler werden 
    dokumentiert und dem Programmierer zur Verfgung gestellt. Nach der Korrektur 
    aller Fehler werden die Testlufe wiederholt, um sicherzustellen, dass 
    wirklich alle Fehler abgefangen wurden und dass keine neuen Fehler entstanden 
    sind.
  - Ergebniskontrolle:
    Um die Richtigkeit der Ergebnisse zu berprfen, werden soweit mglich, die
    Rechenbeispiele und bungsaufgaben aus Hosmer/Lemeshow durch vergleichbare
    Makroaufrufe nachgerechnet. Die Ergebnisse werden denen aus dem Begleitbuch 
    Hosmer/Lemeshow/Cook [hosmer2001] gegenbergestellt und direkt verglichen.
  - Alle Validierungen werden dokumentiert.

  Literatur: 
  - Beiwanger S (2003): Validierung von SAS-Makros. Diplomarbeit, FH Ulm
  - Hosmer DW, Lemeshow S (2000): Applied logistic regression 2nd Ed. 
           J. Wiley & Sons, New York
  - Hosmer DW, Lemeshow S, Cook ED (2001): Solutions manual to accompany
           applied logistic regression 2nd. Ed. J. Wiley & Sons, New York


  AUTHOR
  ------
  R. Muche / C. Ring / C. Ziegler
  University of Ulm
  Department of Biometry and Medical Documentation
  89075 Ulm

  The author does not overtake any responsibility for the correctness
  of results. It must not be modified without removing the heading lines 
  containing the authors name and version number.

----------------------------------------------------------------------------------*/

%MACRO PM_INFLUENCE (	data		= ,
				    	resp_var	= ,
				    	event		= 1,
				 	    xvar		= ,
				   	    cvar		= ,
						ref			= ,
						miss		= 0,
						obs_id		= ,
						difchisq	= 100,
						graphics	= 0,
						ext			= tiffp,
						path		= 'c:\temp',
						macro_path	=
					);

%LET obs=&obs_id;

/*Prfungen / Referenzierungen / Standardbelegungen*/

/*Wichtigster Parameter 'macro_path' berprfen*/
/*bergabe erfolgt?*/

%IF &macro_path= %THEN %DO;

	%WINDOW error color=grey      
     #5  @28 'Fehler!' attr=highlight 
              color=blue        
     #7  @28 "Dem zwingend erforderlichen Parameter 'macro_path' wurde kein Wert"
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.'; 
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Hilfsprogramm welches berprft, ob dem Parameter 'macro_path' ein korrekter Pfad*/
/*bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/

/*Anzahl der Zeichen im String 'macro_path' bestimmen:*/

%LET path_length=%LENGTH(&macro_path);

%LET infile=&macro_path;

%IF %bquote(&infile) NE %THEN %DO;
     %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
      %IF %bquote(&char1) ne %STR(%')
         %THEN %LET command='NOK';
         %ELSE %LET command='OK';

	 %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
      %IF %bquote(&char2) ne %STR(%')
         %THEN %LET command2='NOK';
         %ELSE %LET command2='OK';
%END;

/*falls die Syntax nicht stimmt, Anzeige eines Hinweisfensters*/

%IF &command='NOK' OR &command2='NOK' %THEN %DO;

	%WINDOW error color=grey                          
     #5  @28 'Fehler!' attr=highlight                
              color=blue                        
     #7  @28 "Dem Parameter 'macro_path' wurde ein syntaktisch falscher Wert" 
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                       
    %DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;


/*Test, ob das unter 'macro_path' angegebene Verzeichnis existiert*/

%LET dir=&macro_path;

%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
       %PUT Directory does not exist;
%END;
%ELSE %DO;
       %PUT Directory exists;
       %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;

	%WINDOW error color=grey                                                       
     #5  @28 'Fehler!' attr=highlight        
              color=blue                                                      
     #7  @28 "Das unter dem Parameter 'macro_path' angegebene Verzeichnis existiert"
	 #8  @28 "nicht- Bentigte Untermakros knnen nicht referenziert werden"	 
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                        
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Falls bestimmte Parameter ohne Wert bergeben wurden, diese mit Standard belegen*/

%IF &miss= %THEN %DO;
	%LET miss=0;
%END;

%IF &ext= %THEN %DO;
	%LET ext=tiffp;
%END;

%IF &difchisq= %THEN %DO;
	%LET difchisq=100;
%END;

%IF &event= %THEN %DO;
	%LET event=1;
%END;

%IF &graphics= %THEN %DO;
	%LET graphics=0;
%END;

%IF &path= %THEN %DO;
	%LET path='c:\temp';
%END;

/*Externe Makros, die innerhalb des Makros 'PM_DESCRIPTION.MAC.SAS'*/
/*verwendet werden (Untermakros) einbinden (%include)*/

FILENAME y &macro_path;

%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('words.mac.sas');
%INCLUDE y ('exist.mac.sas');
%INCLUDE y ('exist_var.mac.sas');
%INCLUDE y ('numchar.mac.sas');
%INCLUDE y ('ref.mac.sas');
%INCLUDE y ('fl_heinze.mac.sas');
%INCLUDE y ('dummy_friendly.mac.sas');

/*globale Variable des Untermakros 'numberobs.mac.sas'*/

%GLOBAL nnnn1;

/*Test, ob bei bestimmten Parametern anstatt numerischen Werten*/
/*character- Werte bergeben wurden*/

/*berprfung erfolgt mit Hilfe des Untermakros 'numchar.mac.sas'*/

%NUMCHAR(parameterliste=&difchisq &graphics &event);

%IF &numcharok='NOK' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error8.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Parameter, bei denen keine negativen Werte erlaubt sind, berprfen*/

%IF &difchisq <=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error9.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfen, ob der angegebene Datensatz (Parameter 'data=') auch wirklich existiert*/
/*(mit Hilfe des Hilfs-/Untermakros 'exist.mac.sas')*/

%EXIST(dsn=&data);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error4.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Datenquelle (Parameter 'data=' UND response-Variable*/
/*(Zielgre / Parameter 'resp_var=') angegeben wurde*/

%IF &data= OR &resp_var= %THEN %DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

	%INCLUDE y ('error1.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob berhaupt Variablen (Parameter 'xvar=' / 'cvar=') angegeben wurden*/

%IF &xvar= AND &cvar= %THEN %DO;

/*wenn keine Variablen angegeben wurden,*/
/*Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

		%INCLUDE y ('error2.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;
%END;

/*berprfung, ob die Zielgre (Parameter 'resp_var')*/
/*2 Kategorien (Ausprgungen) besitzt*/

/*missing values ausschliessen*/

DATA freq2;
	SET &data;
	IF &resp_var ne .;
RUN;

/*Anzahl der Ausprgungen mit Hilfe der Prozedur PROC FREQ ermitteln*/

/*Ausgabe unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnis als ODS- Datei zur weiteren Verarbeitung ausgeben*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=freq2;
	TABLES &resp_var;
RUN;

/*ODS- Output schliessen*/

ODS OUTPUT CLOSE;

/*ODS- Ausgabe wieder aktivieren*/

ODS LISTING;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen (= # Ausprgungen der Zielgre)*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq freq2;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

/*falls Zielgre mehr als zwei Ausprgungen besitz--> Fehlermeldung*/

%IF &event_check > 2 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error5.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Prfung, ob alle bergebenen Variablen numerisch sind*/

/*Erzeugung eines temporren Zwischendatensatzes, welcher lediglich die*/
/*bergebenen Variablen enthlt*/

DATA test3;
	SET &data;
	KEEP &xvar &resp_var &cvar &obs;
RUN;

/*mittels der Prozedur PROC CONTENTS kann der Typ einer Variablen bestimmt werden*/

/*Unterdrckung der Ausgabe von PROC CONTENTS*/

PROC CONTENTS DATA=test3 NOPRINT OUT=testcontent;
RUN;

/*lediglich die Variable 'type' der Ergebnisdatei behalten,*/
/*da nur diese von Interesse ist*/

DATA testcontent;
	SET testcontent;
	KEEP type;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=testcontent);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test4=&nnnn1;

/*Prfung ob Variablen vom Typ 'character' vorhanden sind*/

DATA test4;
	SET testcontent;
	IF type=2 THEN DELETE;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test4);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test5=&nnnn1;

/*falls Variablen bergeben wurden, die nicht numerisch sind --> Fehlermeldung*/

%IF &test4 ne &test5 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error60.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE test4 testcontent test3;
RUN;
QUIT;

/*berprfung, ob alle bergebenen Variablen im bergebenen Datensatz*/
/*(Parameter 'data' vorhanden sind)*/

/*Zusammenfassung aller Variablen in einer neuen Makro- Variablen*/

%LET all_variables= &resp_var &xvar &cvar &obs;

%EXIST_VAR(dsn=&data, var=&all_variables);

/*falls irgendwelche Variablen nicht existieren, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error6.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Parameter 'miss' 0 oder 1 ist*/

%IF &miss ne 1  %THEN %DO;
	%IF &miss ne 0  %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error7.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'event' 0 oder 1 ist*/

%IF &event ne 1  %THEN %DO;
	%IF &event ne 0  %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error7.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'graphics' 0 oder 1 ist*/

%IF &graphics ne 1  %THEN %DO;
	%IF &graphics ne 0  %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error10.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*Anzahl der angegebenen kategoriellen Variablen ermitteln:*/

%GLOBAL c_count c_var;
%LET c_count=1;
%LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%DO %WHILE(&c_var ne);
     %LET c_count=%EVAL(&c_count+1);
     %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%END;
%LET c_count=%EVAL(&c_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/

%PUT The cvar contains &c_count variables.;

/*Anzahl der Referenzen ermitteln:*/

%GLOBAL ref_count var_ref;
%LET ref_count=1;
%LET var_ref=%QSCAN(&ref,&ref_count,*);
%DO %WHILE(&var_ref ne);
     %LET ref_count=%EVAL(&ref_count+1);
     %LET var_ref=%QSCAN(&ref,&ref_count,*);
%END;
%LET ref_count=%EVAL(&ref_count-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The refs contains &ref_count variables.;

%IF &ref_count >0 %THEN
	%DO;
		%IF &c_count ne &ref_count %THEN                                                                                  
			%DO;

/*falls Anzahl der kategoriellen Variablen grer Null*/ 
/*ist und nicht mit der Anzahl der angegebenen*/
/*Referenzkategorien bereinstimmt,*/
/*Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

			%INCLUDE y ('error3.sas');

/*Sprungmarke im Fehlerfall*/

			%GOTO ende;

   		%END;
%END;

/*Anzahl der angegebenen stetigen Variablen ermitteln:*/

%GLOBAL x_count x_var;
%LET x_count=1;
%LET x_var=%QSCAN(&xvar,&x_count,%STR( ));
%DO %WHILE(&x_var ne);
     %LET x_count=%EVAL(&x_count+1);
     %LET x_var=%QSCAN(&xvar,&x_count,%STR( ));
%END;
%LET x_count=%EVAL(&x_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/

%PUT The xvar contains &x_count variables.;


/*berprfung, ob etwaige bergebene Referenzkategorien*/
/*vorhanden sind bzw. korrekt bergeben wurden*/

%IF &cvar ne %THEN %DO;
	%IF &ref_count ne 0 %THEN %DO;

	%REF(data=&data, resp_var=&resp_var, event=&event, cvar=&cvar, ref=&ref);

	%IF &correct='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error11.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;

%END;
%END;

/*berprfung, ob dem zwingend erforderlichen*/
/*Parameter 'obs_id' ein Wert bergeben wurde*/

%IF &obs= %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error12.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

/*berprfung, ob Ausgabebildformat korrekt ist:*/
/*(nur wenn Grafiken erwnscht sind)*/

%IF &graphics=1 %THEN %DO;

%LET ext_list=GIF GIF160 GIF260 GIF373 GIF570 GIF733 GIFANIM IMGGIF IMGJPEG JPEG TIFFP IMGTIFB IMGTIFG3 IMGTIFG4 IMGTIFP BMP BMP20 IMGBMP IMGPAINT WMF EMFDRV PDF PDFC; 

%DO h=1 %TO 23;

 %LET extension=%QSCAN(&ext_list,&h,%STR( ));

 	%IF &extension = %UPCASE(&ext) %THEN %DO;

 		%LET ext_check='OK';
		%GOTO endext;

 	%END;
	%ELSE %DO;
		%LET ext_check='NOK';
	%END;

%END;

%endext:

%IF &ext_check ='NOK' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error13.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

/*Hilfsprogramm, welches berprft, ob dem Parameter 'path' ein korrekter Pfad*/
/*bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/

/*Anzahl der Zeichen im String 'path' bestimmen:*/

%LET path_length=%LENGTH(&path);

%LET infile=&path;

%IF %bquote(&infile) NE %THEN %DO;
     %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
      %IF %bquote(&char1) ne %STR(%')
         %THEN %LET command='NOK';
         %ELSE %LET command='OK';

	 %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
      %IF %bquote(&char2) ne %STR(%')
         %THEN %LET command2='NOK';
         %ELSE %LET command2='OK';
%END;

%IF &command='NOK' OR &command2='NOK' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error14.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Test, ob das angegebene Ziel-Verzeichnis existiert*/

%LET dir=&path;

%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
       %PUT Directory does not exist;
%END;
%ELSE %DO;
       %PUT Directory exists;
       %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error15.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;


%END;

/*prfen, ob Variable 'obs_id' missings enthlt*/
/*(diese mu eindeutig sein)*/

/*Erzeugung eines temporren Zwischendatensatzes, der nur diese Variable enthlt*/

DATA test;
	SET &data;
	KEEP=&obs_id;
RUN;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test1=&nnnn1;

/*Elimination aller etwaiger missing values der Variable 'obs_id'*/

DATA test2;
	SET test;
	IF &obs_id=. THEN DELETE;
RUN;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test2);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test2=&nnnn1;

%IF &test1 ne &test2 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error59.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE test test2;
RUN;
QUIT;

/*prfen ob Variable 'obs_id eindeutig ist*/

/*Output von PROC FREQ unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnis in eine ODS- Datei ausgeben*/

ODS OUTPUT OneWayFreqs=freqs;
	PROC FREQ DATA=&data;
		TABLES &obs_id;
	RUN;

/*ODS- Output schliessen*/

ODS OUTPUT CLOSE;

/*Output wieder aktivieren*/

ODS LISTING;

/*Variable 'frequency' in der Ergebnisdatei enthlt die gesuchte Information*/

DATA freqs;
	SET freqs;
	KEEP frequency;
RUN;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freqs);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET freqs2=&nnnn1;

/*Jede Beobachtungsnummer darf lediglich einmal vorkommen (Eindeutigkeit)*/

DATA freqs;
	SET freqs;
	IF frequency =1;
RUN;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freqs);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET freqs=&nnnn1;

%IF &freqs2 ne &freqs %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error61.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*temporr angelegte Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE freqs;
RUN;
QUIT;

/*berprfen ob eine Variable nach Entfernung aller Beobachtungen mit missing*/
/*values nur noch eine Ausprgung besitzt*/

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values*/

DATA work.category_test;
	SET  &data;

	%DO i=1 %TO &count_x;
		%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
		IF &xvar_liste=. THEN DELETE;
	%END;

	%DO i=1 %TO &count;
		%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
		IF &cvar_liste=. THEN DELETE;
	%END;

		IF &resp_var=. THEN DELETE;

RUN;

/*Prfen ob 'resp_var' /'xvar'/ 'cvar' nur 1 Ausprgung besitzen*/

%LET category_test=0;
%LET category_test_fail=;

/*event*/

/*Anzahl der Ausprgungen mittels PROC FREQ bestimmen*/

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &resp_var;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

%IF &event_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &resp_var;

%END;

/*xvar*/

%IF &xvar ne %THEN %DO;

%DO uhgu=1 %TO &count_x;

%LET xvar_liste=%SCAN(&xvar,&uhgu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &xvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET xvar_check=&nnnn1;

%IF &xvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &xvar_liste;

%END;

%END;
%END;

/*cvar*/

%IF &cvar ne %THEN %DO;

%DO uhfu=1 %TO &count;

%LET cvar_liste=%SCAN(&cvar,&uhfu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &cvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET cvar_check=&nnnn1;

%IF &cvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &cvar_liste;

%END;

%END;
%END;

PROC DATASETS NOLIST;
	DELETE category_test;
RUN;
QUIT;

%IF &category_test=1 %THEN %DO;

	DATA category_test;
		var="&category_test_fail";
	RUN;

	PROC PRINT DATA=category_test NOOBS LABEL;
	TITLE1 "Folgende Variablen besitzen nach Entfernung aller";
	TITLE2 "missing values lediglich eine Ausprgung:";
		VAR var;
			LABEL var='Variable';
	RUN;

	PROC DATASETS NOLIST;
		DELETE category_test;
	RUN;
	QUIT;

	%INCLUDE y ('error24.sas');
	%GOTO ende;

%END;

/*ENDE Prfungen / Referenzierungen / Standardbelegungen*/

/*-------------------------------------------------------------------------------*/

/*Initialisierung von Dateien fr die Beobachtungsnummern*/

DATA max_obs_gesamt; SET _NULL_;
DATA max_obs_gesamt2; SET _NULL_;

%IF &graphics=1 %THEN %DO;

/*Globale Optionen fr Plots:*/

/*device=	     : Dateiendung fr Bilder						*/
/*gsfname=grafout: Zielverzeichnis fr Grafiken					*/
/*gsfmode=replace: Falls Datei bereits existiert, berschreiben */

GOPTIONS DEVICE=&ext   GSFNAME=grafout GSFMODE=replace FONTRES=PRESENTATION KEYMAP=NONE 
         FTEXT="Arial" HTITLE=1 HTEXT=1 XMAX=6 in YMAX=6 in XPIXELS=1800 YPIXELS=1800;

%END;

/*-------------------------------------------------------------------------------*/

/*Standardoutput ausgeben*/

%LET datensatz=&data;

%IF &miss=1 %THEN %DO;

/*Fallzahl mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen:*/

%NUMBEROBS(datensatz=&data);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET fallzahl=&nnnn1;

%END;

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls miss=0 angegeben wurde*/

DATA work.fallzahl;
	SET  &data;

		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

RUN;

%LET fallzahldata=fallzahl;

%NUMBEROBS(datensatz=&fallzahldata);

%LET fallzahl=&nnnn1;

%END;

/*Makro-kopf ausgeben*/

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatz       ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

/*temporr angelegte Zwischendatei(en) lschen*/

PROC DATASETS NOLIST;
	DELETE standardoutput 

	%IF &miss=0 %THEN %DO;

	fallzahl

	%END;

	;
RUN;
QUIT;

/*ENDE Standardoutput*/

/*----------------------------------------------------------------------*/

/*Elimination aller missing values des Datensatzes falls miss=0 angegeben wurde*/

%IF &miss=0 %THEN
	%DO;

		DATA work.nomiss;
		SET  &data;
		%DO i=1 %TO &x_count;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &c_count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

		%LET data=nomiss; 
		RUN;
%END;

/*Konvergenzstatus von PROC LOGISTIC #1 berprfen*/

ODS LISTING CLOSE;

ODS OUTPUT ConvergenceStatus=cs;

/*Proc Logistic*/

PROC LOGISTIC DATA=&data NAMELEN=32 OUTEST=fl_fail_est;

/*Falls kategorielle Variablen angegeben wurden Aufnahme in CLASS- Statement*/

%IF &cvar ne %THEN
%DO;


%IF &ref_count=0 %THEN
	%DO;
		CLASS

	%DO i=1 %TO &c_count;
		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
      
		&var_liste (PARAM=REF REF=FIRST)

	%END;
     	;
%END;

%ELSE
	%DO;

/*PROC LOGISTIC CLASS- Statement mit dazugehrigen Referenzkategorien*/

		CLASS

	%DO i=1 %TO &c_count;
   		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
   		%LET ref_liste=%SCAN(&ref,&i,*);

   		&var_liste (PARAM=REF REF=&ref_liste)
	%END;
   		;
%END;


%END;


MODEL &resp_var(

					%IF &event=1 %THEN
						%DO;
							event='1'
						%END;

					%IF &event=0 %THEN
						%DO;
							event='0'
						%END;

					)

	=&cvar &xvar;

/*Daten fr Influence- Diagnostic herausschreiben*/

OUTPUT OUT=cc P=pred RESCHI=reschi RESDEV=resdev DIFCHISQ=difchisq DIFDEV=difdev H=h;
RUN;

ODS OUTPUT CLOSE;
ODS LISTING;

/*(Quasi)- Separation von PROC LOGISTIC #1 immer berprfen*/

%NUMBEROBS(datensatz=cs);

%LET cs_check=&nnnn1;

DATA est_check;
	SET cs;
	IF reason ne 'Quasi-complete separation of data points detected.';
	IF reason ne 'Complete separation of data points detected.';
RUN;

/*Test ob es berhaupt Estimates ohne Warnung gibt:*/

%NUMBEROBS (datensatz=est_check);

%LET est_count=&nnnn1;

PROC DATASETS NOLIST;
	DELETE est_check cs ;
RUN;
QUIT;

%GLOBAL status_separation;

%IF &est_count ne &cs_check %THEN %DO;

%LET status_separation=1;

%END;
%ELSE %DO;

%LET status_separation=0;

%END;

/*Falls (quasi-) separation aufgetreten ist:*/
/*Korrektur:*/

%IF &status_separation=1 %THEN %DO;

/*Daten fr Influence- Diagnostic wieder lschen*/

PROC DATASETS NOLIST;
	DELETE cc;
RUN;
QUIT;

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count_c varc;
%LET count_c=1;
%LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%DO %WHILE(&varc ne);
     %LET count_c=%EVAL(&count_c+1);
     %LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%END;
%LET count_c=%EVAL(&count_c-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The cvar contains &count_c variables.;

/*Dummy- Kodierung (nicht automatisch via CLASS- Statement bei PROC LOGISTIC*/
/*---> Heinze- Makro FL kennt keine kategorischen (klassierten) Variablen*/

%LET all_variables=;

DATA ergebnis_ende (KEEP=&resp_var &cvar &xvar);
	SET &data;
RUN;

/*Referenzen zu jeweiliger Variable zuordnen*/

%DO m=1 %TO &count_c;
	%LET var=%SCAN(&cvar,&m,%STR( ));
	%LET var_ref=%SCAN(&ref,&m,*);

/*Ausprgungen der jeweiligen kategoriellen Variable ermitteln*/

PROC FREQ DATA=&data NOPRINT;
	TABLES &var /  SPARSE OUT=&var;
RUN;

/*Ausgabedatensatz von PROC FREQ anpassen*/

DATA &var (DROP=percent count);
	SET &var;
	IF &var=. THEN DELETE;
	IF count=0 THEN DELETE;
RUN;

/*Anzahl der mglichen Ausprgungen je kategorieller Variable ermitteln*/
	
	%NUMBEROBS(datensatz=&var);

	%LET count_var=&nnnn1;

		%LET underscore=_;

			/*Falls keine Referenzkategorie angegeben wurde, die erste nehmen*/
	
			%IF &ref_count=0 %THEN %DO;
				%LET var_ref=_FIRST_;
			%END;

			/*Referenzkategorie "FIRST" fr bergabe an Dummy- Makro anpassen ("_FIRST_")*/

			%IF &var_ref=FIRST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

			/*Referenzkategorie "LAST" fr bergabe an Dummy- Makro anpassen ("_LAST_")*/

	        %IF &var_ref=LAST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

			/*jeweilige Basiskategorie lschen*/

				DATA var2;
					SET &var end=_eof_;
					%IF 		&var_ref = _FIRST_ | &var_ref = LOW 
					%THEN 		%STR( IF _n_ = 1 THEN DELETE;);
					%ELSE %IF 	&var_ref = _LAST_ | &var_ref = HIGH
					%THEN 		%STR( IF _eof_ THEN DELETE;);
					%ELSE 		%STR(IF &var = &var_ref THEN DELETE;);
				RUN;

			/*Anzahl Kategorien der jeweiligen Variable ermitteln*/

				%NUMBEROBS(datensatz=var2);

			/*mgliche Ausprgungen pro Variable als Makro- Variable zwischenspeichern*/

			%LET count_var2=&nnnn1;

			DATA var2;
				SET var2;
				CALL SYMPUT('auspraeg'||left(_n_), trim(left(put(&var,BEST8.))));
			RUN;

/*Aufruf des Dummy- Kodierungs- Makros*/

	ODS LISTING CLOSE;

	%DUMMY( 
 	 		 data		=work.ergebnis_ende,   /* name of input dataset                  */
   			 out		=work.ergebnis_ende,   /* name of output dataset                 */
  			 var		= &var,         	   /* variable(s) to be dummied              */
  			 base		=&var_ref,     		   /* base category                          */
   			 prefix 	=,    				   /* prefix for dummy variable names        */
   			 format 	=,      			   /* format used to categorize variable     */
  			 name  		= VAL,  			   /* VAL: variable names are D_value        */
  			 fullrank	=1     				   /* Eliminate dummy for baseline category? */
   );

   ODS LISTING;


/*Falls fr eine Variable Dummies generiert wurden, Originalvariable lschen*/

	DATA ergebnis_ende (DROP=&var);
		SET ergebnis_ende;

		/*zustzliche Umbenennung (Anpassung an automatische Ausgabe der Dummy- Variablen*/
		/*von PROC LOGISTIC)*/

		%DO o=1 %TO &count_var2;

		RENAME &var._&&auspraeg&o = &var&&auspraeg&o;

		%END;
	RUN;

/*Zwischendatensatz lschen*/

	PROC DATASETS NOLIST;
		DELETE &var;
	RUN;
	QUIT;


%END;

/*Mittels PROC CONTENTS die Variablen, die fr das FL- Makro selektieren*/
/*ohne Event (resp_var)*/

DATA ergebnis_ende2 (DROP=&resp_var);
	SET ergebnis_ende;
RUN;

PROC CONTENTS DATA=ergebnis_ende2 NOPRINT OUT=ergebnis2 (KEEP=name);
RUN;

%NUMBEROBS(datensatz=ergebnis2);

%LET count_ergebnis2=&nnnn1;

/*Variablennamen als Makro-Variablen speichern*/

DATA ergebnis2;
	SET ergebnis2;
	CALL SYMPUT('y'||left(_N_), name);
RUN;

/*Variablennamen fr FL- Makro zusammenfassen*/

	%DO s=1 %TO &count_ergebnis2;

	%LET all_variables=&all_variables &&y&s;
	%PUT &all_variables;
	%END;


/*Aufruf des FL- Makros*/

%FL		(	data			= ergebnis_ende, 
			y				= &resp_var, 
			varlist			= &all_variables, 
			outest			= outest_fl, 
			print			= 0
       );

/*berflssige Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE ergebnis_ende _work _outtab _offset_ _maxby _fby _estwot_ _cats_ var2 ergebnis_ende2 ergebnis2;
RUN;
QUIT;

/*Datei mit den Regressionskoeffizienten aus dem FL- Makro fr INEST- Statement bei PROC LOGISTIC*/
/*vorbereiten*/

DATA inest_fl (DROP=_rby_ _penlik_ _it_ _resp_ _noresp_ _by_ _total_ _code_);
	SET outest_fl;
	IF _code_ ne -1 THEN DELETE;
	RENAME INTERCEP=Intercept;
RUN;

/*Prfen, ob FL- Makro korrekt abgelaufen ist*/

%NUMBEROBS(datensatz=inest_fl);

%LET fl_fail_check=&nnnn1;

%IF &fl_fail_check=0 %THEN %DO;

%LET fl_false=1;

%END;

%ELSE %DO;

%LET fl_false=0;

%END;

/*Warnung ausgeben, dass Firth- Prozedur benutzt wurde*/

DATA note;
	note="";
RUN;

PROC PRINT DATA=note NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Warnung (PROC LOGISTIC):               ";
	TITLE4 ;
	TITLE5 "There is possibly a quasi-complete separation of data"; 
	TITLE6 "points. The maximum likelihood estimate may not exist.";
	TITLE7 "Validity of the model fit is questionable.";
	TITLE8 ;
	TITLE9 'Es wurde eine automatische Korrektur mittels der ';
	TITLE10 "Firth- Prozedur (Makro 'FL_HEINZE.MAC.SAS') durchgefhrt!";

	
	LABEL note = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
RUN;

PROC DATASETS NOLIST;
	DELETE note;
RUN;
QUIT;

%IF &fl_false=1 %THEN %DO;

/*Hinweis ausgeben, dass Firth- Prozedur (Korrektur) fehlgeschlagen ist*/

DATA note2;
	note2="";
RUN;

PROC PRINT DATA=note2 NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note2;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Warnung:               ";
	TITLE4 "Die Korrektur der Regressionskoeffizienten mittels der";
	TITLE5 "Firth- Prozedur (Makro 'FL_HEINZE.MAC.SAS') ist"; 
	TITLE6 "fehlgeschlagen! Evtl. wurden zu viele Koeffizienten in";
	TITLE7 "Bezug auf die Anzahl der Beobachtungen geschtzt. In der";
	TITLE8 "weiteren Analyse werden die Koeffizienten des letzten";
	TITLE9 "ML-Iterationsschrittes aus PROC LOGISTIC verwendet.";
	TITLE10 "Die Validitt dieses Modells ist fraglich!";
	
	LABEL note2 = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";

RUN;

/*temporre Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE note2;
RUN;
QUIT;

%END;

/*erneute PROC LOGISTIC mit INEST- Statement & MAXITER=0*/

PROC LOGISTIC DATA=&data NOPRINT 

%IF &fl_false=0 %THEN %DO;

INEST=inest_fl 

%END;
%ELSE %DO;

INEST=fl_fail_est

%END;

NAMELEN=32;

/*Falls kategorielle Variablen angegeben wurden Aufnahme in CLASS- Statement*/

%IF &cvar ne %THEN
%DO;


%IF &ref_count=0 %THEN
	%DO;
		CLASS

	%DO i=1 %TO &c_count;
		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
      
		&var_liste (PARAM=REF REF=FIRST)

	%END;
     	;
%END;

%ELSE
	%DO;

/*PROC LOGISTIC CLASS- Statement mit dazugehrigen Referenzkategorien*/

		CLASS
	%DO i=1 %TO &c_count;
   		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
   		%LET ref_liste=%SCAN(&ref,&i,*);

   		&var_liste (PARAM=REF REF=&ref_liste)
	%END;
   		;
%END;


%END;


MODEL &resp_var(

					%IF &event=1 %THEN
						%DO;
							event='1'
						%END;

					%IF &event=0 %THEN
						%DO;
							event='0'
						%END;

					)

	=&cvar &xvar / MAXITER=0;

/*Daten fr Influence- Diagnostic herausschreiben*/

OUTPUT OUT=cc P=pred RESCHI=reschi RESDEV=resdev DIFCHISQ=difchisq DIFDEV=difdev H=h;
RUN;

PROC DATASETS NOLIST;
	DELETE outest_fl inest_fl;
RUN;
QUIT;

%END;

/*Relevante Variablen selektieren, sowie difbeta und reschistd berechnen*/

DATA influence (KEEP= &obs pred difchisq difbeta difdev h reschi resdev reschistd);
	SET cc;
	reschistd=reschi/sqrt(1-h);
	difbeta=(reschistd**2)*(h/(1-h));
RUN;

DATA influence2;
	SET influence;
RUN;

/*Alle Beobachtungen mit difchisq kleiner dem angegebenen Wert eliminieren*/

DATA influence;
	SET influence;
	IF difchisq<&difchisq THEN DELETE;
RUN;

/*Anzahl Beobachtungen, die brig geblieben sind bestimmen*/

%NUMBEROBS(datensatz=influence);
%LET influence_count=&nnnn1;

/*Falls keine Beobachtungen brig geblieben sind Meldung ausgeben*/

%IF &influence_count=0 %THEN %DO;

DATA null;
	note="Es sind keine Beobachtungen mit difchisq>&difchisq im Datensatz enthalten" ;
RUN;

PROC PRINT DATA=null NOOBS;
	TITLE "Residuenanalyse - Influence Diagnostic";
RUN;

PROC SORT DATA=influence2;
	BY DESCENDING difchisq;
RUN;

/*Beobachtungsnummer mit maximalem difchisq- Wert selektieren*/

DATA maxdifchisq (KEEP=&obs);
	SET influence2;
	IF 1=_N_;
RUN;

/*maximaler difchisq- Wert als Makro- Variable speichern*/

DATA maxdifchisq;
	SET maxdifchisq;
	CALL SYMPUT ("max_obs", &obs);
RUN;

PROC PRINT DATA=influence2 NOOBS;
	TITLE "Extremste Beobachtung mit difchisq<&difchisq:";
	WHERE &obs=&max_obs;
	VAR &obs pred difchisq difbeta difdev h reschi resdev reschistd;
RUN;

/*Datensatz lschen*/

PROC DATASETS NOLIST;
	DELETE maxdifchisq null;
	RUN;
QUIT;

%LET hiddenflag=1;

%END;

%ELSE %DO;

%LET hiddenflag=0;

/*Datensatz nach difchisq sortieren und Tabelle ausgeben*/

PROC SORT DATA=influence;
	BY DESCENDING difchisq;
RUN;

PROC PRINT DATA=influence NOOBS;
	TITLE "Residuenanalyse - Influence Diagnostic / Originaldatensatz. Kriterium: difchisq>&difchisq";
	VAR &obs pred difchisq difbeta difdev h reschi resdev reschistd;
RUN;

/*Beobachtungsnummer mit maximalem difchisq- Wert selektieren*/

DATA maxdifchisq (KEEP=&obs);
	SET influence;
	IF 1=_N_;
RUN;

/*maximaler difchisq- Wert als Makro- Variable speichern*/

DATA maxdifchisq;
	SET maxdifchisq;
	CALL SYMPUT ("max_obs", &obs);
RUN;

%LET max_obs_gesamt=&max_obs;

/*berprfen, ob string 'max_obs_gesamt' zu lang ist*/
/*alle gelschten Beobachtungen in einer Tabelle speichern*/

DATA max_obs_gesamt2;
	max_obs_gesamt=&max_obs;
RUN;

DATA max_obs_gesamt;
	SET max_obs_gesamt max_obs_gesamt2;
RUN;

/*Datensatz lschen*/

PROC DATASETS NOLIST;
	DELETE maxdifchisq;
	RUN;
QUIT;

%END;

%IF &graphics=1 %THEN %DO;

/*Datensatz fr Grafik vorbereiten*/

DATA influence_graph (KEEP= &obs pred reschi resdev h difdev difchisq reschistd difbeta);
	SET cc;
	reschistd=reschi/sqrt(1-h);
	difbeta=(reschistd**2)*(h/(1-h));
RUN;

/*Grafiken zeichnen. Influence- Statistik "difchisq" gegen */
/*modellgeschtzte Wahrscheinlichkeit (PRED) mit BUBBLE- Statement "dfbeta"*/

/*Ausgabe der Plots. Standard: TIFF- Format in den Ordner c:/temp*/

FILENAME grafout &path;

PROC GPLOT DATA=influence_graph;

BUBBLE    difchisq*pred=difbeta

  		/HAXIS=axis1
   	 	 VAXIS=axis2
    	 FRAME;

  		 TITLE1 H=1 "Influence Diagnostic Difchsq/Estimated Probability" ;

		 /*Zustzlich extremeste Beobachtungsnummer mit ausgeben*/

		 %IF &nnnn1 ne 0 %THEN %DO;
		 TITLE2 H=1 "Bubble=Difbeta / Extremste Beobachtung: Nummer &max_obs";
		 TITLE3 H=1 "Bisher eliminierte Beobachtungen: Keine";
		 %END;

		 %ELSE %DO;
		 TITLE2 H=1 "Bubble=Difbeta / Extremste Beobachtung mit difchisq<&difchisq: Nummer&max_obs";
		 TITLE3 H=1 "Eliminierte Beobachtungen: keine";
		 %END;

  		 axis1 LABEL  = ('Estimated Probability')
			   ORDER  = (0 to 1 by 0.1);
  		 axis2 LABEL  = (a=90 'Difchisq')
		 	   ;
RUN;
QUIT;

%END;


%IF &influence_count>0 %THEN %DO;

DATA el;
	SET &data;
RUN;

/*Schleife. solange im Datensatz noch Beobachtungen enthalten sind*/

%DO %WHILE (&influence_count ne 0);

/*extremste Beobachtung jeweils entfernen*/

DATA el;
	SET el;
	IF &obs=&max_obs THEN DELETE;
RUN;

/*Konvergenzstatus von PROC LOGISTIC berprfen*/

ODS LISTING CLOSE;

ODS OUTPUT ConvergenceStatus=cs;

/*erneute PROC LOGISTIC*/

PROC LOGISTIC DATA=el NAMELEN=32 OUTEST=fl_fail_est;

/*Falls kategorielle Variablen angegeben wurden Aufnahme in CLASS- Statement*/

%IF &cvar ne %THEN
%DO;


%IF &ref_count=0 %THEN
	%DO;
		CLASS

	%DO i=1 %TO &c_count;
		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
      
		&var_liste (PARAM=REF REF=FIRST)

	%END;
     	;
%END;

%ELSE
	%DO;

/*PROC LOGISTIC CLASS- Statement mit dazugehrigen Referenzkategorien*/

		CLASS
	%DO i=1 %TO &c_count;
   		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
   		%LET ref_liste=%SCAN(&ref,&i,*);

   		&var_liste (PARAM=REF REF=&ref_liste)
	%END;
   		;
%END;


%END;

/*Model- Statement*/

MODEL &resp_var(

					%IF &event=1 %THEN
						%DO;
							event='1'
						%END;

					%IF &event=0 %THEN
						%DO;
							event='0'
						%END;

					)

	=&cvar &xvar;

/*Daten fr Influence- Diagnostic herausschreiben*/

OUTPUT OUT=cc P=pred RESCHI=reschi RESDEV=resdev DIFCHISQ=difchisq DIFDEV=difdev H=h;
RUN;

ODS OUTPUT CLOSE;
ODS LISTING;

/*(Quasi)- Separation von PROC LOGISTIC  immer berprfen*/

%NUMBEROBS(datensatz=cs);

%LET cs_check=&nnnn1;

DATA est_check;
	SET cs;
	IF reason ne 'Quasi-complete separation of data points detected.';
	IF reason ne 'Complete separation of data points detected.';
RUN;

/*Test ob es berhaupt Estimates ohne Warnung gibt:*/

%NUMBEROBS (datensatz=est_check);

%LET est_count=&nnnn1;

PROC DATASETS NOLIST;
	DELETE est_check cs ;
RUN;
QUIT;

%GLOBAL status_separation;

%IF &est_count ne &cs_check %THEN %DO;

%LET status_separation=1;

%END;
%ELSE %DO;

%LET status_separation=0;

%END;

/*Falls (quasi-) separation aufgetreten ist:*/
/*Korrektur:*/

%IF &status_separation=1 %THEN %DO;

/*Daten fr Influence- Diagnostic wieder lschen*/

PROC DATASETS NOLIST;
	DELETE cc;
RUN;
QUIT;

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count_c varc;
%LET count_c=1;
%LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%DO %WHILE(&varc ne);
     %LET count_c=%EVAL(&count_c+1);
     %LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%END;
%LET count_c=%EVAL(&count_c-1);

/*Anzahl ausgeben im LOG- Fenster*/
%PUT The cvar contains &count_c variables.;

/*Dummy- Kodierung (nicht automatisch via CLASS- Statement bei PROC LOGISTIC*/
/*---> Heinze- Makro FL kennt keine kategorischen (klassierten) Variablen*/

%LET all_variables=;

DATA ergebnis_ende (KEEP=&resp_var &cvar &xvar);
	SET &data;
RUN;

/*Referenzen zu jeweiliger Variable zuordnen*/

%DO m=1 %TO &count_c;
	%LET var=%SCAN(&cvar,&m,%STR( ));
	%LET var_ref=%SCAN(&ref,&m,*);

/*Ausprgungen der jeweiligen kategoriellen Variable ermitteln*/

	PROC FREQ DATA=&data NOPRINT;
		TABLES &var /  SPARSE OUT=&var;
	RUN;

/*Ausgabedatensatz von PROC FREQ anpassen*/

	DATA &var (DROP=percent count);
		SET &var;
		IF &var=. THEN DELETE;
		IF count=0 THEN DELETE;
	RUN;

/*Anzahl der mglichen Ausprgungen je kategorieller Variable ermitteln*/
	
	%NUMBEROBS(datensatz=&var);

	%LET count_var=&nnnn1;

		%LET underscore=_;

			/*Falls keine Referenzkategorie angegeben wurde, die erste nehmen*/
	
			%IF &ref_count=0 %THEN %DO;
				%LET var_ref=_FIRST_;
			%END;

			/*Referenzkategorie "FIRST" fr bergabe an Dummy- Makro anpassen ("_FIRST_")*/

			%IF &var_ref=FIRST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

			/*Referenzkategorie "LAST" fr bergabe an Dummy- Makro anpassen ("_LAST_")*/

	        %IF &var_ref=LAST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

			/*jeweilige Basiskategorie lschen*/

				DATA var2;
					SET &var end=_eof_;
					%IF 		&var_ref = _FIRST_ | &var_ref = LOW 
					%THEN 		%STR( IF _n_ = 1 THEN DELETE;);
					%ELSE %IF 	&var_ref = _LAST_ | &var_ref = HIGH
					%THEN 		%STR( IF _eof_ THEN DELETE;);
					%ELSE 		%STR(IF &var = &var_ref THEN DELETE;);
				RUN;

			/*Anzahl Kategorien der jeweiligen Variable ermitteln*/

				%NUMBEROBS(datensatz=var2);

			/*mgliche Ausprgungen pro Variable als Makro- Variable zwischenspeichern*/

			%LET count_var2=&nnnn1;

			DATA var2;
				SET var2;
				CALL SYMPUT('auspraeg'||left(_n_), trim(left(put(&var,BEST8.))));
			RUN;

/*Aufruf des Dummy- Kodierungs- Makros*/

	ODS LISTING CLOSE;

	%DUMMY( 
 	 		 data		= work.ergebnis_ende,  /* name of input dataset                  */
   			 out		= work.ergebnis_ende,  /* name of output dataset                 */
  			 var		= &var,         	   /* variable(s) to be dummied              */
  			 base		= &var_ref,     	   /* base category                          */
   			 prefix 	= ,    				   /* prefix for dummy variable names        */
   			 format 	= ,      			   /* format used to categorize variable     */
  			 name  		= VAL,  			   /* VAL: variable names are D_value        */
  			 fullrank	= 1     			   /* Eliminate dummy for baseline category? */
   );

   ODS LISTING;

/*Falls fr eine Variable Dummies generiert wurden, Originalvariable lschen*/

	DATA ergebnis_ende (DROP=&var);
		SET ergebnis_ende;

		/*zustzliche Umbenennung (Anpassung an automatische Ausgabe der Dummy- Variablen*/
		/*von PROC LOGISTIC)*/

		%DO o=1 %TO &count_var2;

		RENAME &var._&&auspraeg&o = &var&&auspraeg&o;

		%END;
	RUN;

/*Zwischendatensatz lschen*/

	PROC DATASETS NOLIST;
		DELETE &var;
	RUN;
	QUIT;

%END;

/*Mittels PROC CONTENTS die Variablen, die fr das FL- Makro selektieren*/
/*ohne Event (resp_var)*/

DATA ergebnis_ende2 (DROP=&resp_var);
	SET ergebnis_ende;
RUN;

PROC CONTENTS DATA=ergebnis_ende2 NOPRINT OUT=ergebnis2 (KEEP=name);
RUN;

%NUMBEROBS(datensatz=ergebnis2);

%LET count_ergebnis2=&nnnn1;

/*Variablennamen als Makro-Variablen speichern*/

DATA ergebnis2;
	SET ergebnis2;
	CALL SYMPUT('y'||left(_N_), name);
RUN;

/*Variablennamen fr FL- Makro zusammenfassen*/

%DO s=1 %TO &count_ergebnis2;

	%LET all_variables=&all_variables &&y&s;
	%PUT &all_variables;

%END;


/*Aufruf des FL- Makros*/

%FL		(	data			= ergebnis_ende, 
			y				= &resp_var, 
			varlist			= &all_variables, 
			outest			= outest_fl, 
			print			= 0
       );

/*berflssige Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE ergebnis_ende _work _outtab _offset_ _maxby _fby _estwot_ _cats_ var2 ergebnis_ende2 ergebnis2;
RUN;
QUIT;

/*Datei mit den Regressionskoeffizienten aus dem FL- Makro fr INEST- Statement bei PROC LOGISTIC*/
/*vorbereiten*/

DATA inest_fl (DROP=_rby_ _penlik_ _it_ _resp_ _noresp_ _by_ _total_ _code_);
	SET outest_fl;
	IF _code_ ne -1 THEN DELETE;
	RENAME INTERCEP=Intercept;
RUN;

/*Prfen, ob FL- Makro korrekt abgelaufen ist*/

%NUMBEROBS(datensatz=inest_fl);

%LET fl_fail_check=&nnnn1;

%IF &fl_fail_check=0 %THEN %DO;

%LET fl_false=1;

%END;

%ELSE %DO;

%LET fl_false=0;

%END;

/*erneute PROC LOGISTIC mit INEST- Statement & MAXITER=0*/

PROC LOGISTIC DATA=el NOPRINT 

%IF &fl_false=0 %THEN %DO;

INEST=inest_fl 

%END;
%ELSE %DO;

INEST=fl_fail_est

%END;

NAMELEN=32;

/*Falls kategorielle Variablen angegeben wurden Aufnahme in CLASS- Statement*/

%IF &cvar ne %THEN
%DO;


%IF &ref_count=0 %THEN
	%DO;
		CLASS

	%DO i=1 %TO &c_count;
		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
      
		&var_liste (PARAM=REF REF=FIRST)

	%END;
     	;
%END;

%ELSE
	%DO;

/*PROC LOGISTIC CLASS- Statement mit dazugehrigen Referenzkategorien*/

		CLASS
	%DO i=1 %TO &c_count;
   		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
   		%LET ref_liste=%SCAN(&ref,&i,*);

   		&var_liste (PARAM=REF REF=&ref_liste)
	%END;
   		;
%END;


%END;


MODEL &resp_var(

					%IF &event=1 %THEN
						%DO;
							event='1'
						%END;

					%IF &event=0 %THEN
						%DO;
							event='0'
						%END;

					)

	=&cvar &xvar / MAXITER=0;

/*Daten fr Influence- Diagnostic herausschreiben*/

OUTPUT OUT=cc P=pred RESCHI=reschi RESDEV=resdev DIFCHISQ=difchisq DIFDEV=difdev H=h;
RUN;

PROC DATASETS NOLIST;
	DELETE outest_fl inest_fl;
RUN;
QUIT;

%END;

/*Relevante Variablen selektieren, sowie difbeta und reschistd berechnen*/

DATA influence (KEEP= &obs pred reschi resdev h difdev difchisq reschistd difbeta);
	SET cc;
	reschistd=reschi/sqrt(1-h);
	difbeta=(reschistd**2)*(h/(1-h));
RUN;

/*Alle Beobachtungen mit difchisq kleiner dem angegebenen Wert eliminieren*/

DATA influence;
	SET influence;
	IF difchisq<&difchisq THEN DELETE;
RUN;

/*Anzahl Beobachtungen, die brig geblieben sind bestimmen*/

%NUMBEROBS(datensatz=influence);
%LET influence_count=&nnnn1;

%IF &influence_count ne 0 %THEN %DO;

/*Datensatz nach difchisq sortieren und Tabelle ausgeben*/

PROC SORT DATA=influence;
	BY DESCENDING difchisq;
RUN;

PROC PRINT DATA=influence NOOBS;
	TITLE "&max_obs aus dem Datensatz eliminiert ";
	VAR &obs pred difchisq difbeta difdev h reschi resdev reschistd;
RUN;

/*Beobachtungsnummer mit maximalem difchisq- Wert selektieren*/

DATA maxdifchisq (KEEP=&obs);
	SET influence;
	IF 1=_N_;
RUN;

/*maximaler difchisq- Wert als Makro- Variable speichern*/

DATA maxdifchisq;
	SET maxdifchisq;
	CALL SYMPUT ("max_obs", &obs);
RUN;

/*Datensatz lschen*/

PROC DATASETS NOLIST;
	DELETE maxdifchisq;
	RUN;
QUIT;

%IF &graphics=1 %THEN %DO;

/*Datensatz fr Grafik vorbereiten*/

DATA influence_graph (KEEP= &obs pred reschi resdev h difdev difchisq reschistd difbeta);
	SET cc;
	reschistd=reschi/sqrt(1-h);
	difbeta=(reschistd**2)*(h/(1-h));
RUN;

/*Grafiken zeichnen. Influence- Statistik "difchisq" gegen */
/*modellgeschtzte Wahrscheinlichkeit (PRED) mit BUBBLE- Statement "dfbeta"*/

FILENAME grafout &path;

PROC GPLOT DATA=influence_graph;

BUBBLE    difchisq*pred=difbeta

  		/HAXIS=axis1
   	 	 VAXIS=axis2
    	 FRAME;


  		 TITLE1 H=1 "Influence Diagnostic Difchsq/Estimated Probability" ;
		 TITLE2 H=1 "Bubble=Difbeta / Extremste Beobachtung: Nummer&max_obs";

        /*Anzahl der Beobachtungsnummern im String max_obs_gesamt bestimmen*/
        /*falls dieser mehr als 8 Beobachtungsnummern enthlt: auteilen*/

        /*mittels Hilfsmakro %WORDS die # der Beobachtungsnummern im String bestimmen*/

		 %WORDS(&max_obs_gesamt);
		 %LET count_obs=&words;

         /*falls mehr als 8 Beobachtungsnummern enthalten sind: aufteilen zu je 8 Stck + Rest*/

		 %IF &count_obs >8 %THEN %DO;

         /*Anzahl der bentigten Titel- Statements aus der Stringlnge bestimmen*/

			%LET title=%SYSEVALF(&count_obs/8);
			%PUT &title;

			/*runden, falls keine ganze Zahl*/

			%LET title=%SYSEVALF(&title, floor);

            /*Zhler um 1 erhhen*/

			%LET title=%EVAL(&title+1);
			%PUT &title;

			%LET b=1;

			%DO a=1 %TO &title;

				%LET substring&a=;

				%DO s=&b %TO %EVAL(&a*8);

				%LET y=%SCAN(&max_obs_gesamt,&s,%STR( ));

				%LET substring&a=&&substring&a &y;

				%END;

				%LET b=%EVAL(&b+8);

		%IF &&substring&a ne %THEN %DO;

		%IF &a=1 %THEN %DO;

		%LET ch=3;

		TITLE&ch H=1 "Bisher eliminierte Beobachtungen: &&substring&a";

		%END;
		%ELSE %DO;

		%LET ch=%EVAL(&ch+1);

		TITLE&ch H=1 "&&substring&a";

		%END;
		%END;

			%END;

		 %END;

         /*falls String <=8 Beobachtungen enthlt: 1 TITLE- Anweisung*/

		 %ELSE %DO;

		 TITLE3 H=1 "Bisher eliminierte Beobachtungen: &max_obs_gesamt";

		 %END;

  		 axis1 LABEL  = ('Estimated Probability')
			   ORDER  = (0 to 1 by 0.1);
  		 axis2 LABEL  = (a=90 'Difchisq')
		 	   ;
RUN;
QUIT;

%END;

%LET max_obs_gesamt=&max_obs_gesamt &max_obs;

DATA max_obs_gesamt2;
	max_obs_gesamt=&max_obs;
RUN;

DATA max_obs_gesamt;
	SET max_obs_gesamt max_obs_gesamt2;
RUN;


%END;

%END;

/*Falls keine Beobachtungen brig geblieben sind Meldung ausgeben*/

%IF &influence_count=0 %THEN %DO;
	%IF &hiddenflag=0 %THEN %DO;

/*extremste Beobachtung jeweils entfernen*/

DATA el;
	SET el;
	IF &obs=&max_obs THEN DELETE;
RUN;

/*Konvergenzstatus von PROC LOGISTIC #1 berprfen*/

ODS LISTING CLOSE;

ODS OUTPUT ConvergenceStatus=cs;

/*erneute PROC LOGISTIC*/

PROC LOGISTIC DATA=el NAMELEN=32 OUTEST=fl_fail_est;

/*Falls kategorielle Variablen angegeben wurden Aufnahme in CLASS- Statement*/

%IF &cvar ne %THEN
%DO;


%IF &ref_count=0 %THEN
	%DO;
		CLASS

	%DO i=1 %TO &c_count;
		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
      
		&var_liste (PARAM=REF REF=FIRST)

	%END;
     	;
%END;

%ELSE
	%DO;

/*PROC LOGISTIC CLASS- Statement mit dazugehrigen Referenzkategorien*/

		CLASS
	%DO i=1 %TO &c_count;
   		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
   		%LET ref_liste=%SCAN(&ref,&i,*);

   		&var_liste (PARAM=REF REF=&ref_liste)
	%END;
   		;
%END;


%END;

/*Model- Statement*/

MODEL &resp_var(

					%IF &event=1 %THEN
						%DO;
							event='1'
						%END;

					%IF &event=0 %THEN
						%DO;
							event='0'
						%END;

					)

	=&cvar &xvar;

/*Daten fr Influence- Diagnostic herausschreiben*/

OUTPUT OUT=cc P=pred RESCHI=reschi RESDEV=resdev DIFCHISQ=difchisq DIFDEV=difdev H=h;
RUN;

ODS OUTPUT CLOSE;
ODS LISTING;

/*(Quasi)- Separation von PROC LOGISTIC  immer berprfen*/

%NUMBEROBS(datensatz=cs);

%LET cs_check=&nnnn1;

DATA est_check;
	SET cs;
	IF reason ne 'Quasi-complete separation of data points detected.';
	IF reason ne 'Complete separation of data points detected.';
RUN;

/*Test ob es berhaupt Estimates ohne Warnung gibt:*/

%NUMBEROBS (datensatz=est_check);

%LET est_count=&nnnn1;

PROC DATASETS NOLIST;
	DELETE est_check cs ;
RUN;
QUIT;

%GLOBAL status_separation;

%IF &est_count ne &cs_check %THEN %DO;

%LET status_separation=1;

%END;
%ELSE %DO;

%LET status_separation=0;

%END;

/*Falls (quasi-) separation aufgetreten ist:*/
/*Korrektur:*/

/*%LET status_separation=1;*/

%IF &status_separation=1 %THEN %DO;

/*Daten fr Influence- Diagnostic wieder lschen*/

PROC DATASETS NOLIST;
	DELETE cc;
RUN;
QUIT;

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count_c varc;
%LET count_c=1;
%LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%DO %WHILE(&varc ne);
     %LET count_c=%EVAL(&count_c+1);
     %LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%END;
%LET count_c=%EVAL(&count_c-1);

/*Anzahl ausgeben im LOG- Fenster*/
%PUT The cvar contains &count_c variables.;

/*Dummy- Kodierung (nicht automatisch via CLASS- Statement bei PROC LOGISTIC*/
/*---> Heinze- Makro FL kennt keine kategorischen (klassierten) Variablen*/

%LET all_variables=;

DATA ergebnis_ende (KEEP=&resp_var &cvar &xvar);
	SET &data;
RUN;

/*Referenzen zu jeweiliger Variable zuordnen*/

%DO m=1 %TO &count_c;
	%LET var=%SCAN(&cvar,&m,%STR( ));
	%LET var_ref=%SCAN(&ref,&m,*);

/*Ausprgungen der jeweiligen kategoriellen Variable ermitteln*/

PROC FREQ DATA=&data NOPRINT;
	TABLES &var /  SPARSE OUT=&var;
RUN;

/*Ausgabedatensatz von PROC FREQ anpassen*/

DATA &var (DROP=percent count);
	SET &var;
	IF &var=. THEN DELETE;
	IF count=0 THEN DELETE;
RUN;

/*Anzahl der mglichen Ausprgungen je kategorieller Variable ermitteln*/
	
%NUMBEROBS(datensatz=&var);

	%LET count_var=&nnnn1;

		%LET underscore=_;

			/*Falls keine Referenzkategorie angegeben wurde, die erste nehmen*/
	
			%IF &ref_count=0 %THEN %DO;
				%LET var_ref=_FIRST_;
			%END;

			/*Referenzkategorie "FIRST" fr bergabe an Dummy- Makro anpassen ("_FIRST_")*/

			%IF &var_ref=FIRST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

			/*Referenzkategorie "LAST" fr bergabe an Dummy- Makro anpassen ("_LAST_")*/

	        %IF &var_ref=LAST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

			/*jeweilige Basiskategorie lschen*/

				DATA var2;
					SET &var end=_eof_;
					%IF 		&var_ref = _FIRST_ | &var_ref = LOW 
					%THEN 		%STR( IF _n_ = 1 THEN DELETE;);
					%ELSE %IF 	&var_ref = _LAST_ | &var_ref = HIGH
					%THEN 		%STR( IF _eof_ THEN DELETE;);
					%ELSE 		%STR(IF &var = &var_ref THEN DELETE;);
				RUN;

			/*Anzahl Kategorien der jeweiligen Variable ermitteln*/

				%NUMBEROBS(datensatz=var2);

			/*mgliche Ausprgungen pro Variable als Makro- Variable zwischenspeichern*/

			%LET count_var2=&nnnn1;

			DATA var2;
				SET var2;
				CALL SYMPUT('auspraeg'||left(_n_), trim(left(put(&var,BEST8.))));
			RUN;

/*Aufruf des Dummy- Kodierungs- Makros*/

	ODS LISTING CLOSE;

	%DUMMY( 
 	 		 data		= work.ergebnis_ende,  /* name of input dataset                  */
   			 out		= work.ergebnis_ende,  /* name of output dataset                 */
  			 var		= &var,         	   /* variable(s) to be dummied              */
  			 base		= &var_ref,     	   /* base category                          */
   			 prefix 	= ,    				   /* prefix for dummy variable names        */
   			 format 	= ,      			   /* format used to categorize variable     */
  			 name  		= VAL,  			   /* VAL: variable names are D_value        */
  			 fullrank	= 1     			   /* Eliminate dummy for baseline category? */
   );

   ODS LISTING;


/*Falls fr eine Variable Dummies generiert wurden, Originalvariable lschen*/

DATA ergebnis_ende (DROP=&var);
	SET ergebnis_ende;

		/*zustzliche Umbenennung (Anpassung an automatische Ausgabe der Dummy- Variablen*/
		/*von PROC LOGISTIC)*/

		%DO o=1 %TO &count_var2;

		RENAME &var._&&auspraeg&o = &var&&auspraeg&o;

		%END;

RUN;

/*Zwischendatensatz lschen*/

	PROC DATASETS NOLIST;
		DELETE &var;
	RUN;
	QUIT;

%END;

/*Mittels PROC CONTENTS die Variablen, die fr das FL- Makro selektieren*/
/*ohne Event (resp_var)*/

DATA ergebnis_ende2 (DROP=&resp_var);
	SET ergebnis_ende;
RUN;

PROC CONTENTS DATA=ergebnis_ende2 NOPRINT OUT=ergebnis2 (KEEP=name);
RUN;

%NUMBEROBS(datensatz=ergebnis2);

%LET count_ergebnis2=&nnnn1;

/*Variablennamen als Makro-Variablen speichern*/

DATA ergebnis2;
	SET ergebnis2;
	CALL SYMPUT('y'||left(_N_), name);
RUN;

/*Variablennamen fr FL- Makro zusammenfassen*/

%DO s=1 %TO &count_ergebnis2;

	%LET all_variables=&all_variables &&y&s;
	%PUT &all_variables;

%END;


/*Aufruf des FL- Makros*/

%FL		(	data			= ergebnis_ende, 
			y				= &resp_var, 
			varlist			= &all_variables, 
			outest			= outest_fl, 
			print			= 0
       );

/*berflssige Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE ergebnis_ende _work _outtab _offset_ _maxby _fby _estwot_ _cats_ var2 ergebnis_ende2 ergebnis2;
RUN;
QUIT;

/*Datei mit den Regressionskoeffizienten aus dem FL- Makro fr INEST- Statement bei PROC LOGISTIC*/
/*vorbereiten*/

DATA inest_fl (DROP=_rby_ _penlik_ _it_ _resp_ _noresp_ _by_ _total_ _code_);
	SET outest_fl;
	IF _code_ ne -1 THEN DELETE;
	RENAME INTERCEP=Intercept;
RUN;

/*Prfen, ob FL- Makro korrekt abgelaufen ist*/

%NUMBEROBS(datensatz=inest_fl);

%LET fl_fail_check=&nnnn1;

%IF &fl_fail_check=0 %THEN %DO;

%LET fl_false=1;

%END;

%ELSE %DO;

%LET fl_false=0;

%END;

/*erneute PROC LOGISTIC mit INEST- Statement & MAXITER=0*/

PROC LOGISTIC DATA=el NOPRINT 

%IF &fl_false=0 %THEN %DO;

INEST=inest_fl 

%END;
%ELSE %DO;

INEST=fl_fail_est

%END;

NAMELEN=32;

/*Falls kategorielle Variablen angegeben wurden Aufnahme in CLASS- Statement*/

%IF &cvar ne %THEN
%DO;


%IF &ref_count=0 %THEN
	%DO;
		CLASS

	%DO i=1 %TO &c_count;
		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
      
		&var_liste (PARAM=REF REF=FIRST)

	%END;
     	;
%END;

%ELSE
	%DO;

/*PROC LOGISTIC CLASS- Statement mit dazugehrigen Referenzkategorien*/

		CLASS
	%DO i=1 %TO &c_count;
   		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
   		%LET ref_liste=%SCAN(&ref,&i,*);

   		&var_liste (PARAM=REF REF=&ref_liste)
	%END;
   		;
%END;


%END;


MODEL &resp_var(

					%IF &event=1 %THEN
						%DO;
							event='1'
						%END;

					%IF &event=0 %THEN
						%DO;
							event='0'
						%END;

					)

	=&cvar &xvar / MAXITER=0;

/*Daten fr Influence- Diagnostic herausschreiben*/

OUTPUT OUT=cc P=pred RESCHI=reschi RESDEV=resdev DIFCHISQ=difchisq DIFDEV=difdev H=h;
RUN;

PROC DATASETS NOLIST;
	DELETE outest_fl inest_fl;
RUN;
QUIT;

%END;

/*Relevante Variablen selektieren, sowie difbeta und reschistd berechnen*/

DATA influence (KEEP= &obs pred reschi resdev h difdev difchisq reschistd difbeta);
	SET cc;
	reschistd=reschi/sqrt(1-h);
	difbeta=(reschistd**2)*(h/(1-h));
RUN;

PROC SORT DATA=influence;
	BY DESCENDING difchisq;
RUN;

/*Beobachtungsnummer mit maximalem difchisq- Wert selektieren*/

DATA maxdifchisq (KEEP=&obs);
	SET influence;
	IF 1=_N_;
RUN;

/*maximaler difchisq- Wert als Makro- Variable speichern*/

DATA maxdifchisq;
	SET maxdifchisq;
	CALL SYMPUT ("max_obs_last", &obs);
RUN;

/*Datensatz lschen*/

PROC DATASETS NOLIST;
	DELETE maxdifchisq;
	RUN;
QUIT;

%IF &graphics=1 %THEN %DO;

/*Datensatz fr Grafik vorbereiten*/

DATA influence_graph (KEEP= &obs pred reschi resdev h difdev difchisq reschistd difbeta);
	SET cc;
	reschistd=reschi/sqrt(1-h);
	difbeta=(reschistd**2)*(h/(1-h));
RUN;

/*Grafiken zeichnen. Influence- Statistik "difchisq" gegen */
/*modellgeschtzte Wahrscheinlichkeit (PRED) mit BUBBLE- Statement "dfbeta"*/

FILENAME grafout &path;

PROC GPLOT DATA=influence_graph;

BUBBLE    difchisq*pred=difbeta

  		/HAXIS=axis1
   	 	 VAXIS=axis2
    	 FRAME;


  		 TITLE1 H=1 "Influence Diagnostic Difchsq/Estimated Probability" ;
		 TITLE2 H=1 "Bubble=Difbeta / Extremste Beobachtung mit difchisq<&difchisq: Nummer&max_obs_last";
		
		  /*Anzahl der Beobachtungsnummern im String max_obs_gesamt bestimmen*/
        /*falls dieser mehr als 8 Beobachtungsnummern enthlt: auteilen*/

        /*mittels Hilfsmakro %WORDS die # der Beobachtungsnummern im String bestimmen*/

		 %WORDS(&max_obs_gesamt);
		 %LET count_obs=&words;

         /*falls mehr als 8 Beobachtungsnummern enthalten sind: aufteilen zu je 8 Stck + Rest*/

		 %IF &count_obs >8 %THEN %DO;

         /*Anzahl der bentigten Titel- Statements aus der Stringlnge bestimmen*/

			%LET title=%SYSEVALF(&count_obs/8);
			%PUT &title;

			/*runden, falls keine ganze Zahl*/

			%LET title=%SYSEVALF(&title, floor);

            /*Zhler um 1 erhhen*/

			%LET title=%EVAL(&title+1);
			%PUT &title;

			%LET b=1;

			%DO a=1 %TO &title;

				%LET substring&a=;

				%DO s=&b %TO %EVAL(&a*8);

				%LET y=%SCAN(&max_obs_gesamt,&s,%STR( ));

				%LET substring&a=&&substring&a &y;

				%END;

				%LET b=%EVAL(&b+8);

        %IF &&substring&a ne %THEN %DO;
		%IF &a=1 %THEN %DO;

		%LET ch=3;

		TITLE&ch H=1 "Bisher eliminierte Beobachtungen: &&substring&a";

		%END;
		%ELSE %DO;

		%LET ch=%EVAL(&ch+1);

		TITLE&ch H=1 "&&substring&a";

		%END;
		%END;

			%END;

		 %END;

         /*falls String <=8 Beobachtungen enthlt: 1 TITLE- Anweisung*/

		 %ELSE %DO;

		 TITLE3 H=1 "Bisher eliminierte Beobachtungen: &max_obs_gesamt";

		 %END;


  		 axis1 LABEL  = ('Estimated Probability')
			   ORDER  = (0 to 1 by 0.1);
  		 axis2 LABEL  = (a=90 'Difchisq')
		 	   ;
RUN;
QUIT;

%END;

PROC PRINT DATA=max_obs_gesamt NOOBS LABEL;
	TITLE1 "&max_obs aus dem Datensatz eliminiert";
	TITLE2 ;
	TITLE3 "Es sind keine Beobachtungen mehr mit difchisq>&difchisq im Datensatz enthalten" ;
	TITLE4 "Eliminiert wurden:";
	TITLE5 ;
		VAR max_obs_gesamt;
		LABEL max_obs_gesamt='Beobachtungen';
RUN;

PROC PRINT DATA=influence NOOBS;
	TITLE "Extremste Beobachtung mit difchisq<&difchisq nach Elimination:";
	WHERE &obs=&max_obs_last;
	VAR &obs pred difchisq difbeta difdev h reschi resdev reschistd;
RUN;

%END;

/*Datensatz lschen*/

PROC DATASETS NOLIST;
	DELETE el;
	RUN;
QUIT;

%END;
%END;

%ELSE %DO;

%IF &hiddenflag=0 %THEN %DO;

/*extremste Beobachtung jeweils entfernen*/

DATA el;
	SET &data;
	IF &obs=&max_obs THEN DELETE;
RUN;

/*erneute PROC LOGISTIC*/

/*Konvergenzstatus von PROC LOGISTIC #1 berprfen*/

ODS LISTING CLOSE;

ODS OUTPUT ConvergenceStatus=cs;

PROC LOGISTIC DATA=el NAMELEN=32 OUTEST=fl_fail_est;

/*Falls kategorielle Variablen angegeben wurden Aufnahme in CLASS- Statement*/

%IF &cvar ne %THEN
%DO;


%IF &ref_count=0 %THEN
	%DO;
		CLASS

	%DO i=1 %TO &c_count;
		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
      
		&var_liste (PARAM=REF REF=FIRST)

	%END;
     	;
%END;

%ELSE
	%DO;

/*PROC LOGISTIC CLASS- Statement mit dazugehrigen Referenzkategorien*/

		CLASS
	%DO i=1 %TO &c_count;
   		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
   		%LET ref_liste=%SCAN(&ref,&i,*);

   		&var_liste (PARAM=REF REF=&ref_liste)
	%END;
   		;
%END;

%END;

/*Model- Statement*/

MODEL &resp_var(

					%IF &event=1 %THEN
						%DO;
							event='1'
						%END;

					%IF &event=0 %THEN
						%DO;
							event='0'
						%END;

					)

	=&cvar &xvar;

/*Daten fr Influence- Diagnostic herausschreiben*/

OUTPUT OUT=cc P=pred RESCHI=reschi RESDEV=resdev DIFCHISQ=difchisq DIFDEV=difdev H=h;
RUN;

ODS OUTPUT CLOSE;
ODS LISTING;

/*(Quasi)- Separation von PROC LOGISTIC  immer berprfen*/

%NUMBEROBS(datensatz=cs);

%LET cs_check=&nnnn1;

DATA est_check;
	SET cs;
	IF reason ne 'Quasi-complete separation of data points detected.';
	IF reason ne 'Complete separation of data points detected.';
RUN;

/*Test ob es berhaupt Estimates ohne Warnung gibt:*/

%NUMBEROBS (datensatz=est_check);

%LET est_count=&nnnn1;

PROC DATASETS NOLIST;
	DELETE est_check cs ;
RUN;
QUIT;

%GLOBAL status_separation;

%IF &est_count ne &cs_check %THEN %DO;

%LET status_separation=1;

%END;
%ELSE %DO;

%LET status_separation=0;

%END;

/*Falls (quasi-) separation aufgetreten ist:*/
/*Korrektur:*/

/*%LET status_separation=1;*/

%IF &status_separation=1 %THEN %DO;

/*Daten fr Influence- Diagnostic wieder lschen*/

PROC DATASETS NOLIST;
	DELETE cc;
RUN;
QUIT;

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count_c varc;
%LET count_c=1;
%LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%DO %WHILE(&varc ne);
     %LET count_c=%EVAL(&count_c+1);
     %LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%END;
%LET count_c=%EVAL(&count_c-1);

/*Anzahl ausgeben im LOG- Fenster*/
%PUT The cvar contains &count_c variables.;

/*Dummy- Kodierung (nicht automatisch via CLASS- Statement bei PROC LOGISTIC*/
/*---> Heinze- Makro FL kennt keine kategorischen (klassierten) Variablen*/

%LET all_variables=;

DATA ergebnis_ende (KEEP=&resp_var &cvar &xvar);
	SET &data;
RUN;

/*Referenzen zu jeweiliger Variable zuordnen*/

%DO m=1 %TO &count_c;
	%LET var=%SCAN(&cvar,&m,%STR( ));
	%LET var_ref=%SCAN(&ref,&m,*);

/*Ausprgungen der jeweiligen kategoriellen Variable ermitteln*/

PROC FREQ DATA=&data NOPRINT;
	TABLES &var /  SPARSE OUT=&var;
RUN;

/*Ausgabedatensatz von PROC FREQ anpassen*/

DATA &var (DROP=percent count);
	SET &var;
	IF &var=. THEN DELETE;
	IF count=0 THEN DELETE;
RUN;

/*Anzahl der mglichen Ausprgungen je kategorieller Variable ermitteln*/
	
%NUMBEROBS(datensatz=&var);

	%LET count_var=&nnnn1;

		%LET underscore=_;

			/*Falls keine Referenzkategorie angegeben wurde, die erste nehmen*/
	
			%IF &ref_count=0 %THEN %DO;
				%LET var_ref=_FIRST_;
			%END;

			/*Referenzkategorie "FIRST" fr bergabe an Dummy- Makro anpassen ("_FIRST_")*/

			%IF &var_ref=FIRST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

			/*Referenzkategorie "LAST" fr bergabe an Dummy- Makro anpassen ("_LAST_")*/

	        %IF &var_ref=LAST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

			/*jeweilige Basiskategorie lschen*/

				DATA var2;
					SET &var end=_eof_;
					%IF 		&var_ref = _FIRST_ | &var_ref = LOW 
					%THEN 		%STR( IF _n_ = 1 THEN DELETE;);
					%ELSE %IF 	&var_ref = _LAST_ | &var_ref = HIGH
					%THEN 		%STR( IF _eof_ THEN DELETE;);
					%ELSE 		%STR(IF &var = &var_ref THEN DELETE;);
				RUN;

			/*Anzahl Kategorien der jeweiligen Variable ermitteln*/

				%NUMBEROBS(datensatz=var2);

			/*mgliche Ausprgungen pro Variable als Makro- Variable zwischenspeichern*/

			%LET count_var2=&nnnn1;

			DATA var2;
				SET var2;
				CALL SYMPUT('auspraeg'||left(_n_), trim(left(put(&var,BEST8.))));
			RUN;

/*Aufruf des Dummy- Kodierungs- Makros*/

	ODS LISTING CLOSE;

	%DUMMY( 
 	 		 data		= work.ergebnis_ende,  /* name of input dataset                  */
   			 out		= work.ergebnis_ende,  /* name of output dataset                 */
  			 var		= &var,         	   /* variable(s) to be dummied              */
  			 base		= &var_ref,     	   /* base category                          */
   			 prefix 	= ,    				   /* prefix for dummy variable names        */
   			 format 	= ,      			   /* format used to categorize variable     */
  			 name  		= VAL,  			   /* VAL: variable names are D_value        */
  			 fullrank	= 1     			   /* Eliminate dummy for baseline category? */
   );

   ODS LISTING;


/*Falls fr eine Variable Dummies generiert wurden, Originalvariable lschen*/

DATA ergebnis_ende (DROP=&var);
	SET ergebnis_ende;

		/*zustzliche Umbenennung (Anpassung an automatische Ausgabe der Dummy- Variablen*/
		/*von PROC LOGISTIC)*/

		%DO o=1 %TO &count_var2;

		RENAME &var._&&auspraeg&o = &var&&auspraeg&o;

		%END;

RUN;

/*Zwischendatensatz lschen*/

PROC DATASETS NOLIST;
	DELETE &var;
RUN;
QUIT;

%END;

/*Mittels PROC CONTENTS die Variablen, die fr das FL- Makro selektieren*/
/*ohne Event (resp_var)*/

DATA ergebnis_ende2 (DROP=&resp_var);
	SET ergebnis_ende;
RUN;

PROC CONTENTS DATA=ergebnis_ende2 NOPRINT OUT=ergebnis2 (KEEP=name);
RUN;

%NUMBEROBS(datensatz=ergebnis2);

%LET count_ergebnis2=&nnnn1;

/*Variablennamen als Makro-Variablen speichern*/

DATA ergebnis2;
	SET ergebnis2;
	CALL SYMPUT('y'||left(_N_), name);
RUN;

/*Variablennamen fr FL- Makro zusammenfassen*/

%DO s=1 %TO &count_ergebnis2;

	%LET all_variables=&all_variables &&y&s;
	%PUT &all_variables;

%END;

/*Aufruf des FL- Makros*/

%FL		(	data			= ergebnis_ende, 
			y				= &resp_var, 
			varlist			= &all_variables, 
			outest			= outest_fl, 
			print			= 0
       );

/*berflssige Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE ergebnis_ende _work _outtab _offset_ _maxby _fby _estwot_ _cats_ var2 ergebnis_ende2 ergebnis2;
RUN;
QUIT;

/*Datei mit den Regressionskoeffizienten aus dem FL- Makro fr INEST- Statement bei PROC LOGISTIC*/
/*vorbereiten*/

DATA inest_fl (DROP=_rby_ _penlik_ _it_ _resp_ _noresp_ _by_ _total_ _code_);
	SET outest_fl;
	IF _code_ ne -1 THEN DELETE;
	RENAME INTERCEP=Intercept;
RUN;

/*Prfen, ob FL- Makro korrekt abgelaufen ist*/

%NUMBEROBS(datensatz=inest_fl);

%LET fl_fail_check=&nnnn1;

%IF &fl_fail_check=0 %THEN %DO;

%LET fl_false=1;

%END;

%ELSE %DO;

%LET fl_false=0;

%END;

/*erneute PROC LOGISTIC mit INEST- Statement & MAXITER=0*/

PROC LOGISTIC DATA=el NOPRINT 

%IF &fl_false=0 %THEN %DO;

INEST=inest_fl 

%END;
%ELSE %DO;

INEST=fl_fail_est

%END;

NAMELEN=32;

/*Falls kategorielle Variablen angegeben wurden Aufnahme in CLASS- Statement*/

%IF &cvar ne %THEN
%DO;


%IF &ref_count=0 %THEN
	%DO;
		CLASS

	%DO i=1 %TO &c_count;
		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
      
		&var_liste (PARAM=REF REF=FIRST)

	%END;
     	;
%END;

%ELSE
	%DO;

/*PROC LOGISTIC CLASS- Statement mit dazugehrigen Referenzkategorien*/

		CLASS
	%DO i=1 %TO &c_count;
   		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
   		%LET ref_liste=%SCAN(&ref,&i,*);

   		&var_liste (PARAM=REF REF=&ref_liste)
	%END;
   		;
%END;


%END;


MODEL &resp_var(

					%IF &event=1 %THEN
						%DO;
							event='1'
						%END;

					%IF &event=0 %THEN
						%DO;
							event='0'
						%END;

					)

	=&cvar &xvar / MAXITER=0;

/*Daten fr Influence- Diagnostic herausschreiben*/

OUTPUT OUT=cc P=pred RESCHI=reschi RESDEV=resdev DIFCHISQ=difchisq DIFDEV=difdev H=h;
RUN;

PROC DATASETS NOLIST;
	DELETE outest_fl inest_fl;
RUN;
QUIT;

%END;

/*Relevante Variablen selektieren, sowie difbeta und reschistd berechnen*/

DATA influence (KEEP= &obs pred reschi resdev h difdev difchisq reschistd difbeta);
	SET cc;
	reschistd=reschi/sqrt(1-h);
	difbeta=(reschistd**2)*(h/(1-h));
RUN;

PROC SORT DATA=influence;
	BY DESCENDING difchisq;
RUN;

/*Beobachtungsnummer mit maximalem difchisq- Wert selektieren*/

DATA maxdifchisq (KEEP=&obs);
	SET influence;
	IF 1=_N_;
RUN;

/*maximaler difchisq- Wert als Makro- Variable speichern*/

DATA maxdifchisq;
	SET maxdifchisq;
	CALL SYMPUT ("max_obs_last", &obs);
RUN;

%IF &graphics=1 %THEN %DO;

/*Datensatz fr Grafik vorbereiten*/

DATA influence_graph (KEEP= &obs pred reschi resdev h difdev difchisq reschistd difbeta);
	SET cc;
	reschistd=reschi/sqrt(1-h);
	difbeta=(reschistd**2)*(h/(1-h));
RUN;

/*Grafiken zeichnen. Influence- Statistik "difchisq" gegen */
/*modellgeschtzte Wahrscheinlichkeit (PRED) mit BUBBLE- Statement "dfbeta"*/

FILENAME grafout &path;

PROC GPLOT DATA=influence_graph;

BUBBLE    difchisq*pred=difbeta

  		/HAXIS=axis1
   	 	 VAXIS=axis2
    	 FRAME;

  		 TITLE1 H=1 "Influence Diagnostic Difchsq/Estimated Probability" ;
		 TITLE2 H=1 "Bubble=Difbeta / Extremste Beobachtung mit difchisq<&difchisq: Nummer&max_obs_last";

		 /*Anzahl der Beobachtungsnummern im String max_obs_gesamt bestimmen*/
         /*falls dieser mehr als 8 Beobachtungsnummern enthlt: auteilen*/

        /*mittels Hilfsmakro %WORDS die # der Beobachtungsnummern im String bestimmen*/

		 %WORDS(&max_obs_gesamt);
		 %LET count_obs=&words;

         /*falls mehr als 8 Beobachtungsnummern enthalten sind: aufteilen zu je 8 Stck + Rest*/

		 %IF &count_obs >8 %THEN %DO;

         /*Anzahl der bentigten Titel- Statements aus der Stringlnge bestimmen*/

			%LET title=%SYSEVALF(&count_obs/8);
			%PUT &title;

			/*runden, falls keine ganze Zahl*/

			%LET title=%SYSEVALF(&title, floor);

            /*Zhler um 1 erhhen*/

			%LET title=%EVAL(&title+1);
			%PUT &title;

			%LET b=1;

			%DO a=1 %TO &title;

				%LET substring&a=;

				%DO s=&b %TO %EVAL(&a*8);

				%LET y=%SCAN(&max_obs_gesamt,&s,%STR( ));

				%LET substring&a=&&substring&a &y;

				%END;

				%LET b=%EVAL(&b+8);

		%IF &&substring&a ne %THEN %DO;
		%IF &a=1 %THEN %DO;

		%LET ch=3;

		TITLE&ch H=1 "Bisher eliminierte Beobachtungen: &&substring&a";

		%END;
		%ELSE %DO;

		%LET ch=%EVAL(&ch+1);

		TITLE&ch H=1 "&&substring&a";

		%END;
		%END;

			%END;

		 %END;

         /*falls String <=8 Beobachtungen enthlt: 1 TITLE- Anweisung*/

		 %ELSE %DO;

		 TITLE3 H=1 "Bisher eliminierte Beobachtungen: &max_obs_gesamt";

		 %END;

  		 axis1 LABEL  = ('Estimated Probability')
			   ORDER  = (0 to 1 by 0.1);
  		 axis2 LABEL  = (a=90 'Difchisq')
		 	   ;
RUN;
QUIT;

%END;

PROC PRINT DATA=max_obs_gesamt NOOBS LABEL;
	TITLE1 "&max_obs aus dem Datensatz eliminiert";
	TITLE2 ;
	TITLE3 "Es sind keine Beobachtungen mehr mit difchisq>&difchisq im Datensatz enthalten" ;
	TITLE4 "Eliminiert wurden:";
	TITLE5 ;
		VAR max_obs_gesamt;
		LABEL max_obs_gesamt='Beobachtungen';
RUN;

PROC PRINT DATA=influence NOOBS;
	TITLE "Extremste Beobachtung mit difchisq<&difchisq nach Elimination:";
	WHERE &obs=&max_obs_last;
	VAR &obs pred difchisq difbeta difdev h reschi resdev reschistd;
RUN;

/*Datensatz lschen*/

PROC DATASETS NOLIST;
	DELETE el;
	RUN;
QUIT;

%END;

%END;

/*Datenstze lschen*/

PROC DATASETS NOLIST;
	DELETE cc influence influence2 max_obs_gesamt max_obs_gesamt2 fl_fail_est; 

	%IF &graphics=1 %THEN %DO;

	DELETE influence_graph;

	%END;

	%IF &miss=0 %THEN %DO;

	DELETE nomiss;

	%END;

RUN;
QUIT;

%ende:

%MEND PM_INFLUENCE;
