/************************************************************************************/
/*																				  	*/
/*	NAME:		pm_logreg.mac.sas													*/
/*  TITLE: 		Multiple Logistische Regression										*/
/*  KEYS:		logistic regression, variables selection, selection method			*/
/*				(quasi-) complete separation, firth procedure						*/
/*	VERSION:	SAS 8.2																*/
/* 	PRODUCT:	BASE, STAT, IML														*/
/*  PROCS:		LOGISTIC															*/
/*																					*/
/*																					*/
/*	Verzeichnis/Dateiname:			F:\Prognose\pm_logreg.mac.sas					*/
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*	Programmierer:					Christoph Ziegler								*/
/*	Datum der Ersterstellung:		9/2002											*/
/*  Aktuelle Makro- Version:		12/2003											*/
/*																					*/
/*	Kurzbeschreibung:				Allgemeines Makro zur multiplen logistischen	*/
/*							    	Regressionanalyse mit/ ohne Variablenselektion	*/
/*							    	sowie mit Korrektur etwaiger '(quasi-) 			*/
/*							    	complete separation of datapoints' mittels 		*/
/*							    	Firth- Prozedur (FL- Makro, Heinze) bei     	*/
/*							    	Selektionsmethode 'NONE' und 'BACKWARD'     	*/
/*																					*/
/*  Verwendete externe Untermakros:	numberobs.mac.sas								*/
/*									exist.mac.sas									*/
/*									exist_var.mac.sas								*/
/*									numchar.mac.sas									*/
/*									logreg2.mac.sas									*/
/*									fl_heinze.mac.sas								*/
/*									dummy_friendly.mac.sas							*/
/*									ref.mac.sas										*/
/*									error1.sas  error2.sas  error3.sas				*/
/*									error4.sas  error5.sas  error6.sas				*/
/*								    error7.sas  error8.sas  error9.sas				*/
/*									error11.sas	error16.sas	error20.sas				*/
/*									error21.sas	error23.sas	error24.sas				*/
/*									error25.sas	error26.sas	error27.sas				*/
/*									error28.sas	error29.sas	error30.sas				*/
/*									error31.sas	error32.sas	error33.sas				*/
/*									error34.sas	error35.sas	error60.sas				*/
/*									error66.sas										*/
/*																					*/
/*	Bentigte Dateien:				Datensatz (Parameter 'data=')					*/
/*																					*/
/*																					*/
/*  Erzeugte Dateien:				work.out_pred		(falls Parameter 'pred  =1')*/
/*									work.out_roc		(falls Parameter 'roc   =1')*/
/*	(bei Standardeinstellung;		work.out_est		(falls Parameter 'est   =1')*/
/*   Dateinamen frei whlbar)		work.association	(falls Parameter 'c_wert=1')*/
/*									work.out_mi_log		(falls Parameter 'mi    =1')*/
/*																					*/
/*																					*/
/*  Programmnderungen 				Sandra Mller, 02.05.2016, Makrovariable		*/
/*  und -ergnzungen 				binwidth hinzugefgt							*/
/*  (Name, Datum, Beschreibung):     												*/
/*   																   			    */
/*																					*/
/************************************************************************************/
/*

 TITLE
 -----
 pm_logreg.mac.sas

 SAS Makro zur multiplen logistischen Regressionsanalyse

 BACKGROUND
 ----------

 Das Makro zur logistischen Regressionsanalyse ist ein allgemeingltiges Werkzeug zur 
 Entwicklung von logistischen Regressionsmodellen sowie das Kernstck des kompletten 
 Makropaketes zur Prognosemodellierung auf Basis der logistischen Regression. 
 Allgemeingltig bedeutet in diesem Zusammenhang, dass das Makro prinzipiell auf 
 beliebige Datenstze anwendbar ist. Einzige Voraussetzung ist die binre Datenstruktur 
 der Zielgre. Das Makro bietet ber den Aufruf, bzw. die Parameter, eine groe Anzahl 
 von Variations- und Auswahlmglichkeiten, die unter "Syntax" detailliert beschrieben 
 sind. SAS bietet verschiedene Prozeduren (PROC GENMOD, PROC LOGISTIC, PROC PROBIT, 
 PROC CATMOD) zu logistischen Regressionsanalysen an. Das Makro "PM_LOGREG.MAC.SAS" 
 wurde mit Hilfe von PROC LOGISTIC implementiert, da diese Prozedur die wohl mchtigste 
 und weitverbreitetste Prozedur im Bereich der logistischen Regression darstellt, die 
 auch am lngsten auf dem Markt ist [allison1999, S.18].
 
 DESCRIPTION
 -----------

 Das Makro "PM_LOGREG.MAC.SAS" besitzt diverse spezifische Eigenschaften und 
 Mglichkeiten, die zustzlich zu den normalen PROC LOGISTIC-Funktionen zur Verfgung 
 stehen und welche nachfolgend erklrt werden. 

 (I) Erzeugung korrekter Ausgabedateien:
 Wird, unabhngig vom Makro "PM_LOGREG.MAC.SAS", eine Variablenselektion mittels 
 PROC LOGISTIC durchgefhrt, so kann beispielsweise die Option OUTROC angewendet 
 werden, um die Daten fr die sptere Zeichnung einer ROC-Kurve zu erhalten. 
 Das Ergebnis ist jedoch unbefriedigend. Wird diese Option unmittelbar bei der 
 Variablenselektion angewandt, enthlt die Ausgabedatei ROC-Daten fr jeden Schritt 
 der durchgefhrten Variablenselektion. Dasselbe gilt fr andere Ausgabedateien, 
 wie z.B. "out_est" (Regressions-koeffizienten) oder "out_pred" (predicted values). 
 Um diesem Problem Abhilfe zu schaffen, msste eine zweite PROC LOGISTIC mit dem 
 finalen, entwickelten Modell sowie der Selektionsmethode "NONE" gestartet werden. 
 Dies wird innerhalb des "PM_LOGREG. MAC.SAS"-Makros automatisch bewerkstelligt.

 (II) Prfung des Konvergenzstatus:
 Tritt whrend der Durchfhrung einer logistischen Regression eine sogenannte 
 "(quasi-) complete separation of datapoints" auf (entsprechende Warnung im LOG-Fenster), 
 so sind bestimmte Modellparameter (oder mindestens einer) theoretisch unendlich 
 [heinze2002]. Dies ist des fteren ein allgemeines Problem im Zusammenhang mit 
 binren Zielgren und ist immer dann der Fall, wenn das Modell die Zielgre perfekt 
 oder fast perfekt vorhersagt, das heit die Kategorien der Zielgre trennt. 
 Das Problem der Separation tritt hufig in Zusammenhang mit kleinen Datenstzen auf, 
 die einige unbalancierte und sehr einflussreiche Risikofaktoren enthalten 
 [heinze2002]. Tritt eine solche Nicht-Konvergenz ("non-convergence") auf, bricht 
 PROC LOGISTIC automatisch den maximum Likelihood-Iterationsprozess ab und gibt die 
 Ergebnisse des letzten Iterationsprozesses aus. Diese sind jedoch nicht korrekt! 
 Es gibt verschiedene Mglichkeiten, dieses Problem zu lsen. Eine Mglichkeit wre die 
 Reduktion der Einflussgren und/ oder die Transformation stetiger Variablen in 
 Kategorielle [sas2003]. Oft ist es jedoch schwer, die Variablen, die das Problem 
 hervorrufen, zu enttarnen. Des fteren schafft auch die Wahl der Selektionsmethode 
 Abhilfe (FORWARD / STEPWISE anstatt BACKWARD versuchen).

 (III) Korrektur von Regressionskoeffizienten: 
 Das Makro "PM_LOGREG.MAC.SAS" fhrt eine automatische Korrektur der 
 Regressionskoeffizienten mit Hilfe des Makros "FL_HEINZE.MAC.SAS" [heinze2001] durch, 
 falls dieses Problem auftreten sollte (bei Variablenselektionsmethode BACKWARD und NONE).
 Das Hilfsmakro von Heinze beinhaltet eine Implementierung der Firth-Prozedur, welche 
 ursprnglich entwickelt wurde, um den Bias von Maximum Likelihood Schtzungen zu 
 reduzieren [firth1993]. Innerhalb des "FL_HEINZE.MAC.SAS"-Makros produziert diese 
 Prozedur endliche Schtzer mittels einer sogenannten "penalized maximum likelihood 
 estimation" [heinze2002]. Innerhalb des "PM_LOGREG.MAC.SAS"-Makros erfolgt eine 
 automatische Detektion etwaiger "(quasi-) complete separation of datapoints" sowie 
 eine automatische Lsung dieses Problems durch die Korrektur von 
 Regressionskoeffizienten. Dabei werden korrekte Ergebnisse erzeugt wobei jegliche 
 durchgefhrten Korrekturen durch eine entsprechende Hinweismeldung im OUTPUT-Fenster 
 angezeigt werden. Makrointern werden die korrigierten Regressionskoeffizienten an 
 PROC LOGISTIC mit den Optionen INEST= und MAXITER=0 bergeben. Bei der Korrektur von 
 Regressionskoeffizienten ist jedoch zu beachten, dass das Makro "FL_HEINZE.MAC.SAS" 
 unter Umstnden lange Laufzeiten hat, da dieses Makro in IML programmiert ist. Es besteht
 dennoch die Mglichkeit, dass eine Korrektur der Regressionskoeffizienten mit Hilfe des 
 Makros "FL_HEINZE. MAC.SAS" [Heinze, 2001] nicht mglich ist. Das Makro 
 "PM_LOGREG.MAC.SAS" erkennt diesen Fall automatisch und gibt die jeweiligen Ergebnisse, 
 welche auf den nicht korrekten Regressionskoeffizienten basieren, zusammen mit einer 
 entsprechenden Meldung ber die fehlgeschlagene Korrektur aus.

 (IV) Test-Statement:
 "PM_LOGREG.MAC.SAS" bietet die Mglichkeit, den Einfluss einzelner Variablen bzw. 
 Variablenkombinationen auf die Zielgre zu untersuchen (p-Werte). Diese Mglichkeit 
 ist besonders im Hinblick auf sogenannte Fractional Polynomials interessant, welche 
 in der Beschreibung des Makros "UNI_LOGREG.MAC.SAS" nher erlutert sind. Das 
 TEST-Statement von PROC LOGISTIC kann nicht im Zusammenhang mit dem CLASS-Statement 
 (kategorielle Variablen) genutzt werden. "PM_LOGREG.MAC.SAS" fhrt makrointern mit 
 Hilfe des Untermakros "DUMMY_FRIENDLY.MAC.SAS" [friendly1998-2001] in Abhngigkeit 
 etwaiger bergebener benutzerspezifischer Referenzkategorien eine automatische Dummy-
 Kodierung aller bergebenen kategoriellen Variablen durch und ermglicht somit die 
 Nutzung dieses TEST-Statements, wobei ebenfalls die im vorherigen Abschnitt (III) 
 erluterte automatische Konvergenzprfung bzw. die Korrektur der Regressionskoeffizienten
 (falls ntig) durchgefhrt wird. 

 (V) INEST-Statement:
 Mittels "PM_LOGREG.MAC.SAS" knnen ferner feststehende Modelle (Regressionskoeffizienten)
 bergeben werden und auf einen externen Datensatz angewendet werden (INEST-Statement). 
 Es ist ebenfalls mglich, zuerst automatisch ein Modell zu entwickeln (mit der 
 entsprechenden Variablenselektionsmethode) und dieses anschlieend auf einen zweiten 
 Datensatz anzuwenden, der dem Makro bergeben werden kann.

 (VI) Manuelle BACKWARD-Selektion mit Korrektur von Regressionskoeffizienten:
 Innerhalb des Makros "PM_LOGREG.MAC.SAS" wurde eine manuelle BACKWARD-Selektion 
 implementiert, falls das Problem einer "(quasi-) complete separation of datapoints" 
 whrend der automatischen BACKWARD-Selektion von PROC LOGISTIC auftritt. Dabei werden 
 fehlerhafte Regressionskoeffizienten automatisch mit Hilfe des FL-Makros von Georg 
 Heinze [heinze2001], wie oben beschrieben (Kapitel 4.1.2.3), korrigiert. Ist die 
 Korrektur fehlgeschlagen, so werden auch hier die jeweiligen Ergebnisse, die auf den 
 nicht korrekten Regressionskoeffizienten basieren, zusammen mit einer entsprechenden 
 Meldung ber die fehlgeschlagene Korrektur ausgegeben.

 (VII) Anpassung der predicted values innerhalb der "out_roc"-Datei:
 Werden mittels "roc=1" sowie "out_roc=" die Daten fr die sptere ROC-Analyse 
 ausgegeben, so sind die in dieser Datei enthaltenen predicted values stets vom Typ 
 "individual". Die Auswahl der gewnschten Art (CUMULATIVE, INDIVIDUAL oder 
 CROSSVALIDATE) von predicted values mit Hilfe des Parameters "predprobs_art" greift 
 innerhalb von PROC LOGISTIC nur bei der Ausgabe mittels "out_pred". Innerhalb des 
 Makros "PM_LOGREG.MAC.SAS" ist eine Funktion implementiert, welche die Art der 
 vorhergesagten Wahrscheinlichkeiten auch in der Ausgabedatei "out_roc" bercksichtigt, 
 was zur Folge hat, dass die Art der predicted values in den Ausgabedateien "out_pred" 
 und "out_roc" stets identisch sind. Dies ermglicht eine sptere ROC-Analyse auf Basis 
 der selben predicted values. 

 LITERATURE
 ----------

 - SAS Online Doc zu der Prozedur PROC LOGISTIC
 - Allison, PD (1999): Logistic regression using the SAS System. 
				SAS Institute Inc., Cary NC
 - Assfalg I (2003): Die Bootstrap- Methode zur internen Validierung von 
				Prognosemodellen, Diplomarbeit, Fachhochschule Ulm
 - Firth, D (1993). Bias reduction of maximum likelihood estimates. 
				Biometrika 80, 27-38.
 - Friendly M (1998-2001): dummy.sas, A Macro to create dummy variables 
				URL: http://www.math.yorku.ca/SCS/vcd/dummy.html (Zugriff vom 16.10.2003
 - Harrell, FE (2001): Regression Modeling Strategies. Springer, New York
 - Heinze, G (2001): Technical Report 10/1999, updated 2001: The application of 
				Firth's procedure to Cox and logistic regression. Section of Clinical 
				Biometrics, Department of Medical Computer Sciences, University of 
				Vienna, Vienna.
 - Heinze G, Schemper, M (2002): A Solution to the Problem of Separation in logistic 
				regression. Statistics in Medicine, 21, 2409 - 2419.
 - Hosmer DW, Lemeshow S (2000): Applied Logistic Regression, 2. Edition.
				Wiley-Interscience Publication, New York
 - Koch, A (1999): IMBI Uni Heidelberg, logistische Regression zur Modellierung 
				von Binrdaten, URL: http://web.urz.uni-heidelberg.de/statistik/
				sas-ah/B/LogistischeRegression.html (Zugriff vom 16.10.2003)
 - Muche, R (2003): Entwicklung und Validierung von Prognosemodellen auf Basis des 
				logistischen Regressionsmodells 2003, Entwurf Habilitationsarbeit,
                persnliche Mitteilung, Uni Ulm
 - Myers, RH (1990): Classical and Modern Regression with applications 2nd Ed. 
				PWS and Kent Publishing Company Inc., Boston
 - Sauerbrei W, Augustin N., Schumacher M. (2001): Anstze zur Bercksichtigung 
				von Modellselektionsunsicherheiten in Prognosestudien. 
				Abstract zum 47. Biometrisches Kolloquium in Homburg/ Saar, 
				20.-23. Mrz 2001, URL: http://www.med-rz.uni-sb.de/bio-metrie2001/
				biometrie2001/abstractsall.pdf (Zugriff vom 16.10.2003) 
 - So Y, (2002): SAS Institute Inc., Cary, NC, A Tutorial on Logistic Regression, 
				URL: http://www.sas.com/rnd/app/papers/logistic.pdf 
				(Zugriff vom 16.10.2003)
 - SAS Institute, Technical Support: FAQ #960, What do messages about 
				nonconvergence or separation (complete or quasicomplete) mean and 
				how can I fix it? URL: http://support.sas.com/faq/009/FAQ-00960.html 
				(Zugriff vom 16.10.2003)
 - Ziegler C (2003): Ein SAS-Makropaket zur Entwicklung und Validierung von 
				Prognosemodellen auf Basis der logistischen Regression. 
				Diplomarbeit, FH Ulm 

 SYNTAX
 ------
 [Makroaufruf und Beschreibung der zu bergebenden Parameter]

 %PM_LOGREG (
   *data			= ,					Datensatz
   *resp_var		= ,					Zielgre
	event			= 1,				Event-Kategorie der binren Zielgre (0/1)
   *xvar			= ,					stetige Variable(n)
   *cvar			= ,					kategorische Variable(n)
	ref				= ,					Referenzkategorien fr kategorische Variablen
	fxvar			= ,					feste stetige Variablen
	fcvar			= ,					feste kategorische Variablen
	miss			= 0,				Missing Values ausschliessen (ja=0/nein=1)
	sel				= NONE,				Variablenselektionsmethode
    sel_details		= 1,				Detailliertes Ergebnis einer Variablenselektion 
										(ja/nein)
	slstay			= 0.05,				Signifikanzniveau bei backward-Selektion
	slentry			= 0.05,				Signifikanzniveau bei 
										forward-/stepwise-Selektion
	clodds			= 0,				PL-Konfidenzintervalle fr Odds Ratios (ja/nein)
	alpha			= 0.05,				Signifikanzlevel der Konfidenzintervalle
										fr OR
	rsquare			= 1,				R-Quadrat-Wert ausgeben (ja/nein)
	ctable			= 0,				crossvalidierte Klassifikationstabelle
	ctable_pprob	= ,					Cutpoints fr ctable
	pred			= 1,				predicted values ausgeben (ja/nein)
	out_pred		= work.out_pred,	Ausgabedatei fr predicted values
	predprobs_art	= INDIVIDUAL,		Art der predicted values
	roc				= 1,				Daten fr ROC-Tabelle ausgeben (ja/nein)
	out_roc			= work.out_roc,		Ausgabedatei fr ROC-Daten
	est				= 1,				Regressionskoeffizienten ausgeben (ja/nein)
	out_est			= work.out_est,		Ausgabedatei fr Regressionskoeffizienten
	inest_flag		= 0,				bergabe eines Modells 
										(Regressionskoeffizienten)
	inest_data		= ,					Inputdatei (Regressionskoeffizienten)
	boot_data		= ,					Bootstrap-Datensatz
	c_wert			= 1,				Kenngren (c-Wert/ Somer's D) ausgeben 
										(ja/nein)
	c_data			= work.association,	Ausgabedatei fr Kenngren 
										(c-Wert/ Somer's D)
	test			= ,					Test des Einflusses von 
										Variablen(kombinationen)
	mi				= 0,				Ausgabe der TypeIII's fr 
										Multiple Imputation-Analyse
	out_mi			= work.out_mi_log,	Ausgabedatei fr TypeIII's
   *macro_path		= ,					Pfad zur Makrosammlung
	binwidth		= 0					Bin-Breite
 );

 Mit (*) gekennzeichnete Parameter sind beim Makroaufruf zwingend erforderlich.
 Bereits gesetzte Parameter stellen die jeweilige Standardeinstellung dar

   data:		    Parameter zur Angabe des zu analysierenden Datensatzes. Wurde 
					beispielsweise vor dem Aufruf des Makros ein Datensatz mit dem 
					Namen "dat300" angelegt, lautet der korrekte Aufruf: "data=dat300". 
					Zustzlich kann auch die dazugehrige SAS-Library bergeben werden 
					(Beispiel: "data=work.dat300"). Die bergabe dieses Parameters ist 
					zwingend erforderlich.
 
   resp_var:		"resp_var" steht fr "Response Variable", also fr die Zielgre. 
					Dieser bergabeparameter ist ebenfalls zwingend erforderlich. 
					Ferner wird eine binre Datenstruktur (0/1) der Zielgre bentigt. 
					Die Zielgre muss mit "0" und "1" kodiert sein. Sollte dies nicht 
					der Fall, ist eine vorherige Umkodierung erforderlich. Heit 
					beispielsweise die Zielgre "tod" lautet der Aufruf wie folgt: 
					"resp_var=tod".

   event:			Je nach dem welche Kategorie der binren Zielgre untersucht 
					werden soll, kann diese mit Hilfe dieses Parameters bergeben werden. 
					Soll beispielsweise die Zielgre "tod" analysiert werden und steht 
					"1" fr tot sowie "0" fr lebendig, lautet der korrekte Aufruf 
					"event=1". Dies ist zugleich die Standardeinstellung, da blicherweise
					"1" fr das Eintreten eines bestimmten Ereignisses steht.

   xvar:			Dieser Parameter ist zur Angabe der stetigen Variablen 
					(Einflussgren). Allgemein gesagt sollten hier alle Variablen 
					genannt werden, die nicht in das CLASS-Statement von PROC LOGISTIC 
					aufgenommen werden sollen, also beispielsweise auch quasi-stetige 
					Variablen. Werden mehrere stetige oder quasi-stetige Variablen 
					bergeben, so mssen diese durch ein Blank (Leerzeichen) getrennt 
					werden. Beispiel: "xvar=alter bmi".

   cvar:			"cvar" steht fr "categorical variables" und ist fr die bergabe 
					aller kategorischen sowie aller dichotomen Variablen verantwortlich. 
					Diese werden innerhalb der LOGISTIC-Prozedur in das CLASS-Statement 
					aufgenommen und werden automatisch Dummy-kodiert. Werden mehrere 
					kategorische Variablen bergeben, mssen diese getrennt durch ein 
					Leerzeichen (Blank) bergeben werden (Beispiel: "cvar=diagnose msnr").
					Wurden dem Parameter "cvar" dichotome Variablen bergeben, so werden 
					diese zwar in das CLASS-Statement von PROC LOGISTIC aufgenommen, 
					jedoch keine Dummy-Kodierung durchgefhrt, da dichotome Variablen nur 
					zwei Klassen besitzen.

   ref:				Mittels dieses Parameters, der fr "reference" steht, ist es 
					mglich, pro bergebener kategorischer Variable eine sogenannte 
					Referenzkategorie anzugeben. Werden keine Referenzkategorien 
					angegeben, wird der Standard bei jeder Variablen innerhalb des 
					CLASS-Statements von PROC LOGISTIC auf REF=FIRST gesetzt. Dies 
					bedeutet, dass die erste (sortierte) Kategorie jeder bergebenen 
					Variablen als Referenzkategorie gewhlt wird. Werden 
					Referenzkategorien bergeben, so muss deren Anzahl mit der Anzahl 
					der bergebenen kategorischen Variablen bereinstimmen. SAS bietet 
					standardmig die Kategorien FIRST und LAST an. Zustzlich ist es 
					mit Hilfe dieses Parameters mglich, beliebige Kategorien anzugeben. 
					Das Trennungszeichen ist der Stern (*). Eigens gewhlte 
					Referenzkategorien mssen zustzlich in Hochkommata eingeschlossen 
					werden. Die SAS-Standardkategorien FIRST / LAST nicht. Die ntige 
					Form lautet beispielsweise: "ref='1'*FIRST*'4'". Der dazugehrige 
					"cvar"-Parameter: "cvar=icdklass3 msnr_num beginn_b_num". Von der 
					bergebenen Variablen "icdklass" wird die Kategorie "1" als 
					Referenzkategorie gewhlt. Von der Variablen "msnr_num" die erste 
					(sortierte), unabhngig was diese fr einen Wert besitzt und von der 
					Variablen "beginn_b_num" die Kategorie "4". 

   fxvar:			Dieser Parameter ermglicht feststehende stetige Variablen zu 
					bergeben, das heit Variablen, die bei Variablenselektionen auf 
					alle Flle im Modell verbleiben sollen (zum Beispiel aus inhaltlichen 
					oder medizinischen berlegungen). Die hier bergebenen Variablen 
					haben makrointern Auswirkungen auf das INCLUDE-Statement von 
					PROC LOGISTIC, welches dazu da ist, Variablen fest in ein Modell 
					zu integrieren. "fxvar" sowie folglich auch das INCLUDE-Statement 
					von PROC LOGISTIC haben nur Effekt, wenn eine Variablenselektion 
					durchgefhrt wird, also nicht wenn SELECTION=NONE gewhlt wird. Auch 
					hier mssen mehrere Variablen durch je ein Leerzeichen getrennt 
					bergeben werden. Des weiteren mssen Variablen, die unter dem 
					Parameter "fxvar" genannt werden, auch unter dem Parameter "xvar" 
					angegeben werden.

   fcvar:			Dieser bergabeparameter ist mit "fxvar" bis auf die Art der zu 
					bergebenden Variablen identisch. Mittels "fcvar" ist es mglich, 
					feststehende kategorische und dichotome Variablen zu bergeben, das 
					heit kategorische oder binre Variablen, die bei Variablenselektionen
					auf alle Flle im Modell verbleiben sollen, das heit nicht 
					bercksichtigt werden sollen. Wie auch beim Parameter "fxvar" mssen
					Variablen, die unter "fcvar" angegeben werden, auf alle Flle unter 
					"cvar" genannt werden. Bezglich der brigen Beschreibung dieses 
					Parameters siehe "fxvar". 

   miss:			Dieser Parameter legt fest, ob Beobachtungen mit fehlenden Werten 
					(missing values) in den unter "xvar", "cvar" und "resp_var" 
					bergebenen Variablen vor der Analyse aus dem Datensatz eliminiert 
					werden sollen (0) oder nicht (1). Der Standardwert dieses Parameters 
					ist 0 (Complete Case-Analyse). Hierbei ist zu beachten, dass fehlende 
					Werte im Datensatz durch einen Punkt (.) reprsentiert sein mssen.

   sel:				"sel" steht fr "selection" und ermglicht die Auswahl der 
					Variablenselektionsmethode. Dieser Parameter greift auf die 
					SELECTION-Option des MODEL- Statements von PROC LOGISTIC zu. 
					SAS bietet die folgenden Selektionsmethoden an: BACKWARD (oder B), 
					FORWARD (oder F), NONE (oder N), STEPWISE (oder S) und SCORE. 
					Die Selektionsmethode SCORE wird im Makro jedoch nicht angeboten, 
					da diese in Zusammenhang mit kategorischen Variablen (PROC 
					LOGISTIC-CLASS-Statement) nicht anwendbar ist [SAS Online Doc., s. 
					"PROC LOGISTIC-Effect Selection Methods"]. Auerdem ist es nicht 
					mglich, den Konvergenzstatus zu prfen, falls die Selektionsmethode 
					SCORE gewhlt wurde. Der Standardwert dieses Makroparameters ist 
					"sel=NONE".

   sel_details:		Variablenselektionsprozesse knnen unter Umstnden groe Mengen an 
					Output erzeugen, da PROC LOGISTIC jeden Schritt einer Selektion 
					detailliert beschreibt und dessen Ergebnisse im OUTPUT-Fenster 
					ausgibt. "sel_details" ermglicht die Unterdrckung ("sel_details=0")
					der genauen Ergebnisse einer Variablenselektion was zur Folge hat, 
					dass lediglich die wesentlichen Ergebnisse prsentiert werden. Dazu 
					gehrt "Model Information", "Response Profile" sowie "Summary of 
					variable selection process". Wird dem Parameter "sel_details" der 
					Wert 1 bergeben, so wird der gewohnte Output angezeigt. 

   slstay:			Dieser Parameter ermglicht die Spezifizierung des Signifikanzniveaus
					bei der BACKWARD-Elimination. Der Parameter greift makrointern auf 
					die gleichnamige SLSTAY-Option beim MODEL-Statement von PROC LOGISTIC
					zu und hat nur Effekt, wenn gleichzeitig "sel=BACKWARD" angegeben wird.
					Die Werte von "slstay" sollten zwischen 0 und 1 liegen. Standardmig
					ist "slstay" auf den Wert 0.05 gesetzt. 

  slentry:			"slentry" ist im Grunde genommen identisch zu "slstay" nur mit dem 
					Unterschied, dass "slentry" das Siginifikanzniveau bei der FORWARD- 
					sowie bei der STEPWISE-Elminiation festlegt. Der Parameter greift 
					makrointern auf die gleichnamige SLENTRY-Option beim MODEL-Statement 
					von PROC LOGISTIC zu und hat nur Effekt, wenn gleichzeitig 
					"sel=STEPWISE" oder "sel=FORWARD" angegeben wird. Die Werte von 
					"slentry" sollten ebenfalls zwischen 0 und 1 liegen und die 
					Standardvorbelegung ist 0.05.

   clodds:			Mit Hilfe des Parameters "clodds" knnen zustzliche 
					Konfidenzintervalle (PL-Konfidenzintervalle, deren Berechnung auf 
					"Profile Likelihood" basiert) fr Odds Ratios angefordert werden 
					(0, Standard). Falls diese erwnscht sind, mu dem Parameter 
					"clodds" der Wert 1 bergeben werden. Standardmig liefert PROC 
					LOGISTIC stets WALD- Konfidenzintervalle fr Odds Ratios (Berechnung 
					der Konfidenzintervalle basiert auf individuellen Wald- Tests). 

   alpha:			Mittels der "alpha"-Option kann das Signifikanzlevel fr die 
					Konfidenzintervalle der Odds Ratios bestimmt werden. Die 
					Standardeinstellung ist 0.05. Im Allgemeinen sollten bergebene 
					Werte zwischen 0 und 1 liegen. Dieser Parameter zeigt stets Effekte, 
					da PROC LOGISTIC standardmig WALD-Konfidenzintervalle ausgibt. 
					"alpha" steuert ebenfalls das Signifikanzlevel fr die 
					PL-Konfidenzintervalle der Odds Ratios, falls diese mittels 
					"clodds=1" angefordert werden. 

   rsquare:			Der Parameter "rsquare" ermglicht die Ausgabe des R-Quadrat-Wertes 
					(Bestimmtheitsma) fr das angepasste Modell ("rsquare=1", Standard) 
					im Output-Fenster. Falls diese Ausgabe nicht erwnscht ist, kann 
					"rsquare=0" gesetzt werden. Der Parameter greift auf die RSQUARE-Option
					des MODEL-Statements von PROC LOGISTIC zu. 

   ctable:			"ctable" ermglicht die Ausgabe einer crossvalidierten 
					Klassifikationstabelle. Der "ctable"- Parameter greift auf die 
					gleichnamige CTABLE-Option des MODEL-Statements von PROC LOGISTIC zu, 
					welche die Beobachtungen des Datensatzes aufgrund eines Schwellenwertes
					(Cutpoints) einteilt. Dabei werden vorhergesagten event-
					Wahrscheinlichkeiten mit einem Cutpoint zwischen 0 und 1 verglichen. 
					Der Standardwert von "ctable" ist "0", das heit, dass die 
					Klassifikationstabelle nicht angefordert wird. Alternativ kann 
					"ctable=1" bergeben werden, um die Ausgabe der Klassifikationstabelle
					zu initiieren. Zu beachten ist hierbei, dass der "ctable"-Parameter 
					nur Effekt zeigt, wenn zustzlich "ctable_pprob" mit den Cutpoints/ 
					dem Cutpoint bergeben wird. 

   ctable_pprob:	Der "ctable_pprob"-Parameter bietet die Mglichkeit, einen oder mehrere
					Cutpoints zu bergeben. Die Werte mssen ebenfalls zwischen 0 und 1
					liegen. Die bergabe mehrerer Cutpoints erfolgt beispielsweise in der
					Form "ctable_pprob=0.1 0.2 0.3". Dabei mssen die einzelnen Cutpoints 
					durch ein Leerzeichen getrennt werden. Eine weitere Mglichkeit der 
					bergabe besteht durch die schrittweise Angabe der Cutpoints. Die 
					korrekte Syntax lautet beispielsweise "ctable_pprob=0.1 TO 1 BY 0.1".
					Dieser Parameter zeigt nur in Verbindung mit "ctable=1" eine Wirkung.

   pred:			Durch einen Makro-Aufruf mit dem Parameter "pred=1" ist es mglich, 
					die Ausgabetabelle mit den vorhergesagten Wahrscheinlichkeiten 
					anzufordern. Mgliche Werte sind 0 oder 1 (Standard).

   out_pred:		Mit Hilfe dieses Parameters kann ein individueller Name fr die
					Ausgabedatei angegeben werden. Der Parameter greift auf die PROC
					LOGISTIC-Option "OUTPUT OUT=" zu, der Standardwert ist "work.out_pred".
					Diese Tabelle enthlt neben den vorhergesagten Wahrscheinlichkeiten
					alle Variablen des Ausgangsdatensatzes. Der Bibliotheksname ist
					optional.

   predprobs_art:	Wie der Name schon andeutet, kann mit dieser Option die Art der 
					vorhergesagten Wahrscheinlichkeiten bestimmt werden. Der Parameter 
					greift innerhalb der PROC LOGISTIC auf die OUTPUT OUT-Option PREDPROBS 
					zu. SAS stellt drei verschiedene Arten von vorhergesagten 
					Wahrscheinlichkeiten bereit: Individuelle (INDIVIDUAL), kumulative 
					(CUMULATIVE) und crossvalidierte (CROSSVALIDATE). Um individuelle 
					vorhergesagte Wahrscheinlichkeiten auszugeben gibt es sowohl die 
					Mglichkeit dem Parameter "predprobs_art" den Wert INDIVIDUAL als auch 
					den Wert I zu bergeben. Kumulative predicted values (vorhergesagte 
					Wahrscheinlichkeiten) knnen mittels "predprobs_art=CUMULATIVE" oder 
					"predprobs_art=C" angefordert werden. Um crossvalidierte vorhergesagte 
					Wahrscheinlichkeiten zu erhalten gibt es sogar drei Mglichkeiten: 
					CROSSVALIDATE, XVALIDATE und X. Standardmig ist dieser Parameter auf 
					den Wert INDIVIDUAL gesetzt. Die jeweiligen Spaltennamen in der unter 
					"out_pred" angegebenen Datei fr die unterschiedlichen Arten der 
					vorhergesagten Wahrscheinlichkeiten lauten fr "event=1" XP_1 
					(CROSSVALIDATE), CP_1 (CUMULATIVE) oder IP_1 (INDIVIDUAL) und fr 
					"event=0" XP_0 (CROSSVALIDATE), CP_0 (CUMULATIVE) oder IP_0 
					(INDIVIDUAL)

   roc:				Ist "roc" auf den Wert 1 (Standard) gesetzt, wird eine Tabelle 
					ausgegeben mit der es mglich ist, ROC-Kurven zu zeichnen. Der 
					Parameter greift PROC LOGISTIC-intern auf die MODEL-Option OUTROC zu. 
					Darber hinaus wird zu dieser Option automatisch ROCEPS angewendet. 
					Diese Option gibt das Gruppierungskriterium fr geschtzte event- 
					Wahrscheinlichkeiten an. Das Gruppierungskriterium ist standardmig 
					bewusst sehr klein gewhlt (1E-16) um sicherzustellen, dass jede 
					Beobachtung als eigene Gruppe betrachtet wird. Dies hat den Vorteil, 
					dass sptere ROC-Kurven exakter sind. Wird die Tabelle mit den ROC-
					Daten nicht gewnscht, muss roc auf den Wert 0 gesetzt werden. 

   out_roc:			"out_roc" ermglicht einen individuellen Namen fr die ROC Tabelle 
					zu vergeben. Standardeinstellung ist "work.out_roc". Der 
					Bibliotheksname ist optional und kann auch weggelassen werden. 
					Zu beachten ist, dass dieser Parameter nur Wirkung zeigt, wenn 
					zustzlich "roc=1" gesetzt wurde.

   est:				Der Parameter "est" spricht die PROC LOGISTIC-Option OUTEST an und 
					ermglicht die Ausgabe einer Tabelle mit den Parameterschtzungen 
					(Betas, Regressionskoeffizienten). Hierfr muss "est" auf den Wert 1 
					(Standard) gesetzt werden. Ist die Tabelle nicht erwnscht, so muss 
					der Wert 0 bergeben werden. 

   out_est:			Mittels "out_est" kann ein benutzerspezifischer Name fr die 
					Tabelle mit den Parameterschtzungen vergeben werden. Standardname 
					ist "work.out_est". Wie auch bei allen anderen Tabellennamen ist die
					bergabe des Bibliotheksnamens optional. "out_est" zeigt nur Effekte, 
					wenn gleichzeitig "est" auf den Wert 1 gesetzt wird. 

   inest_flag:		Dieser Parameter ermglicht die bergabe eines festen Modells 
					(Regressionskoeffizienten) an PROC LOGISTIC und greift innerhalb des 
					Makros auf die PROC LOGISTIC-Option INEST zu. Diese Option kann 
					beispielsweise genutzt werden, um ein bestimmtes Modell auf einen 
					anderen Datensatz anzuwenden. Die Struktur der Datei, die bergeben 
					wird, sollte mit der Datei, die mittels der PROC LOGISTIC-Option 
					OUTEST erzeugt wird, bereinstimmen, obwohl dies nicht zwingend ntig 
					ist. Auf alle Flle sollte der Achsenabschnitt (Intercept) sowie die 
					Regressionskoeffizienten aller Variablen in der INEST-Datei enthalten 
					sein. "inest_flag" kann drei verschiedene Parameter bergeben werden, 
					wobei 0 der Standardwert ist und bedeutet, dass diese Funktion 
					deaktiviert ist. Wird der Wert 1 bergeben, so wird Makro-intern 
					mittels der ersten PROC LOGISTIC ein Modell entwickelt, die 
					Regressionskoeffizienten ausgegeben und diese Regressionskoeffizienten 
					in einer zweiten PROC LOGISTIC mittels INEST wieder aufgenommen. Die 
					zweite PROC LOGISTIC wird in diesem Fall zustzlich standardmig mit 
					der Option MAXITER=0 aufgerufen. Dies bedeutet, dass das Modell 
					(Parameter) nicht neu geschtzt wird. Es ist anzumerken, dass hierbei 
					eine Warnung im LOG-Fenster erscheint, die aber ignoriert werden kann, 
					da eine erneute Parameterneuschtzung ja bewusst unterdrckt werden 
					soll [assfalg2003]. Die Warnung lautet:

					WARNING:
					Convergence was not attained in 0 iterations. You may want to 
					increase the maximum number of interations (MAXITER= option) or 
					change the convergence criteria (ABSFCONV=, FCONV=, XCONV= options) 
					in the MODEL statement.

					WARNING:
					The LOGISTIC procedure continues in spite of the above warning. 
					Results shown are based on the last maximum likelihood iteration. 
					Validity of the model is questionable. 

					Ferner besteht die Mglichkeit, eine eigene Datei mit 
					Parameterschtzungen (Regressionskoeffizienten) dem Makro zu 
					bergeben. Hierfr muss "inest_flag" auf den Wert 2 gesetzt werden. 
					Zustzlich muss die entsprechende Datei zwingend unter dem Parameter 
					"inest_data" angegeben werden. Auch hierbei wird die bergebene Datei
					in das INEST-Statement von PROC LOGISTIC mit der Option MAXITER=0 
					aufgenommen. 

   inest_data:		Dieser Parameter ist ausschlielich fr die bergabe einer Datei mit
					Regressionskoeffizienten (Modell) vorhanden, falls "inest_flag=2"
					gesetzt ist. Es kann sowohl eine Datei mit zugehrigem 
					Bibliotheksnamen (Beispiel: "work.inest") als auch ohne 
					Bibliotheksnamen bergeben werden. 

   boot_data:		"boot_data" ist ein Spezialparameter fr Bootstrap-Validierungen mit 
					Variablenselektionen. Er ermglicht die bergabe eines Bootstrap-
					Datensatzes. Auf diesen wird mittels einer ersten PROC LOGISTIC eine 
					Variablenselektion angewandt und die entsprechenden Parameterschtzungen
					durch die Option OUTEST ausgegeben. Wichtig ist hierbei, dass der 
					Parameter "inest_flag" auf den Wert 1 gesetzt wird. Anschlieend wird
					eine zweite PROC LOGISTIC gestartet, die ber die Option INEST die
					Parameterschtzungen des Bootstrap-Datensatzes aufnimmt und dieses 
					Modell auf den Originaldatensatz ohne Neuschtzung (MAXITER=0) 
					anwendet. Unter Originaldatensatz wird hier derjenige Datensatz 
					verstanden, der ber den Parameter "data" angegeben wurde 
					[assfalg2003]. Bootstrap-Validierungsmethoden mit 
					Variablenselektionen sind jedoch im Rahmen dieses Makropaketes 
					bisher noch nicht implementiert, weswegen der Parameter "boot_data"
					derzeit nicht bentigt wird. 

   c_wert:			Wird dieser Parameter auf den Wert 1 gesetzt, so erfolgt die Ausgabe 
					einer Tabelle mit den Kenngren Somer's D, c-Wert, R-Square und 
					Max-rescaled-R-Square in die unter dem Parameter "c_data" 
					angegebene Datei. Die Tabelle selber wird fr den Aufruf des 
					ROC-Makros bentigt, wo diese weiterverarbeitet wird. Der 
					Standardwert ist 1. Die Tabelle selber wird innerhalb von PROC 
					LOGISTIC mittels ODS (Output Delivery System) erzeugt 
					(ODS OUTPUT ASSOCIATION=).

   c_data:			Falls der Parameter "c_wert" auf den Wert 1 gesetzt wird besteht 
					via "c_data" die Mglichkeit, einen benutzerspezifische 
					Ausgabedateinamen zu nennen. Wird diese Mglichkeit nicht 
					wahrgenommen, so lautet der Standarddateiname "work.association". 
					Auch bei diesem Parameter ist die Angabe des Bibliotheknamens 
					optional. 

   test:			Der Parameter test ermglicht das individuelle testen des 
					Einflusses von Variablen und Variablenkombinationen auf die 
					unter "resp_var" angegebene Zielgre. Wird der Parameter 
					"test" durch die bergabe von einer oder mehreren Variablen 
					aktiviert, so wird makrointern nur diese Funktion ausgefhrt. 
					Alle anderen Einstellungen zeigen keine Wirkung. Mittels dem 
					Parameter "test" knnen darber hinaus mehrere Test- Statements 
					gleichzeitig bergeben werden. Die Syntax dieses Parameters ist 
					abhngig von der Anzahl der gewnschten Tests. Wird ein 
					individueller Test gewnscht, so lautet die entsprechende 
					Parameterbergabe beispielsweise "test=x y" (Test von zwei 
					Variablen) oder "test=z" (Test von einer Variable). Werden 
					beispielsweise drei Tests angefordert, so verlangt der Parameter 
					die folgende Syntax: "test=x y TEST a b x TEST z v". Die 
					makrointerne Umsetzung dieses Test- Statements wre wie folgt:
 
					PROC LOGISTIC DATA= &data, CLASS= &cvar (Falls kategorische 
					Variablen bergeben wurden (mit etwaigen unter dem Parameter "ref" 
					bergebenen Referenzkategorien)), MODEL &resp_var (Eventkategorie,
					Parameter "event") = &cvar &xvar /SELECTION= NONE, TEST x=y=0, 
					TEST a=b=x=0, TEST z=v=0, RUN. 

					Variablen, die unter dem Parameter "test" angegeben wurden mssen 
					zustzlich unter "xvar" beziehungsweise "cvar" aufgelistet sein. 
					Ferner ist die Angabe einer gltigen response-variable (Zielgre, 
					Parameter "resp_var") sowie eines gltigen zu analysierenden 
					Datensatzes (Parameter "data") zwingend erforderlich. Der 
					Parameter "sel" wird automatisch auf den Wert NONE gesetzt 
					(Keine Variablenselektion). Da das TEST- Statement bei PROC 
					LOGISTIC nur ohne CLASS-Statement funktioniert wird eine 
					automatische manuelle Dummy-Codierung durch das Untermakro 
					"DUMMY_FRIENDLY.MAC.SAS" fr bergebene kategorischen Variablen 
					durchgefhrt. Hierbei werden individuelle, unter dem Parameter 
					"ref" bergebene Referenzkategorien bercksichtigt. Falls keine 
					benutzerspezifischen Referenzkategorien angegeben wurden, so wird 
					fr alle kategorischen Variablen die jeweils erste (sortierte) 
					Referenzkategorie gewhlt. 

   mi:				Wird der Parameter "mi" auf den Wert 1 gesetzt (Standard=0) 
					erfolgt eine Ausgabe der p-Werte (Type III's) aller Variablen 
					des finalen Modells in die unter dem Parameter "out_mi" angegebene 
					Datei. Dieser Parameter, beziehungsweise die hier angeforderte 
					Ausgabedatei, ist fr die sptere Analyse von Multiple Imputation-
					Datenstzen von Bedeutung (siehe Makro "PM_MI_ANALYZE.MAC.SAS"). 

   out_mi:			Der Parameter "out_mi" ermglicht die benutzerspezifische 
					Bestimmung des Ausgabedateinamens fr die p-Werte der Variablen 
					des finalen Modells (Type III's). Wird "mi=1" gewhlt und kein 
					individueller Dateiname angegeben wird "out_mi" standardmig 
					auf den Wert "work.out_mi_log" gesetzt. Die bergabe eines 
					Bibliotheknamens ist auch hier optional.

   macro_path: 		Aufgrund der Tatsache, dass innerhalb des Makros diverse 
					Untermakros aufgerufen werden, ist es zwingend ntig, den 
					Pfad zu dem Verzeichnis, wo die komplette Makrosammlung 
					abgelegt ist, anzugeben. Der Pfad selber muss in Hochkommata 
					eingeschlossen sein. Beispiel: 'f:\makros'. 

   binwidth:		Der Parameter "binwidth" gibt die Bin-Breite fr die Schtzung der 
					Assoziationsstatistik an. Der Wert muss dabei zwischen 0 und kleiner
					1 liegen. SAS verwendet als	Standard 0.002, ab Version 9.4 wird bei 
					binrem Endpunkt und weniger als 5 Millionen Beobachtungen jedoch 
					auf Binnen verzichtet. Der Parameter erleichtert daher den Vergleich
					der Ergebnisse unterschiedlicher SAS Versionen.

  VALIDATION
  ----------

  Validiert durch: Stephanie Heiss
 				   University of Ulm 
                   Department of Biometry and Medical Documentation
                   89075 Ulm
  
  Validiert am:    25. November 2003

  Validierungsprogramm::      PM_LOGREG_VAL.SAS
  Validierungsdokumentation:  PM_LOGREG_VAL.DOC
   
  Validierungsvorgehen:
  - Das Testen der Makros wird durch eine von der Programmierung unabhngige 
    Person durchgefhrt.
  - Fr die Durchfhrung der Validierung wird auf einen in der Literatur 
    genutzten Datensatz zurckgegriffen:  LBW (Low Birth Weight), beschrieben
    in Hosmer, Lemeshow: Applied logistic regression [hosmer2000]
  - Der Datensatz wird fr verschiedene Testlufe verndert bzw. erweitert, 
    damit "die Testdaten neben den normalen Angaben auch Grenzwerte, ungltige
    Angaben, Fehlerbedingungen und Sonderflle beinhalten." [beiwanger2003]
  - Fehlerkontrolle:
    Hierbei werden nur die im LOG-Fenster auftretenden Fehler bzw. Warnungen 
    und offensichtlichen Fehler im OUTPUT betrachtet. Fr jeden Parameter des 
    Makros werden mehrere Durchlufe mit verschiedenen Wertangaben durchgefhrt, 
    wobei auch ungltige Werte angegeben werden, um das Verhalten des Makros
    bei fehlerhaften Aufrufen zu untersuchen. Gefundene Fehler werden 
    dokumentiert und dem Programmierer zur Verfgung gestellt. Nach der Korrektur 
    aller Fehler werden die Testlufe wiederholt, um sicherzustellen, dass 
    wirklich alle Fehler abgefangen wurden und dass keine neuen Fehler entstanden 
    sind.
  - Ergebniskontrolle:
    Um die Richtigkeit der Ergebnisse zu berprfen, werden soweit mglich, die
    Rechenbeispiele und bungsaufgaben aus Hosmer/Lemeshow durch vergleichbare
    Makroaufrufe nachgerechnet. Die Ergebnisse werden denen aus dem Begleitbuch 
    Hosmer/Lemeshow/Cook [hosmer2001] gegenbergestellt und direkt verglichen.
  - Alle Validierungen werden dokumentiert.

  Literatur: 

  - Beiwanger S (2003): Validierung von SAS-Makros. Diplomarbeit, FH Ulm
  - Hosmer DW, Lemeshow S (2000): Applied logistic regression 2nd Ed. 
           J. Wiley & Sons, New York
  - Hosmer DW, Lemeshow S, Cook ED (2001): Solutions manual to accompany
           applied logistic regression 2nd. Ed. J. Wiley & Sons, New York

  AUTHOR
  ------
  R. Muche / C. Ring / C. Ziegler
  University of Ulm
  Department of Biometry and Medical Documentation
  89075 Ulm

  The author does not overtake any responsibility for the correctness
  of results. It must not be modified without removing the heading lines 
  containing the authors name.

----------------------------------------------------------------------------------*/

%MACRO PM_LOGREG	 (  	data			= ,
							resp_var		= ,
							event			= 1,
							xvar			= ,
							cvar			= ,
							ref				= ,
							fxvar			= ,
							fcvar			= ,
							miss			= 0,
							sel				= NONE,
							sel_details		= 1,
							slstay			= 0.05,
							slentry			= 0.05,
							clodds			= 0,
							alpha			= 0.05,
							rsquare			= 1,
							ctable			= 0,
							ctable_pprob	= ,
							pred			= 1,
							out_pred		= work.out_pred,
							predprobs_art	= INDIVIDUAL,
							roc				= 1,
							out_roc			= work.out_roc,
							est				= 1,
							out_est			= work.out_est,
							inest_flag		= 0,
							inest_data		= ,
							boot_data		= ,
							c_wert			= 1,
							c_data			= work.association,
							test			= ,
							mi				= 0,
							out_mi			= work.out_mi_log,
							macro_path		= ,
							binwidth		= 0
					);

/*globale Variable des Untermakros 'numberobs.mac.sas'*/

%GLOBAL nnnn1;

%LET datensatz2=&data;

/*Prfungen / Referenzierungen / Standardbelegungen*/

/*Wichtigster Parameter 'macro_path' berprfen*/
/*bergabe erfolgt?*/

%IF &macro_path= %THEN %DO;

	%WINDOW error color=grey                                                      
	 #5  @28 'Fehler!' attr=highlight
             color=blue
     #7  @28 "Dem zwingend erforderlichen Parameter 'macro_path' wurde kein Wert"
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Hilfsprogramm welches berprft, ob dem Parameter 'macro_path' ein korrekter Pfad*/
/*bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/

/*Anzahl der Zeichen im String 'macro_path' bestimmen:*/

%LET path_length=%LENGTH(&macro_path);

%LET infile=&macro_path;

%IF %bquote(&infile) NE %THEN %DO;
     %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
      %IF %bquote(&char1) ne %STR(%')
         %THEN %LET command='NOK';
         %ELSE %LET command='OK';

	 %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
      %IF %bquote(&char2) ne %STR(%')
         %THEN %LET command2='NOK';
         %ELSE %LET command2='OK';
%END;

/*falls die Syntax nicht stimmt, Anzeige eines Hinweisfensters*/

%IF &command='NOK' OR &command2='NOK' %THEN %DO;

	%WINDOW error color=grey      
	 #5  @28 'Fehler!' attr=highlight            
             color=blue   
	 #7  @28 "Dem Parameter 'macro_path' wurde ein syntaktisch falscher Wert" 
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"	 
	 #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Test, ob das unter 'macro_path' angegebene Verzeichnis existiert*/

%LET dir=&macro_path;

%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
       %PUT Directory does not exist;
%END;
%ELSE %DO;
       %PUT Directory exists;
       %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;

	%WINDOW error color=grey              
	 #5  @28 'Fehler!' attr=highlight      
         color=blue                                    
	 #7  @28 "Das unter dem Parameter 'macro_path' angegebene Verzeichnis existiert"
	 #8  @28 "nicht- Bentigte Untermakros knnen nicht referenziert werden"
	 #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';     
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Falls bestimmte Parameter ohne Wert bergeben wurden, diese mit Standard belegen*/

%IF &event= %THEN %DO;
	%LET event=1;
%END;

%IF &miss= %THEN %DO;
	%LET miss=0;
%END;

%IF &sel= %THEN %DO;
	%LET sel=NONE;
%END;

%IF &sel_details= %THEN %DO;
	%LET sel_details=1;
%END;

%IF &slstay= %THEN %DO;
	%LET slstay=0.05;
%END;

%IF &slentry= %THEN %DO;
	%LET slentry=0.05;
%END;

%IF &clodds= %THEN %DO;
	%LET clodds=0;
%END;

%IF &alpha= %THEN %DO;
	%LET alpha=0.05;
%END;

%IF &rsquare= %THEN %DO;
	%LET rsquare=1;
%END;

%IF &ctable= %THEN %DO;
	%LET ctable=0;
%END;

%IF &pred= %THEN %DO;
	%LET pred=1;
%END;

%IF &out_pred= %THEN %DO;
	%LET out_pred=work.out_pred;
%END;

%IF &predprobs_art= %THEN %DO;
	%LET predprobs_art=INDIVIDUAL;
%END;

%IF &roc= %THEN %DO;
	%LET roc=1;
%END;

%IF &out_roc= %THEN %DO;
	%LET out_roc=work.out_roc;
%END;

%IF &est= %THEN %DO;
	%LET est=1;
%END;

%IF &out_est= %THEN %DO;
	%LET out_est=work.out_est;
%END;

%IF &inest_flag= %THEN %DO;
	%LET inest_flag=0;
%END;

%IF &c_wert= %THEN %DO;
	%LET c_wert=1;
%END;

%IF &c_data= %THEN %DO;
	%LET c_data=work.association;
%END;

%IF &mi= %THEN %DO;
	%LET mi=0;
%END;

%IF &out_mi= %THEN %DO;
	%LET out_mi=work.out_mi_log;
%END;

%IF &binwidth= %THEN %DO;
	%LET binwidth=0;
%END;

/*Externe Makros, die innerhalb des Makros 'PM_LOGREG.MAC.SAS'*/
/*verwendet werden (Untermakros) einbinden (%include)*/

FILENAME y &macro_path;

%INCLUDE y ('logreg2.mac.sas');
%INCLUDE y ('fl_heinze.mac.sas');
%INCLUDE y ('dummy_friendly.mac.sas');
%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('numchar.mac.sas');
%INCLUDE y ('exist.mac.sas');
%INCLUDE y ('exist_var.mac.sas');
%INCLUDE y ('ref.mac.sas');

/*Test, ob bei bestimmten Parametern anstatt numerischen Werten*/
/*character- Werte bergeben wurden*/

/*berprfung erfolgt mit Hilfe des Untermakros 'numchar.mac.sas'*/

%NUMCHAR(parameterliste=&event &miss &slstay &slentry &clodds &alpha 
         &rsquare &ctable &pred &sel_details &roc &est &inest_flag &c_wert &binwidth);

%IF &numcharok='NOK' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error8.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Parameter, bei denen keine negativen Werte erlaubt sind, berprfen*/

%IF &alpha <=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error9.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%IF &slstay <=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error9.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%IF &slentry <=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error9.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%IF &binwidth <0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error66.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfen, ob der angegebene Datensatz (Parameter 'data=') auch wirklich existiert*/
/*(mit Hilfe des Hilfs-/Untermakros 'exist.mac.sas')*/

%EXIST(dsn=&data);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error4.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfen, ob 'inest_data'/ 'boot_data' existiert, falls angegeben*/

%IF &inest_data ne %THEN %DO;

%EXIST(dsn=&inest_data);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error31.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%END;

%IF &boot_data ne %THEN %DO;

%EXIST(dsn=&boot_data);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error31.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%END;

/*berprfung, ob Datenquelle (Parameter 'data=' UND response-Variable*/
/*(Zielgre / Parameter 'resp_var=') angegeben wurde*/

%IF &data= OR &resp_var= %THEN %DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

		%INCLUDE y ('error1.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

/*berprfung, ob berhaupt Variablen (Parameter 'xvar=' / 'cvar=') angegeben wurden*/

%IF &xvar= AND &cvar= %THEN %DO;

/*wenn keine Variablen angegeben wurden, Anzeige eines Fensters mit*/ 
/*Fehlermeldung/ Hinweis*/

		%INCLUDE y ('error2.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;
%END;

/*Prfung, ob alle bergebenen Variablen numerisch sind*/

/*Erzeugung eines temporren Zwischendatensatzes, welcher lediglich die*/
/*bergebenen Variablen enthlt*/

DATA test3;
	SET &data;
	KEEP &xvar &resp_var &cvar;
RUN;

/*mittels der Prozedur PROC CONTENTS kann der Typ einer Variablen bestimmt werden*/

/*Unterdrckung der Ausgabe von PROC CONTENTS*/

PROC CONTENTS DATA=test3 NOPRINT OUT=testcontent;
RUN;

/*lediglich die Variable 'type' der Ergebnisdatei behalten,*/
/*da nur diese von Interesse ist*/

DATA testcontent;
	SET testcontent;
	KEEP type;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=testcontent);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test4=&nnnn1;

/*Prfung ob Variablen vom Typ 'character' vorhanden sind*/

DATA test4;
	SET testcontent;
	IF type=2 THEN DELETE;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test4);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test5=&nnnn1;

/*falls Variablen bergeben wurden, die nicht numerisch sind --> Fehlermeldung*/

%IF &test4 ne &test5 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error60.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE test4 testcontent test3;
RUN;
QUIT;

/*berprfung, ob alle bergebenen Variablen im bergebenen Datensatz*/
/*(Parameter 'data' vorhanden sind)*/

/*Zusammenfassung aller Variablen in einer neuen Makro- Variablen*/

%LET all_variables= &resp_var &xvar &cvar;

/*Prfung erfolgt mit Hilfe des Untermakros 'exist_var.mac.sas'*/

%EXIST_VAR(dsn=&data, var=&all_variables);

/*falls irgendwelche Variablen nicht existieren, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error6.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob alle bergebenen Variablen im bergebenen Bootstrap-Datensatz*/
/*vorhanden sind*/

%IF &boot_data ne %THEN %DO;

%EXIST_VAR(dsn=&boot_data, var=&all_variables);

/*falls irgendwelche Variablen nicht existieren, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error33.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%END;

/*berprfung, ob die Zielgre (Parameter 'resp_var')*/
/*2 Kategorien (Ausprgungen) besitzt*/

/*missing values ausschliessen*/

DATA freq2;
	SET &data;
	IF &resp_var ne .;
RUN;

/*Anzahl der Ausprgungen mit Hilfe der Prozedur PROC FREQ ermitteln*/

/*Ausgabe unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnis als ODS- Datei zur weiteren Verarbeitung ausgeben*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=freq2;
	TABLES &resp_var;
RUN;

/*ODS- Output schliessen*/

ODS OUTPUT CLOSE;

/*ODS- Ausgabe wieder aktivieren*/

ODS LISTING;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen (= # Ausprgungen der Zielgre)*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq freq2;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

/*falls Zielgre mehr als zwei Ausprgungen besitz--> Fehlermeldung*/

%IF &event_check > 2 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error5.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

/*berprfung, ob Parameter 'miss' 0 oder 1 ist*/

%IF &miss ne 1 %THEN %DO;
	%IF &miss ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error7.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'event' 0 oder 1 ist*/

%IF &event ne 1 %THEN %DO;
	%IF &event ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error7.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'sel_details' 0 oder 1 ist*/

%IF &sel_details ne 1 %THEN %DO;
	%IF &sel_details ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error20.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'clodds' 0 oder 1 ist*/

%IF &clodds ne 1 %THEN %DO;
	%IF &clodds ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error20.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'rsquare' 0 oder 1 ist*/

%IF &rsquare ne 1 %THEN %DO;
	%IF &rsquare ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error20.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'ctable' 0 oder 1 ist*/

%IF &ctable ne 1 %THEN %DO;
	%IF &ctable ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error20.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'pred' 0 oder 1 ist*/

%IF &pred ne 1 %THEN %DO;
	%IF &pred ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error20.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'roc' 0 oder 1 ist*/

%IF &roc ne 1 %THEN %DO;
	%IF &roc ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE  y ('error20.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'est' 0 oder 1 ist*/

%IF &est ne 1 %THEN %DO;
	%IF &est ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error20.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'c_wert' 0 oder 1 ist*/

%IF &c_wert ne 1 %THEN %DO;
	%IF &c_wert ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error20.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'mi' 0 oder 1 ist*/

%IF &mi ne 1 %THEN %DO;
	%IF &mi ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error20.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'inest_flag' 0 / 1 oder 2 ist*/

%IF &inest_flag ne 1  %THEN %DO;
	%IF &inest_flag ne 0  %THEN %DO;
		%IF &inest_flag ne 2  %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

			%INCLUDE y ('error21.sas');

/*Sprungmarke im Fehlerfall*/

			%GOTO ende;

		%END;
	%END;
%END;
				
/*Anzahl der angegebenen kategoriellen Variablen ermitteln:*/

%GLOBAL c_count c_var;
%LET c_count=1;
%LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%DO %WHILE(&c_var ne);
     %LET c_count=%EVAL(&c_count+1);
     %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%END;
%LET c_count=%EVAL(&c_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/

%PUT The cvar contains &c_count variables.;

/*Anzahl der Referenzen ermitteln*/

%GLOBAL ref_count var_ref;
%LET ref_count=1;
%LET var_ref=%QSCAN(&ref,&ref_count,*);
%DO %WHILE(&var_ref ne);
     %LET ref_count=%EVAL(&ref_count+1);
     %LET var_ref=%QSCAN(&ref,&ref_count,*);
%END;
%LET ref_count=%EVAL(&ref_count-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The refs contains &ref_count variables.;

%IF &ref_count >0 %THEN
	%DO;
		%IF &c_count ne &ref_count %THEN                                                                                  
			%DO;

/*falls Anzahl der kategoriellen Variablen grer Null ist und
/* nicht mit der Anzahl der  angegebenen*/
/*Referenzkategorien bereinstimmt, Anzeige eines Fensters*/
/*mit Fehlermeldung/ Hinweis*/

			%INCLUDE y ('error3.sas');

/*Sprungmarke im Fehlerfall*/

			%GOTO ende;

   		%END;
%END;

/*berprfung, ob etwaige bergebene Referenzkategorien vorhanden sind bzw.*/
/*korrekt bergeben wurden*/

%IF &cvar ne %THEN %DO;
	%IF &ref_count ne 0 %THEN %DO;

	%REF(data=&data, resp_var=&resp_var, event=&event, cvar=&cvar, ref=&ref);

	%IF &correct='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error11.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;

%END;
%END;	


/*berprfen, ob Parameter, denen Character- Werte bergeben werden*/
/*korrekte Angaben enthalten*/

/*Parameter 'sel'*/

	%IF %UPCASE(&sel) ne FORWARD %THEN %DO;
		%IF %UPCASE(&sel) ne BACKWARD %THEN %DO;
			%IF %UPCASE(&sel) ne STEPWISE %THEN %DO;
				%IF %UPCASE(&sel) ne NONE %THEN %DO;
					%IF %UPCASE(&sel) ne F %THEN %DO;
						%IF %UPCASE(&sel) ne S %THEN %DO;
							%IF %UPCASE(&sel) ne B %THEN %DO;
								%IF %UPCASE(&sel) ne N %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

									%INCLUDE y ('error23.sas');

									/*Sprungmarke im Fehlerfall*/

									%GOTO ende;

								%END;
							%END;
						%END;
					%END;
				%END;
			%END;
		%END;
	%END;

/*Parameter 'predprobs_art'*/

%IF %UPCASE(&predprobs_art) ne CROSSVALIDATE %THEN %DO;
	%IF %UPCASE(&predprobs_art) ne INDIVIDUAL %THEN %DO;
		%IF %UPCASE(&predprobs_art) ne CUMULATIVE %THEN %DO;
			%IF %UPCASE(&predprobs_art) ne C %THEN %DO;
				%IF %UPCASE(&predprobs_art) ne I %THEN %DO;
					%IF %UPCASE(&predprobs_art) ne X %THEN %DO;
						%IF %UPCASE(&predprobs_art) ne XVALIDATE %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

							%INCLUDE y ('error25.sas');

							/*Sprungmarke im Fehlerfall*/

							%GOTO ende;

						%END;
					%END;
				%END;
			%END;
		%END;
	%END;
%END;

/*berprfen ob die Parameter 'alpha', 'slstay' und 'slentry' zwischen 0 und 1 liegen*/

%IF &alpha >1 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error26.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%IF &slstay >1 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error26.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%IF &slentry >1 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error26.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%IF &binwidth >=1 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error66.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%IF &binwidth >0.01 %THEN %DO;

/*Anzeige eines Warnhinweises*/

	DATA hint;
		hint="Bei einem Wert von 0 wird kein Binning angewandt. In manchen SAS Versionen wird der Standard 0.002 verwendet.";
	RUN;
	PROC PRINT DATA=hint NOOBS LABEL HEADING=H WIDTH=UNIFORM;
		VAR   hint;
		TITLE1 "Dem Parameter 'binwidth' wurde ein Wert grer 0.01 bergeben.";
		LABEL hint = "Sinnvoll ist ein Wert zwischen 0 und 0.01."; 
	RUN;

%END;

/*Parameter ctable_pprob berprfen (Syntax und [0,1])*/

%IF &ctable_pprob ne %THEN %DO;

/*Anzahl der angegebenen cutpoints ermitteln:*/

%GLOBAL ctable_count ctable_var;
%LET ctable_count=1;
%LET ctable_var=%QSCAN(&ctable_pprob,&ctable_count,%STR( ));
%DO %WHILE(&ctable_var ne);
     %LET ctable_count=%EVAL(&ctable_count+1);
     %LET ctable_var=%QSCAN(&ctable_pprob,&ctable_count,%STR( ));
%END;
%LET ctable_count=%EVAL(&ctable_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/

%PUT The ctable_pprob-Variable contains &ctable_count variables.;

%DO df=1 %TO &ctable_count;

%LET ctablecheck=%QSCAN(&ctable_pprob,&df,%STR( ));

%NUMCHAR(parameterliste=&ctablecheck);

%IF &numcharok='OK' %THEN %DO;


%IF &ctablecheck > 1 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error26.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%IF &ctablecheck <=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error9.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%END;
%ELSE %DO;

%IF %UPCASE(&ctablecheck) ne TO %THEN %DO;
	%IF %UPCASE(&ctablecheck) ne BY %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error27.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

	%END;
%END;

%END;

%END;

%END;

/*berprfung, ob Libnames der Zieldateien vorhanden sind falls welche*/
/* angegeben wurden:*/

%IF &out_pred ne %THEN %DO;

	%LET length=%LENGTH(&out_pred);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&out_pred,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&out_pred,1,.);
%LET file=%QSCAN(&out_pred,2,.);

DATA &lib..test;
	test='Test';
RUN;

%EXIST(dsn=&lib..test);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE test;
RUN;
QUIT;

%IF &exist ='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error16.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%END;

%END;

%IF &out_roc ne %THEN %DO;

	%LET length=%LENGTH(&out_roc);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&out_roc,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&out_roc,1,.);
%LET file=%QSCAN(&out_roc,2,.);

DATA &lib..test;
	test='Test';
RUN;

%EXIST(dsn=&lib..test);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE test;
RUN;
QUIT;

%IF &exist ='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE  y ('error16.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%END;

%END;

%IF &out_est ne %THEN %DO;

	%LET length=%LENGTH(&out_est);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&out_est,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&out_est,1,.);
%LET file=%QSCAN(&out_est,2,.);

DATA &lib..test;
	test='Test';
RUN;

%EXIST(dsn=&lib..test);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE test;
RUN;
QUIT;

%IF &exist ='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE  y ('error16.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%END;

%END;

%IF &c_data ne %THEN %DO;

	%LET length=%LENGTH(&c_data);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&c_data,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&c_data,1,.);
%LET file=%QSCAN(&c_data,2,.);

DATA &lib..test;
	test='Test';
RUN;

%EXIST(dsn=&lib..test);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE test;
RUN;
QUIT;

%IF &exist ='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error16.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%END;

%END;

%IF &inest_data ne %THEN %DO;

	%LET length=%LENGTH(&inest_data);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&inest_data,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&inest_data,1,.);
%LET file=%QSCAN(&inest_data,2,.);

DATA &lib..test;
	test='Test';
RUN;

%EXIST(dsn=&lib..test);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE test;
RUN;
QUIT;

%IF &exist ='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error16.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%END;

%END;

%IF &boot_data ne %THEN %DO;

	%LET length=%LENGTH(&boot_data);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&boot_data,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&boot_data,1,.);
%LET file=%QSCAN(&boot_data,2,.);

DATA &lib..test;
	test='Test';
RUN;

%EXIST(dsn=&lib..test);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE test;
RUN;
QUIT;

%IF &exist ='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE  y ('error16.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%END;

%END;

%IF &out_mi ne %THEN %DO;

	%LET length=%LENGTH(&out_mi);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&out_mi,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&out_mi,1,.);
%LET file=%QSCAN(&out_mi,2,.);

DATA &lib..test;
	test='Test';
RUN;

%EXIST(dsn=&lib..test);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE test;
RUN;
QUIT;

%IF &exist ='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE  y ('error16.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%END;

%END;

/*Sicherstellen, dass etwaige 'fxvars' in 'xvar' vorkommen sowie*/
/*dass etwaige 'fcvars' in 'cvar' vorkommen*/

/*Anzahl der fixen stetigen Variablen ermitteln*/

%GLOBAL count_fxvar var_fxvar;
%LET count_fxvar=1;
%LET var_fxvar=%QSCAN(&fxvar,&count_fxvar,%STR( ));
%DO %WHILE(&var_fxvar ne);
     %LET count_fxvar=%EVAL(&count_fxvar+1);
     %LET var_fxvar=%QSCAN(&fxvar,&count_fxvar,%STR( ));
%END;
%LET count_fxvar=%EVAL(&count_fxvar-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The fxvar contains &count_fxvar variables.;

%IF &count_fxvar ne 0 %THEN %DO;

%LET xvarcheck=0;
%DO igh=1 %TO &count_fxvar %BY 1;
	%DO jgh=1 %TO &count_x %BY 1;
		%LET x=%SCAN(&fxvar,&igh,%STR( ));
		%LET y=%SCAN(&xvar,&jgh,%STR( ));

		%IF (&x = &y) %THEN
		%DO;
			%LET xvarcheck=%EVAL(&xvarcheck+1);
		%END;
	%END;

%END;


%IF &xvarcheck ne &count_fxvar %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error28.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%END;

/*Anzahl der fixen kategoriellen Variablen ermitteln*/

%GLOBAL count_fcvar var_fcvar;
%LET count_fcvar=1;
%LET var_fcvar=%QSCAN(&fcvar,&count_fcvar,%STR( ));
%DO %WHILE(&var_fcvar ne);
     %LET count_fcvar=%EVAL(&count_fcvar+1);
     %LET var_fcvar=%QSCAN(&fcvar,&count_fcvar,%STR( ));
%END;
%LET count_fcvar=%EVAL(&count_fcvar-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The fcvar contains &count_fcvar variables.;

%IF &count_fcvar ne 0 %THEN %DO;

%LET cvarcheck=0;
%DO ighh=1 %TO &count_fcvar %BY 1;
	%DO jghh=1 %TO &count_c %BY 1;
		%LET x=%SCAN(&fcvar,&ighh,%STR( ));
		%LET y=%SCAN(&cvar,&jghh,%STR( ));

		%IF (&x = &y) %THEN
		%DO;
			%LET cvarcheck=%EVAL(&cvarcheck+1);
		%END;
	%END;

%END;


%IF &cvarcheck ne &count_fcvar %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE  y ('error28.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%END;

/*Sicherstellen, dass Variablen, die unter 'test' angegeben wurden*/
/*auch bei 'cvar'/ 'xvar' angegeben wurden*/

%IF &test ne %THEN %DO;

/*Anzahl der Variablen des 'test'- Parameters bestimmen.*/

%GLOBAL count_teststring var_teststring;
%LET count_teststring=1;
%LET var_teststring=%QSCAN(&test,&count_teststring,%STR( ));
%DO %WHILE(&var_teststring ne);
     %LET count_teststring=%EVAL(&count_teststring+1);
     %LET var_teststring=%QSCAN(&test,&count_teststring,%STR( ));
%END;
%LET count_teststring=%EVAL(&count_teststring-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The test contains &count_teststring variables.;

/*Etwaige Schlsselwrter 'TEST' entfernen*/

%LET teststring_new=;

%DO uzt=1 %TO &count_teststring;

%LET testpart=%SCAN(&test,&uzt,%STR( ));

/*prfen, ob TEST gro geschrieben wurde*/

%IF &testpart = %LOWCASE(test) %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error35.sas');
	%GOTO ende;

/*Sprungmarke im Fehlerfall*/

%END;

%IF &testpart ne %UPCASE(TEST) %THEN %DO;

%LET teststring_new=&teststring_new &testpart;

%END;

%END;

/*Anzahl der Variablen des 'teststring_new'- Parameters bestimmen.*/

%GLOBAL count_teststring_new var_teststring_new;
%LET count_teststring_new=1;
%LET var_teststring_new=%QSCAN(&teststring_new,&count_teststring_new,%STR( ));
%DO %WHILE(&var_teststring_new ne);
     %LET count_teststring_new=%EVAL(&count_teststring_new+1);
     %LET var_teststring_new=%QSCAN(&teststring_new,&count_teststring_new,%STR( ));
%END;
%LET count_teststring_new=%EVAL(&count_teststring_new-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The teststring_new contains &count_teststring_new variables.;

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The cvar contains &count variables.;

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The xvar contains &count_x variables.;

/*Addition*/

%GLOBAL count_cvarxvar;
%LET count_cvarxvar=%EVAL(&count_x+&count);

/*Summe im LOG- Fenster ausgeben*/

%PUT Summe: &count_cvarxvar;

%LET cvarxvar=&xvar &cvar;

%LET xvarcvarcheck=0;
%DO tre=1 %TO &count_cvarxvar %BY 1;
	%DO ert=1 %TO &count_teststring_new %BY 1;
		%LET x=%SCAN(&cvarxvar,&tre,%STR( ));
		%LET y=%SCAN(&teststring_new,&ert,%STR( ));

		%IF (&x = &y) %THEN
		%DO;
			%LET xvarcvarcheck=%EVAL(&xvarcvarcheck+1);
		%END;
	%END;

%END;

%IF &xvarcvarcheck ne &count_teststring_new %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error29.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%END;
			
/*berprfen, ob SEL=NONE angegeben wurde, falls test- Statement aktiviert ist*/

%IF &test ne %THEN %DO;

%IF %UPCASE(&sel) ne NONE %THEN %DO;
%IF %UPCASE(&sel) ne N %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error30.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;
%END;

%END;

/*berprfen ob 'inest_flag=2' ist, jedoch keine Datei unter*/
/*inest_data angegeben wurde*/

%IF &inest_flag=2 %THEN %DO;
	%IF &inest_data= %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error32.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfen ob 'boot_data' angegeben wurde, jedoch nicht 'inest_flag=1'*/

%IF &boot_data ne %THEN %DO;
	%IF &inest_flag ne 1 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error34.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfen ob eine Variable nach Entfernung aller Beobachtungen mit missing*/
/*values nur noch eine Ausprgung besitzt*/

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values*/

DATA work.category_test;
	SET  &data;

	%DO i=1 %TO &count_x;
		%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
		IF &xvar_liste=. THEN DELETE;
	%END;

	%DO i=1 %TO &count;
		%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
		IF &cvar_liste=. THEN DELETE;
	%END;

		IF &resp_var=. THEN DELETE;

RUN;

/*Prfen ob 'resp_var' /'xvar'/ 'cvar' nur 1 Ausprgung besitzen*/

%LET category_test=0;
%LET category_test_fail=;

/*event*/

/*Anzahl der Ausprgungen mittels PROC FREQ bestimmen*/

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &resp_var;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

%IF &event_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &resp_var;

%END;

/*xvar*/

%IF &xvar ne %THEN %DO;

%DO uhgu=1 %TO &count_x;

%LET xvar_liste=%SCAN(&xvar,&uhgu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &xvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET xvar_check=&nnnn1;

%IF &xvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &xvar_liste;

%END;

%END;
%END;

/*cvar*/

%IF &cvar ne %THEN %DO;

%DO uhfu=1 %TO &count;

%LET cvar_liste=%SCAN(&cvar,&uhfu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &cvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET cvar_check=&nnnn1;

%IF &cvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &cvar_liste;

%END;

%END;
%END;

PROC DATASETS NOLIST;
	DELETE category_test;
RUN;
QUIT;

%IF &category_test=1 %THEN %DO;

	DATA category_test;
		var="&category_test_fail";
	RUN;

	PROC PRINT DATA=category_test NOOBS LABEL;
	TITLE1 "Folgende Variablen besitzen nach Entfernung aller";
	TITLE2 "missing values lediglich eine Ausprgung:";
		VAR var;
			LABEL var='Variable';
	RUN;

	PROC DATASETS NOLIST;
		DELETE category_test;
	RUN;
	QUIT;

	%INCLUDE y ('error24.sas');
	%GOTO ende;

%END;

/*ENDE Prfungen / Referenzierungen / Standardbelegungen*/

/*-------------------------------------------------------------------------------*/

/*Separation-Status vorab initialisieren und auf 0 setzen*/

%GLOBAl status_separation;						
%LET status_separation=0;

/*Aufruf des Untermakros LOGREG2 mit den gewnschten Einstellungen*/

%LOGREG2	 (				data			= &data,
							resp_var		= &resp_var,
							event			= &event,
							xvar			= &xvar,
							cvar			= &cvar,
							ref				= &ref,
							fxvar			= &fxvar,
							fcvar			= &fcvar,
							miss			= &miss,
							sel				= &sel,
							sel_details		= &sel_details,
							slstay			= &slstay,
							slentry			= &slentry,
							clodds			= &clodds,
							alpha			= &alpha,
							rsquare			= &rsquare,
							ctable			= &ctable,
							ctable_pprob	= &ctable_pprob,
							pred			= &pred,
							out_pred		= &out_pred,
							predprobs_art	= &predprobs_art,
							roc				= &roc,
							out_roc			= &out_roc,
							est				= 1,
							out_est			= fl_fail_est,
							inest_flag		= &inest_flag,
							inest_data		= &inest_data,
							boot_data		= &boot_data,
							c_wert			= &c_wert,
							c_data			= &c_data,
							test			= &test,
							mi				= &mi,
							out_mi			= &out_mi,
							macro_path		= &macro_path,
							binwidth		= &binwidth
				);

/*(Quasi-) Complete Separation im Zusammenhang mit Selection=NONE prfen*/

%IF &status_separation=1 %THEN %DO;
%IF %UPCASE(&sel)=N OR %UPCASE(&sel)=NONE %THEN %DO;

/*Fehlerhafte Ausgabedateien berschreiben/lschen*/

DM "clear log; clear output;";


/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count_c varc;
%LET count_c=1;
%LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%DO %WHILE(&varc ne);
     %LET count_c=%EVAL(&count_c+1);
     %LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%END;
%LET count_c=%EVAL(&count_c-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The cvar contains &count_c variables.;

/*Dummy- Kodierung (nicht automatisch via CLASS- Statement bei PROC LOGISTIC*/
/*---> Heinze- Makro FL kennt keine kategorischen (klassierten) Variablen*/

%LET all_variables=;

DATA ergebnis_ende (KEEP=&resp_var &cvar &xvar);
	SET &data;
RUN;

/*Referenzen zu jeweiliger Variable zuordnen*/

%DO m=1 %TO &count_c;
	%LET var=%SCAN(&cvar,&m,%STR( ));
	%LET var_ref=%SCAN(&ref,&m,*);

/*Ausprgungen der jeweiligen kategoriellen Variable ermitteln*/

PROC FREQ DATA=&data NOPRINT;
	TABLES &var /  SPARSE OUT=&var;
RUN;

/*Ausgabedatensatz von PROC FREQ anpassen*/

DATA &var (DROP=percent count);
	SET &var;
	IF &var=. THEN DELETE;
	IF count=0 THEN DELETE;
RUN;

/*Anzahl der mglichen Ausprgungen je kategorieller Variable ermitteln*/
	
%NUMBEROBS(datensatz=&var);

%LET count_var=&nnnn1;

		%LET underscore=_;

/*Falls keine Referenzkategorie angegeben wurde, die erste nehmen*/
	
			%IF &count_ref=0 %THEN %DO;
				%LET var_ref=_FIRST_;
			%END;

/*Referenzkategorie "FIRST" fr bergabe an Dummy- Makro anpassen ("_FIRST_")*/

			%IF &var_ref=FIRST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

/*Referenzkategorie "LAST" fr bergabe an Dummy- Makro anpassen ("_LAST_")*/

	        %IF &var_ref=LAST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

			/*jeweilige Basiskategorie lschen*/

				DATA var2;
					SET &var end=_eof_;
					%IF 		&var_ref = _FIRST_ | &var_ref = LOW 
					%THEN 		%STR( IF _n_ = 1 THEN DELETE;);
					%ELSE %IF 	&var_ref = _LAST_ | &var_ref = HIGH
					%THEN 		%STR( IF _eof_ THEN DELETE;);
					%ELSE 		%STR(IF &var = &var_ref THEN DELETE;);
				RUN;

			/*Anzahl Kategorien der jeweiligen Variable ermitteln*/

				%NUMBEROBS(datensatz=var2);

	/*mgliche Ausprgungen pro Variable als Makro- Variable zwischenspeichern*/

			%LET count_var2=&nnnn1;

			DATA var2;
				SET var2;
				CALL SYMPUT('auspraeg'||left(_n_), trim(left(put(&var,BEST8.))));
			RUN;

/*Aufruf des Dummy- Kodierungs- Makros*/

	ODS LISTING CLOSE;

	%DUMMY( 
 	 		 data		= work.ergebnis_ende, /* name of input dataset        */
   			 out		= work.ergebnis_ende, /* name of output dataset       */
  			 var		= &var,         	  /* variable(s) to be dummied    */
  			 base		= &var_ref, /* base category                          */
   			 prefix 	= ,    		/* prefix for dummy variable names        */
   			 format 	= ,      	/* format used to categorize variable     */
  			 name  		= VAL,  	/* VAL: variable names are D_value        */
  			 fullrank	= 1     	/* Eliminate dummy for baseline category? */
   		  );

   ODS LISTING;


/*Falls fr eine Variable Dummies generiert wurden, Originalvariable lschen*/

DATA ergebnis_ende (DROP=&var);
	SET ergebnis_ende;

/*zustzliche Umbenennung (Anpassung an automatische Ausgabe der Dummy- Variablen*/
/*von PROC LOGISTIC)*/

		%DO o=1 %TO &count_var2;

		RENAME &var._&&auspraeg&o = &var&&auspraeg&o;

		%END;

RUN;

/*Zwischendatensatz lschen*/

PROC DATASETS NOLIST;
	DELETE &var;
RUN;
QUIT;

%END;

/*Mittels PROC CONTENTS die Variablen, die fr das FL- Makro selektieren*/
/*ohne Event (resp_var)*/

DATA ergebnis_ende2 (DROP=&resp_var);
	SET ergebnis_ende;
RUN;

PROC CONTENTS DATA=ergebnis_ende2 NOPRINT OUT=ergebnis2 (KEEP=name);
RUN;

%NUMBEROBS(datensatz=ergebnis2);

%LET count_ergebnis2=&nnnn1;

/*Variablennamen als Makro-Variablen speichern*/

DATA ergebnis2;
	SET ergebnis2;
	CALL SYMPUT('y'||left(_N_), name);
RUN;

/*Variablennamen fr FL- Makro zusammenfassen*/

%DO s=1 %TO &count_ergebnis2;

%LET all_variables=&all_variables &&y&s;
%PUT &all_variables;
%END;

/*Aufruf des FL- Makros*/

%FL		(	data			= ergebnis_ende, 
			y				= &resp_var, 
			varlist			= &all_variables, 
			outest			= outest_fl, 
			print			= 0
        );

/*berflssige Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE ergebnis_ende _work _outtab _offset_ _maxby _fby _estwot_ _cats_ 
		   var2 ergebnis_ende2 ergebnis2;
RUN;
QUIT;

/*Datei mit den Regressionskoeffizienten aus dem*/
/*FL- Makro fr INEST- Statement bei PROC LOGISTIC vorbereiten*/

DATA inest_fl (DROP=_rby_ _penlik_ _it_ _resp_ _noresp_ _by_ _total_ _code_);
	SET outest_fl;
	IF _code_ ne -1 THEN DELETE;
	RENAME INTERCEP=Intercept;
RUN;

/*Prfen, ob FL- Makro korrekt abgelaufen ist*/

%NUMBEROBS(datensatz=inest_fl);

%LET fl_fail_check=&nnnn1;

%IF &fl_fail_check=0 %THEN %DO;

%LET fl_false=1;

%END;

%ELSE %DO;

%LET fl_false=0;

%END;

/*-------------------------------------------------------------------------------*/

/*Standardoutput ausgeben*/

%LET datensatz=&data;

%IF &miss=1 %THEN %DO;

/*Fallzahl mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen:*/

%NUMBEROBS(datensatz=&data);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET fallzahl=&nnnn1;

%END;

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls miss=0 angegeben wurde*/

DATA work.fallzahl;
	SET  &data;

		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

RUN;

%LET fallzahldata=fallzahl;

%NUMBEROBS(datensatz=&fallzahldata);

%LET fallzahl=&nnnn1;

%END;

/*Makro-kopf ausgeben*/

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatz       ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

/*temporr angelegte Zwischendatei(en) lschen*/

PROC DATASETS NOLIST;
	DELETE 

	%IF &miss=0 %THEN %DO;

	fallzahl 

	%END;

	standardoutput
;
RUN;
QUIT;

/*ENDE Standardoutput*/

/*----------------------------------------------------------------------*/

/*Warnung ausgeben, dass Firth- Prozedur benutzt wurde*/

DATA note;
	note="";
RUN;

PROC PRINT DATA=note NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note;
    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Warnung (Variablenselektionsmethode='NONE'):";
	TITLE4 ;
	TITLE5 "There is possibly a quasi-complete separation of data"; 
	TITLE6 "points. The maximum likelihood estimate may not exist.";
	TITLE7 "Validity of the model fit is questionable.";
	TITLE8 ;
	TITLE9 'Es wurde dehalb eine automatische Korrektur mittels der';
	TITLE10 "Firth- Prozedur (Makro 'FL_HEINZE.MAC.SAS') durchgefhrt!";
	LABEL note = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
RUN;

DATA note2;
	note2="";
RUN;

PROC PRINT DATA=note2 NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note2;
    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Aufgrund der Korrektur der Regressionskoeffizienten wurde";
	TITLE4 "PROC LOGISTIC mit den Optionen INEST=neue (korrigierte)";
	TITLE5 "Regressionskoeffizienten sowie MAXITER=0 aufgerufen."; 
	TITLE6 "Dadurch erscheint nun unter Umstnden folgende Meldung";
	TITLE7 "im Output, welche ignoriert werden kann:";
	TITLE8 "'Iteration limit reached without convergence'";
	LABEL note2 = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
RUN;

/*lschen*/

PROC DATASETS NOLIST;
	DELETE note note2;
RUN;
QUIT;

%IF &fl_false=1 %THEN %DO;

/*Hinweis ausgeben, dass Firth- Prozedur (Korrektur) fehlgeschlagen ist*/

DATA note2;
	note2="";
RUN;

PROC PRINT DATA=note2 NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note2;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Warnung:               ";
	TITLE4 "Die Korrektur der Regressionskoeffizienten mittels der";
	TITLE5 "Firth- Prozedur (Makro 'FL_HEINZE.MAC.SAS') ist"; 
	TITLE6 "fehlgeschlagen! Evtl. wurden zu viele Koeffizienten in";
	TITLE7 "Bezug auf die Anzahl der Beobachtungen geschtzt. In der";
	TITLE8 "weiteren Analyse werden die Koeffizienten des letzten";
	TITLE9 "ML-Iterationsschrittes aus PROC LOGISTIC verwendet.";
	TITLE10 "Die Validitt dieses Modells ist fraglich!";
	
	LABEL note2 = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";

RUN;

/*temporre Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE note2;
RUN;
QUIT;

%END;

%IF &fl_false=1 %THEN %DO;

%LET inest_fl=fl_fail_est;

%END;

%ELSE %DO;

%LET inest_fl=inest_fl;

%END;

/*Neue Proc- Logistic mit INEST- Statement um korrekten Output zu erhalten*/

%LOGREG2	 (				data			= &data,
							resp_var		= &resp_var,
							event			= &event,
							xvar			= &xvar,
							cvar			= &cvar,
							ref				= &ref,
							fxvar			= &fxvar,
							fcvar			= &fcvar,
							miss			= &miss,
							sel				= &sel,
							sel_details		= &sel_details,
							slstay			= &slstay,
							slentry			= &slentry,
							clodds			= &clodds,
							alpha			= &alpha,
							rsquare			= &rsquare,
							ctable			= &ctable,
							ctable_pprob	= &ctable_pprob,
							pred			= &pred,
							out_pred		= &out_pred,
							predprobs_art	= &predprobs_art,
							roc				= &roc,
							out_roc			= &out_roc,
							est				= &est,
							out_est			= &out_est,
							inest_flag		= 2,			/* <-----*/
							inest_data		= &inest_fl,     /* <-----*/
							boot_data		= &boot_data,
							c_wert			= &c_wert,
							c_data			= &c_data,
							test			= &test,
							mi				= &mi,
							out_mi			= &out_mi,
							macro_path		= &macro_path,
							binwidth		= &binwidth
				);

/*berflssige Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE inest_fl outest_fl;
RUN;
QUIT;

%END;

/*------------------------------------------------------------------------*/

/*Backward- Selektion in Zusammenhang mit "(Quasi-)complete separation"*/

%LET fail_count=0;
%LET final_fl_false=0;

%IF &status_separation=1 %THEN %DO;
%IF %UPCASE(&sel)=B OR %UPCASE(&sel)=BACKWARD %THEN %DO;

/*Aufruf des Untermakros LOGREG2 mit SELECTION=NONE um Regressionskoeffizienten des*/
/*vollen Modells zu erhalten*/

%LOGREG2	 (				data			= &data,
							resp_var		= &resp_var,
							event			= &event,
							xvar			= &xvar,
							cvar			= &cvar,
							ref				= &ref,
							fxvar			= &fxvar,
							fcvar			= &fcvar,
							miss			= &miss,
							sel				= NONE,
							sel_details		= &sel_details,
							slstay			= &slstay,
							slentry			= &slentry,
							clodds			= &clodds,
							alpha			= &alpha,
							rsquare			= &rsquare,
							ctable			= &ctable,
							ctable_pprob	= &ctable_pprob,
							pred			= &pred,
							out_pred		= &out_pred,
							predprobs_art	= &predprobs_art,
							roc				= &roc,
							out_roc			= &out_roc,
							est				= 1,
							out_est			= fl_fail_est,
							inest_flag		= &inest_flag,
							inest_data		= &inest_data,
							boot_data		= &boot_data,
							c_wert			= &c_wert,
							c_data			= &c_data,
							test			= &test,
							mi				= &mi,
							out_mi			= &out_mi,
							macro_path		= &macro_path,
							binwidth		= &binwidth
				);

/*missing values vorab entfernen*/

DATA work.nomissbackward;
	SET  &data;

		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

RUN;

%LET data=nomissbackward;

%LET back_var_summary=;
%LET p_back_var_summary=;

/*Sprungmarke fr Backward- Selektion*/

%weiter:

/*Fehlerhafte Ausgabedateien berschreiben/lschen*/

DM "clear log; clear output;";

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count_c varc;
%LET count_c=1;
%LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%DO %WHILE(&varc ne);
     %LET count_c=%EVAL(&count_c+1);
     %LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%END;
%LET count_c=%EVAL(&count_c-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The cvar contains &count_c variables.;

/*Dummy- Kodierung (nicht automatisch via CLASS- Statement bei PROC LOGISTIC*/
/*---> Heinze- Makro FL kennt keine kategorischen (klassierten) Variablen*/

%LET all_variables=;

DATA ergebnis_ende (KEEP=&resp_var &cvar &xvar);
	SET &data;
RUN;

/*Referenzen zu jeweiliger Variable zuordnen*/

%DO m=1 %TO &count_c;
	%LET var=%SCAN(&cvar,&m,%STR( ));
	%LET var_ref=%SCAN(&ref,&m,*);

/*Ausprgungen der jeweiligen kategoriellen Variable ermitteln*/

PROC FREQ DATA=&data NOPRINT;
	TABLES &var /  SPARSE OUT=&var;
RUN;

/*Ausgabedatensatz von PROC FREQ anpassen*/

DATA &var (DROP=percent count);
	SET &var;
	IF &var=. THEN DELETE;
	IF count=0 THEN DELETE;
RUN;

/*Anzahl der mglichen Ausprgungen je kategorieller Variable ermitteln*/
	
%NUMBEROBS(datensatz=&var);

	%LET count_var=&nnnn1;

		%LET underscore=_;

/*Falls keine Referenzkategorie angegeben wurde, die erste nehmen*/
	
			%IF &count_ref=0 %THEN %DO;
				%LET var_ref=_FIRST_;
			%END;

/*Referenzkategorie "FIRST" fr bergabe an Dummy- Makro anpassen ("_FIRST_")*/

			%IF &var_ref=FIRST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

/*Referenzkategorie "LAST" fr bergabe an Dummy- Makro anpassen ("_LAST_")*/

	        %IF &var_ref=LAST %THEN %DO;
				%LET var_ref=_&&var_ref&&underscore;
			%END;

/*jeweilige Basiskategorie lschen*/

				DATA var2;
					SET &var end=_eof_;
					%IF 		&var_ref = _FIRST_ | &var_ref = LOW 
					%THEN 		%STR( IF _n_ = 1 THEN DELETE;);
					%ELSE %IF 	&var_ref = _LAST_ | &var_ref = HIGH
					%THEN 		%STR( IF _eof_ THEN DELETE;);
					%ELSE 		%STR(IF &var = &var_ref THEN DELETE;);
				RUN;

/*Anzahl Kategorien der jeweiligen Variable ermitteln*/

				%NUMBEROBS(datensatz=var2);

/*mgliche Ausprgungen pro Variable als Makro- Variable zwischenspeichern*/

			%LET count_var2=&nnnn1;

			DATA var2;
				SET var2;
				CALL SYMPUT('auspraeg'||left(_n_), trim(left(put(&var,BEST8.))));
			RUN;

/*Aufruf des Dummy- Kodierungs- Makros*/

	ODS LISTING CLOSE;

	%DUMMY( 
 	 		 data		= work.ergebnis_ende,  /* name of input dataset     */
   			 out		= work.ergebnis_ende,  /* name of output dataset    */
  			 var		= &var,         	   /* variable(s) to be dummied */
  			 base		= &var_ref, /* base category                        */
   			 prefix 	= ,    		/* prefix for dummy variable names      */
   			 format 	= ,      	/* format used to categorize variable   */
  			 name  		= VAL,  	/* VAL: variable names are D_value      */
  			 fullrank	= 1     	/* Eliminate dummy for baseline category? */
          );

   ODS LISTING;


/*Falls fr eine Variable Dummies generiert wurden, Originalvariable lschen*/

DATA ergebnis_ende (DROP=&var);
	SET ergebnis_ende;

/*zustzliche Umbenennung (Anpassung an automatische Ausgabe der Dummy- Variablen*/
/*von PROC LOGISTIC)*/

	%DO o=1 %TO &count_var2;

	RENAME &var._&&auspraeg&o = &var&&auspraeg&o;

	%END;

RUN;

/*Zwischendatensatz lschen*/

PROC DATASETS NOLIST;
	DELETE &var;
RUN;
QUIT;

%END;

/*Mittels PROC CONTENTS die Variablen, die fr das FL- Makro selektieren*/
/*ohne Event (resp_var)*/

DATA ergebnis_ende2 (DROP=&resp_var);
	SET ergebnis_ende;
RUN;

PROC CONTENTS DATA=ergebnis_ende2 NOPRINT OUT=ergebnis2 (KEEP=name);
RUN;

%NUMBEROBS(datensatz=ergebnis2);

%LET count_ergebnis2=&nnnn1;

/*Variablennamen als Makro-Variablen speichern*/

DATA ergebnis2;
	SET ergebnis2;
	CALL SYMPUT('y'||left(_N_), name);
RUN;

/*Variablennamen fr GOFLOGIT- Makro zusammenfassen*/

%DO s=1 %TO &count_ergebnis2;

	%LET all_variables=&all_variables &&y&s;
	%PUT &all_variables;

%END;

/*Aufruf des FL- Makros*/

%FL		(	data			= ergebnis_ende, 
			y				= &resp_var, 
			varlist			= &all_variables, 
			outest			= outest_fl, 
			print			= 0
       );

/*berflssige Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE ergebnis_ende _work _outtab _offset_ _maxby _fby _estwot_ _cats_ 
		   var2 ergebnis_ende2 ergebnis2;
RUN;
QUIT;

/*Datei mit den Regressionskoeffizienten aus dem FL- Makro*/
/*fr INEST- Statement bei PROC LOGISTIC vorbereiten*/

DATA inest_fl (DROP=_rby_ _penlik_ _it_ _resp_ _noresp_ _by_ _total_ _code_);
	SET outest_fl;
	IF _code_ ne -1 THEN DELETE;
	RENAME INTERCEP=Intercept;
RUN;

/*Prfen, ob FL- Makro korrekt abgelaufen ist*/

%NUMBEROBS(datensatz=inest_fl);

%LET fl_fail_check=&nnnn1;

%IF &fl_fail_check=0 %THEN %DO;

%LET fl_false=1;
%LET final_fl_false=1;

%END;

%ELSE %DO;

%LET fl_false=0;

%END;


%IF &fl_false=1 %THEN %DO;

%LET inest_fl=fl_fail_est;
%LET fail_count=%EVAL(&fail_count+1);

%END;

%ELSE %DO;

%LET inest_fl=inest_fl;

%END;

/*Neue Proc- Logistic mit INEST- Statement um korrekten Output zu erhalten*/

%LOGREG2	 (				data			=&data,
							resp_var		=&resp_var,
							event			=&event,
							xvar			=&xvar,
							cvar			=&cvar,
							ref				=&ref,
							fxvar			=&fxvar,
							fcvar			=&fcvar,
							miss			=1,
							sel				=NONE,
							sel_details		=&sel_details,
							slstay			=&slstay,
							slentry			=&slentry,
							clodds			=&clodds,
							alpha			=&alpha,
							rsquare			=&rsquare,
							ctable			=&ctable,
							ctable_pprob	=&ctable_pprob,
							pred			=&pred,
							out_pred		=&out_pred,
							predprobs_art	=&predprobs_art,
							roc				=&roc,
							out_roc			=&out_roc,
							est				=1,
							out_est			=fl_fail_est,
							inest_flag		=2,            /*<-----*/
							inest_data		=&inest_fl,    /*<-----*/
							boot_data		=&boot_data,
							c_wert			=&c_wert,
							c_data			=&c_data,
							test			=&test,
							mi				=&mi,
							out_mi			=&out_mi,
							macro_path		=&macro_path,
							binwidth		=&binwidth
				);

/*berflssige Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE outest_fl;
RUN;
QUIT;

/*Beginn der manuellen BACKWARD- Selektion auf Basis des Parameters SLSTAY*/
/*Variable Intercept lschen, falls vorhanden*/

DATA pwerte2;
	SET pwerte;
	IF Variable='Intercept' THEN DELETE;
RUN;

/*fixe stetigen und fixe kategorielle Variablen aus dem Selektionsprozess nehmen*/

%IF &fcvar ne %THEN %DO;

/*Anzahl der fixen kategoriellen Variablen ermitteln*/

%GLOBAL count_fcvar var_fcvar;
%LET count_fcvar=1;
%LET var_fcvar=%QSCAN(&fcvar,&count_fcvar,%STR( ));
%DO %WHILE(&var_fcvar ne);
     %LET count_fcvar=%EVAL(&count_fcvar+1);
     %LET var_fcvar=%QSCAN(&fcvar,&count_fcvar,%STR( ));
%END;
%LET count_fcvar=%EVAL(&count_fcvar-1);

/*p-werte der fixen Variablen bestimmen*/

DATA pwerte4;
	SET pwerte2;

	%DO u=1 %TO &count_fcvar;

	%LET fixcvar_liste=%SCAN(&fcvar,&u,%STR( ));


	IF Variable ne "&fixcvar_liste" THEN DELETE;

	%END;

RUN;

DATA pwerte2;
	SET pwerte2;

	%DO t=1 %TO &count_fcvar;

	%LET fixcvar_liste=%SCAN(&fcvar,&t,%STR( ));


	IF Variable = "&fixcvar_liste" THEN DELETE;

	%END;

RUN;

%END;

%IF &fxvar ne %THEN %DO;

/*Anzahl der fixen stetigen Variablen ermitteln*/

%GLOBAL count_fxvar var_fxvar;
%LET count_fxvar=1;
%LET var_fxvar=%QSCAN(&fxvar,&count_fxvar,%STR( ));
%DO %WHILE(&var_fxvar ne);
     %LET count_fxvar=%EVAL(&count_fxvar+1);
     %LET var_fxvar=%QSCAN(&fxvar,&count_fxvar,%STR( ));
%END;
%LET count_fxvar=%EVAL(&count_fxvar-1);

/*p-werte der fixen Variablen bestimmen*/

DATA pwerte5;
	SET pwerte2;

	%DO u=1 %TO &count_fcvar;

	%LET fixcvar_liste=%SCAN(&fxvar,&u,%STR( ));


	IF Variable ne "&fixcvar_liste" THEN DELETE;

	%END;

RUN;

DATA pwerte2;
	SET pwerte2;

	%DO z=1 %TO &count_fxvar;

	%LET fixxvar_liste=%SCAN(&fxvar,&z,%STR( ));

	IF variable = "&fixxvar_liste" THEN DELETE;

	%END;

RUN;

%END;

/*# Beobachtungen bestimmen*/

%NUMBEROBS(datensatz=pwerte2);

%LET pwerte2_count=&nnnn1;

DATA pwerte3;
	SET pwerte2;
	IF ProbChiSq < &slstay THEN DELETE;
RUN;

/*# Beobachtungen zum Vergleich bestimmen*/

%NUMBEROBS(datensatz=pwerte3);

%LET pwerte3_count=&nnnn1;

%IF &pwerte3_count=0 %THEN %DO;

DM "clear log; clear output;";

/*---> Keine Variablen mit p-Wert > dem Kriterium SLSTAY vorhanden --->*/
/*Ende der Backward- Selektion*/

/*Ergebnis der Backward- Selektion ausgeben:*/

/*Anzahl der entfernten Variablen ermitteln*/

/*Anzahl der fixen stetigen Variablen ermitteln*/

%GLOBAL count_back var_back;
%LET count_back=1;
%LET var_back=%QSCAN(&back_var_summary,&count_back,%STR( ));
%DO %WHILE(&var_back ne);
     %LET count_back=%EVAL(&count_back+1);
     %LET var_back=%QSCAN(&back_var_summary,&count_back,%STR( ));
%END;
%LET count_back=%EVAL(&count_back-1);

/*Zur Ausgabe, Ergebnisse in Tabelle schreiben*/

/*Angegebene kategorielle Variablen und zugehrige Referenzkategorie*/
/*in Tabelle schreiben*/

DATA backward;
	step   ='                                                                ';
	effect ='                                                                ';
	p_value='                                                                ';
		%DO i=1 %TO &count_back %BY 1; 
			effect="%scan(&back_var_summary,&i," ")";
			p_value="%scan(&p_back_var_summary,&i," ")";
			step=&i;
			output backward;
		%END;
RUN;

/*------------------------------------------------------------------------------*/

/*Standardoutput ausgeben*/

%LET datensatz=&data;

/*Fallzahl bestimmen:*/

%IF &miss=1 %THEN %DO;

%NUMBEROBS(datensatz=&data);

%LET fallzahl=&nnnn1;

%END;

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls miss=0 angegeben wurde*/

DATA work.fallzahl;
	SET  &data;

		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

RUN;

%LET fallzahldata=fallzahl;

%NUMBEROBS(datensatz=&fallzahldata);

%LET fallzahl=&nnnn1;

%END;

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatz2      ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

PROC DATASETS NOLIST;
	DELETE 

%IF &miss=0 %THEN %DO;

fallzahl 

%END;

standardoutput
;
RUN;
QUIT;

/*--------------------------------------------------------------------------------*/

/*Ausgeben*/

PROC PRINT DATA=backward NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR step effect p_value;

	TITLE1 'Summary Of Backward Elimination';
	TITLE2 ;
	TITLE3 "NOTE: No (additional) effects met the &slstay significance";
	TITLE4 "level for removal from the model. ";
	TITLE5 ;
	TITLE6 "Aufgrund einer '(quasi-) complete separation of datapoints'";
	TITLE7 "innerhalb der automatischen Backward- Selektion von PROC LOGISTIC";
	TITLE8 "wurde die Selektion manuell mittels der Firth- Prozedur durch-";
	TITLE9 "gefhrt, welche in der Lage ist, dieses Problem zu beheben";
	
	LABEL step      = 'Step';
	LABEL effect    = 'Effect removed';
	LABEL p_value   = 'Pr > ChiSq';
RUN;

DATA note2;
	note2="";
RUN;

PROC PRINT DATA=note2 NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note2;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Aufgrund der Korrektur der Regressionskoeffizienten wurde";
	TITLE4 "PROC LOGISTIC mit den Optionen INEST=neue (korrigierte)";
	TITLE5 "Regressionskoeffizienten sowie MAXITER=0 aufgerufen."; 
	TITLE6 "Dadurch erscheint nun unter Umstnden folgende Meldung";
	TITLE7 "im Output, welche ignoriert werden kann:";
	TITLE8 "'Iteration limit reached without convergence'";

	LABEL note2 = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
RUN;

PROC DATASETS NOLIST;
	DELETE  note2;
RUN;
QUIT;

%IF &final_fl_false=1 %THEN %DO;

/*Hinweis ausgeben, dass Firth- Prozedur (Korrektur) fehlgeschlagen ist*/

DATA note2;
	note2="";
RUN;

PROC PRINT DATA=note2 NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note2;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 "Warnung:               ";
	TITLE3 "Die Korrektur der Regressionskoeffizienten mittels der";
	TITLE4 "Firth- Prozedur (Makro 'FL_HEINZE.MAC.SAS') ist";
	TITLE5 "im Rahmen der Backward-Selektion &fail_count x fehlgeschlagen!"; 
	TITLE6 "Evtl. wurden zu viele Koeffizienten in Bezug auf die Anzahl";
	TITLE7 "der Beobachtungen geschtzt. In der weiteren Analyse";
	TITLE8 "wurden deshalb jeweils die Koeffizienten des letzten";
	TITLE9 "ML-Iterationsschrittes aus PROC LOGISTIC verwendet.";
	TITLE10 "Die Validitt dieses Modells ist fraglich!";
	
	LABEL note2 = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";

RUN;

/*temporre Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE note2;
RUN;
QUIT;

%END;

%IF &fcvar ne %THEN %DO;

DATA pwerte2;
	SET pwerte2 pwerte4;
RUN;

%END;

%IF &fxvar ne %THEN %DO;

DATA pwerte2;
	SET pwerte2 pwerte5;
RUN;

%END;

/*PROC LOGISTIC mit finalem Modell*/

%LET status_separation=1;

%LOGREG2	 (				data			=&data,
							resp_var		=&resp_var,
							event			=&event,
							xvar			=&xvar,
							cvar			=&cvar,
							ref				=&ref,
							fxvar			=&fxvar,
							fcvar			=&fcvar,
							miss			=1,
							sel				=NONE,
							sel_details		=&sel_details,
							slstay			=&slstay,
							slentry			=&slentry,
							clodds			=&clodds,
							alpha			=&alpha,
							rsquare			=&rsquare,
							ctable			=&ctable,
							ctable_pprob	=&ctable_pprob,
							pred			=&pred,
							out_pred		=&out_pred,
							predprobs_art	=&predprobs_art,
							roc				=&roc,
							out_roc			=&out_roc,
							est				=&est,
							out_est			=&out_est,
							inest_flag		=2,
							inest_data		=&inest_fl,
							boot_data		=&boot_data,
							c_wert			=&c_wert,
							c_data			=&c_data,
							test			=&test,
							mi				=&mi,
							out_mi			=&out_mi,
							macro_path		=&macro_path,
							binwidth		=&binwidth
				);

PROC DATASETS NOLIST;
	DELETE pwerte3 backward pwerte2 inest_fl;

	%IF &fcvar ne %THEN %DO;

	DELETE pwerte4;

	%END;

	%IF &fxvar ne %THEN %DO;

	DELETE pwerte5;

	%END;

RUN;
QUIT;

%END;
%ELSE %DO;

/*Datei mit den Variablen, die p- Werte > SLSTAY besitzen sortieren,*/
/*damit die Variable mit dem hchsten p- Wert bestimmt werden kann*/

PROC SORT DATA=pwerte3;
	BY DESCENDING ProbChiSq;
RUN;

/*alle Variablen bis auf _N_=1 lschen */

DATA pwerte3;
	SET pwerte3;
	IF _N_=1;
RUN;

/*Variablenname und P- Wert als Makro- Variablen abspeichern*/

DATA pwerte3;
	SET pwerte3;
	CALL symput('back_var',variable);
	CALL symput('p_back_var',ProbChiSq);
RUN;

%LET back_var_summary=&back_var_summary &back_var;
%LET p_back_var_summary=&p_back_var_summary &p_back_var;

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Anzahl der Referenzen ermitteln*/

%GLOBAL count_ref var_ref;
%LET count_ref=1;
%LET var_ref=%QSCAN(&ref,&count_ref,*);
%DO %WHILE(&var_ref ne);
     %LET count_ref=%EVAL(&count_ref+1);
     %LET var_ref=%QSCAN(&ref,&count_ref,*);
%END;
%LET count_ref=%EVAL(&count_ref-1);

/*Die selektierte Variable aus der entsprechenden Liste cvar/xvar entfernen*/

/*Elimination der entsprechenden stetigen Variablen aus xvar*/

%GLOBAL var_x;
%LET x='                                                               ';
%LET y='                                                               ';
	%DO j=1 %TO &count_x %BY 1;
		%LET y=%scan(&xvar,&j," ");

		%IF (%LOWCASE(&y) ne %LOWCASE(&back_var)) %THEN
		%DO;
			%LET var_x=&var_x &y;
		%END;
	%END;
	%LET xvar=&var_x;
	%LET var_x=;

%PUT &xvar;

/*Elimination der entsprechenden kategoriellen Variablen aus cvar*/

%GLOBAL var_c;
%LET var_c=;
%GLOBAL ref_c;
%LET ref_c=;
%LET x='                                                               ';
%LET y='                                                               ';
	%DO j=1 %TO &count %BY 1;
		%LET y=%scan(&cvar,&j," ");

/*Falls Referenzkategorien angegeben wurden: Zuordnung zu den brigen cvars*/

		%IF &count_ref ne 0 %THEN %DO;

		%LET refscan=%scan(&ref,&j,*);

		%END;


		
        %IF (%LOWCASE(&y) ne %LOWCASE(&back_var)) %THEN
		%DO;
			%LET var_c=&var_c &y;

/*Falls Referenzkategorien angegeben wurden: Zuordnung zu den brigen cvars*/

				%IF &count_ref ne 0 %THEN %DO;

					%IF &j=1 %THEN %DO;

					%LET ref_c=&ref_c &refscan;

					%END;
					%ELSE %DO;

					%LET ref_c=&ref_c * &refscan;

					%END;
							
				%END;
			
		%END;
	%END;
	%LET cvar=&var_c;

			%IF &count_ref ne 0 %THEN %DO;

					%LET ref=&ref_c;
					%LET ref_c=;
			%END;

	
	%LET var_c=;
	
%PUT &cvar;
%PUT &ref;

%GOTO weiter;

%END;

%END;

%END;

%END;

/*Keine Korrekturen fr die Selektionsmethoden FORWARD/*/
/*STEPWISE implementiert!*/
/*Falls (quasi)- complete separation of datapoints auftritt:*/
/* Verweis auf Backward- Selektion (Korrektur mglich, manuell implementiert)*/

%IF &status_separation=1 %THEN %DO;
%IF %UPCASE(&sel)=F OR %UPCASE(&sel)=FORWARD %THEN %DO; 

/*Fehlerhafte Ausgabedateien berschreiben/lschen*/
DM "clear log; clear output;";

/*++++++++++++++++++++*/

/*Standardoutput ausgeben*/

%LET datensatz=&data;

/*Fallzahl bestimmen:*/

%IF &miss=1 %THEN %DO;

%NUMBEROBS(datensatz=&data);

%LET fallzahl=&nnnn1;

%END;

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls miss=0 angegeben wurde*/

DATA work.fallzahl;
	SET  &data;

		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

RUN;

%LET fallzahldata=fallzahl;

%NUMBEROBS(datensatz=&fallzahldata);

%LET fallzahl=&nnnn1;

%END;

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatz2      ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

PROC DATASETS NOLIST;
	DELETE 

%IF &miss=0 %THEN %DO;

fallzahl 

%END;

standardoutput
;
RUN;
QUIT;

/*++++++++++++++++++++*/

DATA note;
	note="";
RUN;

PROC PRINT DATA=note NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Warnung (Variablenselektionsmethode='FORWARD'):               ";
	TITLE4 "There is possibly a quasi-complete separation of data"; 
	TITLE5 "points. The maximum likelihood estimate may not exist.";
	TITLE6 "Validity of the model fit is questionable.";
	TITLE7 ;
	TITLE8 'Etwaige fehlerhaften Ausgabedateien wurden automatisch gelscht';
	TITLE9 "Empfehlung: Variablenselektionsmethode 'BACKWARD' whlen";
	TITLE10 "(Behebung des Problems mittels der Firth- Prozedur implementiert) ";

	
	LABEL note = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
RUN;

PROC DATASETS NOLIST;
	DELETE note;
RUN;
QUIT;

/*Fehlerhafte Ausgabedateien lschen*/

/*berprfung, ob ein doppelter Dateiname angegeben wurde*/

%IF &pred=1 %THEN %DO;

	%LET length=%LENGTH(&out_pred);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&out_pred,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&out_pred,1,.);
%LET file=%QSCAN(&out_pred,2,.);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE &file;
RUN;
QUIT;

%END;
%ELSE %DO;

PROC DATASETS NOLIST;
	DELETE &out_pred;
RUN;
QUIT;

%END;

%END;

%IF &roc=1 %THEN %DO;

	%LET length=%LENGTH(&out_roc);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&out_roc,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&out_roc,1,.);
%LET file=%QSCAN(&out_roc,2,.);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE &file;
RUN;
QUIT;

%END;
%ELSE %DO;

PROC DATASETS NOLIST;
	DELETE &out_roc;
RUN;
QUIT;

%END;

%END;

%IF &est=1 %THEN %DO;

	%LET length=%LENGTH(&out_est);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&out_est,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&out_est,1,.);
%LET file=%QSCAN(&out_est,2,.);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE &file;
RUN;
QUIT;

%END;
%ELSE %DO;

PROC DATASETS NOLIST;
	DELETE &out_est;
RUN;
QUIT;

%END;

%END;

%IF &c_wert=1 %THEN %DO;

	%LET length=%LENGTH(&c_data);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&c_data,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&c_data,1,.);
%LET file=%QSCAN(&c_data,2,.);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE &file;
RUN;
QUIT;

%END;
%ELSE %DO;

PROC DATASETS NOLIST;
	DELETE &c_data;
RUN;
QUIT;

%END;

%END;

%END;
%END;

%IF &status_separation=1 %THEN %DO;
%IF %UPCASE(&sel)=S OR %UPCASE(&sel)=STEPWISE %THEN %DO;

/*Fehlerhafte Ausgabedateien berschreiben/lschen*/

DM "clear log; clear output;";

/*++++++++++++++++++++*/

/*Standardoutput ausgeben*/

%LET datensatz=&data;

/*Fallzahl bestimmen:*/

%IF &miss=1 %THEN %DO;

%NUMBEROBS(datensatz=&data);

%LET fallzahl=&nnnn1;

%END;

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls miss=0 angegeben wurde*/


DATA work.fallzahl;
	SET  &data;

		%DO i=1 %TO &count_x;
			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;
		%END;

		%DO i=1 %TO &count;
			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;
		%END;

			IF &resp_var=. THEN DELETE;

RUN;

%LET fallzahldata=fallzahl;

%NUMBEROBS(datensatz=&fallzahldata);

%LET fallzahl=&nnnn1;

%END;

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatz2      ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

PROC DATASETS NOLIST;
	DELETE 

%IF &miss=0 %THEN %DO;

fallzahl 

%END;

standardoutput
;
RUN;
QUIT;

/*++++++++++++++++++++*/

DATA note;
	note="";
RUN;

PROC PRINT DATA=note NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Warnung (Variablenselektionsmethode='STEPWISE'):               ";
	TITLE4 "There is possibly a quasi-complete separation of data"; 
	TITLE5 "points. The maximum likelihood estimate may not exist.              ";
	TITLE6 "Validity of the model fit is questionable.";
	TITLE7 ;
	TITLE8 'Etwaige fehlerhaften Ausgabedateien wurden automatisch gelscht';
	TITLE9 "Empfehlung: Variablenselektionsmethode 'BACKWARD' whlen";
	TITLE10 "(Behebung des Problems mittels der Firth- Prozedur implementiert) ";

	
	LABEL note = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
RUN;

PROC DATASETS NOLIST;
	DELETE note;
RUN;
QUIT;

/*Fehlerhafte Ausgabedateien lschen*/

/*berprfung, ob ein doppelter Dateiname angegeben wurde*/

%IF &pred=1 %THEN %DO;

	%LET length=%LENGTH(&out_pred);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&out_pred,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&out_pred,1,.);
%LET file=%QSCAN(&out_pred,2,.);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE &file;
RUN;
QUIT;

%END;
%ELSE %DO;

PROC DATASETS NOLIST;
	DELETE &out_pred;
RUN;
QUIT;

%END;

%END;

%IF &roc=1 %THEN %DO;

	%LET length=%LENGTH(&out_roc);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&out_roc,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&out_roc,1,.);
%LET file=%QSCAN(&out_roc,2,.);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE &file;
RUN;
QUIT;

%END;
%ELSE %DO;

PROC DATASETS NOLIST;
	DELETE &out_roc;
RUN;
QUIT;

%END;

%END;

%IF &est=1 %THEN %DO;

	%LET length=%LENGTH(&out_est);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&out_est,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&out_est,1,.);
%LET file=%QSCAN(&out_est,2,.);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE &file;
RUN;
QUIT;

%END;
%ELSE %DO;

PROC DATASETS NOLIST;
	DELETE &out_est;
RUN;
QUIT;

%END;

%END;

%IF &c_wert=1 %THEN %DO;

	%LET length=%LENGTH(&c_data);
	%LET point='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&c_data,&h,1);

		%IF &check=. %THEN %DO;

			%LET point='YES';

		%END;

	%END;

%IF &point='YES' %THEN %DO;

%LET lib=%QSCAN(&c_data,1,.);
%LET file=%QSCAN(&c_data,2,.);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE &file;
RUN;
QUIT;

%END;
%ELSE %DO;

PROC DATASETS NOLIST;
	DELETE &c_data;
RUN;
QUIT;

%END;

%END;

%END;
%END;

%IF &selection_check ne 0 %THEN %DO;

/*Tabelle p-Werte wird nicht bentigt, wenn keine (quasi-) Separation auftritt*/

PROC DATASETS NOLIST;
	DELETE pwerte;
RUN;
QUIT;

%IF &test= %THEN %DO;
%IF &status_separation=0 %THEN %DO;
%IF &est=1 %THEN %DO;

DATA &out_est;
	SET fl_fail_est;
RUN;

%END;

PROC DATASETS NOLIST;
	DELETE fl_fail_est;
RUN;
QUIT;

%END;
%END;
%END;

/*Sprungmarke fr Fehlerfall*/

%ende:

%MEND PM_LOGREG;
