/************************************************************************************/
/*						        													*/
/*     NAME: pm_mi_analyze.mac.sas	         										*/
/*    TITLE: Analyse von 'Multiple Imputation'- Datenstzen	        				*/
/*     KEYS: missing, missing values, imputation, multiple imputation, 	        	*/
/*			 logistic regression, roc analysis										*/
/*  VERSION: SAS 8.2												      			*/
/*  PRODUCT: BASE, STAT							        							*/
/*    PROCS: MEAN										        					*/
/*															         				*/
/*																					*/
/*  Verzeichnis/Dateiname:			F:\Prognose\pm_mi_analyze.mac.sas	        	*/
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*  Programmierer:					Christoph Ziegler								*/
/*  Datum der Ersterstellung:		8/2003											*/
/*  Aktuelle Makro-Version:			12/2003											*/
/*																					*/
/*  Kurzbeschreibung:				Allgemeines Makro zur zusammenfassenden 		*/
/*									Analyse von 'Multiple Imputation'- Datenstzen	*/
/*																					*/
/*  Verwendete externe Untermakros:	exist.mac.sas   exist_var.mac.sas				*/
/*									error22.sas		error41.sas						*/
/*									error42.sas     error43.sas						*/
/*																					*/
/*  Bentigte Dateien:				Dateien aus Makro PM_LOGREG.MAC.SAS 			*/
/*									(Parameter 'out_mi' / 'mi=1')					*/
/*																					*/
/*									Dateien aus PM_ROC.MAC.SAS						*/
/*									(Parameter 'out_mi' / 'mi=1')					*/
/*															           				*/
/*																					*/
/*  Erzeugte Dateien:				----- 							   				*/
/*																					*/
/*  Programmnderungen																*/ 
/*  und -ergnzungen 																*/
/*  (Name, Datum, Beschreibung):											 		*/
/*  																				*/
/*        																			*/
/************************************************************************************/
/*

 TITLE
 -----
 pm_mi_analyze.mac.sas

 SAS Makro zur zusammenfassenden Analyse von 'Multiple Imputation'- Datenstzen


 BACKGROUND
 ----------
 Die Multiple Imputation basiert darauf, die fehlenden Werte durch mehrere plausible 
 Werte zu ersetzen. Dieses Vorgehen resultiert in mehreren parallelen, aufgefllten 
 Datenstzen. Fr jeden dieser Datenstze kann dann die gewnschte Auswertung 
 durchgefhrt werden (hier Modellentwicklung und Prognosegtebestimmung). Anschlieend 
 werden die Ergebnisse parallel dargestellt und die Prognosegten durch Mittelwert-
 bildung zusammengefasst. (Noch nicht implementiert in diesem Makro ist die Schtzung
 der Varianz (s. [littel1987, rubin1987, yuan2000]), die korrigiert werden muss bzgl. 
 dieses Vorgehens.
 Rubin gibt an, dass eine Anzahl von 3-10 Imputationen ausreicht, abhngig von der 
 Anzahl fehlender Werte [rubin1987]. Fr praktische Belange reichen in der Regel 
 5 Imputationen. 

 DESCRIPTION
 -----------
 Das Makro fasst die Ergebnisse der multiplen logistischen Regression und der 
 jeweiligen ROC-Analysen fr die mit der MI-Methode aufgefllten Datenstze zusammen, 
 damit ein Vergleich mit den Ergebnissen z.B. der Complete Case Analyse im Sinne einer 
 Sensitivittsanalyse mglich ist [vach1997]. 
 Dazu wird in den Makros PM_LOGREG.MAC.SAS und PM_ROC.MAC.SAS jeweils ein Parameter 
 angegeben (outmi=), der die entsprechenden Ergebnisse in einer SAS-Datei abspeichert, 
 um sie dem Makro PM_MI_ANALYZE.MAC.SAS bergeben zu knnen. Es stellt nebeneinander 
 zuerst einmal die in den Modellen enthaltenen Variablen mit zugehrigen p-Werten dar, 
 bevor die auf Grundlage dieser Modelle und einer Reklassifikation geschtzten 
 Prognosegtemae fr jedes Modell und zustzlich Minimum, Maximum und Mittelwert 
 angegeben werden. 

 LITERATURE
 ----------
 - SAS Online Doc zu den Prozeduren  PROC MI, PROC MI_ANALYZE
 - Little RJA, Rubin DB (1987): Statistical analysis with missing data. 
 				J. Wiley & Sons, New York
 - Rubin DB (1987): Multiple imputation for nonresponse in surveys.  
				J. Wiley & Sons, New York
 - Yuan YC (2000): Multiple imputation for missing data: concepts and new 
				development. SAS Institute Inc., Cary NC  
                http://support.sas.com/rnd/app/papers/multipleimputation.pdf
 - Vach W  (1997): Some issues in estimating the effect of prognostic factors
                from incomplete covariate data. Statist. Med. 16: 57-72
 - Ziegler C (2003): Ein SAS-Makropaket zur Entwicklung und Validierung von 
				Prognosemodellen auf Basis der logistischen Regression. 
				Diplomarbeit, FH Ulm 

 SYNTAX
 ------
 [Makroaufruf und Beschreibung der zu bergebenden Parameter]

 %PM_MI_ANALYZE (	  
   *data_mi_log 	 = ,  Ergebnisdateien aus PM_LOGREG.MAC.SAS, getrennt mit Blank
   *data_mi_roc		 = ,  Ergebnisdateien aus PM_ROC.MAC.SAS, getrennt mit Blank
   *macro_path 	 	 =    Pfad, in dem das Makro-Paket abgelegt wurde. Form: 'f:\makros'
  );

  Mit (*) gekennzeichnete Parameter sind beim Makroaufruf zwingend erforderlich.
  Bereits gesetzte Parameter stellen die jeweilige Standardeinstellung dar.

   data_mi_log:		Parameter zur bergabe der Dateien aus dem Makro PM_LOGREG 
					(Parameter 'out_mi'). Bei bergabe mehrerer Dateien Trennung durch 
					Blanks. Bsp.: data_mi_log= out_log1 out_log2 out_log3
				 	Dateien enthalten p- Werte der Variablen des finalen Modells 
                    (Type III) 
					
   data_mi_roc:		Parameter zur bergabe der Dateien aus dem Makro PM_ROC 
					(Parameter 'out_mi'). Bei bergabe mehrerer Dateien, Trennung durch 
					Blanks. Bsp.: data_mi_roc= out_roc1 out_roc2 out_roc3
				 	Dateien enthalten Prognosekenngren des jeweiligen Modells
				 	# data_mi_roc mu mit der # data_mi_log bereinstimmen!
				 	Auch die Reihenfolge mu mit data_mi_log korrespondieren.
					
   macro_path: 		Aufgrund der Tatsache, dass innerhalb des Makros diverse 
					Untermakros aufgerufen werden, ist es zwingend ntig, den Pfad 
					zu dem Verzeichnis, wo die komplette Makrosammlung abgelegt ist,
					anzugeben. Der Pfad selber muss in Hochkommata eingeschlossen 
					sein. Beispiel: 'f:\makros'. 


  VALIDATION
  ----------
  Validiert durch: Stephanie Heiss
 				   University of Ulm 
                   Department of Biometry and Medical Documentation
                   89075 Ulm
  
  Validiert am:    02. September 2003

  Validierungsprogramm::      PM_MI_ANALYZE_VAL.SAS
  Validierungsdokumentation:  PM_MI_ANALYZE_VAL.DOC
   
  Validierungsvorgehen:
  - Das Testen der Makros wird durch eine von der Programmierung unabhngige 
    Person durchgefhrt.
  - Fr die Durchfhrung der Validierung wird auf einen in der Literatur 
    genutzten Datensatz zurckgegriffen:  LBW (Low Birth Weight), beschrieben
    in Hosmer, Lemeshow: Applied logistic regression [hosmer2000]
  - Der Datensatz wird fr verschiedene Testlufe verndert bzw. erweitert, 
    damit "die Testdaten neben den normalen Angaben auch Grenzwerte, ungltige
    Angaben, Fehlerbedingungen und Sonderflle beinhalten." [beiwanger2003]
  - Fehlerkontrolle:
    Hierbei werden nur die im LOG-Fenster auftretenden Fehler bzw. Warnungen 
    und offensichtlichen Fehler im OUTPUT betrachtet. Fr jeden Parameter des 
    Makros werden mehrere Durchlufe mit verschiedenen Wertangaben durchgefhrt, 
    wobei auch ungltige Werte angegeben werden, um das Verhalten des Makros
    bei fehlerhaften Aufrufen zu untersuchen. Gefundene Fehler werden 
    dokumentiert und dem Programmierer zur Verfgung gestellt. Nach der Korrektur 
    aller Fehler werden die Testlufe wiederholt, um sicherzustellen, dass 
    wirklich alle Fehler abgefangen wurden und dass keine neuen Fehler entstanden 
    sind.
  - Ergebniskontrolle:
    Um die Richtigkeit der Ergebnisse zu berprfen, werden soweit mglich, die
    Rechenbeispiele und bungsaufgaben aus Hosmer/Lemeshow durch vergleichbare
    Makroaufrufe nachgerechnet. Die Ergebnisse werden denen aus dem Begleitbuch 
    Hosmer/Lemeshow/Cook [hosmer2001] gegenbergestellt und direkt verglichen.
  - Alle Validierungen werden dokumentiert.

  Literatur: 
  - Beiwanger S (2003): Validierung von SAS-Makros. Diplomarbeit, FH Ulm
  - Hosmer DW, Lemeshow S (2000): Applied logistic regression 2nd Ed. 
           J. Wiley & Sons, New York
  - Hosmer DW, Lemeshow S, Cook ED (2001): Solutions manual to accompany
           applied logistic regression 2nd. Ed. J. Wiley & Sons, New York



  AUTHOR
  ------
  R. Muche / C. Ring / C. Ziegler
  University of Ulm
  Department of Biometry and Medical Documentation
  89075 Ulm

  The author does not overtake any responsibility for the correctness
  of results. It must not be modified without removing the heading lines 
  containing the authors name.

----------------------------------------------------------------------------------*/

%MACRO PM_MI_ANALYZE (	  data_mi_log 	 = ,
					      data_mi_roc	 = ,
					      macro_path 	 =
		             );

/*Prfungen / Referenzierungen / Standardbelegungen*/

/*Wichtigster Parameter 'macro_path' berprfen*/
/*bergabe erfolgt?*/

%IF &macro_path= %THEN %DO;

	%WINDOW error color=grey                                                            
     #5  @28 'Fehler!' attr=highlight        
             color=blue  
     #7  @28 "Dem zwingend erforderlichen Parameter 'macro_path' wurde kein Wert"
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';
    %DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Hilfsprogramm welches berprft, ob dem Parameter 'macro_path' ein korrekter Pfad*/
/*bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/

/*Anzahl der Zeichen im String 'macro_path' bestimmen:*/

%LET path_length=%LENGTH(&macro_path);

%LET infile=&macro_path;

%IF %bquote(&infile) NE %THEN %DO;
     %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
      %IF %bquote(&char1) ne %STR(%')
         %THEN %LET command='NOK';
         %ELSE %LET command='OK';

	 %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
      %IF %bquote(&char2) ne %STR(%')
         %THEN %LET command2='NOK';
         %ELSE %LET command2='OK';
%END;

/*falls die Syntax nicht stimmt, Anzeige eines Hinweisfensters*/

%IF &command='NOK' OR &command2='NOK' %THEN %DO;

	%WINDOW error color=grey
	 #5  @28 'Fehler!' attr=highlight
             color=blue
     #7  @28 "Dem Parameter 'macro_path' wurde ein syntaktisch falscher Wert"
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                          
    %DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Test, ob das unter 'macro_path' angegebene Verzeichnis existiert*/

%LET dir=&macro_path;

%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
       %PUT Directory does not exist;
%END;
%ELSE %DO;
       %PUT Directory exists;
       %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;

	%WINDOW error color=grey
	 #5  @28 'Fehler!' attr=highlight
             color=blue
	 #7  @28 "Das unter dem Parameter 'macro_path' angegebene Verzeichnis existiert"
	 #8  @28 "nicht- Bentigte Untermakros knnen nicht referenziert werden"
	 #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';            
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Externe Makros, die innerhalb des Makros 'PM_MI_ANALYZE.MAC.SAS'*/
/*verwendet werden (Untermakros) einbinden (%include)*/

FILENAME y &macro_path;

%INCLUDE y ('exist.mac.sas');
%INCLUDE y ('exist_var.mac.sas');

/*Prfen, ob berhaupt Dateien zur Analyse angegeben wurden*/

%IF &data_mi_log= %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error42.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%IF &data_mi_roc= %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error42.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Prfen, ob die Anzahl der Dateien aus 'PM_LOGREG.MAC.SAS'*/
/*mit denen aus 'PM_ROC.MAC.SAS' bereinstimmt*/

/*Anzahl der angegebenen Dateien aus 'PM_LOGREG.MAC.SAS' ermitteln:*/

%GLOBAL logreg_count logreg_var;
%LET logreg_count=1;
%LET logreg_var=%QSCAN(&data_mi_log,&logreg_count,%STR( ));
%DO %WHILE(&logreg_var ne);
     %LET logreg_count=%EVAL(&logreg_count+1);
     %LET logreg_var=%QSCAN(&data_mi_log,&logreg_count,%STR( ));
%END;
%LET logreg_count=%EVAL(&logreg_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/

%PUT The logreg_count contains &logreg_count variables.;

/*Anzahl der Dateien aus 'PM_ROC.MAC.SAS' ermitteln*/

%GLOBAL roc_count var_roc;
%LET roc_count=1;
%LET var_roc=%QSCAN(&data_mi_roc,&roc_count,%STR( ));
%DO %WHILE(&var_roc ne);
     %LET roc_count=%EVAL(&roc_count+1);
     %LET var_roc=%QSCAN(&data_mi_roc,&roc_count,%STR( ));
%END;
%LET roc_count=%EVAL(&roc_count-1);

/*Anzahl ausgeben im LOG- Fenster*/

%PUT The roc_count contains &roc_count variables.;

%IF &logreg_count ne &roc_count %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error41.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfen, ob die angegebenen Datenstze auch wirklich existiert*/
/*(mit Hilfe des Hilfs-/Untermakros 'exist.mac.sas')*/

%DO kjh=1 %TO &logreg_count;

%LET tmp_logreg=%QSCAN(&data_mi_log,&kjh,%STR( ));

%EXIST(dsn=&tmp_logreg);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error43.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%END;

%DO kjhl=1 %TO &roc_count;

%LET tmp_roc=%QSCAN(&data_mi_roc,&kjhl,%STR( ));

%EXIST(dsn=&tmp_roc);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error43.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%END;

/*prfen, ob slope, emax & intercept in den ROC- Dateien enthalten sind*/

%LET slope_check=0;
%LET emax_check=0;
%LET intercept_check=0;
%LET emax=0;
%LET slope=0;
%LET intercept=0;

%DO kjhlk=1 %TO &roc_count;

%LET tmp_roc=%QSCAN(&data_mi_roc,&kjhlk,%STR( ));

%EXIST_VAR(dsn=&tmp_roc,var=slope);

%IF &exist ne 'NO' %THEN %DO;

%LET slope_check=%SYSEVALF(&slope_check+1);

%END;

%END;

%IF &slope_check= &roc_count %THEN %DO;

	%LET slope=1;

%END;
%ELSE %DO;
	%IF &slope_check ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error22.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;

%END;

%DO kjhlkc=1 %TO &roc_count;

%LET tmp_roc=%QSCAN(&data_mi_roc,&kjhlkc,%STR( ));

%EXIST_VAR(dsn=&tmp_roc,var=intercept);

%IF &exist ne 'NO' %THEN %DO;

%LET intercept_check=%SYSEVALF(&intercept_check+1);

%END;

%END;

%IF &intercept_check= &roc_count %THEN %DO;

	%LET intercept=1;

%END;
%ELSE %DO;
	%IF &intercept_check ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error22.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;

%END;

%DO kjhlkx=1 %TO &roc_count;

%LET tmp_roc=%QSCAN(&data_mi_roc,&kjhlkx,%STR( ));

%EXIST_VAR(dsn=&tmp_roc,var=emax);

%IF &exist ne 'NO' %THEN %DO;

%LET emax_check=%SYSEVALF(&emax_check+1);

%END;

%END;

%IF &emax_check= &roc_count %THEN %DO;

	%LET emax=1;

%END;
%ELSE %DO;

	%IF &emax_check ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error22.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;

%END;

/*ENDE Prfungen / Referenzierungen / Standardbelegungen*/

/*-------------------------------------------------------------------------------*/

/*Standard- Makro- Kopf ausgeben*/

/*bestimmen, wie viele Dateien bergeben wurden (mi_log/ mi_roc)*/

/*Anzahl der mi_log- Dateien ermitteln:*/

%GLOBAL count_mi_log mi_log;
%LET count_mi_log=1;
%LET mi_log=%QSCAN(&data_mi_log,&count_mi_log,%STR( ));
%DO %WHILE(&mi_log ne);
     %LET count_mi_log=%EVAL(&count_mi_log+1);
     %LET mi_log=%QSCAN(&data_mi_log,&count_mi_log,%STR( ));
%END;
%LET count_mi_log=%EVAL(&count_mi_log-1);

/*Anzahl der mi_roc- Dateien ermitteln:*/

%GLOBAL count_mi_roc mi_roc;
%LET count_mi_roc=1;
%LET mi_roc=%QSCAN(&data_mi_roc,&count_mi_roc,%STR( ));
%DO %WHILE(&mi_roc ne);
     %LET count_mi_roc=%EVAL(&count_mi_roc+1);
     %LET mi_roc=%QSCAN(&data_mi_roc,&count_mi_roc,%STR( ));
%END;
%LET count_mi_roc=%EVAL(&count_mi_roc-1);

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:             ";
	TITLE4 ;
	TITLE5 "Makroname:                &sysmacroname    ";
	TITLE6 "# Analysierte Datenstze: &count_mi_log    ";
	TITLE7 "Datum:                    &sysday &sysdate ";
	TITLE8 "Start der SAS-Session:    &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

PROC DATASETS NOLIST;
	DELETE standardoutput
;
RUN;
QUIT;

/*Ende Standardkopf*/

/*-------------------------------------------------------------------------------*/

/*Datenstze (mi_log) fr Output anpassen*/

%LET all_data=;
%LET temp_mi_log=;

%DO hzu=1 %TO &count_mi_log;

	%LET temp_mi_log&hzu=%QSCAN(&data_mi_log,&hzu,%STR( ));

	DATA temp&hzu (KEEP= variable probchisq&hzu);
		SET &&temp_mi_log&hzu;
		IF variable='Intercept' THEN DELETE;
		RENAME probchisq=probchisq&hzu;
	RUN;

	DATA  temp&hzu;
		SET  temp&hzu;
		LABEL probchisq&hzu= "Pr > Chi-Square / Modell &hzu";
	RUN;

	PROC SORT DATA= temp&hzu;
		BY variable;
	RUN;

	%LET all_data= &all_data temp&hzu;

%END;

DATA ergebnis;
	MERGE &all_data;
	BY variable;
RUN;

PROC PRINT DATA=ergebnis NOOBS LABEL;
	TITLE1 "Endmodellstruktur:";
	TITLE2 ;
	TITLE3 "&data_mi_log";
RUN;

/*Gtemae ausgeben*/

/*zuerst Kenngren berechnen*/

DATA test;
	SET &data_mi_roc;
RUN;

PROC MEANS DATA=test NOPRINT MIN MEAN MEDIAN STD MAX;
OUTPUT OUT=mean_test;
RUN;


DATA mean_test (DROP=_TYPE_ _FREQ_);
	SET mean_test;
	IF _STAT_='N' THEN DELETE;
RUN;

PROC TRANSPOSE DATA=mean_test OUT=transpose_test;
RUN;

DATA transpose_test (DROP=_LABEL_);
	SET transpose_test;
		RENAME COL1=MIN;
		RENAME COL2=MAX;
		RENAME COL3=MEAN;
		RENAME COL4=STD;
		RENAME _Name_=kenngroessen;
RUN;

PROC SORT DATA= transpose_test;
	BY kenngroessen;
RUN;
	
%LET all_data_roc=;
%LET all_var_roc=;

%DO hzy=1 %TO &count_mi_roc;

	%LET temp_mi_roc&hzy=%QSCAN(&data_mi_roc,&hzy,%STR( ));

	DATA temp2&hzy;
		SET &&temp_mi_roc&hzy;
	RUN;

	PROC TRANSPOSE DATA=temp2&hzy OUT=temp2&hzy;
	RUN;

	DATA temp2&hzy (KEEP=_NAME_ COL1);
		SET temp2&hzy;
	RUN;

	DATA temp2&hzy;
		SET temp2&hzy;
			RENAME _NAME_=kenngroessen;
			RENAME COL1=modell&hzy;
	RUN;

	DATA temp2&hzy;
		SET temp2&hzy;
		    LABEL kenngroessen='Kenngrsse';
			LABEL modell&hzy  ="Modell &hzy";
	RUN;

	PROC SORT DATA=temp2&hzy;
		BY kenngroessen;
    RUN; 

	%LET all_data_roc= &all_data_roc temp2&hzy;

	%LET all_var_roc= &all_var_roc modell&hzy;

%END;

DATA ergebnis2;
	MERGE &all_data_roc;
	BY kenngroessen;
RUN;

DATA ergebnis2;
	MERGE ergebnis2 transpose_test;
	BY kenngroessen;
RUN;

/*Umbenennungen*/

DATA ergebnis2;
	SET ergebnis2;
	IF kenngroessen='auc' 		THEN kenngroessen='AUC';
	IF kenngroessen='brier' 	THEN kenngroessen='Brier';
	IF kenngroessen='cutpoint' 	THEN kenngroessen='Cutpoint';
	%IF %EVAL(&emax)=1 %THEN %DO;
	IF kenngroessen='emax' 		THEN kenngroessen='Emax';
	%END;
	%IF %EVAL(&intercept)=1 %THEN %DO;
	IF kenngroessen='intercept' THEN kenngroessen='Intercept';
	%END;
	IF kenngroessen='max_resca' THEN kenngroessen='MaxResc.R';
	IF kenngroessen='maxyouden' THEN kenngroessen='MaxYouden';
	IF kenngroessen='npv' 		THEN kenngroessen='NPV';
	IF kenngroessen='ppv' 		THEN kenngroessen='PPV';
	IF kenngroessen='psep' 		THEN kenngroessen='PSEP';
	IF kenngroessen='r_square' 	THEN kenngroessen='R-Square';
	IF kenngroessen='sens' 		THEN kenngroessen='Sens.';
	%IF %EVAL(&slope)=1 %THEN %DO;
	IF kenngroessen='slope' 	THEN kenngroessen='Slope';
	%END;
	IF kenngroessen='som' 		THEN kenngroessen='Somers D';
	IF kenngroessen='spez' 		THEN kenngroessen='Spez.';
RUN;

/*Reihenfolge der Kenngren anpassen*/

DATA ergebnis2;
	SET ergebnis2;
	newobs='    ';
	IF kenngroessen='AUC' 			THEN newobs=1;
	IF kenngroessen='Brier'			THEN newobs=3;
	IF kenngroessen='Cutpoint'		THEN newobs=9;
	%IF %EVAL(&emax)=1 %THEN %DO;
	IF kenngroessen='Emax'			THEN newobs=6;
	%END;
	%IF %EVAL(&intercept)=1 %THEN %DO;
	IF kenngroessen='Intercept'		THEN newobs=4;
	%END;
	IF kenngroessen='MaxResc.R'		THEN newobs=8;
	IF kenngroessen='MaxYouden'		THEN newobs=12;
	IF kenngroessen='NPV'			THEN newobs=14;
	IF kenngroessen='PPV'			THEN newobs=13;
	IF kenngroessen='PSEP'			THEN newobs=15;
	IF kenngroessen='R-Square'		THEN newobs=7;
	IF kenngroessen='Sens.'			THEN newobs=10;
	%IF %EVAL(&slope)=1 %THEN %DO;
	IF kenngroessen='Slope'			THEN newobs=5;
	%END;
	IF kenngroessen='Somers D'		THEN newobs=2;
	IF kenngroessen='Spez.'			THEN newobs=11;
RUN;

PROC SORT DATA=ergebnis2;
	BY newobs;
RUN;

PROC PRINT DATA=ergebnis2 NOOBS LABEL;
	TITLE1 "Prognosekenngrssen:";
	TITLE2 ;
	TITLE3 "&data_mi_roc";
	LABEl kenngroessen='Kenngrsse';
	VAR kenngroessen &all_var_roc MIN MEAN MAX;
RUN;

/*berflssige Dateien lschen*/

PROC DATASETS NOLIST;
	DELETE ergebnis2 &all_data_roc &all_data ergebnis mean_test transpose_test test; 
RUN;
QUIT;

%ende:

%MEND PM_MI_ANALYZE;