/************************************************************************************/
/*						        													*/
/*     NAME: pm_multicollin.mac.sas	         										*/
/*    TITLE: Multikollinearittsanalyse	        									*/
/*     KEYS: multicollinearity, collinearity, correlation, tolerance, spearman,	   	*/
/*			 variance inflation factor, proportion of variation, condition index	*/
/*  VERSION: SAS 8.2												      			*/
/*  PRODUCT: BASE, STAT							        							*/
/*    PROCS: CORR, REG										        				*/
/*															         				*/
/*																					*/
/*  Verzeichnis/Dateiname:			F:\Prognose\pm_multicollin.mac.sas	        	*/
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*  Programmierer:					Christoph Ziegler								*/
/*  Datum der Ersterstellung:		10/2002											*/
/*  Aktuelle Makro-Version:			12/2003											*/
/*																					*/
/*  Kurzbeschreibung:				Allgemeines Makro zur Prfung auf  				*/
/*									Multikollinearitt, d.h. zur Detektion etwaiger	*/
/*									Abhngigkeitsstrukturen zwischen Variablen		*/
/*																					*/
/*  Verwendete externe Untermakros:	numberobs.mac.sas								*/
/*									exist.mac.sas									*/
/*									exist_var.mac.sas								*/
/*									numchar.mac.sas									*/
/*									fl_heinze.mac.sas								*/
/*									dummy_friendly.mac.sas							*/
/*									error1.sas	error2.sas	error4.sas				*/
/*									error5.sas	error6.sas	error7.sas	error8.sas	*/
/*									error9.sas	error24.sas error60.sas				*/
/*																					*/
/*  Bentigte Dateien:				Datensatz (Parameter 'data=')		 			*/
/*																					*/
/*  Erzeugte Dateien:				----- 							   				*/
/*																					*/
/*  Programmnderungen																*/
/*  und -ergnzungen 																*/
/*  (Name, Datum, Beschreibung):											 		*/
/*  																				*/
/*        																			*/
/************************************************************************************/
/*

 TITLE
 -----
 pm_multicollin.mac.sas

 SAS Makro zur Prfung auf Multikollinearitt, d.h. zur Detektion etwaiger
 Abhngigkeitsstrukturen zwischen den Einflussgren 


 BACKGROUND
 ----------
 Die gleichzeitige Modellierung der Einflsse mehrerer Variablen in einem 
 logistischen Regressionsmodell durch Adjustierung der Variablen ist ein groer 
 Vorteil gegenber der Einzelbetrachtung von Variablen mit einer univariaten 
 Regression. Die so bercksichtigte Abhngigkeitsstruktur der Variablen untereinander 
 ist erwnscht, kann aber bei starken Abhngigkeiten zu Problemen bei der 
 Modellierung fhren. Diese Abhngigkeiten zwischen 2 und mehr Einflussgren 
 wird als Multikollinearitt bezeichnet. Dabei sind diese Abhngigkeiten so zu 
 verstehen, dass es fast lineare Beziehungen zwischen einigen der Einflussgren 
 gibt [armitage2002]. 

 Das Vorliegen von Multikollinearitt ist keine Verletzung der Modellvoraussetzungen, 
 beeintrchtigt dementsprechend nicht die Schtzung der Regressionskoeffizienten 
 [freund1991, harrell2001]. Die Nutzbarkeit des Modells ist aber trotzdem stark 
 eingeschrnkt, da die gleichzeitige Verwendung hoch korrelierter Variablen im 
 logistischen Regressionsmodell zu einer zu groen Varianz der Parameterschtzer 
 und somit zu einer Instabilitt des Modells fhren kann [freund1991]. Dies bedingt 
 wiederum eine eingeschrnkte Aussagefhigkeit der Ergebnisse, da durch diese zu hohe 
 Varianzschtzung Konfidenz- und Signifikanzbetrachtungen der Parameter inkorrekt 
 sind. In extremen Fllen knnen die Regressionskoeffizienten unplausibel (z.B. 
 in Bezug auf das Vorzeichen) und dementsprechend nutzlos sein [belsley1991]. 

 In Bezug auf Prognosen beschreiben Freund und Little [freund1991], dass es zwar 
 nicht bei der Schtzung der "predicted values" einen Bias, aber eine Tendenz zu 
 einer erhhten Varianz der Prognosen gibt, so dass dies bei den Schtzungen fr 
 Beobachtungen, die nicht im Originaldatensatz vorgekommen sind, zu ungenauen 
 Ergebnissen fhrt. 

 Lassen sich Gruppen von korrelierten Variablen identifizieren, so knnen Methoden 
 zur Datenreduktion herangezogen und eventuell durch eine irgendwie geartete 
 Zusammenfassung der betroffenen Variablen das Problem umgangen werden. Die 
 Identifizierung von Multikollinearitt ist allerdings nicht ganz einfach. Belsley 
 [belsley1991] gibt einige Kriterien an, die man bei der Untersuchung von 
 Multikollinearitt untersuchen sollte:
 - falsches Vorzeichen (gegenber Deskription der Daten oder inhaltlichem Vorwissen)
 - groer geschtzter Standardfehler 
 - hohe Korrelationen in der Korrelationsmatrix
 - Untersuchung des Einflusses einzelner Variablen durch Varianzuntersuchungen   
   (Varianzinflationsfaktoren (VIF))
 - Untersuchung der Struktur von XX (Eigenwerte, -vektoren und Varianzanteile)
 Die hier beschriebenen Methoden sind Standardmethoden der Analyse linearer 
 Regressionsmodelle. Das Argument zur Nutzung dieser Analysen in der logistischen 
 Regression ist, dass hier eine Untersuchung der Abhngigkeitsstrukturen zwischen 
 den Einflussgren durchgefhrt wird, die unabhngig von der Zielgre ist 
 [allison1999]. In den meisten Fllen werden so die Abhngigkeiten identifiziert, 
 es kann allerdings bei direkter Nutzung zur Nichtentdeckung relevanter Multi-
 kollinearitten im logistischen Regressionsmodell fhren. Allison gibt eine 
 Korrekturmglichkeit an, indem die lineare Regression zur Bestimmung der 
 Multikollinearitten auf Basis der angegebenen Analysen gewichtet wird bzgl. 
 der Gewichtungsmatrix in der Maximum-Likelihood-Schtzung der logistischen 
 Regression, die im Falle einer Separation der Datenpunkte mit dem FL-Makro von 
 Heinze [heinze2002] geschtzt werden.  

 Eine Untersuchung der paarweisen Korrelationen mit den blichen Korrelations-
 koeffizienten (Faustformel: r > 0.7) sollte deshalb nach einer genauen Betrachtung 
 des geschtzten Modells (Vorzeichen, Standardfehler) zu Beginn einer 
 Multikollinearittsprfung stehen. Hohe Korrelationen knnen hier ein 
 Multikollinearittsproblem aufzeigen, fehlen hoher Korrelationen kann allerdings 
 aber nicht als Evidenz fr das Fehlen einer Multikollinearitt gewertet werden. 

 Der Varianzinflationsfaktor (VIF) ist definiert als VIF=1/(1-Ri-Quadrat), wobei 
 Ri-Quadrat der Anteil der Variabilitt in der Variable Xi ist, der durch die anderen 
 im Modell befindlichen Variablen erklrt wird [armitage2002]. Bestimmt wird er als 
 lineares Bestimmtheitsma einer linearen Regression mit Xi als abhngiger und alle 
 anderen Variablen als unabhngige Einflussgren. Es kann gezeigt werden, dass die 
 Varianz des Regressionskoeffizienten um den Faktor VIF grer ist als wenn keine 
 Multikollinearitt vorliegt [freund1991]. Es gibt fr den VIF kein formelles 
 Kriterium, ab welchem Wert von einer Multikollinearitt ausgegangen werden muss. 
 Myers [myers1990] und viele andere setzen diese Grenze auf 10. 

 Ein zweiter Ansatz zur Identifikation von Multikollinearitt untersucht die 
 Struktur der Matrix XX durch Analyse der Eigenwerte und Eigenvektoren [belsley1991]. 
 Dabei ist X die Designmatrix der linearen Regression mit allen Einflussgren. Zwei 
 Eigenschaften der Eigenwerte knnen ausgenutzt werden, Kollinearitten zu entdecken:
 - Ist ein Eigenwerte 0, so zeigt er eine exakte lineare Abhngigkeit an. Durch 
   Betrachtung der kleinsten Eigenwerte kann somit untersucht werden, ob es Gruppen 
   von abhngigen Variablen gibt.
 - Die Quadratwurzel aus dem Quotienten des grten zum kleinsten Eigenwert wird mit 
   Condition-Index (CI) bezeichnet und gibt ein Ma fr das Ausma der Kollinearitt 
   im Datensatz an. Belsley [belsley1991] gibt an, dass bei einem CI > 100 die 
   Parameterschtzungen relevant beeinflusst sein knnen und dass ab einem CI von 10 
   schwache Abhngigkeiten das Ergebnis beeinflussen knnten. Freund [freund1991] gibt 
   eine Grenze von 30 an, ab der mit Multikollinearitt gerechnet werden muss.
 Welche Variablen voneinander linear abhngen, kann dann durch die Bestimmung der 
 Varianzanteile (Proportion of Variance, POV) der Parameterschtzer zum jeweiligen 
 Eigenwert bestimmt werden (durch eine jeweilige Hauptkomponentenanalyse). Die 
 Variablen mit einem hohen Varianzanteil (> 0.5, Belsley [belsley1991]) sind somit 
 in die lineare Abhngigkeit involviert, die zu dem hohen Condition-Index fhrt 
 [freund1991]. 

 Es gibt zwei Mglichkeiten der Analyse von XX: Einmal wird diese Matrix so skaliert, 
 dass auf der Hauptdiagonale alle Elemente 1 sind (Option COLLIN in der SAS-Prozedur 
 PROC REG). Zweitens kann dann noch die Matrix so zentriert werden, dass bzgl. des 
 Intercepts adjustiert wird (Option COLLINOINT). Damit hat der Intercept keinen Effekt 
 auf die Multikollinearittsuntersuchung zwischen den Variablen. Diese zweite Methode 
 wird in der Praxis am hufigsten gebraucht, da die erste Methode nur Sinn macht, wenn 
 der Intercept selber eine vernnftige Interpretation hat und im beobachteten 
 Datenbereich liegt.  


 DESCRIPTION
 -----------
 Im Makro sind die angesprochenen Methoden der Identifizierung mglicher 
 Multikollinearitt folgendermaen implementiert worden:
 - Bestimmung der Spearman-Korelationskoeffizienten zwischen allen Einflussgren 
   (mit PROC CORR). Ausgegeben werden die hchsten Korrelationskoeffizienten ab einem 
   vom Benutzer festgelegten Wert. 
 - Da in PROC LOGISTIC keine Multikollinearittsdiagnostik implementiert ist, wird 
   u.a. in Allison [allison1999] vorgeschlagen, die entsprechenden Mglichkeiten 
   der SAS-Prozedur fr lineare Regression (PROC REG) zu nutzen, aber im Output nur 
   die Multikollinearittsdiagnostik zu betrachten. Dies ist gerechtfertigt, da hier 
   eine Abhngigkeitsstruktur unter den Einflussgren untersucht wird und dies 
   unabhngig von der Zielgre betrachtet werden kann. Dabei werden hier die 
   Kenngren zur Multikollinearitt mit der oben beschriebenen Gewichtung aus der 
   logistischen Regression nach Allison korrigiert bestimmt (entweder direkt aus 
   PROC LOGISTIC oder bei Datenseparation ber das FL-Makro von Heinze [heinze2002]. 
   Im Output des Makros werden nur die Ergebnisse der Kollinearittsdiagnostik 
   benutzt, alle anderen Ergebnisse unterdrckt. Folgende Optionen in PROC REG fhren 
   eine entsprechende Diagnostik durch: 
   - Bestimmung von CI und POV ber die Optionen COLLIN bzw. COLLINOINT (s.o.). 
     Dabei werden alle Condition-Indizes ber einem vom Benutzer vorgegebenen Wert 
     ausgegeben (Standard=30), bei denen zwei oder mehr Variablen POV grer einem 
     vorgegebenen Wert (Standard=0.5) zeigen. Sollten keine Variablen mit groen 
     Varianzanteilen vorhanden sein wird der maximale CI zur Information ausgegeben.
   - Bestimmung der VIF ber die Option VIF. Es werden alle Variablen mit einem VIF 
     grer einem vom Benutzer vorgegebenen Wert (Standard=10) ausgegeben. Sollte 
     keine Variable grere Werte haben wird dem Nutzer der maximale VIF zur Kenntnis 
     ausgegeben.


 LITERATURE
 ----------
 - SAS Online Doc zu PROC REG: Collinearity Diagnostics
 - Allison PD (1999):  Logistic regression using the SAS System. 
                     SAS Institute Inc., Cary NC 
 - Armitage P, Berry G, Matthews JNS (2002): Statistical methods in medical research.
                     Blackwell Science Ltd., London 
 - Belsley DA (1991): Conditioning Diagnostics - Collinearity and weak data in 
                     regression. John Wiley and Sons Inc., New York
 - Freund RJ, LITTLE RC (1991):  SAS system for regression 2nd Ed.  
                     SAS Institute Inc., Cary NC 
 - Harrell FE JR (2001):  Regression modeling strategies.  Springer, New York 
 - Heinze G, Schemper M (2002):  A solution to the problem of separation in 
                     logistic regression. Statistics in Medicine 21, 2409-2419
 - Myers RH (1990):  Classical and Modern Regression with applications 2nd Ed.  
                     PWS and Kent Publishing Company Inc., Boston   
 - Ziegler C (2003): Ein SAS-Makropaket zur Entwicklung und Validierung von 
					 Prognosemodellen auf Basis der logistischen Regression. 
					 Diplomarbeit, FH Ulm 


 SYNTAX
 ------
 [Makroaufruf und Beschreibung der zu bergebenden Parameter]

 %PM_MULTICOLLIN ( 	
   *data			= ,			Datensatz
   *resp_var		= ,			Zielgre
    event           = 1,        Event- Kategorie der binren Zielgre (0/1)
   *xvar			= ,			stetige Variable(n)
   *cvar			= ,			kategorische Variable(n)
	miss			= 0,		Missing Values ausschliessen (ja/ nein)
	spearman		= 0.6,		Kriterium fr Spearman Rangkorrelationskoeffizienten
	condition_index	= 30,		Kriterium fr Condition Index
	pov				= 0.5,		Kriterium fr Proportion Of Variation
	coll			= 1,		PROC REG mit Option COLLIN oder COLLINOINT
	vif				= 10,		Kriterium fr Variance Inflation Factor
   *macro_path		=			Pfad zur Makrosammlung
  );

  Mit (*) gekennzeichnete Parameter sind beim Makroaufruf zwingend erforderlich.
  Bereits gesetzte Parameter stellen die jeweilige Standardeinstellung dar.

   data:		    Parameter zur Angabe des zu analysierenden Datensatzes. Wurde 
					beispielsweise vor dem Aufruf des Makros ein Datensatz mit dem 
					Namen "dat300" angelegt, lautet der korrekte Aufruf: "data=dat300". 
					Zustzlich kann auch die dazugehrige SAS-Library bergeben werden 
					(Beispiel: "data=work.dat300"). Die bergabe dieses Parameters ist 
					zwingend erforderlich.
 
   resp_var:		"resp_var" steht fr "Response Variable", also fr die Zielgre. 
					Dieser bergabeparameter ist ebenfalls zwingend erforderlich. 
					Ferner wird eine binre Datenstruktur (0/1) der Zielgre bentigt. 
					Die Zielgre muss mit "0" und "1" kodiert sein. Sollte dies nicht 
					der Fall, ist eine vorherige Umkodierung erforderlich. Heit 
					beispielsweise die Zielgre "tod" lautet der Aufruf wie folgt: 
					"resp_var=tod".

   event:			Je nach dem welche Kategorie der binren Zielgre untersucht 
					werden soll, kann diese mit Hilfe dieses Parameters bergeben werden. 
					Soll beispielsweise die Zielgre "tod" analysiert werden und steht 
					"1" fr tot sowie "0" fr lebendig, lautet der korrekte Aufruf 
					"event=1". Dies ist zugleich die Standardeinstellung, da blicherweise
					"1" fr das Eintreten eines bestimmten Ereignisses steht.

   xvar:			Dieser Parameter ist zur Angabe der stetigen Variablen 
					(Einflussgren). Allgemein gesagt sollten hier alle Variablen 
					genannt werden, die nicht in das CLASS-Statement von PROC LOGISTIC 
					aufgenommen werden sollen, also beispielsweise auch quasi-stetige 
					Variablen. Werden mehrere stetige oder quasi-stetige Variablen 
					bergeben, so mssen diese durch ein Blank (Leerzeichen) getrennt 
					werden. Beispiel: "xvar=alter bmi".

   cvar:			"cvar" steht fr "categorical variables" und ist fr die bergabe 
					aller kategorischen sowie aller dichotomen Variablen verantwortlich. 
					Diese werden innerhalb der LOGISTIC-Prozedur in das CLASS-Statement 
					aufgenommen und werden automatisch Dummy-kodiert. Werden mehrere 
					kategorische Variablen bergeben, mssen diese getrennt durch ein 
					Leerzeichen (Blank) bergeben werden (Beispiel: "cvar=diagnose msnr").
					Wurden dem Parameter "cvar" dichotome Variablen bergeben, so werden 
					diese zwar in das CLASS-Statement von PROC LOGISTIC aufgenommen, 
					jedoch keine Dummy-Kodierung durchgefhrt, da dichotome Variablen nur 
					zwei Klassen besitzen.

   miss:			Dieser Parameter legt fest, ob Beobachtungen mit fehlenden Werten 
					(missing values) in den unter "xvar", "cvar" und "resp_var" 
					bergebenen Variablen vor der Analyse aus dem Datensatz eliminiert 
					werden sollen (0) oder nicht (1). Der Standardwert dieses Parameters 
					ist 0 (Complete Case-Analyse). Hierbei ist zu beachten, dass fehlende 
					Werte im Datensatz durch einen Punkt (.) reprsentiert sein mssen.

   spearman:		Dieser Parameter ermglicht die Spezifizierung des Kriteriums fr
					die Kollinearitts-Detektion von Variablenkombinationen nach 
					Spearman-Rangkorrelationskoeffizienten. Der Standardwert ist 
					0.6, d.h. es werden Variablenkombinationen mit Spearman- 
					Rangkorrelationskoeffizienten zwischen 0.6 und 1 ermittelt 
					und ausgegeben. Die makrointerne Berechnung der Spearman 
					Rangkorrelationskoeffizienten erfolgt mittels PROC CORR 
					(Option SPEARMAN).

   condition_index:	Der Parameter 'condition_index' erlaubt eine benutzerspezifische
					Angabe des Kriteriums fr den Condition Index. Die 
					Standardeinstellung dieses Parameters ist 'condition_index=10'. 
					Die Kollinearitts-Detektion von Variablenkombinationen auf 
					Basis des Condition Index erfolgt zustzlich im Zusammenhang 
					mit dem Parameter 'pov' (Proportion Of Variation). Man geht im 
					allgemeinen davon aus, dass ein Kollinearittsproblem dann 
					auftritt, wenn eine Komponente mit hohem Condition Index 
					groen Anteil (>0.5) an der Varianz zweier (oder mehrerer) 
					Variablen hat. Faustregeln fr den Condition Index sind 
					[belsey1991]: 
					CI > 10: Schwache Abhngigkeiten knnen dazu fhren, die
							 Parameterschtzungen zu beeinflussen
					CI >100: Parameterschtzungen knnen relevant beeinflusst werden
					Die Berechnung der Condition Indizes erfolgt mittels PROC REG 
                    (Option COLLIN oder COLLINOINT).
					
   pov:				Der Parameter 'pov' steht fr 'Proportion of Variation' und 
					ermglicht die Spezifizierung dieses Kriteriums fr die 
					Kollinearitts-Detektion auf Basis der 'Proportion Of Variation'. 
					Dieser Parameter interagiert mit dem Condition Index (siehe dort). 
					Die Standardbelegung ist 0.5. Die Berechnung der Varianzanteile 
                    erfolgt ebenfalls mittels PROC REG.
	
   coll:			Dieser Parameter legt fest, ob PROC REG mittels der Option 
					'COLLIN' (coll=0) oder der Option 'COLLINOINT' (coll=1, Standard) 
					ausgefhrt werden soll. Der Unterschied zwischen den beiden 
					mglichen Optionen besteht darin, dass 'COLLINOINT' den Intercept 
					bei der Kollinearittsdiagnose zunchst anpasst.
					
   vif:				Der Parameter 'vif'(Variance Inflation Factor) dient zur 
					Erkennung von Variablen, die mglicherweise in 
					Multikollienarittsprobleme verwickelt sind. Das 
					Standardkriterium hierbei ist 'vif=10' [myers1990]. 
					Dies bedeutet, dass nach Variablen mit einem Variance Inflation 
					Factor > 10 gesucht wird. Finden sich keine dem Kriterium 
					entsprechenden Variablen, so wird im Output die Variable mit
					dem hchsten Variance Inflation Factor ausgegeben. Die Berechung 
					der Variance Inflation Faktoren erfolgt makrointern mittels der 
					Prozedur REG (Option VIF).	

   macro_path: 		Aufgrund der Tatsache, dass innerhalb des Makros diverse 
					Untermakros aufgerufen werden, ist es zwingend ntig, den Pfad 
					zu dem Verzeichnis, wo die komplette Makrosammlung abgelegt ist,
					anzugeben. Der Pfad selber muss in Hochkommata eingeschlossen 
					sein. Beispiel: 'f:\makros'. 


  VALIDATION
  ----------
  Validiert durch: Stephanie Heiss
 				   University of Ulm 
                   Department of Biometry and Medical Documentation
                   89075 Ulm
  
  Validiert am:    24. November 2003

  Validierungsprogramm::      PM_MULTICOLLIN_VAL.SAS
  Validierungsdokumentation:  PM_MULTICOLLIN_VAL.DOC
   
  Validierungsvorgehen:
  - Das Testen der Makros wird durch eine von der Programmierung unabhngige 
    Person durchgefhrt.
  - Fr die Durchfhrung der Validierung wird auf einen in der Literatur 
    genutzten Datensatz zurckgegriffen:  LBW (Low Birth Weight), beschrieben
    in Hosmer, Lemeshow: Applied logistic regression [hosmer2000]
  - Der Datensatz wird fr verschiedene Testlufe verndert bzw. erweitert, 
    damit "die Testdaten neben den normalen Angaben auch Grenzwerte, ungltige
    Angaben, Fehlerbedingungen und Sonderflle beinhalten." [beiwanger2003]
  - Fehlerkontrolle:
    Hierbei werden nur die im LOG-Fenster auftretenden Fehler bzw. Warnungen 
    und offensichtlichen Fehler im OUTPUT betrachtet. Fr jeden Parameter des 
    Makros werden mehrere Durchlufe mit verschiedenen Wertangaben durchgefhrt, 
    wobei auch ungltige Werte angegeben werden, um das Verhalten des Makros
    bei fehlerhaften Aufrufen zu untersuchen. Gefundene Fehler werden 
    dokumentiert und dem Programmierer zur Verfgung gestellt. Nach der Korrektur 
    aller Fehler werden die Testlufe wiederholt, um sicherzustellen, dass 
    wirklich alle Fehler abgefangen wurden und dass keine neuen Fehler entstanden 
    sind.
  - Ergebniskontrolle:
    Um die Richtigkeit der Ergebnisse zu berprfen, werden soweit mglich, die
    Rechenbeispiele und bungsaufgaben aus Hosmer/Lemeshow durch vergleichbare
    Makroaufrufe nachgerechnet. Die Ergebnisse werden denen aus dem Begleitbuch 
    Hosmer/Lemeshow/Cook [hosmer2001] gegenbergestellt und direkt verglichen.
  - Alle Validierungen werden dokumentiert.

  Literatur: 
  - Beiwanger S (2003): Validierung von SAS-Makros. Diplomarbeit, FH Ulm
  - Hosmer DW, Lemeshow S (2000): Applied logistic regression 2nd Ed. 
           J. Wiley & Sons, New York
  - Hosmer DW, Lemeshow S, Cook ED (2001): Solutions manual to accompany
           applied logistic regression 2nd. Ed. J. Wiley & Sons, New York


  AUTHOR
  ------
  R. Muche / C. Ring / C. Ziegler
  University of Ulm
  Department of Biometry and Medical Documentation
  89075 Ulm

  The author does not overtake any responsibility for the correctness
  of results. It must not be modified without removing the heading lines 
  containing the authors name.

----------------------------------------------------------------------------------*/

%MACRO PM_MULTICOLLIN ( 	data			= ,
							resp_var		= ,
							event			= 1,
							xvar			= ,
							cvar			= ,
							miss			= 0,
							spearman		= 0.6,
							condition_index	= 30,
							pov				= 0.5,
							coll			= 1,
							vif				= 10,
							macro_path		=
					  );

%LET datensatz2=&data;

/*Prfungen / Referenzierungen / Standardbelegungen*/

/*Wichtigster Parameter 'macro_path' berprfen*/
/*bergabe erfolgt?*/

%IF &macro_path= %THEN %DO;

	%WINDOW error color=grey                                                  
	 #5  @28 'Fehler!' attr=highlight  
         color=blue                                                         
	 #7  @28 "Dem zwingend erforderlichen Parameter 'macro_path' wurde kein Wert"
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"	
	 #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.'; 
 
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Hilfsprogramm welches berprft, ob dem Parameter 'macro_path' ein korrekter Pfad*/
/*bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/

/*Anzahl der Zeichen im String 'macro_path' bestimmen:*/

%LET path_length=%LENGTH(&macro_path);

%LET infile=&macro_path;

%IF %BQUOTE(&infile) NE %THEN %DO;
     %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
      %IF %BQUOTE(&char1) ne %STR(%')
         %THEN %LET command='NOK';
         %ELSE %LET command='OK';

	 %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
      %IF %BQUOTE(&char2) ne %STR(%')
         %THEN %LET command2='NOK';
         %ELSE %LET command2='OK';
%END;

/*falls die Syntax nicht stimmt, Anzeige eines Hinweisfensters*/

%IF &command='NOK' OR &command2='NOK' %THEN %DO;

	%WINDOW error color=grey                                                    
	 #5  @28 'Fehler!' attr=highlight                             
   			 color=blue                                            
	 #7  @28 "Dem Parameter 'macro_path' wurde ein syntaktisch falscher Wert" 
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
	 #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.'; 
 
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Test, ob das unter 'macro_path' angegebene Verzeichnis existiert*/

%LET dir=&macro_path;

%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
       %PUT Directory does not exist;
%END;
%ELSE %DO;
       %PUT Directory exists;
       %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;

	%WINDOW error color=grey       
	 #5  @28 'Fehler!' attr=highlight       
       	 color=blue                
	 #7  @28 "Das unter dem Parameter 'macro_path' angegebene Verzeichnis existiert"
	 #8  @28 "nicht- Bentigte Untermakros knnen nicht referenziert werden"
	 #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                  
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

%GOTO ende;

%END;

/*Falls bestimmte Parameter ohne Wert bergeben wurden, dises mit Standard belegen*/

%IF &miss= %THEN %DO;
	%LET miss=0;
%END;

%IF &event= %THEN %DO;
	%LET event=1;
%END;

%IF &spearman= %THEN %DO;
	%LET spearman=0.6;
%END;

%IF &condition_index= %THEN %DO;
	%LET condition_index=30;
%END;

%IF &pov= %THEN %DO;
	%LET pov=0.5;
%END;

%IF &vif= %THEN %DO;
	%LET vif=10;
%END;

%IF &coll= %THEN %DO;
	%LET coll=1;
%END;

/*Externe Makros, die innerhalb des Makros 'PM_MULTICOLLIN.MAC.SAS'*/
/*verwendet werden (Untermakros) einbinden (%include)*/

FILENAME y &macro_path;

%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('exist.mac.sas');
%INCLUDE y ('exist_var.mac.sas');
%INCLUDE y ('numchar.mac.sas');
%INCLUDE y ('dummy_friendly.mac.sas');
%INCLUDE y ('fl_heinze.mac.sas');

/*globale Variable des Untermakros 'numberobs.mac.sas'*/

%GLOBAL nnnn1;

/*Test, ob bei bestimmten Parametern anstatt numerischen Werten*/
/*character- Werte bergeben wurden*/

/*berprfung erfolgt mit Hilfe des Untermakros 'numchar.mac.sas'*/

%NUMCHAR(parameterliste=&spearman &condition_index &pov &vif &coll);

%IF &numcharok='NOK' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error8.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Parameter, bei denen keine negativen Werte erlaubt sind, berprfen*/

%IF &vif <=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error9.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%IF &condition_index <=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error9.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfen, ob der angegebene Datensatz (Parameter 'data=') auch wirklich existiert*/
/*(mit Hilfe des Hilfs-/Untermakros 'exist.mac.sas')*/

%EXIST(dsn=&data);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error4.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Datenquelle (Parameter 'data=' UND response-Variable*/
/*(Zielgre / Parameter 'resp_var=') angegeben wurde*/

%IF &data= OR &resp_var= %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error1.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

/*berprfung, ob berhaupt Variablen (Parameter 'xvar=' / 'cvar=') angegeben wurden*/

%IF &xvar= AND &cvar= %THEN %DO;

/*Falls keine Variablen angegeben wurden,*/
/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error2.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;
%END;

/*berprfung, ob die Zielgre (Parameter 'resp_var')*/
/*2 Kategorien (Ausprgungen) besitzt*/

/*missing values ausschliessen*/

DATA freq2;
	SET &data;
	IF &resp_var ne .;
RUN;

/*Anzahl der Ausprgungen mit Hilfe der Prozedur PROC FREQ ermitteln*/

/*Ausgabe unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnis als ODS- Datei zur weiteren Verarbeitung ausgeben*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=freq2;
	TABLES &resp_var;
RUN;

/*ODS- Output schliessen*/

ODS OUTPUT CLOSE;

/*ODS- Ausgabe wieder aktivieren*/

ODS LISTING;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen (= # Ausprgungen der Zielgre)*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq freq2;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

/*falls Zielgre mehr als zwei Ausprgungen besitz--> Fehlermeldung*/

%IF &event_check > 2 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error5.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

/*Prfung, ob alle bergebenen Variablen numerisch sind*/

/*Erzeugung eines temporren Zwischendatensatzes, welcher lediglich die*/
/*bergebenen Variablen enthlt*/

DATA test3;
	SET &data;
	KEEP &xvar &resp_var &cvar;
RUN;

/*mittels der Prozedur PROC CONTENTS kann der Typ einer Variablen bestimmt werden*/

/*Unterdrckung der Ausgabe von PROC CONTENTS*/

PROC CONTENTS DATA=test3 NOPRINT OUT=testcontent;
RUN;

/*lediglich die Variable 'type' der Ergebnisdatei behalten,*/
/*da nur diese von Interesse ist*/

DATA testcontent;
	SET testcontent;
	KEEP type;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=testcontent);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test4=&nnnn1;

/*Prfung ob Variablen vom Typ 'character' vorhanden sind*/

DATA test4;
	SET testcontent;
	IF type=2 THEN DELETE;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test4);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test5=&nnnn1;

/*falls Variablen bergeben wurden, die nicht numerisch sind --> Fehlermeldung*/

%IF &test4 ne &test5 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error60.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE test4 testcontent test3;
RUN;
QUIT;

/*berprfung, ob alle bergebenen Variablen im bergebenen Datensatz*/
/*(Parameter 'data' vorhanden sind)*/

/*Zusammenfassung aller Variablen in einer neuen Makro- Variablen*/

%LET all_variables= &resp_var &xvar &cvar;

/*Prfung erfolgt mit Hilfe des Untermakros 'exist_var.mac.sas'*/

%EXIST_VAR(dsn=&data, var=&all_variables);

/*falls irgendwelche Variablen nicht im angegebenen Datensatz existieren,*/
/*Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error6.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Parameter 'missing' 0 oder 1 ist*/

%IF &miss ne 1  %THEN %DO;
	%IF &miss ne 0  %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error7.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'event' 0 oder 1 ist*/

%IF &event ne 1  %THEN %DO;
	%IF &event ne 0  %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error7.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfen ob eine Variable nach Entfernung aller Beobachtungen mit missing*/
/*values nur noch eine Ausprgung besitzt*/

/*Anzahl der stetigen Variablen ermitteln:*/

%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values*/

DATA work.category_test;
	SET  &data;

	%DO i=1 %TO &count_x;
		%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
		IF &xvar_liste=. THEN DELETE;
	%END;

	%DO i=1 %TO &count;
		%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
		IF &cvar_liste=. THEN DELETE;
	%END;

		IF &resp_var=. THEN DELETE;

RUN;

/*Prfen ob 'resp_var' /'xvar'/ 'cvar' nur 1 Ausprgung besitzen*/

%LET category_test=0;
%LET category_test_fail=;

/*event*/

/*Anzahl der Ausprgungen mittels PROC FREQ bestimmen*/

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &resp_var;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

%IF &event_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &resp_var;

%END;

/*xvar*/

%IF &xvar ne %THEN %DO;

%DO uhgu=1 %TO &count_x;

%LET xvar_liste=%SCAN(&xvar,&uhgu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &xvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET xvar_check=&nnnn1;

%IF &xvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &xvar_liste;

%END;

%END;
%END;

/*cvar*/

%IF &cvar ne %THEN %DO;

%DO uhfu=1 %TO &count;

%LET cvar_liste=%SCAN(&cvar,&uhfu,%STR( ));

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnisse mittels ODS bestimmen*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=category_test;
	TABLES &cvar_liste;
RUN;

/*ODS Output wieder schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET cvar_check=&nnnn1;

%IF &cvar_check=1 %THEN %DO;

	%LET category_test=1;
	%LET category_test_fail=&category_test_fail &cvar_liste;

%END;

%END;
%END;

PROC DATASETS NOLIST;
	DELETE category_test;
RUN;
QUIT;

%IF &category_test=1 %THEN %DO;

	DATA category_test;
		var="&category_test_fail";
	RUN;

	PROC PRINT DATA=category_test NOOBS LABEL;
	TITLE1 "Folgende Variablen besitzen nach Entfernung aller";
	TITLE2 "missing values lediglich eine Ausprgung:";
		VAR var;
			LABEL var='Variable';
	RUN;

	PROC DATASETS NOLIST;
		DELETE category_test;
	RUN;
	QUIT;

	%INCLUDE y ('error24.sas');
	%GOTO ende;

%END;


/*ENDE Prfungen / Referenzierungen / Standardbelegungen*/

/*-------------------------------------------------------------------------------*/

/*Standardoutput ausgeben*/

%LET datensatz=&data;

/*falls Auswertung mit missing-values erwnscht:*/

%IF &miss=1 %THEN %DO;

/*Fallzahl mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen:*/

%NUMBEROBS(datensatz=&data);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET fallzahl=&nnnn1;

%END;

/*falls Auswertung ohne missing-values erwnscht:*/

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/
%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls 'miss=0' angegeben wurde*/

DATA work.fallzahl;
	SET &data;

	%DO i=1 %TO &count_x;
		%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));

		IF &xvar_liste=. THEN DELETE;

	%END;

	%DO i=1 %TO &count;
		%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));

		IF &cvar_liste=. THEN DELETE;

	%END;

		IF &resp_var=. THEN DELETE;

RUN;

%LET fallzahldata=fallzahl;

/*# Beobachtungen im ohne missing values*/
/*mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen:*/

%NUMBEROBS(datensatz=&fallzahldata);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET fallzahl=&nnnn1;

%END;

/*Makro-kopf ausgeben*/

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatz       ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';

RUN;

/*temporr angelegte Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE standardoutput;

	%IF &miss=0 %THEN %DO;

	DELETE	fallzahl;

	%END;
RUN;
QUIT;

/*ENDE Standardoutput*/

/*----------------------------------------------------------------------*/

/*Anzahl der angegebenen Variablen ('xvar' / 'cvar') ermitteln:*/

%LET vars=&xvar &cvar;

%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&vars,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&vars,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Anzahl ausgeben im LOG- Fenster*/
%PUT The vars contains &count variables.;

/*Elimination aller missing values falls 'miss=0' angegeben wurde*/

%IF &miss=0 %THEN %DO;

DATA work.nomiss;
	SET &data;

	%DO i=1 %TO &count;

		%LET var_liste=%SCAN(&vars,&i,%STR( ));
		IF &var_liste=. THEN DELETE;

	%END;

		IF &resp_var=. THEN DELETE;

%LET data=nomiss; 
RUN;

%END;

/*Anzahl der angegebenen kategoriellen Variablen ermitteln:*/
%GLOBAL c_count c_var;
%LET c_count=1;
%LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%DO %WHILE(&c_var ne);
     %LET c_count=%EVAL(&c_count+1);
     %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%END;
%LET c_count=%EVAL(&c_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/
%PUT The cvar contains &c_count variables.;

/*PROC LOGISTIC um Gewichte zu erhalten*/
/*S. 50 'Logistic Regression Using The SAS System', Theory and Application, */
/*Paul D. Allison, 1999*/

/*Ausgabe unterdrcken*/

ODS LISTING CLOSE;

/*(Quasi-) Separation prfen*/

ODS OUTPUT ConvergenceStatus=cs;

/*NAMELEN- Option definiert die Breite von Spalten in der Ausgabedatei*/
/*---> 32=maximal*/

PROC LOGISTIC DATA=&data NAMELEN=32 OUTEST=fl_fail_est;

%IF &c_count ne 0 %THEN %DO;

	CLASS

	%DO i=1 %TO &c_count;

		%LET var_liste=%SCAN(&cvar,&i,%STR( ));
      
		&var_liste (PARAM=REF REF=FIRST)

	%END;

     ;

%END;


	MODEL &resp_var (

	%IF &event=1 %THEN %DO;
		event='1'
	%END;

	%IF &event=0 %THEN %DO;
		event='0'
	%END;

	)=&xvar &cvar;
	OUTPUT OUT=a PRED=phat;
RUN;

/*ODS- Output schliessen*/

ODS OUTPUT CLOSE;

/*ODS- Ausgabe wieder aktivieren*/

ODS LISTING;


/*(Quasi)- Separation von PROC LOGISTIC immer berprfen*/

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=cs);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET cs_check=&nnnn1;

DATA est_check;
	SET cs;
	IF reason ne 'Quasi-complete separation of data points detected.';
	IF reason ne 'Complete separation of data points detected.';
RUN;

/*Test ob es berhaupt Estimates ohne Warnung gibt:*/

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS (datensatz=est_check);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET est_count=&nnnn1;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE est_check cs ;
RUN;
QUIT;

/*Globale Variable anlegen*/

%GLOBAL status_separation;

/*(Quasi)- Separation vorhanden*/

%IF &est_count ne &cs_check %THEN %DO;

%LET status_separation=1;

%END;

/*keine (Quasi)- Separation vorhanden*/

%ELSE %DO;

%LET status_separation=0;

%END;

/*Falls (quasi-) separation aufgetreten ist:*/
/*Korrektur:*/

%IF &status_separation=1 %THEN %DO;

/*Output- Datei / LOG- und OUTPUT- Fenster wieder lschen*/

DM "clear log; clear output;";

PROC DATASETS NOLIST;
	DELETE a;
RUN;
QUIT;

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count_c varc;
%LET count_c=1;
%LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%DO %WHILE(&varc ne);
     %LET count_c=%EVAL(&count_c+1);
     %LET varc=%QSCAN(&cvar,&count_c,%STR( ));
%END;
%LET count_c=%EVAL(&count_c-1);

/*Anzahl ausgeben im LOG- Fenster*/
%PUT The cvar contains &count_c variables.;

/*Dummy- Kodierung (nicht automatisch via CLASS- Statement bei PROC LOGISTIC*/
/*---> Heinze- Makro FL kennt keine kategorischen (klassierten) Variablen*/

%LET all_variables=;

DATA ergebnis_ende (KEEP=&resp_var &cvar &xvar);
	SET &data;
RUN;

/*Referenzen zu jeweiliger Variable zuordnen*/

%DO m=1 %TO &count_c;
	%LET var=%SCAN(&cvar,&m,%STR( ));
	%LET count_ref=0;

/*Ausprgungen der jeweiligen kategoriellen Variable ermitteln*/

	PROC FREQ DATA=&data NOPRINT;
		TABLES &var / SPARSE OUT=&var;
	RUN;

/*Ausgabedatensatz von PROC FREQ anpassen*/

	DATA &var (DROP=percent count);
		SET &var;
		IF &var=. THEN DELETE;
		IF count=0 THEN DELETE;
	RUN;

/*Anzahl der mglichen Ausprgungen je kategorieller Variable ermitteln*/
	
	%NUMBEROBS(datensatz=&var);

	%LET count_var=&nnnn1;

	%LET underscore=_;

/*Falls keine Referenzkategorie angegeben wurde, die erste nehmen*/
	
	%IF &count_ref=0 %THEN %DO;

		%LET var_ref=_FIRST_;

	%END;

/*Referenzkategorie "FIRST" fr bergabe an Dummy- Makro anpassen ("_FIRST_")*/

	%IF &var_ref=FIRST %THEN %DO;

		%LET var_ref=_&&var_ref&&underscore;

	%END;

/*Referenzkategorie "LAST" fr bergabe an Dummy- Makro anpassen ("_LAST_")*/

	%IF &var_ref=LAST %THEN %DO;

		%LET var_ref=_&&var_ref&&underscore;

	%END;

/*jeweilige Basiskategorie lschen*/

	DATA var2;
		SET &var end=_eof_;

			%IF 		&var_ref = _FIRST_ | &var_ref = LOW 
			%THEN 		%STR( IF _n_ = 1 THEN DELETE;);
			%ELSE %IF 	&var_ref = _LAST_ | &var_ref = HIGH
			%THEN 		%STR( IF _eof_ THEN DELETE;);
			%ELSE 		%STR(IF &var = &var_ref THEN DELETE;);
	RUN;

/*Anzahl Kategorien der jeweiligen Variable ermitteln*/

	%NUMBEROBS(datensatz=var2);

/*mgliche Ausprgungen pro Variable als Makro- Variable zwischenspeichern*/

	%LET count_var2=&nnnn1;

	DATA var2;
		SET var2;
			CALL SYMPUT('auspraeg'||left(_n_), trim(left(put(&var,BEST8.))));
	RUN;

/*Aufruf des Dummy- Kodierungs- Makros*/

/*Output unterdrcken*/

ODS LISTING CLOSE;

%DUMMY( 
 	 		 data		= work.ergebnis_ende,   /* name of input dataset      */
   			 out		= work.ergebnis_ende,   /* name of output dataset     */
  			 var		= &var,         	    /* variable(s) to be dummied  */
  			 base		= &var_ref, /* base category                          */
   			 prefix 	= ,    		/* prefix for dummy variable names        */
   			 format 	= ,      	/* format used to categorize variable     */
  			 name  		= VAL,  	/* VAL: variable names are D_value        */
  			 fullrank	= 1     	/* Eliminate dummy for baseline category? */
      );

/*Output wieder aktivieren*/

ODS LISTING;

/*Falls fr eine Variable Dummies generiert wurden, Originalvariable lschen*/

DATA ergebnis_ende (DROP=&var);
	SET ergebnis_ende;

/*zustzliche Umbenennung (Anpassung an automatische Ausgabe der Dummy- Variablen*/
/*von PROC LOGISTIC)*/

	%DO o=1 %TO &count_var2;

	RENAME &var._&&auspraeg&o = &var&&auspraeg&o;

	%END;

RUN;

/*Temporrer Zwischendatensatz lschen*/

PROC DATASETS NOLIST;
	DELETE &var;
RUN;
QUIT;

%END;

/*Mittels PROC CONTENTS die Variablen fr das FL- Makro selektieren*/
/*ohne Event ('resp_var')*/

DATA ergebnis_ende2 (DROP=&resp_var);
	SET ergebnis_ende;
RUN;

/*Output unterdrcken*/

PROC CONTENTS DATA=ergebnis_ende2 NOPRINT OUT=ergebnis2 (KEEP=name);
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=ergebnis2);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET count_ergebnis2=&nnnn1;

/*Variablennamen als Makro-Variablen speichern*/

DATA ergebnis2;
	SET ergebnis2;
		CALL SYMPUT('y'||left(_N_), name);
RUN;

/*Variablennamen fr FL- Makro zusammenfassen*/

%DO s=1 %TO &count_ergebnis2;

	%LET all_variables=&all_variables &&y&s;

/*	Ausgabe im LOG- Fenster zur bersicht*/

	%PUT &all_variables;

%END;


/*Aufruf des FL- Makros*/

%FL		(	data			= ergebnis_ende, 
			y				= &resp_var, 
			varlist			= &all_variables, 
			outest			= outest_fl, 
			print			= 0
       );

/*berflssige Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE ergebnis_ende _work _outtab _offset_ _maxby _fby _estwot_ _cats_ 
		   var2 ergebnis_ende2 ergebnis2;
RUN;
QUIT;

/*Datei mit den Regressionskoeffizienten aus dem*/
/*FL- Makro fr INEST- Statement bei PROC LOGISTIC vorbereiten*/

DATA inest_fl (DROP=_rby_ _penlik_ _it_ _resp_ _noresp_ _by_ _total_ _code_);
	SET outest_fl;
		IF _code_ ne -1 THEN DELETE;
		RENAME INTERCEP=Intercept;
RUN;

/*Prfen, ob FL- Makro korrekt abgelaufen ist*/

%NUMBEROBS(datensatz=inest_fl);

%LET fl_fail_check=&nnnn1;

%IF &fl_fail_check=0 %THEN %DO;

%LET fl_false=1;

%END;

%ELSE %DO;

%LET fl_false=0;

%END;

/*----------------------------------------------------------------------*/

/*Standardoutput erneut ausgeben*/

%LET datensatz=&data;

/*Fallzahl bestimmen:*/

/*falls mit missing values*/

%IF &miss=1 %THEN %DO;

/*Anzahl der Beobachtungen mit missing values mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=&data);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET fallzahl=&nnnn1;

%END;

/*falls ohne missing values*/

%IF &miss=0 %THEN %DO;

/*Anzahl der stetigen Variablen ermitteln:*/
%GLOBAL count_x varx;
%LET count_x=1;
%LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%DO %WHILE(&varx ne);
     %LET count_x=%EVAL(&count_x+1);
     %LET varx=%QSCAN(&xvar,&count_x,%STR( ));
%END;
%LET count_x=%EVAL(&count_x-1);

/*Anzahl der kategoriellen Variablen ermitteln:*/
%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&cvar,&count,%STR( ));
%DO %WHILE(&var ne);
     %LET count=%EVAL(&count+1);
     %LET var=%QSCAN(&cvar,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Elimination aller missing values falls miss=0 angegeben wurde*/

DATA work.fallzahl;
	SET  &data;

		%DO i=1 %TO &count_x;

			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;

		%END;

		%DO i=1 %TO &count;

			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;

		%END;

			IF &resp_var=. THEN DELETE;

RUN;

%LET fallzahldata=fallzahl;

/*Anzahl der Beobachtungen ohne missing values mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=&fallzahldata);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET fallzahl=&nnnn1;

%END;

/*Standardoutput ausgeben*/

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatz2      ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

/*temporre Zwischendatei(en) lschen*/

PROC DATASETS NOLIST;
	DELETE standardoutput;

	%IF &miss=0 %THEN %DO;

		DELETE	fallzahl;

	%END;
RUN;
QUIT;

/*ENDE Standardoutput*/

/*----------------------------------------------------------------------*/


/*Hinweis ausgeben, dass Firth- Prozedur benutzt wurde*/
/*und eine Korrektur stattgefunden hat*/

DATA note;
	note="";
RUN;

PROC PRINT DATA=note NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Warnung:               ";
	TITLE4 ;
	TITLE5 "There is possibly a quasi-complete separation of data"; 
	TITLE6 "points.The maximum likelihood estimate may not exist.";
	TITLE7 "Validity of the model fit is questionable.";
	TITLE8 ;
	TITLE9 'Es wurde dehalb eine automatische Korrektur mittels der';
	TITLE10 "Firth- Prozedur (Makro 'FL_HEINZE.MAC.SAS') durchgefhrt!";
	
	LABEL note = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";

RUN;

/*temporre Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE note;
RUN;
QUIT;

%IF &fl_false=1 %THEN %DO;

/*Hinweis ausgeben, dass Firth- Prozedur (Korrektur) fehlgeschlagen ist*/

DATA note2;
	note2="";
RUN;

PROC PRINT DATA=note2 NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   note2;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Warnung:               ";
	TITLE4 "Die Korrektur der Regressionskoeffizienten mittels der";
	TITLE5 "Firth- Prozedur (Makro 'FL_HEINZE.MAC.SAS') ist"; 
	TITLE6 "fehlgeschlagen! Evtl. wurden zu viele Koeffizienten in";
	TITLE7 "Bezug auf die Anzahl der Beobachtungen geschtzt. In der";
	TITLE8 "weiteren Analyse werden die Koeffizienten des letzten";
	TITLE9 "ML-Iterationsschrittes aus PROC LOGISTIC verwendet.";
	TITLE10 "Die Validitt dieses Modells ist fraglich!";
	
	LABEL note2 = "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";

RUN;

/*temporre Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE note2;
RUN;
QUIT;

%END;

/*neue PROC LOGISTIC mit INEST- Statement*/

PROC LOGISTIC DATA=&data 

%IF &fl_false=0 %THEN %DO;

INEST=inest_fl 

%END;
%ELSE %DO;

INEST=fl_fail_est

%END;



NOPRINT NAMELEN=32;

/*PROC LOGISTIC CLASS- Statement ohne dazugehrigen Referenzkategorien*/

	CLASS

		%DO i=1 %TO &c_count;
			%LET var_liste=%SCAN(&cvar,&i,%STR( ));
      
			&var_liste (PARAM=REF REF=FIRST)

		%END;

     	;

	MODEL &resp_var (

	%IF &event=1 %THEN %DO;
		event='1'
	%END;

	%IF &event=0 %THEN %DO;
		event='0'
	%END;

	)=&xvar &cvar / MAXITER=0;
	OUTPUT OUT=a PRED=phat;
RUN;

/*temporre Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE inest_fl outest_fl;
RUN;
QUIT;

%END;

/*Gewichte berechnen*/

DATA b;
	SET a;
	w=(phat*(1-phat));
RUN;

/*Output unterdrcken*/

ODS LISTING CLOSE;

/*ODS- Output in Abhngigkeit der gewnschten PROC REG- MODEL- Option:*/

%IF &coll=1 %THEN %DO;

ODS OUTPUT COLLINDIAGNOINT=condition_index;

%END;
%ELSE %DO;

ODS OUTPUT COLLINDIAG=condition_index;

%END;

ODS OUTPUT ParameterEstimates=parameter_estimates;

/*Start von Proc Reg auf die angegebenen Daten*/

PROC REG DATA=b;
	WEIGHT w;
	MODEL &resp_var

	=&vars /

	%IF &coll=1 %THEN %DO;

		COLLINOINT 

	%END;
	%ELSE %DO;

		COLLIN

	%END;

		VIF;

RUN;
QUIT;

/*ODS- Output schliessen*/

ODS OUTPUT CLOSE;
ODS LISTING;

/*Start von Proc Corr (Berechnung der Spearman Rangkorrelationskoeffizienten)*/

PROC CORR DATA=&data SPEARMAN NOPRINT OUTS=out;
	VAR &vars;
RUN;

/*berflssige Informationen lschen*/

DATA spearman;
	SET out;
	IF _TYPE_ ='MEAN' THEN DELETE;
	IF _TYPE_ ='STD'  THEN DELETE;
	IF _TYPE_ ='N'    THEN DELETE;
RUN;

DATA null;
	Prozedur	="The CORR Procedure";
	Analyse		="Spearman Correlation Coefficients";
	Bedingung	="Spearman Rangkorrelationskoeffizienten >&spearman und <1";
RUN;

/*berschrift ausgeben*/

PROC PRINT DATA=null NOOBS;
	TITLE "Multikollinearittsprfung";
RUN;

/*berschrifts-Datei lschen*/

PROC DATASETS NOLIST;
	DELETE null;
RUN;
QUIT;

%GLOBAL xy;
%LET xy=1;

/*Ergebnisdatei null-initialisieren*/

DATA ergebnis;
	SET _NULL_;
RUN;

%DO i=1 %TO &count;

%LET var_string=%SCAN(&vars,&i,%STR( ));

DATA spearman&i (KEEP=_NAME_ &var_string);
	SET spearman;
RUN;

/*Beobachtungen, die Kriterien nicht erfllen lschen*/

DATA spearman&i;
	SET spearman&i;
	IF &var_string > 0.999 THEN DELETE;
	IF &var_string < &spearman THEN DELETE;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS (datensatz=spearman&i);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET zaehler=&nnnn1;

%IF &zaehler=0 %THEN %DO;

%LET xy=%EVAL(&xy+1);

%END;
%ELSE %DO;

DATA ergebnis;
	SET ergebnis spearman&i;
	LABEL &var_string=&var_string;
RUN;

%END;

/*Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE spearman&i;
RUN;
QUIT;

%END;

/*Anzahl Beobachtungen in der Ergebnisdatei ermitteln*/

%NUMBEROBS (datensatz=ergebnis);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET ergebniszaehler=&nnnn1;

/*Falls Ergebnisdatei Beobachtungen enthlt:*/

%IF &ergebniszaehler ne 0 %THEN %DO;

/*Doppelte Werte aus dem Ergebnisdatensatz entfernen*/

PROC SORT DATA=ergebnis OUT=ergebnis_trans;
	BY _NAME_;
RUN;

PROC TRANSPOSE DATA=ergebnis_trans OUT=ergebnis_trans;
	BY _NAME_;
RUN;

/*Anzahl und Namen der Variablen in der Datei 'ergebnis_trans' feststellen*/

PROC CONTENTS DATA=ergebnis_trans out=yy POSITION NOPRINT;
RUN;

DATA yy (KEEP=name varnum format type);
	SET yy;
	IF type=1;
	IF format NOT IN("DDMMYYP", "DATE");
RUN;

PROC SORT DATA=yy;
	BY varnum;
RUN;

DATA yy;
	SET YY;
	LENGTH varlist $2000.;
	RETAIN varlist;
	IF _n_=1 THEN varlist=name;
/*Funktion "compbl" --> blanks zwischen den Variablen eliminieren*/
	ELSE varlist=COMPBL(varlist || name);
	CALL symput ("varlist", COMPBL(varlist));
RUN;

PROC SORT DATA=ergebnis_trans OUT=ergebnis_trans;
	BY &varlist;
RUN;

%NUMBEROBS(datensatz=yy);
%LET count_var=&nnnn1;

%DO i=1 %TO &count_var;

DATA x&i (KEEP= _Name_ _Label_ COL&i);
	SET ergebnis_trans;
RUN;

DATA x&i;
	SET x&i;
	RENAME COL&i=spearman;
RUN;

%END;

DATA gesamt;
	SET _NULL_;
RUN;

%DO i=1 %TO &count_var;

DATA gesamt;
	SET gesamt x&i;
RUN;

/*temporre Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE x&i; 
RUN;
QUIT;

%END;

DATA gesamt;
	SET gesamt;
	IF spearman=. THEN DELETE;
RUN;

PROC SORT DATA=gesamt NODUPKEY OUT=gesamt;
	BY DESCENDING spearman;
RUN;

/*Umbenennungen*/

DATA gesamt;
	SET gesamt;
	RENAME _Label_=Variable2
    	   _Name_ =Variable1;
    LABEL  _Label_=Variable2;
RUN;

/*Ergebnis ausgeben*/

PROC PRINT DATA=gesamt NOOBS;
	TITLE;
RUN;

/*temporre Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE yy ergebnis_trans ergebnis gesamt; 
RUN;
QUIT;

%END;

%LET xy=%EVAL(&xy-1);

%IF &xy=&count %THEN %DO;

DATA null2;
	Note="Keine Spearman Rangkorrelationskoeffizienten >&spearman und <1 vorhanden!";
RUN;

PROC PRINT DATA=null2 NOOBS;
RUN;

PROC DATASETS NOLIST;
	DELETE null2 ergebnis;
	RUN;
QUIT;

%END;

/*Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE spearman out;
	RUN;
QUIT;

/*Selektion der Variablen mit Condition Index > angegebenem Wert*/

DATA condition_analysis;
	SET condition_index;
	IF ConditionIndex < &condition_index THEN DELETE;
RUN;

/*Selektion der Variablen mit Proportion of Variance > angegebenem Wert*/

DATA condition_analysis2 (DROP=model dependent number eigenvalue);
	SET condition_analysis;

	IF

	%DO i=1 %TO &count;

  		%LET var_string=%SCAN(&vars,&i,%STR( ));

		%IF &i ne &count %THEN %DO;

         &var_string < &pov AND

		%END;

		%ELSE %DO;
		
 		&var_string < &pov

		%END;

	%END;

	THEN DELETE;
RUN;

/*Anzahl Beobachtungen im Datensatz ermitteln*/

%NUMBEROBS(datensatz=condition_analysis2);

/*Transponieren*/

PROC TRANSPOSE DATA=condition_analysis2 OUT=condition_analysis3;
RUN;

DATA condition_analysis4;
	SET condition_analysis3;

	%LET x=&nnnn1;
	%IF &x ne 0 %THEN %DO;

	IF

	%DO i=1 %TO &nnnn1;
  		
		%IF &i ne &nnnn1 %THEN
		%DO;

        COL&i < &pov AND

		%END;

		%ELSE %DO;
		
 		COL&i < &pov

		%END;

	%END;

	THEN DELETE;

	%END;
RUN;

/*Fr Ausgabe vorbereiten*/

DATA condition_analysis4 (DROP=_LABEL_);
	SET condition_analysis4;
RUN;

/*Falls Beobachtungen vorhanden sind, die Kriterien erfllen, Ergebnisse ausgeben*/

%IF &x ne 0 %THEN %DO;
TITLE "Kollinearittsdiagnose mit Condition Index >&condition_index und Proportion Of Variation >&pov";

/*Ausgabe des Ergebnisses*/

PROC PRINT DATA=condition_analysis4 NOOBS;
RUN;

%END;
%ELSE %DO;

/*Falls keine Beobachtungen vorhanden sind, die die Kriterien erfllen:*/

DATA null3;
	TITLE "Kollinearittsdiagnose mit Condition Index >&condition_index und Proportion Of Variation >&pov";
	Note="Keine Variablen  mit Condition Index >&condition_index und Proportion Of Variation >&pov vorhanden!";
RUN;

PROC PRINT DATA=null3 NOOBS;
RUN;

/*maximalen CI bestimmen*/

DATA max_ci;
	SET condition_index;
RUN;

PROC SORT DATA=max_ci OUT=max_ci_sorted;
	BY DESCENDING ConditionIndex ;
RUN;

DATA max_ci_sorted;
	SET max_ci_sorted;
	obs=_N_;
RUN;

DATA max_ci_sorted (DROP=model dependent number eigenvalue);
	SET max_ci_sorted;
	IF obs ne 1 THEN DELETE;
RUN;

PROC TRANSPOSE DATA=max_ci_sorted OUT=max_ci_sorted_trans;
RUN;

DATA max_ci_sorted_trans (KEEP=_Name_ Col1);
	SET max_ci_sorted_trans;
	IF _Name_ ne 'ConditionIndex' THEN DELETE;
RUN;

DATA max_ci_sorted_trans;
	SET max_ci_sorted_trans;
	LABEL _Name_='Maximaler'
	COL1 ='Condition Index';
RUN;

PROC PRINT DATA=max_ci_sorted_trans NOOBS LABEL;
	TITLE;
RUN;

/*temporre Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE null3 max_ci_sorted_trans max_ci_sorted max_ci;
RUN;
QUIT;
    
%END;

/*"Variance Inflation Factor"- Kriterium berprfen*/

DATA vif (KEEP=VarianceInflation Variable);
	SET parameter_estimates;
	IF variable='Intercept' THEN DELETE;
RUN;

DATA vif2;
	SET vif;
	IF VarianceInflation<&vif THEN DELETE;
RUN;

%NUMBEROBS (datensatz=vif2);
%LET xy=&nnnn1;

%IF &xy=0 %THEN %DO;

/*Keine Variablen vorhanden, welche das Kriterium erfllen*/

DATA null4;
	TITLE "Kollinearittsdiagnose mit Variance Inflation Factor >&vif";
	Note="Keine Variablen mit Variance Inflation Factor >&vif vorhanden!";
RUN;

PROC PRINT DATA=null4 NOOBS;
RUN;

PROC DATASETS NOLIST;
	DELETE null4;
RUN;
QUIT;

/*maximalen VIF bestimmen*/

DATA max_vif;
	SET vif;
RUN;

PROC SORT DATA=max_vif OUT=max_vif_sorted ;
	BY DESCENDING VarianceInflation ;
RUN;

DATA max_vif_sorted;
	SET max_vif_sorted;
	obs=_N_;
RUN;

DATA max_vif_sorted;
	SET max_vif_sorted;
	IF obs ne 1 THEN DELETE;
RUN;

/*maximalen VIF ausgeben*/

PROC PRINT DATA=max_vif_sorted NOOBS LABEL;
	TITLE;
	LABEL VarianceInflation='Maximaler Variance Inflation Factor';
	VAR Variable VarianceInflation;
RUN;

/*Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE max_vif max_vif_sorted;
RUN;
QUIT;

%END;
%ELSE %DO;

PROC PRINT DATA=vif2 NOOBS LABEL;
	TITLE "Kollinearittsdiagnose mit Variance Inflation Factor >&vif";
	LABEL VarianceInflation='Variance Inflation Factor';
RUN;

%END;

/*Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE condition_index condition_analysis condition_analysis2 condition_analysis3 
		   condition_analysis4 parameter_estimates vif vif2 a b fl_fail_est;

	%IF &miss=0 %THEN %DO;
	
	DELETE nomiss;	
	
	%END;

RUN;
QUIT;

/*Sprungmarke im Fehlerfall*/

%ende:

%MEND PM_MULTICOLLIN;
