/************************************************************************************/
/*						        													*/
/*     NAME: pm_roc.mac.sas	         												*/
/*    TITLE: ROC-Analyse / Prognosegtemae	        								*/
/*     KEYS: roc- analysis, roc- curve, confidence limit, area under the curve (auc)*/
/*			 sensitivity, specificity, cutpoint, (max.) youden-index, separation-	*/
/*			 index, ppv, npv, predicted probablities, false positive, 				*/
/*			 false negative, prevalence, slope, intercept, emax, brier-score,		*/
/*			 r-square, max-rescaled-r-square, somer's d								*/
/*  VERSION: SAS 8.2												      			*/
/*  PRODUCT: BASE, STAT, GRAPH							        					*/
/*    PROCS: UNIVARIATE, LOGISTIC, MEANS, GPLOT, GCHART			     				*/
/*															         				*/
/*																					*/
/*  Verzeichnis/Dateiname:			F:\Prognose\pm_roc.mac.sas	        			*/
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*  Programmierer:					Christoph Ziegler								*/
/*  Datum der Ersterstellung:		9/2002											*/
/*  Aktuelle Makro-Version:			12/2003											*/
/*																					*/
/*  Kurzbeschreibung:				Makro zur ROC-Analyse (Prognosegte)			*/
/*																					*/
/*  Verwendete externe Untermakros:	cibinom_daly.mac.sas							*/
/*									rocdat_hilgers.mac.sas							*/
/*									auroc_koenig.mac.sas							*/
/*									printit.mac.sas									*/
/*									numberobs.mac.sas								*/
/*									numchar.mac.sas									*/
/*									exist.mac.sas									*/
/*									exist_var.mac.sas								*/
/*									error1.sas	error4.sas	error5.sas				*/
/*									error6.sas	error8.sas	error9.sas				*/
/*									error16.sas	error36.sas	error37.sas				*/
/*									error38.sas	error39.sas	error40.sas	error44.sas	*/
/*									error45.sas	error46.sas	error47.sas	error60.sas	*/
/*									error66.sas										*/
/*																					*/
/*  Bentigte Dateien:				Datensatz 			(Parameter 'data=')			*/
/*									ROC- Daten 			(Parameter 'data_roc=')		*/
/*     								Predicted Values  	(Parameter 'data_pred=')	*/
/*									C- Value/ Somer's D (Parameter 'data_asso=')	*/
/*																					*/
/*  Erzeugte Dateien:				work.result 									*/
/*  								(enthlt Cutpoint-abhngige Prognosegtemae:	*/
/*	(Dateinamen nicht 				 Max. Youdenindex, Cutpoint, Sensitivitt,		*/
/*	 frei whlbar)					 Spezifitt, PPV, NPV, PSEP)					*/
/*																					*/
/* 									work.result2	            					*/
/*									(enthlt, falls individueller Cutpoint unter 	*/
/*									 'cp=' angegeben wurde, Prognosegtemae zu 	*/
/*									 diesem Cutpoint: 								*/
/*									 Youdenindex, Cutpoint (wie bergeben), 		*/
/*									 Sensitivitt, Spezifitt, PPV, NPV, PSEP, POS, */
/*									 NEG, FALPOS, FALNEG							*/
/*																					*/
/*									 enthlt, falls kein individueller Cutpoint 	*/
/*									 unter 'cp=' angegeben wurde, Prognosegtemae  */
/*									 zum Cutpoint, der nach maximalem Youdenindex	*/
/*									 ausgewhlt wurde (siehe work.result): 			*/
/*									 Max. Youdenindex, Sensitivitt, Spezifitt,	*/
/*									 POS, NEG, FALPOS, FALNEG)						*/
/*																					*/
/*  						 		work.index_ori					        		*/
/*									(enthlt Cutpoint-unabhngige Prognosegtemae:	*/
/*									 AUC, Somer's D, Brier- Score, Intercept, Slope,*/
/*									 Emax, R-Square, max-rescaled-R-Square)			*/
/*																					*/
/*									'path'/gplot      (falls Parameter 'printit=1'/	*/
/*														 	 Parameter 'cibinom=1')	*/
/*									'path'/gplot1	  (falls Parameter 'printit=1')	*/
/*									'path'/gplot2	  (falls Parameter 'printit=1'/ */
/*															 Parameter 'rocdat =1')	*/
/*									'path'/gplot3	  (falls Parameter 'printit=1')	*/
/*									'path'/gchart	  (falls Parameter 'printit=1')	*/
/*																					*/
/*	(bei Standardeinstellung;		work.out_mi_log	  (falls Parameter 'mi	   =1')	*/
/*	 Dateiname frei whlbar)														*/
/*																					*/
/*  Programmnderungen				Sandra Mller, 04.05.2016, Makrovariable		*/
/*  und -ergnzungen 				binwidth hinzugefgt							*/ 
/*  (Name, Datum, Beschreibung):											 		*/
/*  																				*/
/*        																			*/
/************************************************************************************/
/*

 TITLE
 -----
 pm_roc.mac.sas

 SAS Makro zur ROC-Analyse (Bestimmung der Prognosegte)

 BACKGROUND
 ----------
 Die Bestimmung der Prognosegte multivariater logistischer Regressionsmodelle erfolgt 
 im wesentlichen in zwei Schritten und wird als Reklassifikation bezeichnet. Zuerst 
 werden die Daten der Patienten in die entwickelte Modellgleichung gesetzt, woraus 
 danach die Wahrscheinlichkeit fr den Outcome pro Patient berechnet wird [muche2002]. 
 Diese Wahrscheinlichkeiten liegen im Intervall [0,1]. 
 Daraufhin schliet sich die Frage an, ab welcher prognostizierten Wahrscheinlichkeit ein 
 Patient zuknftig in die eine oder in die andere Gruppe der Zielgre eingeordnet werden 
 soll. Diese "Trennlinie" wird als sogenannter Cutpoint (CP), die Einordnung der 
 Patienten als Klassifikation bezeichnet. Ziel ist es, diesen Cutpoint so zu legen, 
 dass ein hchstmglicher Anteil von richtigen Prognosen getroffen werden kann. 
 Durch die Klassifikation entsteht eine Vierfeldertafel (Classification Table).
 Aus dieser Vierfeldertafel ergeben sich die Cutpoint-abhngigen Gtemae der Prognose, 
 welche allesamt Mazahlen fr die Gte diagnostischer Verfahren darstellen, jedoch ohne 
 weiteres auf Prognosemodelle bertragen werden knnen. Aus dieser Vierfeldertafel lassen sich 
 die folgenden Cutpoint-abhngigen Prognosegtekenngren bestimmen, die auch innerhalb 
 des Makros "PM_ROC.MAC.SAS" Verwendung finden:

 - Sensitivitt 
 - Spezifitt 
 - Youden- Index (Sensitivitt + Spezifitt - 1)
 - Positive Korrektheit (positiver prdiktiver Wert / PPV)
 - Prvalenzabhngige Positive Korrektheit
 - Negative Korrektheit (negativer prdiktiver Wert / NPV)
 - Prvalenzabhngige Negative Korrektheit
 - Separation- Index (PPV + NPV - 1 / PSEP)

 Diese Kenngren sowie der Cutpoint werden anhand des maximalen Youden-Index bestimmt, so 
 dass hier eine Gleichgewichtung von Sensitivitt und Spezifitt angenommen wird. 
 Eine Visualisierung des Sachverhalts erhlt man durch die sogenannte ROC-Kurve 
 (Receiver Operating Characteristic). Dabei werden fr alle mglichen Cutpoints die 
 Wertepaare von Sensitivitt (y-Achse) und 1-Spezifitt (x-Achse) in einem 
 Koordinatensystem gegeneinander aufgetragen. Besitzt die ROC- Kurve die Form der 
 Winkelhalbierenden, so ist die Prognosemglichkeit anhand des Modells nicht besser als eine 
 Zufallszuteilung, unterhalb der Winkelhalbierenden wre die ZUteilung schlechter als der 
 Zufall. Im extremsten Fall geht die Kurve durch den linken oberen Punkt (rechteckige Form).
 Dies bedeutet, dass das Modell in der Lage ist, jede Beobachtung perfekt 
 bezglich der Zielgre zu klassifizieren. Ziel ist es stets, eine mglichst hohe 
 Sensitivitt sowie eine mglichst hohe Spezifitt zu erhalten.
 Neben diesen Cutpoint-abhngigen Prognosegtemaen existieren darber hinaus Cutpoint-
 unabhngige Kenngren, die zur zusammenfassenden Bestimmung der Vorhersagegenauigkeit 
 herangezogen werden knnen. Innerhalb des Makropaketes finden die folgenden Kenngren
 Anwendung:

 - Area Under The Curve (Flche unter der ROC-Kurve / AUC / c-Statistik)
 - Somer's D
 - Brier-Score
 - Slope
 - Intercept
 - Emax
 - R-Square
 - Max-Rescaled-R-Square

 Bezglich genauerer Beschreibungen aller Kenngren sei auf Harrell [harrell2001, S.247-250] 
 sowie auf Assfalg (Kapitel "Prognosegte") [assfalg2003, S.17-19] verwiesen. 
 Die spezielle Problematik der Kenngren Slope, Intercept und Emax wird ebenfalls von 
 Harrell [harrell2003, S.95] nher ausgefhrt und ist im Rahmen des Parameters 
 "validation" nher erlutert. 


 DESCRIPTION
 -----------
 Das Ziel dieses Makros ist die Analyse der Prognosegte eines logistischen 
 Regressionsmodells, welche im Allgemeinen als "ROC-Analyse" bezeichnet wird, mittels 
 Prognosegtekenngren sowie untersttzenden Grafiken, die den Sachverhalt visualisieren.
 Das Makro "PM_ROC.MAC.SAS" beinhaltet die im Folgenden erklrten spezifischen 
 Eigenschaften und Funktionsmglichkeiten, welche in keiner SAS-Prozedur auf diese 
 Art und Weise implementiert sind. 

 Automatische Verarbeitung der Ausgabedateien von "PM_LOGREG.MAC.SAS":
 SAS bietet keine Mglichkeit, um die Ausgabedateien aus einer logistischen Regression 
 (predicted values, ROC-Daten) automatisch mittels einer Prozedur im Rahmen einer 
 umfassenden ROC-Analyse zu untersuchen. Dem Makro "PM_ROC.MAC.SAS" gengen hingegen die 
 standardmig von PROC LOGISTIC mittels "OUTPUT OUT=" (predicted values) und "OUTROC=" 
 (ROC-Daten) ausgegebenen Dateien um eine vollstndige ROC-Analyse durchzufhren, und um 
 somit alle relevanten Prognosegtekenngren zu berechen. Um das ROC-Makro korrekt nutzen
 zu knnen, ist es jedoch empfehlenswert, dass die bentigten Dateien zuvor mittels dem
 Makro "PM_LOGREG.MAC.SAS" erzeugt worden sind. Diese Empfehlung beruht auf der 
 spezifischen Eigenschaft des logistischen Regressions-Makros, welches die Berechnungen 
 in der ROC-Datei auf Basis der gewhlten predicted values Art automatisch durchfhrt.
 Dies ermglicht eine ROC-Analyse auf Basis der gleichen predicted values.

 Grafiken:
 Das "PM_ROC.MAC.SAS"-Makro liefert darber hinaus automatisch erzeugte, begleitende 
 Grafiken zur Visualisierung der Sachverhalte. Eine genauere Beschreibung findet sich 
 in der entsprechenden Parameterbeschreibung ("printit") wieder. Das Grafikformat, 
 sowie das Zielverzeichnis kann mittels Makroparametern gesteuert werden. Die Auflsung 
 der Grafiken ist stets 300 dpi (Standardwert fr Drucksachen). 

 Konfidenzintervalle/ -bnder:
 Durch die Einbindung der Makros von Hilgers [hilgers1991] sowie Daly [daly1992] 
 besteht die Mglichkeit, ROC-Kurven mit asymmetrischen Konfidenzbndern (Hilgers) 
 sowie ROC-Kurven mit punktweisen Konfidenzintervallen (Daly) auszugeben. Die jeweiligen 
 Daten zum Plot der Grafiken stammen aus den entsprechenden Makros von Hilgers und Daly. 
 Die Steuerung der Ausfhrung dieser externen Makros wird durch lediglich je zwei 
 Makroparameter bewerkstellig. Alle anderen Einstellungen, bergabeparameter und bentigte
 Dateien, welche den Untermakros bergeben werden mssen, werden innerhalb des Makros 
 "PM_ROC.MAC.SAS" automatisch vorgenommen, beziehungsweise erzeugt. 

 Flche unter der Kurve (AUC) mit Standardfehler und Konfidenzintervallen:
 Aufgrund der Integration des Makros "AUROC_KOENIG.MAC.SAS" [koenig1998] ist es mglich, 
 den Standardfehler der Flche unter der Kurve nach DeLong et. al [delongdelong1988] 
 auszugeben. Zustzlich ist die Berechnung eines 95%-Konfidenzintervalls fr den AUC-Wert 
 implementiert, welches ebenfalls mit ausgegeben wird.

 PPV, NPV und PSEP zu bestimmten Prvalenzen der Zielgre: 
 Das Makro zur ROC-Analyse bietet die Mglichkeit, die Prognosegtekenngren PPV, NPV und
 PSEP in Abhngigkeit unterschiedlicher Prvalenzen der Zielgre zu berechnen. Die 
 gewnschten Prvalenzen knnen mittels eines Makroparameters bergeben werden. Darber
 hinaus wird die reale Prvalenz der Zielgre im angegebenen Datensatz ermittelt und die
 genannten Kenngren auch zu dieser Prvalenz berechnet.

 Prognosegtekenngren zu einem bestimmten Cutpoint:
 Bei der ROC-Analyse werden die cutpoint-abhngigen Prognosegtekenngren zum Cutpoint,
 welcher nach maximalem Youden-Index automatisch ermittelt wird, berechnet. Oft ist es 
 jedoch erwnscht, die Kenngren zu einem anderen, spezifischen Cutpoint zu berechnen.
 Das Makro "PM_ROC.MAC.SAS" bietet die Mglichkeit, einen solchen individuellen 
 Schwellenwert anzugeben, fr den dann diese Kenngren berechnet werden. 


 LITERATURE
 ----------
 - SAS Online Doc zu den Prozeduren PROC UNIVARIATE, PROC LOGISTIC, PROC MEANS, 
                     PROC GPLOT, PROC GCHART
 - Assfalg I (2003): Die Bootstrap- Methode zur internen Validierung von 
					 Prognosemodellen. Diplomarbeit, Fachhochschule Ulm
 - Daly L (1992):    Simple SAS-macros for the calculation of exact binomial and Poisson 
				     confidence limits. Comput. Biol. Med. 22: 351-361
 - DeLong ER, DeLong DM and Clarke-Pearson DL (1988): Comparing the areas under two 
					 or more correlated receiver operating characteristic curves: A 
					 nonparametric approach. Biometrics 44: 837-845.
 - Harrell, FE (2001): Regression Modeling Strategies. Springer, New York
 - Hilgers R (1991): R. Hilgers: Distribution- free confidence bounds for ROC curves.
					 Meth Inform Med 30: 96-101
 - Muche R, Ring Ch, Hgel J (2002): Kursunterlagen zur Vorlesung "Entwicklung und 
					 Validierung von Prognosemodellen". Universitt Ulm.
 - Koenig J (1998):  Version 1.1 March 1998. Area under the ROC curve including
					 standard errors and inference concerning several variables.
					 See DeLong & DeLong (1988). Bioms pp 837      
 - Ziegler C (2003): Ein SAS-Makropaket zur Entwicklung und Validierung von 
				     Prognosemodellen auf Basis der logistischen Regression. 
				     Diplomarbeit, FH Ulm 


 SYNTAX
 ------
 [Makroaufruf und Beschreibung der zu bergebenden Parameter]

 %PM_ROC(
   *data			= ,					Datensatz
   *resp_var		= ,					Zielgre
	event			= 1,				Event- Kategorie der binren Zielgre (0/1)
   *data_roc		= ,					ROC- Daten aus Makro PM_LOGREG ('out_roc=')
   *data_pred		= ,					Predicted Values aus Makro PM_LOGREG 
										('out_pred=')
	data_asso		= ,					Somer's D / c- Wert aus Makro PM_LOGREG
										('c_data=')
	validation		= NONE,				Kenngren Emax, Slope, Intercept
	praevalenz		= ,					PPV, NPV & PSEP bei bestimmten Prvalenzen 
									    der Zielgre
	all_cp			= 0,				Sensitivitt, Spezifitt, Youdenindex, PPV,
										NPV, PSEP fr alle Cutpoints (0/1)
	cp				= ,					Sensitivitt, Spezifitt, Cutpoint, Youdenindex, 
										PPV, NPV, PSEP fr bestimmten Cutpoint
    auroc			= 1,				Berechnung AUC (Koenig) (ja/nein)
    printit			= 1,				Ausgabe der Grafiken (ja/nein)
    winkel			= 1,				Winkelhalbierende in den Grafiken (ja/nein)
	point			= 1,				Spezielle Markierung in den Grafiken (ja/nein)
	cibinom			= 1,				Berechnung Konfidenzintervall (Daly) (ja/nein)
	rocdat			= 1,				Berechnung Konfidenzband (Hilgers) (ja/nein)
	alpha_cl		= 0.95,				% Konfidenzband (Hilgers) / -intervall (Daly)
	ext				= tiffp,			Format fr Grafikdateien
	path			= 'c:\temp',		Pfad fr Grafikausgaben
	mi				= 0,				Ausgabe der Prognosekenngren fr 
										Multiple Imputation- Analyse (ja/nein)
	out_mi			= work.out_mi_roc,	Ausgabedatei fr Prognosekenngren
   *macro_path		= ,					Pfad zur Makrosammlung
	binwidth		= 0					Bin-Breite
   );

  Mit (*) gekennzeichnete Parameter sind beim Makroaufruf zwingend erforderlich.
  Bereits gesetzte Parameter stellen die jeweilige Standardeinstellung dar.

   data:			Parameter zur Angabe des zu analysierenden Datensatzes. Wurde 
					beispielsweise vor dem Aufruf des Makros ein Datensatz mit dem 
					Namen "dat300" angelegt, lautet der korrekte Aufruf: "data=dat300". 
					Zustzlich kann auch die dazugehrige SAS-Library bergeben werden 
					(Beispiel: "data=work.dat300"). Die bergabe dieses Parameters ist 
					zwingend erforderlich.

   resp_var:		"resp_var" steht fr "Response Variable", also fr die Zielgre.
					Dieser bergabeparameter ist ebenfalls zwingend erforderlich. 
					Ferner wird eine binre Datenstruktur (0/1) der Zielgre bentigt. 
					Die Zielgre muss mit "0" und "1" kodiert sein. Heit beispielsweise 
					die Zielgre "tod" lautet der Aufruf wie folgt: "resp_var=tod".

   event:			Je nach dem welche Kategorie der binren Zielgre untersucht 
					werden soll, kann diese mit Hilfe dieses Parameters bergeben werden. 
					Soll beispielsweise die Zielgre "tod" analysiert werden und steht 
					"1" fr tot sowie "0" fr lebendig, lautet der korrekte Aufruf 
					"event=1". Dies ist zugleich die Standardeinstellung, da 
					blicherweise "1" fr das Eintreten eines bestimmten Ereignisses 
					steht.

   data_roc:	    Dieser zwingend erforderliche Parameter dient zur bergabe der 
					Datei, welche die ROC-Daten enthlt, um eine ROC-Kurve zu zeichnen. 
					Im allgemeinen erhlt man die ntigen Daten mittels der PROC 
					LOGISTIC-Option "OUTROC", speziell im Rahmen dieses Makropaketes 
					zur Prognosemodellierung aus dem Makro "PM_LOGREG.MAC.SAS" 
					(Parameter "roc=1" / "out_roc="). Diese Datei enthlt neben den 
					vorhergesagten Wahrscheinlichkeiten ("predicted values") unter 
					anderem auch die jeweiligen Sensitivitten, die entsprechenden 
					Spezifitten (in der Form 1-Spezifitt) sowie die korrespondierende 
					Vierfeldertafel (in Zeilenform) aus der sich die Sensitivitten und 
					Spezifitten errechnen. Die bergabe dieses Parameters ist sowohl 
					mit (Bsp.: "data_roc=mydat.rocdat") als auch ohne zugehrigen 
					Bibliotheksnamen mglich (Bsp.: "data_roc=rocdat" (work)).
	
   data_pred:		Der Parameter "data_pred" dient zur Angabe der zwingend notwendigen 
					Datei mit den predicted values. Diese erhlt man aus dem Makro 
					"PM_LOGREG.MAC.SAS" mittels den Optionen "pred=1" in Zusammenhang 
					mit "out_pred=" und dem Parameter "predprobsart=". Hierbei ist zu 
					beachten, dass die Variablennamen fr die predicted values IP_1/ 
					IP_0 (individuelle predicted values) oder XP_1/ XP_0 (crossvalidierte 
					predicted values) beziehungsweise CP_1 / CP_0 (kumulative predicted 
					values) heien mssen. Wird die bentigte Datei mittels des Makros 
					"PM_LOGREG.MAC.SAS" erzeugt, so ist diese Namenskonvention 
					automatisch erfllt. Hierbei steht 0/1 fr die zu untersuchende 
					Ausprgung der Zielgre. Die bergabe dieses Parameters ist 
					ebenfalls sowohl mit (Bsp.: "data_pred=mydat.preddat") als auch 
					ohne zugehrigen Bibliotheksnamen mglich 
					(Bsp.: "data_pred=preddat" (work)).
	
   data_asso:		Dieser Parameter ist optional und erlaubt die bergabe der Datei, 
					die mittels den Optionen "c_wert=1" sowie "c_data=" (Makro 
					"PM_LOGREG.MAC.SAS") erzeugt wurde. Diese beinhaltet die Kenngren 
					Somer's D, c-Wert, R-Square und Max-rescaled-R-Square. Wird mittels 
					"data_asso" diese Datei bergeben, so erzeugt das Makro 
					"PM_ROC.MAC.SAS" eine zusammenfassende Ergebniszeile mit allen 
					Cutpoint-unabhngigen Prognosegtemaen (siehe Beschreibung des 
					Outputs). Wird dem Parameter "data_asso" keine Datei bergeben, 
					so findet sich im Output die erwhnte zusammenfassende Ergebniszeile 
					nicht. Die bergabesyntax lautet beispielsweise 
					"data_asso=mydat.association" oder "data_asso=association".

   validation:		Der Parameter "validation" dient der Festlegung, ob es sich um eine 
					ROC- Analyse fr eine Validierung handelt oder nicht. Findet eine 
					normale ROC-Analyse statt, so lautet der korrekte Aufruf 
					"validation=NONE" (Standard). Handelt es sich um eine Validierung, 
					so kann zwischen "validation=BOOTSTRAP" und 
					"validation=CROSSVALIDATION" gewhlt werden. Der Grund fr die 
					Existenz dieses Parameters liegt in den Prognosegtekenngren Emax, 
					Slope und Intercept. Emax und Slope sind nur gltig, wenn zuvor 
					eine Crossvalidierung oder eine Bootstrap-Validierung stattgefunden 
					hat, das heit ein Modell auf den Originaldatensatz angewendet wurde. 
					Die Kenngre Intercept ist sogar nur im Zusammenhang mit 
					Bootstrap-Validierungen sinnvoll [Harrell, 2001, S. 95]. Dieser 
					Parameter hat zur Folge, dass die Kenngren Emax, Slope und 
					Intercept in Abhngigkeit der Einstellungen des Parameters 
					"validation" berechnet und ausgegeben werden. 
	
   praevalenz:		Der optionale Parameter "praevalenz" erlaubt die bergabe einer 
					oder mehrerer Prvalenzen der Zielgre, fr die dann die jeweilige 
					positive und negative Korrektheit, sowie der Separation-Index PSEP 
					berechnet wird. Die bergabe erfolgt beispielsweise in der Form 
					"praevalenz=0.10" (fr eine Prvalenz) beziehungsweise in der 
					Form "praevalenz=0.01 0.10 0.20" (fr mehrere Prvalenzen). Als 
					Trennzeichen bei der bergabe mehrerer Prvalenzen fungiert hierbei 
					je ein Leerzeichen (Blank).
	
   all_cp:			Mittels dieses Parameters kann eine Tabelle mit den Cutpoint-
					abhngigen Kenngren (Sensitivitt, Spezifitt, Youden-Index, 
					positive Korrektheit, negative Korrektheit, Separation Index PSEP) 
					zu jedem mglichen Cutpoint angefordert werden ("all_cp=1"). 
					Standardmig ist dieser Parameter auf den Wert 0 gesetzt.  
	
   cp:				Der Parameter "cp" ermglicht die bergabe eines ganz bestimmten 
					Cutpoints, fr den dann die jeweiligen Cutpoint-abhngigen 
					Prognosegtekenngren ausgegeben werden.
	
   auroc:			Wird dem Parameter "auroc" der Wert 1 bergeben, so wird zustzlich 
					das Makro "AUROC_KOENIG.MAC.SAS" [Knig, 1998] aufgerufen, welches 
					den Standardfehler der Flche unter der Kurve (AUC) nach DeLong
					et. al [DeLong & DeLong, 1988] berechnet. Zustzlich wird ein 95%- 
					Konfidenzintervall fr den AUC- Wert berechnet. Standardmig ist 
					dieser Parameter auf den Wert 1 gesetzt, das heit, dass die 
					Berechnung erfolgt. Falls dies nicht erwnscht ist, so muss dem 
					Parameter "auroc" der Wert 0 bergeben werden.

   printit:			"printit" steuert die Ausgabe von Grafiken in Zusammenhang mit der 
					ROC- Analyse. Wird diesem Parameter der Wert 1 bergeben 
					(Standardeinstellung), so werden folgende Grafiken in das unter 
					dem Parameter "path" bergeben Verzeichnis geschrieben:

					1) normale ROC- Kurve (Sensitivitt aufgetragen gegen 1- Spezifitt). 
					   Pfad: "path"/ gplot1.

					2) ROC- Kurve mit punktweisem Konfidenzintervall [Daly, 1992] 
					   (falls Parameter "cibinom=1" gesetzt wurde). 
		   			   Pfad: "path"/ gplot.

					3) ROC- Kurve mit asymmetrischem Konfidenzband [Hilgers, 1991] 
					   (falls Parameter "rocdat=1" gesetzt wurde). 
					   Pfad: "path"/gplot2.

					4) Identifizierung des Cutpoints bei maximalem Youden-Index 
					   (Cutpoint aufgetragen gegen Youden-Index). 
					   Pfad: "path"/ gplot3.

					5) Verteilung der predicted values nach Reklassifikation 
					   (Hufigkeiten, unterteilt nach event (Zielgre)= 0/1). 
					   Pfad: "path"/ gchart.

	
   winkel:			Wird dem Parameter "winkel" der Wert 1 (Standard) bergeben, so 
					wird in die Grafiken 1, 2 und 3 jeweils die Winkelhalbierende 
					eingezeichnet. Ist dies nicht erwnscht, so muss der Parameter 
					"winkel" auf den Wert 0 gesetzt werden. Dieser Parameter zeigt 
					nur im Zusammenhang mit "printit=1" Wirkung.
	
   point:			"point=1" (Standard) fordert bestimmte Markierungen in den 
					jeweiligen Grafiken an. In Grafik 1 wird die berechnete 
					Sensitivitt sowie die berechnete Spezifitt (1- Spezifitt) 
					ausgezeichnet. In Grafik 4 wird der maximale Youden- Index und 
					der zugehrige Cutpoint hervorgehoben, wobei in den Grafiken 
					2,3 und 5 "point=1" keine Wirkung zeigt. Sind die genannten 
					speziellen Markierungen nicht erwnscht, so muss diesem Parameter 
					der Wert 0 bergeben werden. Auch der Parameter "point" macht nur 
					Sinn, falls "printit" mit dem Wert 1 aufgerufen wurde.
	
   cibinom:			Der Parameter "cibinom" steuert die Berechnung eines punktweisen 
					Konfidenzintervalls [Daly, 1992]. Wird "cibinom" der Wert 1 
					bergeben, so erfolgt der Aufruf des Makros "CIBINOM_DALY.MAC.SAS", 
					wobei das punktweise Konfidenzintervall berechnet wird, sowie die 
					Ausgabe der entsprechenden Grafik (Nr.2). "cibinom=1" zeigt nur 
					Wirkung, falls Grafiken generell erwnscht sind 
					(Parameter  "printit=1").
	
   rocdat:			"rocdat=1" initiiert die Berechnung eines asymmetrischen 
					Konfidenzbandes mit Hilfe des Makros "ROCDAT_HILGERS.MAC.SAS" 
					[Hilgers, 1991] sowie die Ausgabe der entsprechenden Grafik (Nr.3) 
					mittels dem Untermakro "PRINTIT.MAC.SAS" (falls "printit=1" gesetzt 
					wurde). Wird diese Grafik nicht erwnscht, so muss dem Parameter 
					"rocdat" der Wert 0 bergeben werden. 
	
   alpha_cl:		Dieser Parameter dient der Spezifizierung der Breite von 
					Konfidenzintervallen (siehe Parameter "cibinom" und "rocdat") 
					und zeigt nur Effekte, wenn "cibinom=1" und/ oder "rocdat=1" sowie 
					"printit=1" gesetzt wurde. Wir dem Parameter beispielsweise der Wert 
					0.95 bergeben, so werden 95 % Konfidenzintervalle ("cibinom")/ 
					- bnder ("rocdat") erzeugt. Dieser Wert ist zugleich 
					Standardeinstellung. 

   ext:				Der Parameter "ext" (Extension) dient der Bestimmung des 
					Grafikformates, falls Grafiken angefordert wurden 
					(Parameter "printit=1"). Die Standardeinstellung ist "ext=tiffp", 
					das heit, dass alle Grafiken im TIFF- Format ausgegeben werden. 
					Mgliche Angaben sind:

					GIF 		GIF format 
					GIF160 		GIF format (160 by 120 pixels) 
					GIF260 		GIF format (260 by 195 pixels) 
					GIF373 		GIF format (373 by 280 pixels) 
					GIF570 		GIF format (570 by 480 pixels) 
					GIF733 		GIF format (733 by 550 pixels) 
					GIFANIM 	GIF animation driver 
					IMGGIF 		GIF format 
					IMGJPEG 	JPEG format 
					JPEG 		JPEG format 
					TIFFP 		TIFF format 
					IMGTIFB 	TIFF format (black and white, uncompressed)
					IMGTIFG3 	TIFF format (black and white, CCITT Group 3) 
					IMGTIFG4 	TIFF format (black and white, CCITT Group 4) 
					IMGTIFP 	TIFF format (256 colors, uncompressed) 
					BMP 		BMP format (256 colors) (Windows host driver) 
					BMP20 		BMP 2.0 file format (256 colors) 
					IMGBMP		BMP format 
					IMGPAINT 	BMP format (MS Windows Paintbrush) 
					WMF 		Windows Metafile format 
					EMFDRV 		Windows Enhanced Metafile format 
					PDF 		PDF format 
					PDFC 		PDF format (color) 

   path:		  	Dieser Parameter erlaubt die benutzerspezifische bergabe des 
					Zielverzeichnisses fr die erzeugten Grafiken, falls der 
					Parameter "printit" auf den Wert 1 gesetzt wurde. Die 
					Standardeinstellung ist path='c:\temp'.
	
   mi:				Wird der Parameter "mi" auf den Wert 1 gesetzt (Standard=0) 
					erfolgt eine Ausgabe der jeweiligen Prognosegtekenngren 
					(cutpoint- abhngige und cutpoint- unabhngige) in die unter 
					dem Parameter "out_mi" angegebene Datei. Dieser Parameter, 
					beziehungsweise die hier angeforderte Ausgabedatei, ist fr 
					die sptere Analyse von Multiple Imputation- Datenstzen von 
					Bedeutung (siehe Makro "PM_MI_ANALYZE.MAC.SAS").

   out_mi:			Wie auch im Makro "PM_LOGREG.MAC.SAS" ermglicht der Parameter 
					"out_mi" die benutzerspezifische Bestimmung des Ausgabedateinamens 
					fr die mittels "mi=1" angeforderte Datei. Wird "mi=1" gewhlt und 
					kein individueller Dateiname angegeben wird "out_mi" standardmig 
					auf den Wert "work.out_mi_roc" gesetzt. Die bergabe eines 
					Bibliotheknamens ist auch hier optional.
	
   macro_path: 		Aufgrund der Tatsache, dass innerhalb des Makros diverse 
					Untermakros aufgerufen werden, ist es zwingend ntig, den Pfad 
					zu dem Verzeichnis, wo die komplette Makrosammlung abgelegt ist,
					anzugeben. Der Pfad selber muss in Hochkommata eingeschlossen 
					sein. Beispiel: 'f:\makros'. 

   binwidth:		Der Parameter "binwidth" gibt die Bin-Breite fr die Schtzung der 
					Assoziationsstatistik an. Der Wert muss dabei zwischen 0 und kleiner
					1 liegen. SAS verwendet als	Standard 0.002, ab Version 9.4 wird bei 
					binrem Endpunkt und weniger als 5 Millionen Beobachtungen jedoch 
					auf Binnen verzichtet. Der Parameter erleichtert daher den Vergleich
					der Ergebnisse unterschiedlicher SAS Versionen.

  VALIDATION
  ----------

  Validiert durch: Stephanie Heiss
 				   University of Ulm 
                   Department of Biometry and Medical Documentation
                   89075 Ulm
  
  Validiert am:    08. September 2003

  Validierungsprogramm::      PM_ROC_VAL.SAS
  Validierungsdokumentation:  PM_ROC_VAL.DOC
   
  Validierungsvorgehen:
  - Das Testen der Makros wird durch eine von der Programmierung unabhngige 
    Person durchgefhrt.
  - Fr die Durchfhrung der Validierung wird auf einen in der Literatur 
    genutzten Datensatz zurckgegriffen:  LBW (Low Birth Weight), beschrieben
    in Hosmer, Lemeshow: Applied logistic regression [hosmer2000]
  - Der Datensatz wird fr verschiedene Testlufe verndert bzw. erweitert, 
    damit "die Testdaten neben den normalen Angaben auch Grenzwerte, ungltige
    Angaben, Fehlerbedingungen und Sonderflle beinhalten." [beiwanger2003]
  - Fehlerkontrolle:
    Hierbei werden nur die im LOG-Fenster auftretenden Fehler bzw. Warnungen 
    und offensichtlichen Fehler im OUTPUT betrachtet. Fr jeden Parameter des 
    Makros werden mehrere Durchlufe mit verschiedenen Wertangaben durchgefhrt, 
    wobei auch ungltige Werte angegeben werden, um das Verhalten des Makros
    bei fehlerhaften Aufrufen zu untersuchen. Gefundene Fehler werden 
    dokumentiert und dem Programmierer zur Verfgung gestellt. Nach der Korrektur 
    aller Fehler werden die Testlufe wiederholt, um sicherzustellen, dass 
    wirklich alle Fehler abgefangen wurden und dass keine neuen Fehler entstanden 
    sind.
  - Ergebniskontrolle:
    Um die Richtigkeit der Ergebnisse zu berprfen, werden soweit mglich, die
    Rechenbeispiele und bungsaufgaben aus Hosmer/Lemeshow durch vergleichbare
    Makroaufrufe nachgerechnet. Die Ergebnisse werden denen aus dem Begleitbuch 
    Hosmer/Lemeshow/Cook [hosmer2001] gegenbergestellt und direkt verglichen.
  - Alle Validierungen werden dokumentiert.

  Literatur: 
  - Beiwanger S (2003): Validierung von SAS-Makros. Diplomarbeit, FH Ulm
  - Hosmer DW, Lemeshow S (2000): Applied logistic regression 2nd Ed. 
           J. Wiley & Sons, New York
  - Hosmer DW, Lemeshow S, Cook ED (2001): Solutions manual to accompany
           applied logistic regression 2nd. Ed. J. Wiley & Sons, New York

  AUTHOR
  ------
  R. Muche / C. Ring / C. Ziegler
  University of Ulm
  Department of Biometry and Medical Documentation
  89075 Ulm

  The author does not overtake any responsibility for the correctness
  of results. It must not be modified without removing the heading lines 
  containing the authors name.

----------------------------------------------------------------------------------*/

%MACRO PM_ROC(	 data				= ,
				 resp_var			= ,
				 event				= 1,
				 data_roc			= ,
				 data_pred			= ,
				 data_asso			= ,
				 validation			= NONE,
				 praevalenz			= ,
				 all_cp				= 0,
			     cp					= ,
				 auroc				= 1,
				 printit			= 1,
				 winkel				= 1,
				 point				= 1,
				 cibinom			= 1,
				 rocdat				= 1,
				 alpha_cl			= 0.95,
				 ext				= tiffp,
				 path				= 'c:\temp',
				 mi					= 0,
				 out_mi				= work.out_mi_roc,
				 macro_path			= ,
				 binwidth 			= 0
			);

%GLOBAL nnnn1;

/*Prfungen / Referenzierungen / Standardbelegungen*/

/*Wichtigster Parameter 'macro_path' berprfen*/
/*bergabe erfolgt?*/

%IF &macro_path= %THEN %DO;

	%WINDOW error color=grey                                                   
      #5  @28 'Fehler!' attr=highlight                          
               color=blue                                            
      #7  @28 "Dem zwingend erforderlichen Parameter 'macro_path' wurde kein Wert"
	  #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
      #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                        
    %DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Hilfsprogramm welches berprft, ob dem Parameter 'macro_path' ein korrekter Pfad*/
/*bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/

/*Anzahl der Zeichen im String 'macro_path' bestimmen:*/

%LET path_length=%LENGTH(&macro_path);

%LET infile=&macro_path;

%IF %bquote(&infile) NE %THEN %DO;
     %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
      %IF %bquote(&char1) ne %STR(%')
         %THEN %LET command='NOK';
         %ELSE %LET command='OK';

	 %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
      %IF %bquote(&char2) ne %STR(%')
         %THEN %LET command2='NOK';
         %ELSE %LET command2='OK';
%END;

%IF &command='NOK' OR &command2='NOK' %THEN %DO;

	%WINDOW error color=grey                
	 #5  @28 'Fehler!' attr=highlight                            
             color=blue                                               
     #7  @28 "Dem Parameter 'macro_path' wurde ein syntaktisch falscher Wert"
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';    
    %DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Test, ob das unter 'macro_path' angegebene Verzeichnis existiert*/

%LET dir=&macro_path;

%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
       %PUT Directory does not exist;
%END;
%ELSE %DO;
       %PUT Directory exists;
       %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;

	%WINDOW error color=grey                                                
	 #5  @28 'Fehler!' attr=highlight                                 
             color=blue    
	 #7  @28 "Das unter dem Parameter 'macro_path' angegebene Verzeichnis existiert"
	 #8  @28 "nicht- Bentigte Untermakros knnen nicht referenziert werden"
	 #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';     
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*gleichzeitig Makroparameter 'path' prfen!*/

%IF &path= %THEN %DO;
	%LET path='c:\temp';
%END;

/*Hilfsprogramm, welches berprft, ob dem Parameter 'path' ein korrekter Pfad*/
/*bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/

/*(nur von Bedeutung, falls Grafiken erwnscht)*/

%IF &printit=1 %THEN %DO;

%LET path_length=%LENGTH(&path);

/*Anzahl der Zeichen im String 'path' bestimmen:*/

%LET infile=&path;

%IF %bquote(&infile) NE %THEN %DO;
     %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
      %IF %bquote(&char1) ne %STR(%')
         %THEN %LET command='NOK';
         %ELSE %LET command='OK';

	 %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
      %IF %bquote(&char2) ne %STR(%')
         %THEN %LET command2='NOK';
         %ELSE %LET command2='OK';
%END;

%IF &command='NOK' OR &command2='NOK' %THEN %DO;

	%WINDOW error color=grey                
	 #5  @28 'Fehler!' attr=highlight      
             color=blue                     
     #7  @28 "Dem Parameter 'path' wurde ein syntaktisch falscher Wert" 
	 #8  @28 "bergeben- Grafiken knnen nicht gespeichert werden"
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                         
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Test, ob das angegebene Verzeichnis existiert*/

%LET dir=&path;

%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
       %PUT Directory does not exist;
%END;
%ELSE %DO;
       %PUT Directory exists;
       %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;

	%WINDOW error color=grey                                                 
	 #5  @28 'Fehler!' attr=highlight                           
              color=blue                                           
	 #7  @28 "Das unter dem Parameter 'path' angegebene Verzeichnis existiert"
	 #8  @28 "nicht- Grafiken knnen nicht gespeichert werden"
	 #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';              
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;
%END;

/*Falls bestimmte Parameter ohne Wert bergeben wurden, diese mit Standard belegen*/

%IF &event= %THEN %DO;
	%LET event=1;
%END;

%IF &validation= %THEN %DO;
	%LET validation=NONE;
%END;

%IF &all_cp= %THEN %DO;
	%LET all_cp=0;
%END;

%IF &cibinom= %THEN %DO;
	%LET cibinom=1;
%END;

%IF &rocdat= %THEN %DO;
	%LET rocdat=1;
%END;

%IF &auroc= %THEN %DO;
	%LET auroc=1;
%END;

%IF &printit= %THEN %DO;
	%LET printit=1;
%END;

%IF &alpha_cl= %THEN %DO;
	%LET alpha_cl=0.95;
%END;

%IF &ext= %THEN %DO;
	%LET ext=tiffp;
%END;

%IF &winkel= %THEN %DO;
	%LET winkel=1;
%END;

%IF &point= %THEN %DO;
	%LET point=1;
%END;

%IF &mi= %THEN %DO;
	%LET mi=0;
%END;

%IF &out_mi= %THEN %DO;
	%LET out_mi=work.out_mi_roc;
%END;

/*Externe Makros, die innerhalb des Makros 'PM_ROC.MAC.SAS'*/
/*verwendet werden (Untermakros) einbinden (%include)*/

FILENAME y &macro_path;

%INCLUDE y ('cibinom_daly.mac.sas');
%INCLUDE y ('rocdat_hilgers.mac.sas');
%INCLUDE y ('auroc_koenig.mac.sas');
%INCLUDE y ('printit.mac.sas');
%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('numchar.mac.sas');
%INCLUDE y ('exist.mac.sas');
%INCLUDE y ('exist_var.mac.sas');

/*Test, ob bei bestimmten Parametern anstatt numerischen Werten*/
/*character- Werte bergeben wurden*/

/*berprfung erfolgt mit Hilfe des Untermakros 'numchar.mac.sas'*/

%NUMCHAR(parameterliste=&event &praevalenz &all_cp &cp &cibinom &auroc 
		                &printit &rocdat &alpha_cl &winkel &point &mi &binwidth);

%IF &numcharok='NOK' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error8.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Parameter, bei denen keine negativen Werte erlaubt sind, berprfen*/

%IF &alpha_cl ne %THEN %DO;
	%IF &alpha_cl <=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error9.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

%IF &cp ne %THEN %DO;
	%IF &cp <=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error9.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

%IF &binwidth <0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error66.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfen, ob der angegebene Datensatz (Parameter 'data=') auch wirklich existiert*/
/*(mit Hilfe des Hilfs-/Untermakros 'exist.mac.sas')*/

%EXIST(dsn=&data);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error4.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Datenquelle (Parameter 'data=' UND response-Variable*/
/*(Zielgre / Parameter 'resp_var=') angegeben wurde*/

%IF &data= or &resp_var= %THEN %DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

		%INCLUDE y ('error1.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

/*Prfen, ob den Parametern 'data_roc' & 'data_pred' Werte bergeben wurden*/

%IF &data_roc= %THEN %DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

		%INCLUDE y ('error40.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%IF &data_pred= %THEN %DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

		%INCLUDE y ('error40.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

/*Prfung, ob alle bergebenen Variablen numerisch sind*/

/*Erzeugung eines temporren Zwischendatensatzes, welcher lediglich die*/
/*bergebenen Variablen enthlt*/

DATA test3;
	SET &data;
	KEEP &resp_var;
RUN;

/*mittels der Prozedur PROC CONTENTS kann der Typ einer Variablen bestimmt werden*/

/*Unterdrckung der Ausgabe von PROC CONTENTS*/

PROC CONTENTS DATA=test3 NOPRINT OUT=testcontent;
RUN;

/*lediglich die Variable 'type' der Ergebnisdatei behalten,*/
/*da nur diese von Interesse ist*/

DATA testcontent;
	SET testcontent;
	KEEP type;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=testcontent);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test4=&nnnn1;

/*Prfung ob Variablen vom Typ 'character' vorhanden sind*/

DATA test4;
	SET testcontent;
	IF type=2 THEN DELETE;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test4);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test5=&nnnn1;

/*falls Variablen bergeben wurden, die nicht numerisch sind --> Fehlermeldung*/

%IF &test4 ne &test5 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error60.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE test4 testcontent test3;
RUN;
QUIT;

/*berprfung, ob alle bergebenen Variablen im bergebenen Datensatz*/
/*(Parameter 'data' vorhanden sind)*/

/*Zusammenfassung aller Variablen in einer neuen Makro- Variablen*/

%EXIST_VAR(dsn=&data, var=&resp_var);

/*falls irgendwelche Variablen nicht existieren, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error6.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob die Zielgre (Parameter 'resp_var')*/
/*2 Kategorien (Ausprgungen) besitzt*/

/*missing values ausschliessen*/

DATA freq2;
	SET &data;
	IF &resp_var ne .;
RUN;

/*Anzahl der Ausprgungen mit Hilfe der Prozedur PROC FREQ ermitteln*/

/*Ausgabe unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnis als ODS- Datei zur weiteren Verarbeitung ausgeben*/

ODS OUTPUT OneWayFreqs=freq;

PROC FREQ DATA=freq2;
	TABLES &resp_var;
RUN;

/*ODS- Output schliessen*/

ODS OUTPUT CLOSE;

/*ODS- Ausgabe wieder aktivieren*/

ODS LISTING;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen (= # Ausprgungen der Zielgre)*/

%NUMBEROBS(datensatz=freq);

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE freq freq2;
RUN;
QUIT;

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET event_check=&nnnn1;

/*falls Zielgre mehr als zwei Ausprgungen besitz--> Fehlermeldung*/

%IF &event_check > 2 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error5.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob Parameter 'event' 0 oder 1 ist*/

%IF &event ne 1 %THEN %DO;
	%IF &event ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE  y ('error36.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'all_cp' 0 oder 1 ist*/

%IF &all_cp ne 1 %THEN %DO;
	%IF &all_cp ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error36.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'cibinom' 0 oder 1 ist*/

%IF &cibinom ne 1 %THEN %DO;
	%IF &cibinom ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error36.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'rocdat' 0 oder 1 ist*/

%IF &rocdat ne 1 %THEN %DO;
	%IF &rocdat ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error36.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'auroc' 0 oder 1 ist*/

%IF &auroc ne 1 %THEN %DO;
	%IF &auroc ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE  y ('error36.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'printit' 0 oder 1 ist*/

%IF &printit ne 1 %THEN %DO;
	%IF &printit ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE  y ('error36.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'winkel' 0 oder 1 ist*/

%IF &winkel ne 1 %THEN %DO;
	%IF &winkel ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error36.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'point' 0 oder 1 ist*/

%IF &point ne 1 %THEN %DO;
	%IF &point ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error36.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob Parameter 'mi' 0 oder 1 ist*/

%IF &mi ne 1 %THEN %DO;
	%IF &mi ne 0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error36.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*Parameter 'praevalenz' berprfen (innerhalb [0,1])*/

%IF &praevalenz ne %THEN %DO;

/*Anzahl der angegebenen cutpoints ermitteln:*/

%GLOBAL praevalenz_count praevalenz_var;
%LET praevalenz_count=1;
%LET praevalenz_var=%QSCAN(&praevalenz,&praevalenz_count,%STR( ));
%DO %WHILE(&praevalenz_var ne);
     %LET praevalenz_count=%EVAL(&praevalenz_count+1);
     %LET praevalenz_var=%QSCAN(&praevalenz,&praevalenz_count,%STR( ));
%END;
%LET praevalenz_count=%EVAL(&praevalenz_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/

%PUT The praevalenz-Variable contains &praevalenz_count variables.;

%DO df=1 %TO &praevalenz_count;

%LET praevalenzcheck=%QSCAN(&praevalenz,&df,%STR( ));

%NUMCHAR(parameterliste=&praevalenzcheck);

%IF &numcharok='OK' %THEN %DO;

%IF &praevalenzcheck > 1 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error37.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%IF &praevalenzcheck <=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error9.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;


%END;

%END;
%ELSE %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error8.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

%END;

/*berprfung, ob Libname der Zieldatei "out_mi" vorhanden ist falls*/
/*eine angegeben wurde:*/

%IF &mi=1 %THEN %DO;

%IF &out_mi ne %THEN %DO;

	%LET length=%LENGTH(&out_mi);
	%LET pointx='NO';

	%DO h=1 %TO &length;

	%LET check=%SUBSTR(&out_mi,&h,1);

		%IF &check=. %THEN %DO;

			%LET pointx='YES';

		%END;

	%END;

%IF &pointx='YES' %THEN %DO;

%LET lib=%QSCAN(&out_mi,1,.);
%LET file=%QSCAN(&out_mi,2,.);

DATA &lib..test;
	test='Test';
RUN;

%EXIST(dsn=&lib..test);

PROC DATASETS NOLIST LIBRARY=&lib;
	DELETE test;
RUN;
QUIT;

%IF &exist ='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error16.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%END;

%END;

%END;

/*berprfen, ob die angegebenen Datenstze auch wirklich existieren*/

%EXIST(dsn=&data_roc);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error38.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%EXIST(dsn=&data_pred);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error38.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%IF &data_asso ne %THEN %DO;

%EXIST(dsn=&data_asso);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%INCLUDE y ('error38.sas');

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

%END;

/*berprfen, ob Parameter, denen Character- Werte bergeben werden*/
/*korrekte Angaben enthalten*/

/*(nur wenn Grafiken erwnscht sind)*/

%IF &printit=1 %THEN %DO;

%IF %UPCASE(&ext) ne GIF 		%THEN %DO;
%IF %UPCASE(&ext) ne GIF160 	%THEN %DO;
%IF %UPCASE(&ext) ne GIF260 	%THEN %DO;
%IF %UPCASE(&ext) ne GIF373 	%THEN %DO;
%IF %UPCASE(&ext) ne GIF570 	%THEN %DO;
%IF %UPCASE(&ext) ne GIF733 	%THEN %DO;
%IF %UPCASE(&ext) ne GIFANIM 	%THEN %DO;
%IF %UPCASE(&ext) ne IMGGIF 	%THEN %DO;
%IF %UPCASE(&ext) ne IMGJPEG 	%THEN %DO;
%IF %UPCASE(&ext) ne JPEG 		%THEN %DO;
%IF %UPCASE(&ext) ne TIFFP 		%THEN %DO;
%IF %UPCASE(&ext) ne IMGTIFB 	%THEN %DO;
%IF %UPCASE(&ext) ne IMGTIFG3 	%THEN %DO;
%IF %UPCASE(&ext) ne IMGTIFG4 	%THEN %DO;
%IF %UPCASE(&ext) ne IMGTIFP 	%THEN %DO;
%IF %UPCASE(&ext) ne BMP 		%THEN %DO;
%IF %UPCASE(&ext) ne BMP20 		%THEN %DO;
%IF %UPCASE(&ext) ne IMGBMP 	%THEN %DO;
%IF %UPCASE(&ext) ne IMGPAINT 	%THEN %DO;
%IF %UPCASE(&ext) ne WMF 		%THEN %DO;
%IF %UPCASE(&ext) ne EMFDRV 	%THEN %DO;
%IF %UPCASE(&ext) ne PDF 		%THEN %DO;
%IF %UPCASE(&ext) ne PDFC		%THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

%INCLUDE y ('error39.sas');

/*Sprungmarke im Fehlerfall*/

%GOTO ende;

%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;
%END;

%END;

/*Prfen, ob Parameter 'validation' korrekte Angaben enthlt*/

%IF %UPCASE(&validation) ne NONE %THEN %DO;
	%IF %UPCASE(&validation) ne BOOTSTRAP %THEN %DO;
		%IF %UPCASE(&validation) ne CROSSVALIDATION %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

			%INCLUDE y ('error46.sas');

/*Sprungmarke im Fehlerfall*/

			%GOTO ende;

		%END;
	%END;
%END;

/*Prfen, ob die Datei mit den predicted values ('data_pred') die Variablen*/
/*XP_&event oder IP_&event oder CP_&event enthlt*/

/*immer testen*/

%EXIST_VAR(dsn=&data_pred, var=XP_&event);

%IF &exist='NO' %THEN %DO;
		
	%EXIST_VAR(dsn=&data_pred, var=IP_&event);

	%IF &exist='NO' %THEN %DO;
		
		%EXIST_VAR(dsn=&data_pred, var=CP_&event);

		%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

			%INCLUDE y ('error44.sas');

/*Sprungmarke im Fehlerfall*/

			%GOTO ende;

		%END;
		%ELSE %DO;

			%LET predprobsart=CP_&event;
			%LET label_predprobsart=CP_&event;

		%END;

	%END;
	%ELSE %DO;

		%LET predprobsart=IP_&event;
		%LET label_predprobsart=IP_&event;

	%END;

%END;
%ELSE %DO;

	%LET predprobsart=XP_&event;
	%LET label_predprobsart=XP_&event;

%END;

/*falls auroc=1 mu jeweils XP_0 & XP_1 oder IP_0 & IP_1 oder CP_0 & CP_1*/
/*in der Datei 'data_pred' enthalten sein*/

%IF &auroc=1 %THEN %DO;

%IF &event=1 %THEN %DO;

%EXIST_VAR(dsn=&data_pred, var=XP_0);

%IF &exist='NO' %THEN %DO;

	%EXIST_VAR(dsn=&data_pred, var=IP_0);

		%IF &exist='NO' %THEN %DO;

			%EXIST_VAR(dsn=&data_pred, var=CP_0);

				%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

					%INCLUDE y ('error45.sas');

/*Sprungmarke im Fehlerfall*/

					%GOTO ende;

				%END;	
				%ELSE %DO;

					%LET predprobsart2=CP_0;
					%LET label_predprobsart2=CP_0;

				%END;
		%END;
		%ELSE %DO;

			%LET predprobsart2=IP_0;
			%LET label_predprobsart2=IP_0;

		%END;

%END;
%ELSE %DO;

	%LET predprobsart2=XP_0;
	%LET label_predprobsart2=XP_0;

%END;
%END;
%ELSE %DO;

%EXIST_VAR(dsn=&data_pred, var=XP_1);

%IF &exist='NO' %THEN %DO;

	%EXIST_VAR(dsn=&data_pred, var=IP_1);

		%IF &exist='NO' %THEN %DO;

			%EXIST_VAR(dsn=&data_pred, var=CP_1);

				%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

					%INCLUDE y ('error45.sas');
					%GOTO ende;

/*Sprungmarke im Fehlerfall*/

				%END;	
				%ELSE %DO;

					%LET predprobsart2=CP_1;
					%LET label_predprobsart2=CP_1;

				%END;
		%END;
		%ELSE %DO;

			%LET predprobsart2=IP_1;
			%LET label_predprobsart2=IP_1;

		%END;

%END;
%ELSE %DO;

	%LET predprobsart2=XP_1;
	%LET label_predprobsart2=XP_1;

%END;

%END;

%END;

/*prfen ob 'alpha_cl' in [0,1] ist*/

%IF &alpha_cl >1 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error47.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

/*prfen ob 'binwidth' in [0,1) ist*/

%IF &binwidth >=1 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%INCLUDE y ('error66.sas');

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

%END;

%IF &binwidth >0.01 %THEN %DO;

/*Anzeige eines Warnhinweises*/

	DATA hint;
		hint="Bei einem Wert von 0 wird kein Binning angewandt. In manchen SAS Versionen wird der Standard 0.002 verwendet.";
	RUN;
	PROC PRINT DATA=hint NOOBS LABEL HEADING=H WIDTH=UNIFORM;
		VAR   hint;
		TITLE1 "Dem Parameter 'binwidth' wurde ein Wert grer 0.01 bergeben.";
		LABEL hint = "Sinnvoll ist ein Wert zwischen 0 und 0.01."; 
	RUN;

%END;


/*ENDE Prfungen / Referenzierungen / Standardbelegungen*/

/*-------------------------------------------------------------------------------*/

/*Standardoutput ausgeben*/

%LET datensatz=&data;

/*Fallzahl bestimmen:*/

%numberobs(datensatz=&data);

%LET fallzahl=&nnnn1;

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatz       ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

PROC DATASETS NOLIST;
	DELETE standardoutput;
RUN;
QUIT;

/*Ende Standardoutput*/
/*-------------------------------------------------------------------------------*/

%LET eventx=&event;

/*Fr Konfidenzbnder Angaben anpassen:*/

%IF &printit=1 %THEN %DO;

/*rocdat*/

%LET cl=&alpha_cl;

/*cibinom*/

%LET alpha=%SYSEVALF(1-&alpha_cl);

%IF &cibinom=1 %THEN %DO;

/*Ermittlung # event =1 aus out_roc Datei*/
/*(n1:"Number of correctly predicted events") _POS_)*/

PROC UNIVARIATE NOPRINT DATA=&data_roc;
	VAR _POS_;
	OUTPUT OUT=n1 MAX=n1;
RUN;

/*Ermittlung # event =0 aus out_roc Datei*/
/*(n2:"Number of correctly predicted nonevents") _NEG_)*/

PROC UNIVARIATE NOPRINT DATA=&data_roc;
	VAR _NEG_;
	OUTPUT OUT=n2 MAX=n2;
RUN;

/*dummy- Variable zum zusammenfhren der Dateien anlegen*/

DATA n1;
	SET n1;
	dummy=1;
RUN;

/*dummy- Variable zum zusammenfhren der Dateien anlegen*/

DATA n2;
	SET n2;
	dummy=1;
RUN;

/*Zusammenfhren von n1 und n2*/

DATA n1n2;
	MERGE n1 n2;
	BY dummy;
RUN;

/*n1 und n2 als Makro- Variablen zur weiteren Verarbeitung abspeichern*/

DATA n1n2;
	SET n1n2;
	call symput('n1',n1);
	call symput('n2',n2);
RUN;

/*Data-step um n1 und n2 zu ROC- Daten hinzuzufgen*/

DATA work.roc1; 
	SET &data_roc;
	n1=&n1; /*fr %cibinom, # event=1*/
	n2=&n2; /*fr %cibinom, # event=0*/
RUN;

%END;
%END;

%IF &auroc=1 %THEN %DO;

/*data_pred=Ergebnisdatei vom Makro "Logreg"*/
/*(Datei, die unter "out_pred="	angegeben wurde)*/
/*---> p=predicted values*/

DATA pred1;
	SET &data_pred;
	RENAME &predprobsart=x &resp_var=gruppe;
	IF &predprobsart=. THEN DELETE;
RUN;

PROC SORT DATA=pred1;
	BY gruppe x;
RUN;

%END;

/*Data-Step um Spezifitt und Youden-Index zu bestimmen*/
/*Auerdem: positive Korrektheit (ppv) und negative Korrektheit (npv) bestimmen*/

DATA youden;
	SET &data_roc;
	spez=(1-_1mspec_);
	youden=_sensit_ + spez -1;
	ppv=_pos_/(_pos_ + _falpos_);
    npv=_neg_/(_neg_ + _falneg_);
	dummy=1;
RUN;

/*PSEP berechnen und hinzufgen*/

DATA youden;
	SET youden;
	psep=ppv + npv -1;
RUN;

/*Neuen max. Youden bestimmen (output out -> youdenindex)*/

PROC UNIVARIATE NOPRINT DATA=youden;
	VAR youden;
	OUTPUT OUT=youdenindex MAX=maxyouden;
RUN;

*Variable dummy hinzufgen (als By-Variable zum Mergen);

DATA youden_sens_spez;
	SET youdenindex;
	dummy=1;
RUN;

*Max. Youden zu Sens., Spez., Youden-Index hinzufgen;

DATA result (KEEP=_Sensit_ spez maxyouden _PROB_ ppv npv psep);
	MERGE youden_sens_spez youden;
	BY dummy;
	IF youden=maxyouden;
	call symput('maxyouden',trim(left(put(maxyouden,BEST6.))));
	call symput('cutpoint',trim(left(put(_PROB_,BEST6.))));
	call symput('sensit',trim(left(put(_Sensit_,BEST6.))));
	call symput('spez',trim(left(put(spez,BEST6.))));
	call symput('_pos_',trim(left(put(_pos_,BEST6.))));
	call symput('_neg_',trim(left(put(_neg_,BEST6.))));
	call symput('_falpos_',trim(left(put(_falpos_,BEST6.))));
	call symput('_falneg_',trim(left(put(_falneg_,BEST6.))));
RUN;

PROC FORMAT;

RUN;

/*Ausgabe der Ergebnisse*/

PROC PRINT DATA=result LABEL NOOBS;
	VAR _SENSIT_ spez _PROB_ maxyouden ppv npv psep;
	LABEL _SENSIT_ ='Sensitivitt'
	spez     ='Spezifitt'
	maxyouden='maximaler Youden-Index'
	_PROB_	 ='Cutpoint'
	psep	 ='Separation- Index PSEP'
	ppv      ='positive Korrektheit'
	npv      ='negative Korrektheit';
	TITLE1 "ROC- Analyse";
	TITLE2 "Response: &resp_var=&event"; 
	TITLE3 "Sensitivitt & Spezifitt bei Cutpoint ausgewhlt nach maximalem Youden-Index";
RUN;

/*Vierfeldertafel generieren & ausgeben*/

DATA vierfeld;

		%DO kj=0 %TO 5;
			obs=&kj;
			OUTPUT;
		%END;

RUN;

DATA vierfeld;
	SET vierfeld;
	IF _N_=2 THEN &resp_var._cutpoint='';
	IF _N_=3 THEN &resp_var._cutpoint='1';
	IF _N_=4 THEN &resp_var._cutpoint='0';
	IF _N_=5 THEN &resp_var._cutpoint='';
	spalte1='                ';
	spalte2='                ';
	spalte3='                ';
	IF _N_=3 THEN spalte1="&_pos_";
	IF _N_=4 THEN spalte1="&_falpos_";
	IF _N_=6 THEN spalte1="%EVAL(&_pos_+&_falpos_)";
	IF _N_=3 THEN spalte2="&_falneg_";
	IF _N_=4 THEN spalte2="&_neg_";
	IF _N_=6 THEN spalte2="%EVAL(&_neg_+&_falneg_)";
	IF _N_=2 THEN spalte3='';
	IF _N_=3 THEN spalte3="%EVAL(&_pos_+&_falneg_)";
	IF _N_=4 THEN spalte3="%EVAL(&_neg_+&_falpos_)";
	IF _N_=6 THEN spalte3="%EVAL(&_neg_+&_falpos_+&_pos_+&_falneg_)";
	IF obs=0 THEN DELETE;
	IF obs=1 THEN DELETE;
RUN;

PROC PRINT DATA=vierfeld LABEL NOOBS;
	VAR &resp_var._cutpoint spalte1 spalte2 spalte3;
	LABEL &resp_var._cutpoint 	="&resp_var"
					spalte1     ='CP>'
					spalte2		='CP<='
					spalte3		='Summe';

	TITLE1 'Aufteilung der Beobachtungen zum gewhlten Cutpoint';
	TITLE2 ; 
	TITLE3 ;
RUN;

/*PPV & NPV in Abhngigkeit der Prvalenz der Zielgre berechnen*/

DATA result2 (KEEP=_Sensit_ spez maxyouden _pos_  _falpos_ _neg_ _falneg_);
	MERGE youden_sens_spez youden;
	BY dummy;
	IF youden=maxyouden;
RUN;

%LET n=;

DATA praev_result;
	SET result2;
	praevalenz=(_pos_ + _falneg_)/(_pos_ + _falpos_ + _neg_ + _falneg_);
	anzahl=(_pos_ + _falpos_ + _neg_ + _falneg_);
	CALL SYMPUT('n', anzahl);
	ppv_praev=(_sensit_*praevalenz)/((_sensit_*praevalenz)+(1-spez)*(1-praevalenz));
	npv_praev=(spez*(1-praevalenz))/((spez*(1-praevalenz))+((1-_sensit_)*praevalenz));
RUN;

DATA praev_result;
	SET praev_result;
	psep=ppv_praev + npv_praev -1;
	CALL SYMPUT('psep', psep);
RUN;

/*Anzahl der angegebenen Prvalenzen ermitteln:*/
%GLOBAL count var;
%LET count=1;
%LET var=%QSCAN(&praevalenz,&count,%STR( ));
%DO %WHILE(&var ne);
	%LET count=%EVAL(&count+1);
	%LET var=%QSCAN(&praevalenz,&count,%STR( ));
%END;
%LET count=%EVAL(&count-1);

/*Anzahl ausgeben im LOG- Fenster*/
%PUT Anzahl Prvalenzen: &count .;

%DO i=1 %TO &count;

	DATA praev_result&i;
	    SET result2;
		praevalenz=%SCAN(&praevalenz,&i,%STR( ));
		ppv_praev=(_sensit_*praevalenz)/((_sensit_*praevalenz)+(1-spez)*(1-praevalenz));
		npv_praev=(spez*(1-praevalenz))/((spez*(1-praevalenz))+((1-_sensit_)*praevalenz));
		psep=ppv_praev + npv_praev -1;
	RUN;

	DATA praev_result;
		SET praev_result praev_result&i;
	RUN;

	PROC DATASETS NOLIST;
		DELETE praev_result&i;
	RUN;
	QUIT;

%END;

/*Bemerkung hinzufgen*/

DATA praev_result;
	SET praev_result;
	IF _N_='1' THEN note='<-(im Datensatz)';
RUN;

/*Sortierung nach Praevalenz*/

PROC SORT DATA=praev_result;
	BY praevalenz;
RUN;

/*Ausgabe*/

PROC PRINT DATA=praev_result LABEL NOOBS;
	VAR praevalenz ppv_praev npv_praev psep note;
	LABEL praevalenz="Prvalenz von &resp_var=&event  n=&n"
		  note      ="Bemerkung"	
      	  ppv_praev ="Prvalenzabhngige positive Korrektheit" 
	      npv_praev ="Prvalenzabhngige negative Korrektheit"
		  psep	    ="Separation- Index PSEP";
TITLE 'Positive und negative Korrektheit in Abhngigkeit der Prvalenz der Zielgre';
RUN;

/*Zwischendatenstze lschen*/

PROC DATASETS NOLIST;
	DELETE praev_result;
RUN;
QUIT;
	
/*Falls spezifischer Cutpoint angegeben wurde, Sensitivitt & Spezifitt zu diesem berechnen*/
/*und ausgeben*/

%IF &cp ne %THEN %DO;

/*ROC- Datei nach _PROB_ sortieren*/

PROC SORT DATA=&data_roc OUT=predicted;
	BY _PROB_;
RUN;

DATA bigger;
	SET predicted;
	IF (ROUND(_PROB_,.00001)) < &cp THEN DELETE;
RUN;

/*_PROB_ -Minimum bestimmen. Diese Zeile enthlt die gesuchten Kenngren*/

PROC UNIVARIATE NOPRINT DATA=bigger;
	VAR _PROB_;
	OUTPUT OUT=cutpoint MIN=next_cutpoint;
RUN;

/*Dummy- Variable zum mergen einfhren*/

DATA cutpoint;
	SET cutpoint;
	dummy=1;
RUN;

/*Ergebniszeile bestimmen*/

DATA result2 (KEEP=_Sensit_ spez youden next_cutpoint ppv npv psep _pos_  _falpos_ _neg_ _falneg_);
	MERGE cutpoint youden;
	BY dummy;
	IF _PROB_=next_cutpoint;
RUN;

/*Ausgabe der Ergebnisse*/

TITLE "Sensitivitt & Spezifitt bei angegebenem Cutpoint &cp";
PROC PRINT DATA=result2 LABEL NOOBS;
	VAR _SENSIT_ spez next_cutpoint youden ppv npv psep;
	LABEL _SENSIT_ ='Sensitivitt'
	spez     ='Spezifitt'
	next_cutpoint='Cutpoint'
	youden	 ='Youdenindex'
	ppv      ='positive Korrektheit'
	npv      ='negative Korrektheit'
	psep	 ='Separation- Index PSEP';
RUN;

PROC DATASETS NOLIST;
	DELETE predicted bigger cutpoint;
RUN;
QUIT;

/*+++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

DATA praev_result;
	SET result2;
	praevalenz=(_pos_ + _falneg_)/(_pos_ + _falpos_ + _neg_ + _falneg_);
	anzahl=(_pos_ + _falpos_ + _neg_ + _falneg_);
	CALL SYMPUT('n', anzahl);
	ppv_praev=(_sensit_*praevalenz)/((_sensit_*praevalenz)+(1-spez)*(1-praevalenz));
	npv_praev=(spez*(1-praevalenz))/((spez*(1-praevalenz))+((1-_sensit_)*praevalenz));
RUN;

DATA praev_result;
	SET praev_result;
	psep=ppv_praev + npv_praev -1;
	CALL SYMPUT('psep', psep);
RUN;

%DO i=1 %TO &count;

	DATA praev_result&i;
	    SET result2;
		praevalenz=%SCAN(&praevalenz,&i,%STR( ));
		ppv_praev=(_sensit_*praevalenz)/((_sensit_*praevalenz)+(1-spez)*(1-praevalenz));
		npv_praev=(spez*(1-praevalenz))/((spez*(1-praevalenz))+((1-_sensit_)*praevalenz));
		psep=ppv_praev + npv_praev -1;
	RUN;

	DATA praev_result;
		SET praev_result praev_result&i;
	RUN;

	PROC DATASETS NOLIST;
		DELETE praev_result&i;
	RUN;
	QUIT;

%END;

/*Bemerkung hinzufgen*/

DATA praev_result;
	SET praev_result;
	IF _N_='1' THEN note='<-(im Datensatz)';
RUN;

/*Sortierung nach Praevalenz*/

PROC SORT DATA=praev_result;
	BY praevalenz;
RUN;

/*Ausgabe*/

PROC PRINT DATA=praev_result LABEL NOOBS;
	VAR praevalenz ppv_praev npv_praev psep note;
	LABEL praevalenz="Prvalenz von &resp_var=&event  n=&n"
		  note      ="Bemerkung"	
      	  ppv_praev ="Prvalenzabhngige positive Korrektheit" 
	      npv_praev ="Prvalenzabhngige negative Korrektheit"
		  psep	    ="Separation- Index PSEP";
TITLE 'Positive und negative Korrektheit in Abhngigkeit der Prvalenz der Zielgre';
RUN;

/*Zwischendatenstze lschen*/

PROC DATASETS NOLIST;
	DELETE praev_result;
RUN;
QUIT;
	
%END;

/*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++*/

%IF &all_cp=1 %THEN %DO;

/*Youden-Index, Sensitivitt und Spezifitt fr alle mglichen Cutpoints ausgeben*/

TITLE "Youden-Index, Sensitivitt und Spezifitt fr alle mglichen Cutpoints";
PROC PRINT DATA=youden LABEL NOOBS;
	VAR _SENSIT_ spez  _PROB_ youden ppv npv psep;
	LABEL _SENSIT_ 	='Sensitivitt'
	spez     		='Spezifitt'
	_PROB_  		='Cutpoints'
	youden   		='Youden-Index'
	ppv      		='positive Korrektheit'
	npv      		='negative Korrektheit'
	psep			='Separation- Index PSEP';
RUN;

%END;

%IF &auroc=1 %THEN %DO;

/*Fr AUROC*/

DATA pred2;
	SET pred1;
	RENAME x   			 =&label_predprobsart
	       &predprobsart2=z_&label_predprobsart2;
RUN;

/*input parameter auroc*/

%LET var		=&label_predprobsart z_&label_predprobsart2;
%LET event		=gruppe;
%LET data		=work.pred2;
%LET vardef		=;
%LET biascorr	=0;
%LET test		=0;

ODS SELECT NONE;
%AUROC;
ODS SELECT ALL;

proc print data=modwilse label;id __varnam;var __n0 __n1 w se_w ki_auc_u ki_auc_o ; 

%END;

%IF &printit=1 %THEN %DO;

%IF &rocdat=1 %THEN %DO;

/*input parameter rocdat:*/

%LET daten=work.pred1;

/*Model I oder II fr Makro Rocdat bestimmen*/

DATA model_test1;
	SET pred1;
	IF gruppe=0 THEN DELETE;
RUN;

%NUMBEROBS(datensatz=model_test1);

%LET model_test1=&nnnn1;

DATA model_test2;
	SET pred1;
	IF gruppe=1 THEN DELETE;
RUN;

%NUMBEROBS(datensatz=model_test2);

%LET model_test2=&nnnn1;

%IF &model_test1 > &model_test2 %THEN %DO;

%LET model=I;

%END;
%ELSE %DO;

%LET model=II;

%END;

%END;

/*input parameter printit*/

%LET cut_youden_data=&data_roc;
%LET outset1=work.graph1;
%LET outset2=work.graph2; 

%IF &cibinom=1 %THEN %DO;

%LET inset	=work.roc1;
%LET alpha	=&alpha;
%LET p		=estp;
%LET exlcl	=exlcl;
%LET exucl	=exucl;
%LET aslcl	=aslcl;
%LET asucl	=asucl;

%LET n		=n1;
%LET r		=_pos_;
%LET outset	=&outset1;

/*SENSITIVITT*/

%CIBINOM;

%LET n		=n2;
%LET r		=_neg_;
%LET outset	=&outset2;

/*SPEZIFITT*/

%CIBINOM;

%END;

%IF &rocdat=1 %THEN %DO;
%ROCDAT;
%END;

%PRINTIT (		
				data			=&datensatz,
				resp_var		=&resp_var,
				data_roc		=&data_roc,
				data_pred		=&data_pred,
				cibinom			=&cibinom,
				rocdat			=&rocdat,
				maxyouden		=&maxyouden,
				cutpoint		=&cutpoint,
				sensit			=&sensit,
				spez			=&spez,
				outset1			=&outset1,
				outset2			=&outset2,
				alpha			=&alpha,
				cl				=&cl,
				cut_youden_data	=&cut_youden_data,
				ext				=&ext,
				winkel			=&winkel,
				point			=&point,
				path			=&path,
				macro_path  	=&macro_path
          );

%END;


%IF &data_asso ne %THEN %DO;

/*Bearbeitung der bergebenen Datei in der die c-Statistik und SomersD enthalten ist*/

PROC TRANSPOSE DATA=&data_asso OUT=association1_ori;
	ID Label2;
	IDLABEL Label2;
RUN;

/*Variablen umbenennen und nicht mehr bentigte Variablen lschen*/
/*(c und Somers__D sind als Char formatiert -> mssen nach Int konvertiert werden)*/

DATA association1_ori;
	SET association1_ori;
		DROP _NAME_;
		auc_ori=(c*1);
		som_ori=(Somers__D*1);
		DROP c;
		DROP Somers__D;
		dummy=1;
RUN;

/*	Berechnung des Brier-Scores*/

DATA brier brier_ori (keep= anz summe brier_ori file);
	SET &data_pred end=last;
		retain summe;
		b=(&predprobsart-&resp_var);
		br=b**2;
		anz=_n_;
        if anz=1 then summe=br;
		   else       summe=summe+br;
		output brier;
	    if last then do;
           file="&data_pred";
           brier_ori=summe/anz;
           output brier_ori ;
		end;
RUN;

/*Dummy Variable zum spteren mergen hinzufgen*/

DATA brier_ori;
	SET brier_ori;
		dummy=1;
RUN;

%IF %UPCASE(&validation) ne NONE %THEN %DO;

/*emax / slope / intercept*/
/*neuer Vektor nach Harrell  (0-1 in 0.0005- Schritten) einbinden*/
/*(liegt im makro- Paket)*/
/*p's des Vektors mittels flag markiert*/

LIBNAME x  &macro_path;

/*Zielgre auf missing setzen*/
/*logit berechnen*/

DATA vektor;
	SET x.vektor;
	logit_p_stern=log(p_stern/(1-p_stern));
	&resp_var=.;
RUN;

/*Berechnung von Emax: zunchst logit(p) ausrechnen*/

DATA emax;
	SET &data_pred;
		logit=log(&predprobsart/(1-&predprobsart));
RUN;

/*zum Vektor, die Original p's dazumergen*/

DATA vektor2 (KEEP=p_stern logit_p_stern &resp_var);
	SET emax;
	RENAME logit=logit_p_stern;
	RENAME &predprobsart=p_stern;
RUN;

PROC SORT DATA=vektor2;
	BY p_stern;
RUN;

DATA vektor3;
	MERGE vektor vektor2;
	BY p_stern;
RUN;

/*Slope & Intercept extrahieren (mittels ODS)*/
/*erhlt man aus der ODS- Tabelle "Analysis Of Maximum Likelihood Estimates"*/

ODS LISTING CLOSE;
ODS OUTPUT ParameterEstimates=ParameterEstimates;

/*p-Value kalibrieren*/

PROC LOGISTIC DATA=vektor3 NAMELEN=32;
	MODEL &resp_var (

	%IF &eventx=1 %THEN
	%DO;
		event='1'
	%END;

	%IF &eventx=0 %THEN
	%DO;
		event='0'
	%END;

	)=logit_p_stern 
	
/*falls es sich um Crossvalidierung handelt, PROC LOGISTIC mit Option NOINT ausfhren*/
/*ansonsten (Bootstrap) mit Intercept*/

	/ BINWIDTH = &binwidth

	%IF %UPCASE(&validation)=CROSSVALIDATION %THEN %DO;

	NOINT

	%END;

	;
	OUTPUT OUT=pred_ori_c pred=p_corr;
RUN;

ODS OUTPUT CLOSE;
ODS LISTING;

/*ODS- Ausgabedatei bearbeiten und Slope/ Intercept extrahieren*/

DATA slope_intercept (KEEP=variable estimate);
	SET ParameterEstimates;
RUN;

PROC TRANSPOSE DATA=slope_intercept OUT=slope_intercept2;
RUN;

%IF %UPCASE(&validation)=CROSSVALIDATION %THEN %DO;

DATA slope_intercept2 (KEEP= slope dummy);
	SET slope_intercept2;
		RENAME col1=slope;
		dummy=1;
RUN;

%END;
%ELSE %DO;

DATA slope_intercept2 (KEEP= intercept slope dummy);
	SET slope_intercept2;
		RENAME col1=intercept;
		RENAME col2=slope;
		dummy=1;
RUN;

%END;

/*maximaler Fehler in den vorhergesagten Wahrscheinlichkeiten:*/
/*hierfr zunchst Differenz zwischen der vorhergesagten prob und der kalibrierten */
/*vorhergesagten prob*/

DATA emax1 (KEEP=p_stern logit_p_stern p_corr p_c);
	SET pred_ori_c;
	IF vektor_flag=1;
		p_c=abs(p_stern-p_corr);
RUN;

/*Maximum dieser Differenz ist Emax*/

PROC MEANS NOPRINT DATA=emax1 MAX;
	VAR p_c;
	OUTPUT OUT=emax2 MAX=emax_ori;
RUN;

/*Dummy Variable zum spteren mergen hinzufgen*/

DATA emax2;
	SET emax2;
	dummy=1;
RUN;

/*Datei mit Brier-Score und Datei mit Emax zusammenmergen*/

DATA brierEmax;
	MERGE brier_ori emax2;
	BY dummy;
RUN;

%IF %UPCASE(&validation)=CROSSVALIDATION %THEN %DO;

/*Diese Datei mit c-Statistik und Somers D zusammenmergen*/

DATA index_ori (KEEP= brier_ori auc_ori som_ori emax_ori dummy max_resca r_square slope);
	MERGE brierEmax association1_ori slope_intercept2;
	BY dummy;
RUN;

DATA index_ori (DROP=dummy);
	SET index_ori;
	RENAME max_resca 	= max_resca_ori;
	RENAME r_square 	= r_square_ori;
	RENAME slope		= slope_ori;
RUN;

/*Ausgabe der Ergebnisdatei*/

PROC PRINT DATA=index_ori NOOBS LABEL;
	TITLE 'Zusammenfassung von cutpoint- unabhngigen Prognosegtemaen';
	VAR auc_ori som_ori brier_ori slope_ori emax_ori r_square_ori max_resca_ori ;
	LABEL emax_ori		='Emax'
	      brier_ori		='Brier-Score'
		  auc_ori		='AUC'
		  som_ori		='Somers D'
		  max_resca_ori ='Max-rescaled R-Square'
		  r_square_ori  ='R-Square'	
		  slope_ori		='Slope'
;
RUN;

%END;
%ELSE %DO;

/*Diese Datei mit c-Statistik und Somers D zusammenmergen*/

DATA index_ori (KEEP= brier_ori auc_ori som_ori emax_ori dummy max_resca r_square slope intercept);
	MERGE brierEmax association1_ori slope_intercept2;
	BY dummy;
RUN;

DATA index_ori (DROP=dummy);
	SET index_ori;
	RENAME max_resca 	= max_resca_ori;
	RENAME r_square 	= r_square_ori;
	RENAME slope		= slope_ori;
	RENAME intercept	= intercept_ori;
RUN;

/*Ausgabe der Ergebnisdatei*/

PROC PRINT DATA=index_ori NOOBS LABEL;
	TITLE 'Zusammenfassung von cutpoint- unabhngigen Prognosegtemaen';
	VAR auc_ori som_ori brier_ori intercept_ori slope_ori emax_ori r_square_ori max_resca_ori ;
	LABEL emax_ori		='Emax'
	      brier_ori		='Brier-Score'
		  auc_ori		='AUC'
		  som_ori		='Somers D'
		  max_resca_ori ='Max-rescaled R-Square'
		  r_square_ori  ='R-Square'	
		  slope_ori		='Slope'
		  intercept_ori	='Intercept'
;
RUN;

%END;

%END;
%ELSE %DO;

/*Diese Datei mit c-Statistik und Somers D zusammenmergen*/
/*ohne emax, slope, intercept*/

DATA index_ori (KEEP= brier_ori auc_ori som_ori dummy max_resca r_square);
	MERGE brier_ori association1_ori;
	BY dummy;
RUN;

DATA index_ori (DROP=dummy);
	SET index_ori;
	RENAME max_resca 	= max_resca_ori;
	RENAME r_square 	= r_square_ori;
RUN;

/*Ausgabe der Ergebnisdatei*/

PROC PRINT DATA=index_ori NOOBS LABEL;
	TITLE 'Zusammenfassung von cutpoint- unabhngigen Prognosegtemaen';
	VAR auc_ori som_ori brier_ori r_square_ori max_resca_ori ;
	LABEL brier_ori		='Brier-Score'
		  auc_ori		='AUC'
		  som_ori		='Somers D'
		  max_resca_ori ='Max-rescaled R-Square'
		  r_square_ori  ='R-Square'	

;
RUN;

%END;

%END;

/*Temporre Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE youden youdenindex youden_sens_spez vierfeld;

	%IF &auroc=1 %THEN %DO;

    DELETE __wilse __sm __sesp __sc __s1 __s0  __r1 __r __nq __n1 __n0 __n __m __lang
		   __dcum __cmp __chi __c   __s modwilse pred2 pred1;

	%END;

	%IF &printit=1 %THEN %DO;

	DELETE model_test1 model_test2;

    %IF &cibinom=1 %THEN %DO;

    DELETE graph1 graph2 _tmp_ n1 n2 n1n2 roc1 ;

	%END;


	%IF &rocdat=1 %THEN %DO;

	DELETE rocnobs rocfn rocf0 roc1fn roc1f0 roc1f roc0fn roc0f0 roc0f roc roc0 plotdat;

	%END;

	%END;

	%IF &data_asso ne %THEN %DO;

		%IF %UPCASE(&validation) ne NONE %THEN %DO;

		DELETE  association1_ori brier brier_ori pred_ori_c
			    emax emax1 emax2 
			    vektor vektor2 vektor3
				brierEmax slope_intercept2 slope_intercept parameterestimates;
		%END;
		%ELSE %DO;

		DELETE  association1_ori brier brier_ori;

		%END;

	%END;

	RUN;
QUIT;

/*falls mi=1: Zusammenfassung aller Prognosegtemae in eine Datei (out_mi)*/

%IF &mi=1 %THEN %DO;

/*CP- unabhngige Prognosegtemae umbenennen*/

DATA tmp1;
	SET index_ori;
		RENAME brier_ori		=brier;
		%IF %UPCASE(&validation) ne NONE %THEN %DO;
		RENAME emax_ori			=emax;
		%IF %UPCASE(&validation) ne CROSSVALIDATION %THEN %DO;
		RENAME intercept_ori	=intercept;
		%END;
		RENAME slope_ori		=slope;
		%END;
		RENAME max_resca_ori	=max_resca;
		RENAME r_square_ori		=r_square;
		RENAME auc_ori			=auc;
		RENAME som_ori			=som;
		dummy=1;
RUN;

/*CP- abhngige Prognosegtemae umbenennen*/

DATA tmp2;
	SET result;
		RENAME _prob_			=cutpoint;
		RENAME _sensit_			=sens;
		dummy=1;
RUN;

/*Zusmammenfhren*/

DATA tmp3;
	MERGE tmp2 tmp1;
	BY dummy;
RUN;

/*dummy lschen und in gewnschte, angegebene Datei speichern*/

DATA &out_mi (DROP=dummy);
	SET tmp3;
RUN;

/*temporre Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE tmp1 tmp2 tmp3;
RUN;
QUIT;

%END;

/*Sprungmarke fr Fehlerfall:*/

%ende:

%MEND PM_ROC;
