/************************************************************************************/
/*																				  	*/
/*	NAME:		reclass.mac.sas														*/
/*  TITLE: 		Manuelle Erzeugung von ROC- Daten auf Basis der predicted values	*/
/*				(gleiche Funktion wie PROC LOGISTIC OPTION OUTROC=)					*/
/*  KEYS: 		roc analysis, logistic regression, predicted values, sensitivity,	*/
/*				specificity, classification, cutpoint								*/
/*	VERSION:	SAS 8.2																*/
/* 	PRODUCT:	BASE, STAT															*/
/*  PROCS:		FREQ																*/
/*																					*/
/*																					*/
/*	Verzeichnis/Dateiname:			F:\Prognose\reclass.mac.sas						*/
/*  Autoren:						R. Muche, C. Ring, C. Ziegler					*/
/*	Programmierer:					Christoph Ziegler								*/
/*	Datum der Ersterstellung:		4/2003											*/
/*  Aktuelle Makro- Version:		12/2003											*/
/*																					*/
/*	Kurzbeschreibung:				Allgemeines Makro zur manuellen Erzeugung von	*/
/*							    	ROC- Daten auf Basis der predicted values		*/
/*									(gleiche Funktion wie PROC LOGISTIC OPTION		*/
/*									OUTROC=)										*/
/*																					*/
/*	Hilfs-/Untermakro von:			JACKKNIFE_CROSSVALIDATION.MAC.SAS				*/
/*																					*/
/*  Verwendete externe Untermakros:	numberobs.mac.sas								*/
/*																					*/
/*  Bentigte Dateien:				Datei mit predicted values sowie Zielgre		*/
/*									(Parameter 'pred_data=')						*/
/*																					*/
/*  Erzeugte Dateien:				work.ergebnis									*/
/*																					*/
/*  Programmnderungen 																*/
/*  und -ergnzungen 																*/
/*  (Name, Datum, Beschreibung):     												*/
/*   																   			    */
/*																					*/
/************************************************************************************/

%MACRO RECLASS (	macro_path	=,
					pred_data	=,
					resp_var	=,
					prob		=
			   );

/*Verzeichnis referenzieren*/

FILENAME y &macro_path;

/*Verwendete Untermakros referenzieren*/

%INCLUDE y ('numberobs.mac.sas');

/*Dateien initialisieren*/

DATA y_index;
	SET _NULL_;
RUN;

/*Probabilities und event- Spalte aus Datei selektieren*/

DATA roc (KEEP=&prob event);
	SET &pred_data;
RUN;

/*Datei nach den Wahrscheinlichkeiten sortieren*/

PROC SORT DATA = roc;
	BY DESCENDING &prob;
RUN;

/*Beobachtungsnummern hinzufgen*/

DATA roc;
	SET roc;
	__obs__=_N_;
RUN;

/*Anzahl Beobachtungen bestimmen*/

%numberobs(datensatz=roc);

%LET zaehler=&nnnn1;

/*Start der Schleife*/

%DO i=1 %TO &zaehler;

/*aktuelle Beobachtungsnummer selektieren*/

DATA _tmp_;
	SET roc;
	IF _N_ ne &i THEN DELETE;
RUN;

DATA cutpoints;
	SET roc;
    IF __obs__ = &i THEN p_cp = 0;
	IF __obs__ > &i THEN p_cp = 0;
    ELSE p_cp = 1;
RUN;

/*Hufigkeitsauszhlung*/

PROC FREQ DATA= cutpoints NOPRINT;
	TABLES p_cp * &resp_var / OUT= roc1;
RUN;

/*POS, NEG, FALPOS, FALNEG bestimmen*/

DATA positiv; 
	SET ROC1;
	IF 		(p_cp = 1 AND &resp_var = 1) THEN positiv = count;
	ELSE DELETE;
RUN;

DATA negativ; 
    SET ROC1;
	IF 		(p_cp = 0 AND &resp_var = 0) THEN negativ = count;
	ELSE DELETE;
RUN;

DATA false_pos; 
    SET ROC1;
	IF 		(p_cp = 1 AND &resp_var = 0) THEN false_pos = count;
	ELSE DELETE;
RUN;

DATA false_neg; 
    SET ROC1;
	IF 		(p_cp = 0 AND &resp_var = 1) THEN false_neg = count;
	ELSE DELETE;
RUN;

/*Dateien zusammenfhren*/

DATA tmp_y_index (DROP= p_cp EVENT COUNT PERCENT); 
	MERGE positiv negativ false_pos false_neg;
RUN;

/*An Ergebnisdatei anhngen*/

DATA y_index;
   	SET y_index tmp_y_index;
RUN;

/*Log- Fenster lschen um berlauf zu vermeiden*/

DM "clear log;";

/*Ende der Schleife*/

%END;

/*Spalten umbenennen*/

DATA y_index;
	SET y_index;
	RENAME   positiv=_POS_;
	RENAME	 negativ=_NEG_;
	RENAME	 false_pos=_FALPOS_;
	RENAME	 false_neg=_FALNEG_;
RUN;

/*Missing values auf Null setzen*/

DATA y_index;
	SET y_index;
	IF  _POS_    ='.' THEN _POS_    = 0;
	IF	_NEG_    ='.' THEN _NEG_    = 0;
	IF	_FALPOS_ ='.' THEN _FALPOS_ = 0;
	IF	_FALNEG_ ='.' THEN _FALNEG_ = 0;
RUN;

/*Sens / Spez berechnen*/

DATA y_index;
	SET y_index;
	_SENSIT_=(_POS_ / (_POS_ + _FALNEG_));
	_SPEC_  =(_NEG_ / (_FALPOS_ + _NEG_));
	_1MSPEC_=1-_SPEC_;
RUN;

/*# Beobachtungen hinzufgen um spter zu mergen*/

DATA roc;
	SET roc;
	_obs_=_N_;
RUN;

/*# Beobachtungen hinzufgen um spter zu mergen*/

DATA y_index;
	SET y_index;
	_obs_=_N_;
RUN;

DATA ergebnis (KEEP=&prob _POS_ _NEG_ _FALPOS_ _FALNEG_ _SENSIT_ _1MSPEC_);
	MERGE roc y_index;
	BY _obs_;
RUN;

/*Ergebnisdatei*/

DATA ergebnis;
	SET ergebnis;
	RENAME &prob=_PROB_;
RUN;

/*temporre Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE _tmp_ y_index tmp_y_index roc1 positiv negativ false_pos false_neg cutpoints roc;
RUN;
QUIT;

%MEND RECLASS;